@@ -1,5 +1,5 @@
 /**************************************************************************
-  Copyright (c) 2004-2013, Cray Inc.  (See LICENSE file for more details)
+  Copyright (c) 2004-2014, Cray Inc.  (See LICENSE file for more details)
 **************************************************************************/
 
 
@@ -12,6 +12,11 @@
 #include "type.h"
 
 
+static void pruneUnusedAggregateTypes(Vec<TypeSymbol*>& types);
+static void pruneUnusedRefs(Vec<TypeSymbol*>& types);
+static void changeDeadTypesToVoid(Vec<TypeSymbol*>& types);
+
+
 void collectFnCalls(BaseAST* ast, Vec<CallExpr*>& calls) {
   AST_CHILDREN_CALL(ast, collectFnCalls, calls);
   if (CallExpr* call = toCallExpr(ast))
@@ -353,7 +358,8 @@ bool isOpEqualPrim(CallExpr* call) {
 //
 int isDefAndOrUse(SymExpr* se) {
   if (CallExpr* call = toCallExpr(se->parentExpr)) {
-    if (call->isPrimitive(PRIM_MOVE) && call->get(1) == se) {
+    if ((call->isPrimitive(PRIM_MOVE) || call->isPrimitive(PRIM_ASSIGN)) &&
+        call->get(1) == se) {
       return 1;
     } else if (isOpEqualPrim(call) && call->get(1) == se) {
       return 3;
@@ -582,6 +588,12 @@ visitVisibleFunctions(Vec<FnSymbol*>& fns, Vec<TypeSymbol*>& types)
   // chpl_gen_main is always visible (if it exists).
   pruneVisit(chpl_gen_main, fns, types);
 
+  // When present, the printModuleInitOrder function is always visible;
+  // it will be NULL for --minimal-modules compilations
+  if (gPrintModuleInitFn) {
+    pruneVisit(gPrintModuleInitFn, fns, types);
+  }
+
   // Functions appearing the function pointer table are visible.
   // These are blocks that can be started through a forall, coforall or on statement.
   forv_Vec(FnSymbol, fn, ftableVec)
@@ -605,59 +617,106 @@ visitVisibleFunctions(Vec<FnSymbol*>& fns, Vec<TypeSymbol*>& types)
 
 static void
 pruneUnusedTypes(Vec<TypeSymbol*>& types)
+{
+
+  pruneUnusedAggregateTypes(types);
+  pruneUnusedRefs(types);
+  changeDeadTypesToVoid(types);
+}
+
+
+static void pruneUnusedAggregateTypes(Vec<TypeSymbol*>& types)
 {
   //
-  // delete unused ClassType types, only deleting references to such
+  // delete unused AggregateType types, only deleting references to such
   // types when the value types are deleted
   //
-  forv_Vec(TypeSymbol, ts, gTypeSymbols) {
+  forv_Vec(TypeSymbol, ts, gTypeSymbols)
+  {
+    // Do not delete reference types.
+    // We delete the reference type if the base type is dead (below).
+    if (ts->hasFlag(FLAG_REF))
+      continue;
+
     // Ignore types flagged as primitive.
     if (ts->hasFlag(FLAG_PRIMITIVE_TYPE))
       continue;
 
+    // Ignore types that are not class types
+    if (!isAggregateType(ts->type))
+      continue;
+
+    // Do not delete the object class.  It's pretty crucial.
+    if (ts->hasFlag(FLAG_OBJECT_CLASS))
+      continue;
+
     // Visit only those types not marked as visible.
     if (types.set_in(ts))
       continue;
 
-    if (isClassType(ts->type)) {
-      //
-      // delete reference types for classes/records/unions only if
-      // deleting value types
-      //
-      if (ts->hasFlag(FLAG_REF) && isClassType(ts->getValType()))
+    ts->defPoint->remove();
+  }
+}
+
+
+static void pruneUnusedRefs(Vec<TypeSymbol*>& types)
+{
+  forv_Vec(TypeSymbol, ts, gTypeSymbols)
+  {
+    // This pass, we are interested only in ref types.
+    if (!ts->hasFlag(FLAG_REF))
+      continue;
+
+    // Ignore types flagged as primitive.
+    if (ts->hasFlag(FLAG_PRIMITIVE_TYPE))
+      continue;
+
+    // Ignore types that are not class types
+    if (!isAggregateType(ts->type))
+    {
+      // hilde sez: Ref types are always class types.
+      // So we can't get here.
+      INT_ASSERT(false);
+      continue;
+    }
+
+    // Visit only those types not marked as visible.
+    if (types.set_in(ts))
+      continue;
+
+    if (Type* vt = ts->getValType())
+    {
+      // Don't remove a ref type if it refers to a class type
+      if (isAggregateType(vt))
+        // and the class type is still alive.
+        if (types.set_in(vt->symbol))
+          continue;
+
+      // Don't delete nil ref as it is used in widening.
+      if (vt == dtNil)
         continue;
 
-      //
-      // delete reference type if type is not used
-      //
-      if (!ts->hasFlag(FLAG_REF))
-        if (Type* refType = ts->getRefType())
-          refType->symbol->defPoint->remove();
-
-      //
-      // unlink reference type from value type
-      //
-      if (ts->hasFlag(FLAG_REF)) {
-        if (Type* vt = ts->getValType()) {
-          if (vt == dtNil) // don't delete nil ref as it is used when widening
-            continue;
-          vt->refType = NULL;
-        }
-      }
-
-      ts->defPoint->remove();
+      // Unlink reference type from value type.
+      vt->refType = NULL;
     }
-  }
 
+    ts->defPoint->remove();
+  }
+}
+
+
+static void changeDeadTypesToVoid(Vec<TypeSymbol*>& types)
+{
   //
   // change symbols with dead types to void (important for baseline)
   //
   forv_Vec(DefExpr, def, gDefExprs) {
-    if (def->parentSymbol && def->sym->type && isClassType(def->sym->type) && !isTypeSymbol(def->sym) && !types.set_in(def->sym->type->symbol))
+    if (def->parentSymbol && def->sym->type && isAggregateType(def->sym->type) && !isTypeSymbol(def->sym) && !types.set_in(def->sym->type->symbol))
       def->sym->type = dtVoid;
   }
 }
 
+
 // Done this way because the log letter and hence the pass name for
 // each pass must be unique.  See initLogFlags() in runpasses.cpp.
 void prune2() { prune(); } // Synonym for prune.
