@@ -1,5 +1,5 @@
 /**************************************************************************
-  Copyright (c) 2004-2013, Cray Inc.  (See LICENSE file for more details)
+  Copyright (c) 2004-2014, Cray Inc.  (See LICENSE file for more details)
 **************************************************************************/
 
 
@@ -101,6 +101,7 @@ static void addPragmaFlags(Symbol* sym, Vec<const char*>* pragmas) {
         INT_ASSERT(mod);
         INT_ASSERT(mod->initFn);
         mod->initFn->addFlag(FLAG_EXPORT);
+        mod->initFn->addFlag(FLAG_LOCAL_ARGS);
       } else if (flag == FLAG_RUNTIME_TYPE_INIT_FN) {
         //
         // These functions must be marked as type functions early in
@@ -175,16 +176,16 @@ Expr* buildFormalArrayType(Expr* iterator, Expr* eltType, Expr* index) {
     if (indexCall->numActuals() != 1)
       USR_FATAL(iterator, "invalid index expression");
     return new CallExpr("chpl__buildArrayRuntimeType",
-             new CallExpr("chpl__buildDomainExpr", iterator),
+             new CallExpr("chpl__ensureDomainExpr", iterator),
              eltType, indexCall->get(1)->remove(),
-             new CallExpr("chpl__buildDomainExpr", iterator->copy()));
+             new CallExpr("chpl__ensureDomainExpr", iterator->copy()));
   } else {
     CallExpr* call = toCallExpr(iterator);
     if (call->numActuals() == 1 && isDefExpr(call->get(1))) {
       return new CallExpr("chpl__buildArrayRuntimeType", call->get(1)->remove(), eltType);
     } else
       return new CallExpr("chpl__buildArrayRuntimeType",
-               new CallExpr("chpl__buildDomainExpr", iterator), eltType);
+               new CallExpr("chpl__ensureDomainExpr", iterator), eltType);
   }
 }
 
@@ -435,13 +436,6 @@ void createInitFn(ModuleSymbol* mod) {
   mod->initFn->retType = dtVoid;
   mod->initFn->addFlag(FLAG_MODULE_INIT);
   mod->initFn->addFlag(FLAG_INSERT_LINE_FILE_INFO);
-  // All module initialization functions should be exported.
-  // But that means we have to adopt some new naming conventions.
-  //
-  // BLC: Actually, I think we should only be exporting module
-  // initialization functions for modules that have exported symbols.
-  //
-  // mod->initFn->addFlag(FLAG_EXPORT);
 
   //
   // move module-level statements into module's init function
@@ -541,11 +535,11 @@ BlockStmt* buildWhileDoLoopStmt(Expr* cond, BlockStmt* body) {
   body = new BlockStmt(body);
   body->blockInfo = new CallExpr(PRIM_BLOCK_WHILEDO_LOOP, condVar);
   LabelSymbol* continueLabel = new LabelSymbol("_continueLabel");
-  continueLabel->addFlag(FLAG_TEMP);
+  continueLabel->addFlag(FLAG_COMPILER_GENERATED);
   continueLabel->addFlag(FLAG_LABEL_CONTINUE);
   body->continueLabel = continueLabel;
   LabelSymbol* breakLabel = new LabelSymbol("_breakLabel");
-  breakLabel->addFlag(FLAG_TEMP);
+  breakLabel->addFlag(FLAG_COMPILER_GENERATED);
   breakLabel->addFlag(FLAG_LABEL_BREAK);
   body->breakLabel = breakLabel;
   body->insertAtTail(new DefExpr(continueLabel));
@@ -571,10 +565,10 @@ BlockStmt* buildDoWhileLoopStmt(Expr* cond, BlockStmt* body) {
   }
 
   LabelSymbol* continueLabel = new LabelSymbol("_continueLabel");
-  continueLabel->addFlag(FLAG_TEMP);
+  continueLabel->addFlag(FLAG_COMPILER_GENERATED);
   continueLabel->addFlag(FLAG_LABEL_CONTINUE);
   LabelSymbol* breakLabel = new LabelSymbol("_breakLabel");
-  breakLabel->addFlag(FLAG_TEMP);
+  breakLabel->addFlag(FLAG_COMPILER_GENERATED);
   breakLabel->addFlag(FLAG_LABEL_BREAK);
   BlockStmt* block = new BlockStmt(body);
   block->continueLabel = continueLabel;
@@ -592,18 +586,13 @@ BlockStmt* buildDoWhileLoopStmt(Expr* cond, BlockStmt* body) {
 
 BlockStmt* buildSerialStmt(Expr* cond, BlockStmt* body) {
   cond = new CallExpr("_cond_test", cond);
-  if (fSerial) {
-    body->insertAtHead(cond);
-    return body;
-  } else {
-    BlockStmt *sbody = new BlockStmt();
-    VarSymbol *serial_state = newTemp();
-    sbody->insertAtTail(new DefExpr(serial_state, new CallExpr(PRIM_GET_SERIAL)));
-    sbody->insertAtTail(new CondStmt(cond, new CallExpr(PRIM_SET_SERIAL, gTrue)));
-    sbody->insertAtTail(body);
-    sbody->insertAtTail(new CallExpr(PRIM_SET_SERIAL, serial_state));
-    return sbody;
-  }
+  BlockStmt *sbody = new BlockStmt();
+  VarSymbol *serial_state = newTemp();
+  sbody->insertAtTail(new DefExpr(serial_state, new CallExpr(PRIM_GET_SERIAL)));
+  sbody->insertAtTail(new CondStmt(cond, new CallExpr(PRIM_SET_SERIAL, gTrue)));
+  sbody->insertAtTail(body);
+  sbody->insertAtTail(new CallExpr(PRIM_SET_SERIAL, serial_state));
+  return sbody;
 }
 
 
@@ -726,7 +715,7 @@ handleArrayTypeCase(FnSymbol* fn, Expr* indices, Expr* iteratorExpr, Expr* expr)
   // counting domains within runtime array types
   thenStmt->insertAtTail(new CallExpr(PRIM_MOVE, domain,
                            new CallExpr("chpl__autoCopy",
-                             new CallExpr("chpl__buildDomainExpr",
+                             new CallExpr("chpl__ensureDomainExpr",
                                           iteratorExpr->copy()))));
   if (hasSpecifiedIndices) {
     // we want to swap something like the below commented-out
@@ -784,11 +773,95 @@ buildForLoopExpr(Expr* indices, Expr* iteratorExpr, Expr* expr, Expr* cond, bool
 }
 
 
+//
+// build serial iterator function
+//
+static void buildSerialIteratorFn(FnSymbol* fn, const char* iteratorName,
+                                  Expr* expr, Expr* cond, Expr* indices,
+                                  bool zippered, Expr*& stmt)
+{
+  FnSymbol* sifn = new FnSymbol(iteratorName);
+  sifn->addFlag(FLAG_ITERATOR_FN);
+  ArgSymbol* sifnIterator = new ArgSymbol(INTENT_BLANK, "iterator", dtAny);
+  sifn->insertFormalAtTail(sifnIterator);
+  fn->insertAtHead(new DefExpr(sifn));
+  stmt = new CallExpr(PRIM_YIELD, expr);
+  if (cond)
+    stmt = new CondStmt(new CallExpr("_cond_test", cond), stmt);
+  sifn->insertAtTail(buildForLoopStmt(indices, new SymExpr(sifnIterator), new BlockStmt(stmt), false, zippered));
+}
+
+
+//
+// build leader iterator function
+//
+static void buildLeaderIteratorFn(FnSymbol* fn, const char* iteratorName,
+                                  bool zippered)
+{
+  FnSymbol* lifn = new FnSymbol(iteratorName);
+
+  Expr* tag = buildDotExpr(buildDotExpr(new UnresolvedSymExpr("ChapelBase"),
+                                        iterKindTypename),
+                           iterKindLeaderTagname);
+  ArgSymbol* lifnTag = new ArgSymbol(INTENT_PARAM, "tag", dtUnknown,
+                                     new CallExpr(PRIM_TYPEOF, tag));
+  lifn->insertFormalAtTail(lifnTag);
+  ArgSymbol* lifnIterator = new ArgSymbol(INTENT_BLANK, "iterator", dtAny);
+  lifn->insertFormalAtTail(lifnIterator);
+
+  lifn->where = new BlockStmt(new CallExpr("==", lifnTag, tag->copy()));
+  fn->insertAtHead(new DefExpr(lifn));
+
+  VarSymbol* leaderIterator = newTemp("_leaderIterator");
+  leaderIterator->addFlag(FLAG_EXPR_TEMP);
+  lifn->insertAtTail(new DefExpr(leaderIterator));
+  
+  if( !zippered ) {
+    lifn->insertAtTail(new CallExpr(PRIM_MOVE, leaderIterator, new CallExpr("_toLeader", lifnIterator)));
+  } else {
+    lifn->insertAtTail(new CallExpr(PRIM_MOVE, leaderIterator, new CallExpr("_toLeaderZip", lifnIterator)));
+  }
+
+  lifn->insertAtTail(new CallExpr(PRIM_RETURN, leaderIterator));
+}
+
+
+//
+// build follower iterator function
+//
+static FnSymbol* buildFollowerIteratorFn(FnSymbol* fn, const char* iteratorName,
+                                         bool zippered, VarSymbol*& followerIterator)
+{
+  FnSymbol* fifn = new FnSymbol(iteratorName);
+  fifn->addFlag(FLAG_ITERATOR_FN);
+
+  Expr* tag = buildDotExpr(buildDotExpr(new UnresolvedSymExpr("ChapelBase"),
+                                  iterKindTypename), iterKindFollowerTagname);
+  ArgSymbol* fifnTag = new ArgSymbol(INTENT_PARAM, "tag", dtUnknown,
+                                     new CallExpr(PRIM_TYPEOF, tag));
+  fifn->insertFormalAtTail(fifnTag);
+  ArgSymbol* fifnFollower = new ArgSymbol(INTENT_BLANK, iterFollowthisArgname, dtAny);
+  fifn->insertFormalAtTail(fifnFollower);
+  ArgSymbol* fifnIterator = new ArgSymbol(INTENT_BLANK, "iterator", dtAny);
+  fifn->insertFormalAtTail(fifnIterator);
+
+  fifn->where = new BlockStmt(new CallExpr("==", fifnTag, tag->copy()));
+  fn->insertAtHead(new DefExpr(fifn));
+  followerIterator = newTemp("_followerIterator");
+  followerIterator->addFlag(FLAG_EXPR_TEMP);
+  fifn->insertAtTail(new DefExpr(followerIterator));
+
+  if( !zippered ) {
+    fifn->insertAtTail(new CallExpr(PRIM_MOVE, followerIterator, new CallExpr("_toFollower", fifnIterator, fifnFollower)));
+  } else {
+    fifn->insertAtTail(new CallExpr(PRIM_MOVE, followerIterator, new CallExpr("_toFollowerZip", fifnIterator, fifnFollower)));
+  }
+  return fifn;
+}
+
+
 CallExpr*
 buildForallLoopExpr(Expr* indices, Expr* iteratorExpr, Expr* expr, Expr* cond, bool maybeArrayType, bool zippered) {
-  if (fSerial || fSerialForall)
-    return buildForLoopExpr(indices, iteratorExpr, expr, cond, maybeArrayType, zippered);
-
   FnSymbol* fn = new FnSymbol(astr("_parloopexpr", istr(loopexpr_uid++)));
   BlockStmt* block = fn->body;
 
@@ -804,77 +877,18 @@ buildForallLoopExpr(Expr* indices, Expr* iteratorExpr, Expr* expr, Expr* cond, b
   const char* iteratorName = astr("_iterator_for_loopexpr", istr(loopexpr_uid-1));
   block->insertAtTail(new CallExpr(PRIM_RETURN, new CallExpr(iteratorName, iterator)));
 
-  //
-  // build serial iterator function
-  //
-  FnSymbol* sifn = new FnSymbol(iteratorName);
-  sifn->addFlag(FLAG_ITERATOR_FN);
-  ArgSymbol* sifnIterator = new ArgSymbol(INTENT_BLANK, "iterator", dtAny);
-  sifn->insertFormalAtTail(sifnIterator);
-  fn->insertAtHead(new DefExpr(sifn));
-  Expr* stmt = new CallExpr(PRIM_YIELD, expr);
-  if (cond)
-    stmt = new CondStmt(new CallExpr("_cond_test", cond), stmt);
-  sifn->insertAtTail(buildForLoopStmt(indices, new SymExpr(sifnIterator), new BlockStmt(stmt), false, zippered));
-
-  //
-  // build leader iterator function
-  //
-  FnSymbol* lifn = new FnSymbol(iteratorName);
-  ArgSymbol* lifnIterator = new ArgSymbol(INTENT_BLANK, "iterator", dtAny);
-  lifn->insertFormalAtTail(lifnIterator);
-  Expr* tag = buildDotExpr(buildDotExpr(new UnresolvedSymExpr("ChapelBase"),
-                                        iterKindTypename),
-                           iterKindLeaderTagname);
-  ArgSymbol* lifnTag = new ArgSymbol(INTENT_PARAM, "tag", dtUnknown,
-                                     new CallExpr(PRIM_TYPEOF, tag));
-  lifn->insertFormalAtTail(lifnTag);
-  lifn->where = new BlockStmt(new CallExpr("==", lifnTag, tag->copy()));
-  fn->insertAtHead(new DefExpr(lifn));
-  VarSymbol* leaderIterator = newTemp("_leaderIterator");
-  leaderIterator->addFlag(FLAG_EXPR_TEMP);
-  lifn->insertAtTail(new DefExpr(leaderIterator));
-  
-  if( !zippered ) {
-    lifn->insertAtTail(new CallExpr(PRIM_MOVE, leaderIterator, new CallExpr("_toLeader", lifnIterator)));
-  } else {
-    lifn->insertAtTail(new CallExpr(PRIM_MOVE, leaderIterator, new CallExpr("_toLeaderZip", lifnIterator)));
-  }
-
-  lifn->insertAtTail(new CallExpr(PRIM_RETURN, leaderIterator));
-
-  //
-  // build follower iterator function
-  //
-  FnSymbol* fifn = new FnSymbol(iteratorName);
-  fifn->addFlag(FLAG_ITERATOR_FN);
-  ArgSymbol* fifnIterator = new ArgSymbol(INTENT_BLANK, "iterator", dtAny);
-  fifn->insertFormalAtTail(fifnIterator);
-
-  tag = buildDotExpr(buildDotExpr(new UnresolvedSymExpr("ChapelBase"),
-                                  iterKindTypename), iterKindFollowerTagname);
-  ArgSymbol* fifnTag = new ArgSymbol(INTENT_PARAM, "tag", dtUnknown,
-                                     new CallExpr(PRIM_TYPEOF, tag));
-  fifn->insertFormalAtTail(fifnTag);
-  ArgSymbol* fifnFollower = new ArgSymbol(INTENT_BLANK, iterFollowthisArgname, dtAny);
-  fifn->insertFormalAtTail(fifnFollower);
-  fifn->where = new BlockStmt(new CallExpr("==", fifnTag, tag->copy()));
-  fn->insertAtHead(new DefExpr(fifn));
-  VarSymbol* followerIterator = newTemp("_followerIterator");
-  followerIterator->addFlag(FLAG_EXPR_TEMP);
-  fifn->insertAtTail(new DefExpr(followerIterator));
-
-  if( !zippered ) {
-    fifn->insertAtTail(new CallExpr(PRIM_MOVE, followerIterator, new CallExpr("_toFollower", fifnIterator, fifnFollower)));
-  } else {
-    fifn->insertAtTail(new CallExpr(PRIM_MOVE, followerIterator, new CallExpr("_toFollowerZip", fifnIterator, fifnFollower)));
-  }
+  Expr* stmt; // Initialized by buildSerialIteratorFn.
+  buildSerialIteratorFn(fn, iteratorName, expr, cond, indices, zippered, stmt);
+  buildLeaderIteratorFn(fn, iteratorName, zippered);
+  VarSymbol* followerIterator; // Initialized by buildFollowerIteratorFn.
+  FnSymbol* fifn = buildFollowerIteratorFn(fn, iteratorName, zippered, followerIterator);
 
   // do we need to use this map since symbols have not been resolved?
   SymbolMap map;
   Expr* indicesCopy = (indices) ? indices->copy(&map) : NULL;
   Expr* bodyCopy = stmt->copy(&map);
   fifn->insertAtTail(buildForLoopStmt(indicesCopy, new SymExpr(followerIterator), new BlockStmt(bodyCopy), false, zippered));
+
   return new CallExpr(new DefExpr(fn));
 }
 
@@ -934,11 +948,11 @@ BlockStmt* buildForLoopStmt(Expr* indices,
   body = new BlockStmt(body);
   BlockStmt* stmts = buildChapelStmt();
   LabelSymbol* continueLabel = new LabelSymbol("_continueLabel");
-  continueLabel->addFlag(FLAG_TEMP);
+  continueLabel->addFlag(FLAG_COMPILER_GENERATED);
   continueLabel->addFlag(FLAG_LABEL_CONTINUE);
   body->continueLabel = continueLabel;
   LabelSymbol* breakLabel = new LabelSymbol("_breakLabel");
-  breakLabel->addFlag(FLAG_TEMP);
+  breakLabel->addFlag(FLAG_COMPILER_GENERATED);
   breakLabel->addFlag(FLAG_LABEL_BREAK);
   body->breakLabel = breakLabel;
 
@@ -1018,9 +1032,6 @@ buildForallLoopStmt(Expr* indices,
                     bool zippered) {
   checkControlFlow(loopBody, "forall statement");
 
-  if (fSerial || fSerialForall)
-    return buildForLoopStmt(indices, iterExpr, loopBody, false, zippered);
-
   SET_LINENO(loopBody);
 
   //
@@ -1124,10 +1135,6 @@ BlockStmt* buildCoforallLoopStmt(Expr* indices,
 {
   checkControlFlow(body, "coforall statement");
 
-  if (fSerial)
-    // dropping byref_vars, if any
-    return buildForLoopStmt(indices, iterator, body, false, zippered);
-
   //
   // insert temporary index when elided by user
   //
@@ -1173,7 +1180,7 @@ BlockStmt* buildCoforallLoopStmt(Expr* indices,
     body->insertAtHead(new CallExpr("_upEndCount", coforallCount));
     block->insertAtTail(new CallExpr("_waitEndCount", coforallCount));
     block->insertAtTail(new CallExpr("_endCountFree", coforallCount));
-    onBlock->blockInfo->primitive = primitives[PRIM_BLOCK_ON_NB];
+    onBlock->blockInfo->primitive = primitives[PRIM_BLOCK_COFORALL_ON];
     addByrefVars(onBlock, byref_vars);
     BlockStmt* innerOnBlock = new BlockStmt();
     for_alist(tmp, onBlock->body) {
@@ -1229,6 +1236,13 @@ BlockStmt* buildParamForLoopStmt(const char* index, Expr* range, BlockStmt* stmt
     high = call->get(1)->remove();
   } else
     USR_FATAL(range, "iterators for param-for-loops must be literal ranges");
+
+  LabelSymbol* breakLabel = new LabelSymbol("_breakLabel");
+  breakLabel->addFlag(FLAG_COMPILER_GENERATED);
+  breakLabel->addFlag(FLAG_LABEL_BREAK);
+  block->breakLabel = breakLabel;
+  outer->insertAtTail(new DefExpr(breakLabel));
+
   Symbol* lowVar = insertBeforeCompilerTemp(block, low);
   Symbol* highVar = insertBeforeCompilerTemp(block, high);
   Symbol* strideVar = insertBeforeCompilerTemp(block, stride);
@@ -1422,60 +1436,56 @@ CallExpr* buildReduceExpr(Expr* opExpr, Expr* dataExpr, bool zippered) {
   serialBlock->insertAtTail(new DefExpr(index));
   serialBlock->insertAtTail(buildForLoopStmt(new SymExpr(index), new SymExpr(data), new BlockStmt(new CallExpr(new CallExpr(".", globalOp, new_StringSymbol("accumulate")), index)), false, zippered));
 
-  if (fSerial || fSerialForall) {
-    fn->insertAtTail(serialBlock);
+  VarSymbol* leadIdx = newTemp("chpl__leadIdx");
+  VarSymbol* leadIter = newTemp("chpl__leadIter");
+  VarSymbol* leadIdxCopy = newTemp("chpl__leadIdxCopy");
+  VarSymbol* followIdx = newTemp("chpl__followIdx");
+  VarSymbol* followIter = newTemp("chpl__followIter");
+  VarSymbol* localOp = newTemp();
+  leadIdxCopy->addFlag(FLAG_INDEX_VAR);
+  leadIdxCopy->addFlag(FLAG_INSERT_AUTO_DESTROY);
+  BlockStmt* followBody = new BlockStmt();
+  followBody->insertAtTail(".(%S, 'accumulate')(%S)", localOp, followIdx);
+  followBody->blockInfo = new CallExpr(PRIM_BLOCK_FOR_LOOP, followIdx, followIter);
+  BlockStmt* followBlock = new BlockStmt();
+  followBlock->insertAtTail(new DefExpr(followIter));
+  followBlock->insertAtTail(new DefExpr(followIdx));
+  followBlock->insertAtTail(new DefExpr(localOp));
+
+  if( !zippered ) {
+    followBlock->insertAtTail("'move'(%S, _getIterator(_toFollower(%S, %S)))", followIter, data, leadIdxCopy);
   } else {
-    VarSymbol* leadIdx = newTemp("chpl__leadIdx");
-    VarSymbol* leadIter = newTemp("chpl__leadIter");
-    VarSymbol* leadIdxCopy = newTemp("chpl__leadIdxCopy");
-    VarSymbol* followIdx = newTemp("chpl__followIdx");
-    VarSymbol* followIter = newTemp("chpl__followIter");
-    VarSymbol* localOp = newTemp();
-    leadIdxCopy->addFlag(FLAG_INDEX_VAR);
-    leadIdxCopy->addFlag(FLAG_INSERT_AUTO_DESTROY);
-    BlockStmt* followBody = new BlockStmt();
-    followBody->insertAtTail(".(%S, 'accumulate')(%S)", localOp, followIdx);
-    followBody->blockInfo = new CallExpr(PRIM_BLOCK_FOR_LOOP, followIdx, followIter);
-    BlockStmt* followBlock = new BlockStmt();
-    followBlock->insertAtTail(new DefExpr(followIter));
-    followBlock->insertAtTail(new DefExpr(followIdx));
-    followBlock->insertAtTail(new DefExpr(localOp));
-
-    if( !zippered ) {
-      followBlock->insertAtTail("'move'(%S, _getIterator(_toFollower(%S, %S)))", followIter, data, leadIdxCopy);
-    } else {
-      followBlock->insertAtTail("'move'(%S, _getIteratorZip(_toFollowerZip(%S, %S)))", followIter, data, leadIdxCopy);
-    }
-    
-    followBlock->insertAtTail("{TYPE 'move'(%S, iteratorIndex(%S))}", followIdx, followIter);
-    followBlock->insertAtTail("'move'(%S, 'new'(%E(%E)))", localOp, opExpr->copy(), new NamedExpr("eltType", new SymExpr(eltType)));
-    followBlock->insertAtTail(followBody);
-    followBlock->insertAtTail("chpl__reduceCombine(%S, %S)", globalOp, localOp);
-    followBlock->insertAtTail("'delete'(%S)", localOp);
-    followBlock->insertAtTail("_freeIterator(%S)", followIter);
-    BlockStmt* leadBody = new BlockStmt();
-    leadBody->insertAtTail(new DefExpr(leadIdxCopy));
-    leadBody->insertAtTail("'move'(%S, %S)", leadIdxCopy, leadIdx);
-    leadBody->insertAtTail(followBlock);
-    leadBody->blockInfo = new CallExpr(PRIM_BLOCK_FOR_LOOP, leadIdx, leadIter);
-    BlockStmt* leadBlock = buildChapelStmt();
-    leadBlock->insertAtTail(new DefExpr(leadIdx));
-    leadBlock->insertAtTail(new DefExpr(leadIter));
-
-    if( !zippered ) {
-      leadBlock->insertAtTail("'move'(%S, _getIterator(_toLeader(%S)))", leadIter, data);
-    } else {
-      leadBlock->insertAtTail("'move'(%S, _getIterator(_toLeaderZip(%S)))", leadIter, data);
-    }
-
-    leadBlock->insertAtTail("{TYPE 'move'(%S, iteratorIndex(%S))}", leadIdx, leadIter);
-    leadBlock->insertAtTail(leadBody);
-    leadBlock->insertAtTail("_freeIterator(%S)", leadIter);
-    serialBlock->insertAtHead("compilerWarning('reduce has been serialized (see note in $CHPL_HOME/STATUS)')");
-    
-    fn->insertAtTail(new CondStmt(new SymExpr(gTryToken), leadBlock, serialBlock));
+    followBlock->insertAtTail("'move'(%S, _getIteratorZip(_toFollowerZip(%S, %S)))", followIter, data, leadIdxCopy);
   }
 
+  followBlock->insertAtTail("{TYPE 'move'(%S, iteratorIndex(%S))}", followIdx, followIter);
+  followBlock->insertAtTail("'move'(%S, 'new'(%E(%E)))", localOp, opExpr->copy(), new NamedExpr("eltType", new SymExpr(eltType)));
+  followBlock->insertAtTail(followBody);
+  followBlock->insertAtTail("chpl__reduceCombine(%S, %S)", globalOp, localOp);
+  followBlock->insertAtTail("'delete'(%S)", localOp);
+  followBlock->insertAtTail("_freeIterator(%S)", followIter);
+  BlockStmt* leadBody = new BlockStmt();
+  leadBody->insertAtTail(new DefExpr(leadIdxCopy));
+  leadBody->insertAtTail("'move'(%S, %S)", leadIdxCopy, leadIdx);
+  leadBody->insertAtTail(followBlock);
+  leadBody->blockInfo = new CallExpr(PRIM_BLOCK_FOR_LOOP, leadIdx, leadIter);
+  BlockStmt* leadBlock = buildChapelStmt();
+  leadBlock->insertAtTail(new DefExpr(leadIdx));
+  leadBlock->insertAtTail(new DefExpr(leadIter));
+
+  if( !zippered ) {
+    leadBlock->insertAtTail("'move'(%S, _getIterator(_toLeader(%S)))", leadIter, data);
+  } else {
+    leadBlock->insertAtTail("'move'(%S, _getIterator(_toLeaderZip(%S)))", leadIter, data);
+  }
+
+  leadBlock->insertAtTail("{TYPE 'move'(%S, iteratorIndex(%S))}", leadIdx, leadIter);
+  leadBlock->insertAtTail(leadBody);
+  leadBlock->insertAtTail("_freeIterator(%S)", leadIter);
+  serialBlock->insertAtHead("compilerWarning('reduce has been serialized (see note in $CHPL_HOME/STATUS)')");
+
+  fn->insertAtTail(new CondStmt(new SymExpr(gTryToken), leadBlock, serialBlock));
+
   VarSymbol* result = new VarSymbol("result");
   fn->insertAtTail(new DefExpr(result, new CallExpr(new CallExpr(".", globalOp, new_StringSymbol("generate")))));
   fn->insertAtTail("'delete'(%S)", globalOp);
@@ -1496,8 +1506,7 @@ CallExpr* buildScanExpr(Expr* opExpr, Expr* dataExpr, bool zippered) {
 
   buildReduceScanPreface(fn, data, eltType, globalOp, opExpr, dataExpr, zippered);
 
-  if (!fSerial && !fSerialForall)
-    fn->insertAtTail("compilerWarning('scan has been serialized (see note in $CHPL_HOME/STATUS)')");
+  fn->insertAtTail("compilerWarning('scan has been serialized (see note in $CHPL_HOME/STATUS)')");
 
   if( !zippered ) {
     fn->insertAtTail("'return'(chpl__scanIterator(%S, %S))", globalOp, data);
@@ -1604,7 +1613,7 @@ buildVarDecls(BlockStmt* stmts, Flag externconfig, Flag varconst, char* docs) {
 
 DefExpr*
 buildClassDefExpr(const char* name, Type* type, Expr* inherit, BlockStmt* decls, Flag isExtern, char *docs) {
-  ClassType* ct = toClassType(type);
+  AggregateType* ct = toAggregateType(type);
   INT_ASSERT(ct);
   TypeSymbol* ts = new TypeSymbol(name, ct);
   DefExpr* def = new DefExpr(ts);
@@ -1712,6 +1721,28 @@ FnSymbol* buildLambda(FnSymbol *fn) {
   return fn;
 }
 
+
+// Replaces the dummy function name "_" with the real name, sets the 'this'
+// intent tag. For methods, it also adds a method tag and "this" declaration.
+FnSymbol* buildFunctionSymbol(FnSymbol* fn, const char* name,
+                              IntentTag thisTag, const char* class_name)
+{
+  fn->cname = fn->name = astr(name);
+  fn->thisTag = thisTag;
+  if (fn->name[0] == '~' && fn->name[1] != '\0')
+    fn->addFlag(FLAG_DESTRUCTOR);
+  if (class_name)
+  {
+    fn->_this = new ArgSymbol(thisTag, "this", dtUnknown,
+                              new UnresolvedSymExpr(class_name));
+    fn->_this->addFlag(FLAG_ARG_THIS);
+    fn->insertFormalAtHead(new DefExpr(fn->_this));
+    ArgSymbol* mt = new ArgSymbol(INTENT_BLANK, "_mt", dtMethodToken);
+    fn->insertFormalAtHead(new DefExpr(mt));
+  }
+  return fn;
+}
+
 // Called like:
 // buildFunctionDecl($4, $6, $7, $8, $9, @$.comment);
 BlockStmt*
@@ -1882,8 +1913,7 @@ buildOnStmt(Expr* expr, Expr* stmt) {
     Symbol* tmp = newTemp();
     body->insertAtHead(new CallExpr(PRIM_MOVE, tmp, onExpr));
     body->insertAtHead(new DefExpr(tmp));
-    beginBlock->blockInfo = new CallExpr(PRIM_BLOCK_ON, tmp);
-    beginBlock->blockInfo->primitive = primitives[PRIM_BLOCK_ON_NB];
+    beginBlock->blockInfo = new CallExpr(PRIM_BLOCK_BEGIN_ON, tmp);
     // If there are beginBlock->byrefVars, they will be preserved.
     return body;
   } else {
@@ -1904,10 +1934,6 @@ BlockStmt*
 buildBeginStmt(CallExpr* byref_vars, Expr* stmt) {
   checkControlFlow(stmt, "begin statement");
 
-  if (fSerial)
-    // dropping byref_vars, if any
-    return buildChapelStmt(new BlockStmt(stmt));
-
   BlockStmt* body = toBlockStmt(stmt);
   
   //
@@ -1933,7 +1959,7 @@ buildBeginStmt(CallExpr* byref_vars, Expr* stmt) {
   if (onBlock) {
     body->insertAtHead(new CallExpr("_upEndCount"));
     onBlock->insertAtTail(new CallExpr("_downEndCount"));
-    onBlock->blockInfo->primitive = primitives[PRIM_BLOCK_ON_NB];
+    onBlock->blockInfo->primitive = primitives[PRIM_BLOCK_BEGIN_ON];
     addByrefVars(onBlock, byref_vars);
     return body;
   } else {
@@ -1953,8 +1979,6 @@ buildBeginStmt(CallExpr* byref_vars, Expr* stmt) {
 BlockStmt*
 buildSyncStmt(Expr* stmt) {
   checkControlFlow(stmt, "sync statement");
-  if (fSerial)
-    return buildChapelStmt(new BlockStmt(stmt));
   BlockStmt* block = new BlockStmt();
   VarSymbol* endCountSave = newTemp("_endCountSave");
   block->insertAtTail(new DefExpr(endCountSave));
@@ -1986,10 +2010,6 @@ buildCobeginStmt(CallExpr* byref_vars, BlockStmt* block) {
     return buildChapelStmt(block);
   }
 
-  if (fSerial)
-    // dropping byref_vars, if any
-    return buildChapelStmt(block);
-
   VarSymbol* cobeginCount = newTemp("_cobeginCount");
   cobeginCount->addFlag(FLAG_TEMP);
 
