@@ -1,5 +1,5 @@
 /**************************************************************************
-  Copyright (c) 2004-2013, Cray Inc.  (See LICENSE file for more details)
+  Copyright (c) 2004-2014, Cray Inc.  (See LICENSE file for more details)
 **************************************************************************/
 
 
@@ -12,6 +12,13 @@
 #include "optimizations.h"
 #include "view.h"
 
+//
+// This file implements lowerIterator() called by the lowerIterators pass
+//
+// lowerIterator() rewrites the original iterator function to fill in
+// the iterator record and build the various functions needed to build
+// the final loop: zip*, hasMore, advance, getValue, and getIterator.
+//
 
 //#define DEBUG_LIVE
 
@@ -43,41 +50,6 @@ static inline CallExpr* parentYieldExpr(SymExpr* se) {
 }
 
 
-// The set of FnSymbols where we have removed the initialization
-// (via PRIM_INIT) of 'ret' symbols.
-Vec<FnSymbol*> iteratorsWithRemovedRetInitSet;
-
-// Helper for removeRetSymbolAndUses().
-static inline void
-removeRetInitialization(DefExpr* rdef, Symbol* rsym) {
-  for (Expr* stmt = rdef->next; stmt; stmt = stmt->next) {
-    // If the iterator has a declared return type, 'sym' is
-    // initialized to its type's default value.
-    // Find that and yank it.
-    if (CallExpr* call = toCallExpr(stmt)) {
-      if (call->isPrimitive(PRIM_MOVE)) {
-        if (SymExpr* callTarget = toSymExpr(call->get(1))) {
-          if (callTarget->var == rsym) {
-            call->remove();
-            return;
-          }
-        }
-      }
-    }
-  }
-
-  // We did not find the initialization even though we expected it.
-  //
-  // BTW the above is executed only when we expect the initialization.
-  // I.e. when the return type was specified explicitly and we have
-  // not removed the initialization. Otherwise we avoid traversing the
-  // (top-level?) statement list (from rdef->next), in the name of
-  // reducing the compilation time.
-  //
-  INT_ASSERT(false);
-}
-
-
 //
 // Now that we have localized yield symbols, the return symbol
 // and the PRIM_RETURN CallExpr are not needed and would cause trouble.
@@ -91,23 +63,12 @@ removeRetSymbolAndUses(FnSymbol* fn) {
   SymExpr* rse = toSymExpr(ret->get(1));
   INT_ASSERT(rse);
   Symbol*  rsym = rse->var;
-  DefExpr* rdef = rsym->defPoint;
 
   // Yank the return statement.
   ret->remove();
-  // Yank rsym's initialization. We expect it only when
-  // the iterator's return type was specified explicitly.
-  if (fn->hasFlag(FLAG_SPECIFIED_RETURN_TYPE) &&
-      !iteratorsWithRemovedRetInitSet.set_in(fn))
-    removeRetInitialization(rdef, rsym);
-  // Yank rsym's definition.
-  rdef->remove();
-
-  // We could assert here that no uses of rsym remain.
-  // But that's expensive for large iterators. Instead,
-  // if we overlooked something, we will get a verification
-  // failure "SymExpr::var::defPoint is not in AST".
 
+  // We cannot remove rsym's definition, because rsym
+  // may also be referenced in an autoDestroy call.
   return rsym->type;
 }
 
@@ -164,10 +125,19 @@ isSingleLoopIterator(FnSymbol* fn, Vec<BaseAST*>& asts) {
       return NULL;
     }
   }
-  if (singleFor && singleYield)
+  if (singleFor && singleYield) {
+    if (fReportOptimizedLoopIterators) {
+      ModuleSymbol *mod = toModuleSymbol(fn->defPoint->parentSymbol);
+      INT_ASSERT(mod);
+      if (developer ||
+          ((mod->modTag != MOD_INTERNAL) && (mod->modTag != MOD_STANDARD))) {
+        printf("Optimized single yield/loop iterator (%s) in module %s (%s:%d)\n",
+               fn->cname, mod->name, fn->fname(), fn->linenum());
+      }
+    }
     return singleYield;
-  else
-    return NULL;
+  } else
+      return NULL;
 }
 
 //  se -- A sym expression which accesses a live local variable.
@@ -222,18 +192,15 @@ static void replaceLocalWithFieldTemp(SymExpr* se, Symbol* ic, Symbol* field,
     }
   }
 
-// This conditional assertion fails.
-// That means we probably need to update the definition of findDefUseSets()
-// so it recognizes PRIM_SET_MEMBER as a definition 
-// (of the member, tho maybe of the containing struct as well).
-//  if (call && call->isPrimitive(PRIM_SET_MEMBER)) {
-//    INT_ASSERT(is_def);
-//  }
-
   // If the symexpr is set here,
   if (is_def ||
-      // Isn't this redundant? If not, why not?
-      (call && call->isPrimitive(PRIM_SET_MEMBER))) {
+      // Currently buildDefUseSets() does not identify PRIM_SET_MEMBER and
+      // PRIM_SVEC_SET_MEMBER as defs.
+      (call &&
+       (call->isPrimitive(PRIM_SET_MEMBER) ||
+        call->isPrimitive(PRIM_SET_SVEC_MEMBER)) &&
+       call->get(1) == se))
+  {
     ArgSymbol* arg = toArgSymbol(se->var);
     if (arg)
     {
@@ -278,6 +245,27 @@ static void replaceLocalWithFieldTemp(SymExpr* se, Symbol* ic, Symbol* field,
 }
 
 
+//
+// What is oneLocalYS ?
+//
+// oneLocalYS is set when all PRIM_YIELDs in the iterator
+// yield the same Symbol *and* that Symbol is defined locally
+// within the iterator.
+//
+// When oneLocalYS: that Symbol is represented directly by ic.value.
+// I.e. the Symbol is mapped directly to valField, in
+// local2field, replaceLocalsWithFields, etc.
+// ic.value is not updated explicitly upon a yield - because it always
+// has the up-to-date value of that Symbol.
+//
+// When !oneLocalYS: each local Symbol is represented by its own field
+// ic.FNN_xxx. Non-local Symbols are not represented in ic's fields.
+// Upon each yield, ic.value aka valField is explicitly updated
+// with yield's expression.
+// E.g. 'yield localvar' is converted to ic.value = ic.FNN_localvar.
+// 
+
+
 // In the body of an iterator function, replace references to local variables
 // with references to fields in the iterator class instead.
 static void
@@ -286,6 +274,7 @@ replaceLocalsWithFields(FnSymbol* fn,           // the iterator function
                         SymbolMap& local2field, // Map: local symbol --> class field
                         Vec<Symbol*>& yldSymSet,// The set of locals that appear in yields.
                         Symbol* valField,       // ic.value field - value being yielded.
+                        bool oneLocalYS,        // fn's yields has exactly one yield symbol, and it is local.
                         Vec<Symbol*>& locals) { // The list of live locals in the iterator body.
 
   IteratorInfo* ii = fn->iteratorInfo;
@@ -307,19 +296,24 @@ replaceLocalsWithFields(FnSymbol* fn,           // the iterator function
         // SymExpr is in a yield.
 
         Symbol* ySym = se->var;
-        if (ySym->defPoint->parentSymbol == fn) {
-          // This is a local symbol. It must have been assigned to,
-          // and that assignment must have been converted to a store
-          // to ic.value. No need to do anything. The yield remains as-is.
+        if (oneLocalYS) {
+          // ySym is already tracked in ic.value. No need to do anything.
 
           // While here, sanity-check a couple of things.
+          INT_ASSERT(ySym->defPoint->parentSymbol == fn);
           INT_ASSERT(yldSymSet.set_in(ySym));
           INT_ASSERT(local2field.get(ySym) == valField);
 
         } else {
-          // This is a non-local symbol. We do not track assignments to these.
-          // So we need to assign it to ic.value. The yield remains as-is.
-          pc->insertBefore(new CallExpr(PRIM_SET_MEMBER, ic, valField, ySym));
+          // Update ic.value.
+          SymExpr* upd = new SymExpr(ySym);
+          pc->insertBefore(new CallExpr(PRIM_SET_MEMBER, ic, valField, upd));
+          if (ySym->defPoint->parentSymbol == fn) {
+            // Need to convert 'upd' itself, too.
+            Symbol* field = local2field.get(se->var);
+            INT_ASSERT(field && field != valField);
+            replaceLocalWithFieldTemp(upd, ic, field, false, true, asts);
+          }
         }
       } else if (useSet.set_in(se) || defSet.set_in(se)) {
         // SymExpr is among those we are interested in: def or use of a live local.
@@ -344,8 +338,18 @@ replaceLocalsWithFields(FnSymbol* fn,           // the iterator function
   }
 }
 
+//
+// Build zip functions for the single loop iterator optimization
+//
+// The single iterator loop optimization is applied to loops
+// recognized as above in isSingleLoopIterator().  Such loops can be
+// generated in a specialized form with simplified control flow.  When
+// the optimization does not apply, zip1 and zip3 simply call advance,
+// and zip2 and zip4 are empty (see below in lowerIterator()).
+//
 
-// Build the zip1 function, copying expression out of the iterator body up to the singleLoop.
+// Build the zip1 function, copying expressions out of the iterator
+// body that occur *before* the singleLoop
 static void
 buildZip1(IteratorInfo* ii, Vec<BaseAST*>& asts, BlockStmt* singleLoop) {
 
@@ -354,32 +358,40 @@ buildZip1(IteratorInfo* ii, Vec<BaseAST*>& asts, BlockStmt* singleLoop) {
   Symbol* ic = ii->advance->getFormal(1);
   map.put(ic, ii->zip1->_this);
 
-  // Copy def expressions which are not arg defs.
+  // This block will replace the body of the zip1 function stubbed in
+  // during resolution.
+  BlockStmt* zip1body = new BlockStmt();
+
+  // Copy non-arg def expressions from the original iterator
   forv_Vec(BaseAST, ast, asts) {
     if (DefExpr* def = toDefExpr(ast))
       if (!isArgSymbol(def->sym))
-        ii->zip1->insertAtTail(def->copy(&map));
+        zip1body->insertAtTail(def->copy(&map));
   }
 
-  // Copy all iterator body expressions which are not declarations.
+  // Copy all iterator body expressions before singleLoop that are not
+  // declarations
   for_alist(expr, ii->iterator->body->body) {
-    // Quit if we reach singleLoop (?)
+    // Quit when we reach singleLoop
     if (expr == singleLoop)
       break;
     if (!isDefExpr(expr))
-      ii->zip1->insertAtTail(expr->copy(&map));
+      zip1body->insertAtTail(expr->copy(&map));
   }
 
-  // Complete the rest of the function.
+  // Check for more
   CallExpr* blockInfo = singleLoop->blockInfo->copy(&map);
-  ii->zip1->insertAtTail(new CondStmt(blockInfo->get(1)->remove(),
+  zip1body->insertAtTail(new CondStmt(blockInfo->get(1)->remove(),
                                       new CallExpr(PRIM_SET_MEMBER, ii->zip1->_this, ii->iclass->getField("more"), new_IntSymbol(1)),
                                       new CallExpr(PRIM_SET_MEMBER, ii->zip1->_this, ii->iclass->getField("more"), new_IntSymbol(0))));
-  ii->zip1->insertAtTail(new CallExpr(PRIM_RETURN, gVoid));
+  zip1body->insertAtTail(new CallExpr(PRIM_RETURN, gVoid));
+
+  ii->zip1->body->replace(zip1body);
 }
 
 
-// Build the zip2 function, copying expressions out of the singleLoop body up to the first yield.
+// Build the zip2 function, copying expressions out of the singleLoop
+// body that are *before* the yield
 static void
 buildZip2(IteratorInfo* ii, Vec<BaseAST*>& asts, BlockStmt* singleLoop) {
 
@@ -388,38 +400,56 @@ buildZip2(IteratorInfo* ii, Vec<BaseAST*>& asts, BlockStmt* singleLoop) {
   Symbol* ic = ii->advance->getFormal(1);
   map.put(ic, ii->zip2->_this);
 
-  // Copy non-arg def expressions.
+  // This block will replace the body of the zip2 function stubbed in
+  // during resolution.
+  BlockStmt* zip2body = new BlockStmt();
+
+  // Copy non-arg def expressions from the original iterator
   forv_Vec(BaseAST, ast, asts) {
     if (DefExpr* def = toDefExpr(ast))
       if (!isArgSymbol(def->sym))
-        ii->zip2->insertAtTail(def->copy(&map));
+        zip2body->insertAtTail(def->copy(&map));
   }
 
+  // Copy all non-defs in singleLoop before the yield
   for_alist(expr, singleLoop->body) {
     if (CallExpr* call = toCallExpr(expr))
       if (call->isPrimitive(PRIM_YIELD))
         break;
     if (!isDefExpr(expr))
-      ii->zip2->insertAtTail(expr->copy(&map));
+      zip2body->insertAtTail(expr->copy(&map));
   }
-  ii->zip2->insertAtTail(new CallExpr(PRIM_RETURN, gVoid));
+  zip2body->insertAtTail(new CallExpr(PRIM_RETURN, gVoid));
+
+  ii->zip2->body->replace(zip2body);
 }
 
 
-// Build the zip3 function, copying expressions out of the singleLoop body up to the yield
-// but also copy non-declarations following the yield.
+// Build the zip3 function, copying expressions out of the singleLoop
+// body that are *after* the yield
 static void
 buildZip3(IteratorInfo* ii, Vec<BaseAST*>& asts, BlockStmt* singleLoop) {
+
+  // In copied expressions, replace _ic with zip2->_this .
   Symbol* ic = ii->advance->getFormal(1);
   SymbolMap map;
   map.put(ic, ii->zip3->_this);
+
+  // This block will replace the body of the zip3 function stubbed in
+  // during resolution.
+  BlockStmt* zip3body = new BlockStmt();
+
+  // Copy non-arg def expressions from the original iterator
   forv_Vec(BaseAST, ast, asts) {
     if (DefExpr* def = toDefExpr(ast))
       if (!isArgSymbol(def->sym))
-        ii->zip3->insertAtTail(def->copy(&map));
+        zip3body->insertAtTail(def->copy(&map));
   }
+
+  // Copy all non-defs in singleLoop after the yield
   bool flag = true;
   for_alist(expr, singleLoop->body) {
+    // Skip everything before the yield
     if (flag) {
       if (CallExpr* call = toCallExpr(expr))
         if (call->isPrimitive(PRIM_YIELD))
@@ -427,39 +457,57 @@ buildZip3(IteratorInfo* ii, Vec<BaseAST*>& asts, BlockStmt* singleLoop) {
       continue;
     }
     if (!isDefExpr(expr))
-      ii->zip3->insertAtTail(expr->copy(&map));
+      zip3body->insertAtTail(expr->copy(&map));
   }
+ 
+  // Check for more
   CallExpr* blockInfo = singleLoop->blockInfo->copy(&map);
-  ii->zip3->insertAtTail(new CondStmt(blockInfo->get(1)->remove(),
+  zip3body->insertAtTail(new CondStmt(blockInfo->get(1)->remove(),
                                       new CallExpr(PRIM_SET_MEMBER, ii->zip3->_this, ii->iclass->getField("more"), new_IntSymbol(1)),
                                       new CallExpr(PRIM_SET_MEMBER, ii->zip3->_this, ii->iclass->getField("more"), new_IntSymbol(0))));
-  ii->zip3->insertAtTail(new CallExpr(PRIM_RETURN, gVoid));
+  zip3body->insertAtTail(new CallExpr(PRIM_RETURN, gVoid));
+
+  ii->zip3->body->replace(zip3body);
 }
 
 
-// Build the zip4 function, copying expressions out of the iterator body, up to singleLoop,
-// but also copy exprs that have a next pointer.
+// Build the zip4 function, copy expressions out of the iterator body
+// that are *after* the singleLoop
 static void
 buildZip4(IteratorInfo* ii, Vec<BaseAST*>& asts, BlockStmt* singleLoop) {
+
+  // In copied expressions, replace _ic with zip4->_this .
   Symbol* ic = ii->advance->getFormal(1);
   SymbolMap map;
   map.put(ic, ii->zip4->_this);
+
+  // This block will replace the body of the zip4 function stubbed in
+  // during resolution.
+  BlockStmt* zip4body = new BlockStmt();
+
+  // Copy non-arg def expressions from the original iterator
   forv_Vec(BaseAST, ast, asts) {
       if (DefExpr* def = toDefExpr(ast))
         if (!isArgSymbol(def->sym))
-          ii->zip4->insertAtTail(def->copy(&map));
+          zip4body->insertAtTail(def->copy(&map));
     }
+
+  // Copy all iterator body expressions after singleLoop that are not
+  // declarations
   bool flag = true;
   for_alist(expr, ii->iterator->body->body) {
+    // Skip everything before the singleLoop
     if (flag) {
       if (expr == singleLoop)
         flag = false;
       continue;
     }
     if (!isDefExpr(expr) && expr->next)
-      ii->zip4->insertAtTail(expr->copy(&map));
+      zip4body->insertAtTail(expr->copy(&map));
   }
-  ii->zip4->insertAtTail(new CallExpr(PRIM_RETURN, gVoid));
+  zip4body->insertAtTail(new CallExpr(PRIM_RETURN, gVoid));
+
+  ii->zip4->body->replace(zip4body);
 }
 
 
@@ -474,12 +522,14 @@ buildAdvance(FnSymbol* fn,
   //
   // build standard iterator advance method (advance)
   //
+  // This block will replace the stubbed-in body.
+  BlockStmt* advanceBody = new BlockStmt();
 
   for_alist(expr, fn->body->body)
-    ii->advance->insertAtTail(expr->remove());
+    advanceBody->insertAtTail(expr->remove());
 
   Symbol* end = new LabelSymbol("_end");
-  ii->advance->insertAtTail(new DefExpr(end));
+  advanceBody->insertAtTail(new DefExpr(end));
 
   // change yields to labels and gotos
   int i = 2; // 1 = not started, 0 = finished
@@ -511,13 +561,15 @@ buildAdvance(FnSymbol* fn,
   forv_Vec(LabelSymbol, label, labels) {
     GotoStmt* igs = new GotoStmt(GOTO_ITER_RESUME, label);
     label->iterResumeGoto = igs;
-    ii->advance->insertAtHead(new CondStmt(new SymExpr(tmp), igs));
-    ii->advance->insertAtHead(new CallExpr(PRIM_MOVE, tmp, new CallExpr(PRIM_EQUAL, more, new_IntSymbol(i++))));
+    advanceBody->insertAtHead(new CondStmt(new SymExpr(tmp), igs));
+    advanceBody->insertAtHead(new CallExpr(PRIM_MOVE, tmp, new CallExpr(PRIM_EQUAL, more, new_IntSymbol(i++))));
   }
-  ii->advance->insertAtHead(new CallExpr(PRIM_MOVE, more, new CallExpr(PRIM_GET_MEMBER_VALUE, ic, ii->iclass->getField("more"))));
-  ii->advance->insertAtHead(new DefExpr(tmp));
-  ii->advance->insertAtHead(new DefExpr(more));
-  ii->advance->insertAtTail(new CallExpr(PRIM_RETURN, gVoid));
+  advanceBody->insertAtHead(new CallExpr(PRIM_MOVE, more, new CallExpr(PRIM_GET_MEMBER_VALUE, ic, ii->iclass->getField("more"))));
+  advanceBody->insertAtHead(new DefExpr(tmp));
+  advanceBody->insertAtHead(new DefExpr(more));
+  advanceBody->insertAtTail(new CallExpr(PRIM_RETURN, gVoid));
+
+  ii->advance->body->replace(advanceBody);
 }
 
 
@@ -525,9 +577,12 @@ buildAdvance(FnSymbol* fn,
 static void
 buildHasMore(IteratorInfo* ii) {
   VarSymbol* tmp = newTemp(ii->hasMore->retType);
-  ii->hasMore->insertAtTail(new DefExpr(tmp));
-  ii->hasMore->insertAtTail(new CallExpr(PRIM_MOVE, tmp, new CallExpr(PRIM_GET_MEMBER_VALUE, ii->hasMore->getFormal(1), ii->iclass->getField("more"))));
-  ii->hasMore->insertAtTail(new CallExpr(PRIM_RETURN, tmp));
+  // This block will replace the stubbed-in body of ii->hasMore.
+  BlockStmt* hasMoreBody = new BlockStmt();
+  hasMoreBody->insertAtTail(new DefExpr(tmp));
+  hasMoreBody->insertAtTail(new CallExpr(PRIM_MOVE, tmp, new CallExpr(PRIM_GET_MEMBER_VALUE, ii->hasMore->getFormal(1), ii->iclass->getField("more"))));
+  hasMoreBody->insertAtTail(new CallExpr(PRIM_RETURN, tmp));
+  ii->hasMore->body->replace(hasMoreBody);
 }
 
 
@@ -535,9 +590,12 @@ buildHasMore(IteratorInfo* ii) {
 static void
 buildGetValue(IteratorInfo* ii) {
   VarSymbol* tmp = newTemp(ii->getValue->retType);
-  ii->getValue->insertAtTail(new DefExpr(tmp));
-  ii->getValue->insertAtTail(new CallExpr(PRIM_MOVE, tmp, new CallExpr(PRIM_GET_MEMBER_VALUE, ii->getValue->getFormal(1), ii->iclass->getField("value"))));
-  ii->getValue->insertAtTail(new CallExpr(PRIM_RETURN, tmp));
+  // This block will replace the stubbed-in body of ii->getValue.
+  BlockStmt* getMoreBody = new BlockStmt();
+  getMoreBody->insertAtTail(new DefExpr(tmp));
+  getMoreBody->insertAtTail(new CallExpr(PRIM_MOVE, tmp, new CallExpr(PRIM_GET_MEMBER_VALUE, ii->getValue->getFormal(1), ii->iclass->getField("value"))));
+  getMoreBody->insertAtTail(new CallExpr(PRIM_RETURN, tmp));
+  ii->getValue->body->replace(getMoreBody);
 }
 
 
@@ -631,6 +689,7 @@ static void insertLocalsForRefs(Vec<Symbol*>& syms, FnSymbol* fn,
         INT_FATAL(sym, "invalid assumption about reference");
       }
 
+      // Do we need to consider PRIM_ASSIGN as well?
       CallExpr* move = toCallExpr(defs->v[0]->parentExpr);
       INT_ASSERT(move->isPrimitive(PRIM_MOVE));
 
@@ -823,8 +882,10 @@ rebuildGetIterator(IteratorInfo* ii) {
 // All "newRet" symbols used in yield expressions will need special handling,
 // so we collect them into a set.
 static void
-collectYieldSymbols(FnSymbol* fn, Vec<BaseAST*>& asts, Vec<Symbol*>& yldSymSet)
+collectYieldSymbols(FnSymbol* fn, Vec<BaseAST*>& asts, Vec<Symbol*>& yldSymSet,
+                    bool* oneLocalYSRef)
 {
+  bool gotNonlocalYS = false;
   forv_Vec(BaseAST, ast, asts) {
     if (CallExpr* yCall = asYieldExpr(ast)) {
       SymExpr* ySymExpr = toSymExpr(yCall->get(1));
@@ -834,12 +895,14 @@ collectYieldSymbols(FnSymbol* fn, Vec<BaseAST*>& asts, Vec<Symbol*>& yldSymSet)
         // It is a local symbol, add it.
         yldSymSet.set_add(ySym);
       } else {
-        // Non-local symbols should not be considered.
-        // FYI. Or can this be anything else?
+        // Non-local symbols are not included in yldSymSet.
+        gotNonlocalYS = true;
+        // FYI.
         INT_ASSERT(ySym->isConstant() || ySym->isParameter());
       }
     }
   }
+  *oneLocalYSRef = !gotNonlocalYS && (yldSymSet.count() == 1);
 }
 
 
@@ -868,10 +931,10 @@ removeLocals(Vec<Symbol*>& locals, Vec<BaseAST*>& asts, Vec<Symbol*>& yldSymSet,
 // Creates (and returns) an iterator class field.
 // 'type' is used if local==NULL.
 static inline Symbol* createICField(int& i, Symbol* local, Type* type,
-                                    bool isYieldSym, FnSymbol* fn) {
+                                    bool isValueField, FnSymbol* fn) {
   // The field name is "value" for the return value of the iterator,
   // or F<int>_<local->name> otherwise.
-  const char* fieldName = isYieldSym
+  const char* fieldName = isValueField
     ? "value"
     : astr("F", istr(i++), "_", local->name);
 
@@ -894,7 +957,7 @@ static inline Symbol* createICField(int& i, Symbol* local, Type* type,
 // and live at any yield.
 static void addLocalsToClassAndRecord(Vec<Symbol*>& locals, FnSymbol* fn, 
                                       Vec<Symbol*>& yldSymSet, Type* yieldedType,
-                                      Symbol** valFieldRef,
+                                      Symbol** valFieldRef, bool oneLocalYS,
                                       SymbolMap& local2field, SymbolMap& local2rfield)
 {
   IteratorInfo* ii = fn->iteratorInfo;
@@ -902,29 +965,19 @@ static void addLocalsToClassAndRecord(Vec<Symbol*>& locals, FnSymbol* fn,
 
   int i = 0;    // This numbers the fields.
   forv_Vec(Symbol, local, locals) {
-    Symbol* field = NULL;
     bool isYieldSym = yldSymSet.set_in(local);
+    Symbol* field = createICField(i, local, NULL, isYieldSym && oneLocalYS, fn);
+    local2field.put(local, field);
     if (isYieldSym) {
       INT_ASSERT(local->type == yieldedType);
-    }
-
-    if (isYieldSym && valField) {
-      // We have already computed the corresponding field.
-      // Return symbols are never arguments, so not in local2rfield.
-      local2field.put(local, valField);
-
-    } else {
-      field = createICField(i, local, NULL, isYieldSym, fn);
-      local2field.put(local, field);
-      if (isYieldSym) {
+      if (oneLocalYS) {
+        INT_ASSERT(valField == NULL); // there is exactly 1 yield symbol
         valField = field;
       }
     }
 
     // Only (live) arguments are added to the record.
     if (isArgSymbol(local)) {
-      // 'field' is NULL only if isYieldSym - but then 'local' is not an arg.
-      INT_ASSERT(field);
       Symbol* rfield = new VarSymbol(field->name, field->type);
       local2rfield.put(local, rfield);
       ii->irecord->fields.insertAtTail(new DefExpr(rfield));
@@ -962,13 +1015,14 @@ void lowerIterator(FnSymbol* fn) {
   SymbolMap local2field;  // map from arg/local to class field
   SymbolMap local2rfield; // map from arg to record field
   Vec<Symbol*> locals;
-  Vec<Symbol*> yldSymSet;
+  Vec<Symbol*> yldSymSet; // local symbols appearing in yield(s)
   Symbol* valField;
+  bool oneLocalYS;  // see comment earlier
 
   for_formals(formal, fn)
     locals.add(formal);
 
-  collectYieldSymbols(fn, asts, yldSymSet);
+  collectYieldSymbols(fn, asts, yldSymSet, &oneLocalYS);
 
   if (fNoLiveAnalysis)
     addAllLocalVariables(locals, asts);
@@ -978,9 +1032,10 @@ void lowerIterator(FnSymbol* fn) {
   addYieldSymbols(locals, yldSymSet);
 
   addLocalsToClassAndRecord(locals, fn, yldSymSet, yieldedType, &valField,
-                            local2field, local2rfield);
+                            oneLocalYS, local2field, local2rfield);
 
-  replaceLocalsWithFields(fn, asts, local2field, yldSymSet, valField, locals);
+  replaceLocalsWithFields(fn, asts, local2field, yldSymSet, valField,
+                          oneLocalYS, locals);
 
   removeLocals(locals, asts, yldSymSet, fn);
 
@@ -995,12 +1050,14 @@ void lowerIterator(FnSymbol* fn) {
       buildZip3(ii, asts, singleLoop);
       buildZip4(ii, asts, singleLoop);
     } else {
-      ii->zip1->insertAtTail(new CallExpr(ii->advance, ii->zip1->_this));
-      ii->zip1->insertAtTail(new CallExpr(PRIM_RETURN, gVoid));
-      ii->zip2->insertAtTail(new CallExpr(PRIM_RETURN, gVoid));
-      ii->zip3->insertAtTail(new CallExpr(ii->advance, ii->zip3->_this));
-      ii->zip3->insertAtTail(new CallExpr(PRIM_RETURN, gVoid));
-      ii->zip4->insertAtTail(new CallExpr(PRIM_RETURN, gVoid));
+      // zip functions are already normal, because we normalized them in
+      // protoIteratorMethod (functionResolution.cpp).
+      ii->zip1->insertAtHead(new CallExpr(ii->advance, ii->zip1->_this));
+//      ii->zip1->insertAtTail(new CallExpr(PRIM_RETURN, gVoid));
+//      ii->zip2->insertAtTail(new CallExpr(PRIM_RETURN, gVoid));
+      ii->zip3->insertAtHead(new CallExpr(ii->advance, ii->zip3->_this));
+//      ii->zip3->insertAtTail(new CallExpr(PRIM_RETURN, gVoid));
+//      ii->zip4->insertAtTail(new CallExpr(PRIM_RETURN, gVoid));
     }
     buildAdvance(fn, asts, local2field, locals);
     buildHasMore(ii);
