@@ -1,5 +1,5 @@
 /**************************************************************************
-  Copyright (c) 2004-2013, Cray Inc.  (See LICENSE file for more details)
+  Copyright (c) 2004-2014, Cray Inc.  (See LICENSE file for more details)
 **************************************************************************/
 
 
@@ -123,13 +123,6 @@ returnInfoFirstDeref(CallExpr* call) {
   return call->get(1)->getValType();
 }
 
-static Type*
-returnIteratorType(CallExpr* call) {
-  Type* ict = call->get(1)->typeInfo();
-  INT_ASSERT(ict->symbol->hasFlag(FLAG_ITERATOR_CLASS));
-  return ict->initializer->getReturnSymbol()->type;
-}
-
 static Type*
 returnInfoCast(CallExpr* call) {
   Type* t1 = call->get(1)->typeInfo();
@@ -145,7 +138,7 @@ returnInfoCast(CallExpr* call) {
 
 static Type*
 returnInfoVal(CallExpr* call) {
-  ClassType* ct = toClassType(call->get(1)->typeInfo());
+  AggregateType* ct = toAggregateType(call->get(1)->typeInfo());
   if (!ct || !ct->symbol->hasFlag(FLAG_REF))
     INT_FATAL(call, "attempt to get value type of non-reference type");
   return ct->getField(1)->type;
@@ -203,9 +196,9 @@ returnInfoGetMember(CallExpr* call) {
   SymExpr* sym1 = toSymExpr(call->get(1));
   if (!sym1)
     INT_FATAL(call, "bad member primitive");
-  ClassType* ct = toClassType(sym1->var->type);
+  AggregateType* ct = toAggregateType(sym1->var->type);
   if (ct->symbol->hasFlag(FLAG_REF))
-    ct = toClassType(ct->getValType());
+    ct = toAggregateType(ct->getValType());
   if (!ct)
     INT_FATAL(call, "bad member primitive");
   SymExpr* sym = toSymExpr(call->get(2));
@@ -228,7 +221,7 @@ returnInfoGetMember(CallExpr* call) {
 
 static Type*
 returnInfoGetTupleMember(CallExpr* call) {
-  ClassType* ct = toClassType(call->get(1)->getValType());
+  AggregateType* ct = toAggregateType(call->get(1)->getValType());
   INT_ASSERT(ct && ct->symbol->hasFlag(FLAG_STAR_TUPLE));
   return ct->getField("x1")->type;
 }
@@ -241,18 +234,27 @@ returnInfoGetTupleMemberRef(CallExpr* call) {
 
 static Type*
 returnInfoGetMemberRef(CallExpr* call) {
-  ClassType* ct = toClassType(call->get(1)->getValType());
+  AggregateType* ct = toAggregateType(call->get(1)->getValType());
   INT_ASSERT(ct);
   SymExpr* se = toSymExpr(call->get(2));
   INT_ASSERT(se);
   VarSymbol* var = toVarSymbol(se->var);
   INT_ASSERT(var);
-  if (var->immediate) {
-    const char* name = var->immediate->v_string;
-    for_fields(field, ct) {
-      if (!strcmp(field->name, name))
-        return field->type->refType ? field->type->refType : field->type;
+  if (Immediate* imm = var->immediate)
+  {
+    Symbol* field = NULL;
+    if (imm->const_kind == CONST_KIND_STRING)
+    {
+      const char* name = var->immediate->v_string;
+      field = ct->getField(name);
     }
+    if (imm->const_kind == NUM_KIND_INT)
+    {
+      int64_t i = imm->int_value();
+      field = ct->getField(i);
+    }
+    INT_ASSERT(field);
+    return field->type->refType ? field->type->refType : field->type;
   } else
     return var->type->refType ? var->type->refType : var->type;
   INT_FATAL(call, "bad member primitive");
@@ -367,6 +369,7 @@ initPrimitive() {
   prim_def(PRIM_NOOP, "noop", returnInfoVoid);
   prim_def(PRIM_MOVE, "move", returnInfoVoid, false, true);
   prim_def(PRIM_INIT, "init", returnInfoFirstDeref);
+  prim_def(PRIM_NO_INIT, "no init", returnInfoFirstDeref);
   prim_def(PRIM_REF_TO_STRING, "ref to string", returnInfoString);
   prim_def(PRIM_RETURN, "return", returnInfoFirst, true);
   prim_def(PRIM_YIELD, "yield", returnInfoFirst, true);
@@ -392,6 +395,7 @@ initPrimitive() {
   prim_def(PRIM_XOR, "^", returnInfoFirst);
   prim_def(PRIM_POW, "**", returnInfoNumericUp);
 
+  prim_def(PRIM_ASSIGN, "=", returnInfoVoid, true);
   prim_def(PRIM_ADD_ASSIGN, "+=", returnInfoVoid, true);
   prim_def(PRIM_SUBTRACT_ASSIGN, "-=", returnInfoVoid, true);
   prim_def(PRIM_MULT_ASSIGN, "*=", returnInfoVoid, true);
@@ -476,7 +480,6 @@ initPrimitive() {
   prim_def(PRIM_CAST, "cast", returnInfoCast, false, true);
   prim_def(PRIM_DYNAMIC_CAST, "dynamic_cast", returnInfoCast, false, true);
   prim_def(PRIM_TYPEOF, "typeof", returnInfoFirstDeref);
-  prim_def(PRIM_GET_ITERATOR_RETURN, "get iterator return", returnIteratorType);
   prim_def(PRIM_USE, "use", returnInfoVoid, true);
   prim_def(PRIM_USED_MODULES_LIST, "used modules list", returnInfoVoid);
   prim_def(PRIM_TUPLE_EXPAND, "expand_tuple", returnInfoVoid);
@@ -511,7 +514,9 @@ initPrimitive() {
   prim_def(PRIM_BLOCK_COBEGIN, "cobegin block", returnInfoVoid);
   prim_def(PRIM_BLOCK_COFORALL, "coforall loop", returnInfoVoid);
   prim_def(PRIM_BLOCK_ON, "on block", returnInfoVoid);
-  prim_def(PRIM_BLOCK_ON_NB, "non-blocking on block", returnInfoVoid);
+  prim_def(PRIM_BLOCK_BEGIN_ON, "begin on block", returnInfoVoid);
+  prim_def(PRIM_BLOCK_COBEGIN_ON, "cobegin on block", returnInfoVoid);
+  prim_def(PRIM_BLOCK_COFORALL_ON, "coforall on block", returnInfoVoid);
   prim_def(PRIM_BLOCK_LOCAL, "local block", returnInfoVoid);
   prim_def(PRIM_BLOCK_UNLOCAL, "unlocal block", returnInfoVoid);
 
@@ -538,7 +543,6 @@ initPrimitive() {
 
   prim_def(PRIM_ON_LOCALE_NUM, "chpl_on_locale_num", returnInfoLocaleID);
 
-  prim_def(PRIM_ALLOC_GVR, "allocchpl_globals_registry", returnInfoVoid);
   prim_def(PRIM_HEAP_REGISTER_GLOBAL_VAR, "_heap_register_global_var", returnInfoVoid, true, true);
   prim_def(PRIM_HEAP_BROADCAST_GLOBAL_VARS, "_heap_broadcast_global_vars", returnInfoVoid, true, true);
   prim_def(PRIM_PRIVATE_BROADCAST, "_private_broadcast", returnInfoVoid, true, true);
@@ -556,18 +560,15 @@ initPrimitive() {
   prim_def("string_index", returnInfoString, true, true);
   prim_def(PRIM_STRING_COPY, "string_copy", returnInfoString, false, true);
   prim_def(PRIM_STRING_NORMALIZE, "string_normalize", returnInfoVoid, true, false);
-  prim_def(PRIM_STRING_FROM_C_STRING, "string_from_c_string", returnInfoString, true, true);
-  prim_def(PRIM_C_STRING_FROM_STRING, "c_string_from_string", returnInfoStringC, true, true);
+  prim_def(PRIM_STRING_FROM_C_STRING, "string_from_c_string", returnInfoString, false, true);
+  prim_def(PRIM_C_STRING_FROM_STRING, "c_string_from_string", returnInfoStringC, false, true);
   prim_def(PRIM_CAST_TO_VOID_STAR, "cast_to_void_star", returnInfoOpaque, true, false);
   prim_def("string_select", returnInfoString, true, true);
-  prim_def("string_strided_select", returnInfoString, true, true);
   prim_def("sleep", returnInfoVoid, true);
   prim_def("real2int", returnInfoDefaultInt);
   prim_def("object2int", returnInfoDefaultInt);
   prim_def("chpl_exit_any", returnInfoVoid, true);
 
-  prim_def("chpl_setMemFlags", returnInfoVoid, true);
-
   prim_def(PRIM_RT_ERROR, "chpl_error", returnInfoVoid, true, true);
   prim_def(PRIM_RT_WARNING, "chpl_warning", returnInfoVoid, true, true);
 
@@ -581,6 +582,9 @@ initPrimitive() {
 
   prim_def(PRIM_FTABLE_CALL, "call ftable function", returnInfoVoid, true);
 
+  prim_def(PRIM_IS_SYNC_TYPE, "is sync type", returnInfoBool);
+  prim_def(PRIM_IS_SINGLE_TYPE, "is single type", returnInfoBool);
+  prim_def(PRIM_IS_TUPLE_TYPE, "is tuple type", returnInfoBool);
   prim_def(PRIM_IS_STAR_TUPLE_TYPE, "is star tuple type", returnInfoBool);
   prim_def(PRIM_SET_SVEC_MEMBER, "set svec member", returnInfoVoid, true, true);
   prim_def(PRIM_GET_SVEC_MEMBER, "get svec member", returnInfoGetTupleMemberRef);
@@ -594,6 +598,7 @@ initPrimitive() {
   prim_def(PRIM_FIELD_ID_BY_NUM, "field id by num", returnInfoInt32);
   prim_def(PRIM_FIELD_VALUE_BY_NAME, "field value by name", returnInfoUnknown);
   prim_def(PRIM_IS_UNION_TYPE, "is union type", returnInfoBool);
+  prim_def(PRIM_IS_ATOMIC_TYPE, "is atomic type", returnInfoBool);
 
   prim_def(PRIM_ENUM_MIN_BITS, "enum min bits", returnInfoInt32);
   prim_def(PRIM_ENUM_IS_SIGNED, "enum is signed", returnInfoBool);
