@@ -1,5 +1,5 @@
 /**************************************************************************
-  Copyright (c) 2004-2013, Cray Inc.  (See LICENSE file for more details)
+  Copyright (c) 2004-2014, Cray Inc.  (See LICENSE file for more details)
 **************************************************************************/
 
 
@@ -37,6 +37,7 @@ ModuleSymbol* theProgram = NULL;
 ModuleSymbol* mainModule = NULL;
 ModuleSymbol* baseModule = NULL;
 ModuleSymbol* standardModule = NULL;
+ModuleSymbol* printModuleInitModule = NULL;
 Symbol *gNil = NULL;
 Symbol *gUnknown = NULL;
 Symbol *gMethodToken = NULL;
@@ -60,6 +61,8 @@ VarSymbol *gBoundsChecking = NULL;
 VarSymbol* gPrivatization = NULL;
 VarSymbol* gLocal = NULL;
 VarSymbol* gNodeID = NULL;
+VarSymbol *gModuleInitIndentLevel = NULL;
+FnSymbol *gPrintModuleInitFn = NULL;
 FnSymbol* gChplHereAlloc = NULL;
 FnSymbol* gChplHereFree = NULL;
 Symbol *gCLine = NULL;
@@ -115,6 +118,9 @@ bool Symbol::isConstant(void) {
   return false;
 }
 
+bool Symbol::isConstValWillNotChange(void) {
+  return false;
+}
 
 bool Symbol::isParameter(){
   return false;
@@ -202,6 +208,7 @@ void VarSymbol::verify() {
 VarSymbol*
 VarSymbol::copyInner(SymbolMap* map) {
   VarSymbol* newVarSymbol = new VarSymbol(name, type);
+  newVarSymbol->copyFlags(this);
   newVarSymbol->cname = cname;
   INT_ASSERT(!newVarSymbol->immediate);
   return newVarSymbol;
@@ -218,6 +225,11 @@ bool VarSymbol::isConstant(void) {
 }
 
 
+bool VarSymbol::isConstValWillNotChange(void) {
+  return hasFlag(FLAG_CONST);
+}
+
+
 bool VarSymbol::isParameter(void){
   return hasFlag(FLAG_PARAM) || immediate;
 }
@@ -284,7 +296,7 @@ llvm::Value* codegenImmediateLLVM(Immediate* i)
           break;
       }
       break;
-    case NUM_KIND_FLOAT:
+    case NUM_KIND_REAL:
     case NUM_KIND_IMAG:
       switch(i->num_index) {
         case FLOAT_SIZE_32:
@@ -423,12 +435,49 @@ GenRet VarSymbol::codegen() {
         } else if (iconst <= -2147483648ll || iconst >= 2147483647ll) {
           ret.c = "INT64(" + int64_to_string(iconst) + ")";
         } else {
-          ret.c = int64_to_string(iconst);
+          const char* castString = "(";
+          switch (immediate->num_index) {
+          case INT_SIZE_8:
+            castString = "INT8(";
+            break;
+          case INT_SIZE_16:
+            castString = "INT16(";
+            break;
+          case INT_SIZE_32:
+            castString = "INT32(";
+            break;
+          case INT_SIZE_64:
+            castString = "INT64(";
+            break;
+          case INT_SIZE_1:
+          default:
+            INT_FATAL("Unexpected immediate->num_index: %d\n", immediate->num_index);
+          }
+            
+          ret.c = castString + int64_to_string(iconst) + ")";
         }
       } else if (immediate->const_kind == NUM_KIND_UINT) {
         uint64_t uconst = immediate->uint_value();
         if( uconst <= (uint64_t) INT32_MAX ) {
-          ret.c = uint64_to_string(uconst);
+          const char* castString = "(";
+          switch (immediate->num_index) {
+          case INT_SIZE_8:
+            castString = "UINT8(";
+            break;
+          case INT_SIZE_16:
+            castString = "UINT16(";
+            break;
+          case INT_SIZE_32:
+            castString = "UINT32(";
+            break;
+          case INT_SIZE_64:
+            castString = "UINT64(";
+            break;
+          case INT_SIZE_1:
+          default:
+            INT_FATAL("Unexpected immediate->num_index: %d\n", immediate->num_index);
+          }
+          ret.c = castString + uint64_to_string(uconst) + ")";
         } else {
           ret.c = "UINT64(" + uint64_to_string(uconst) + ")";
         }
@@ -536,30 +585,6 @@ GenRet VarSymbol::codegen() {
 }
 
 
-static void zeroInitializeRecord(FILE* outfile, ClassType* ct) {
-  bool first = true;
-  fprintf(outfile, "{");
-  for_fields(field, ct) {
-    if (!first) {
-      fprintf(outfile, ", ");
-    }
-    if (ClassType* ct = toClassType(field->type)) {
-      if (ct->classTag == CLASS_RECORD) {
-        zeroInitializeRecord(outfile, ct);
-      } else if (ct->classTag == CLASS_UNION) {
-        fprintf(outfile, "{0, 0}");
-      } else {
-        fprintf(outfile, "NULL");
-      }
-    } else {
-      fprintf(outfile, "0");
-    }
-    first = false;
-  }
-  fprintf(outfile, "}");
-}
-
-
 void VarSymbol::codegenDefC(bool global) {
   GenInfo* info = gGenInfo;
   if (this->hasFlag(FLAG_EXTERN))
@@ -568,7 +593,7 @@ void VarSymbol::codegenDefC(bool global) {
   if (type == dtVoid)
     return;
 
-  ClassType* ct = toClassType(type);
+  AggregateType* ct = toAggregateType(type);
   std::string typestr =  (this->hasFlag(FLAG_SUPER_CLASS) ?
                           std::string(ct->classStructName(true)) :
                           type->codegen().c);
@@ -581,7 +606,7 @@ void VarSymbol::codegenDefC(bool global) {
 
   std::string str = (isStatic ? "static " : "") + typestr + " " + cname;
   if (ct) {
-    if (ct->classTag == CLASS_CLASS) {
+    if (ct->isClass()) {
       if (isFnSymbol(defPoint->parentSymbol)) {
         str += " = NULL";  
       }
@@ -589,7 +614,12 @@ void VarSymbol::codegenDefC(bool global) {
                ct->symbol->hasFlag(FLAG_WIDE_CLASS)) {
       if (isFnSymbol(defPoint->parentSymbol)) {
         if( widePointersStruct || isWideString(ct) ) {
-          str += " = {{0,0},NULL}";
+
+          //
+          // CHPL_LOCALEID_T_INIT is #defined in the chpl-locale-model.h
+          // file in the runtime, for the selected locale model.
+          //
+          str += " = {CHPL_LOCALEID_T_INIT, NULL}";
         } else {
           str += " = ((wide_ptr_t) NULL)";
         }
@@ -631,11 +661,26 @@ void VarSymbol::codegenGlobalDef() {
         }
       }
     } else {
-      // Anything that we don't have already
-      //
-      llvm::GlobalVariable *gVar = llvm::cast<llvm::GlobalVariable>(
-          info->module->getOrInsertGlobal(cname, type->codegen().type));
-      gVar->setInitializer(llvm::Constant::getNullValue(type->codegen().type));
+      bool existing;
+
+      existing = (info->module->getNamedValue(cname) != NULL);
+
+      if( existing )
+        INT_FATAL(this, "Redefinition of a global variable %s", cname);
+
+      // Now, create a global variable with appropriate linkage.
+      llvm::Type* llTy = type->codegen().type;
+      INT_ASSERT(llTy);
+
+      llvm::GlobalVariable *gVar =
+        new llvm::GlobalVariable(
+            *info->module,
+            llTy,
+            false, /* is constant */
+            hasFlag(FLAG_EXPORT) ? llvm::GlobalVariable::ExternalLinkage
+                                 : llvm::GlobalVariable::InternalLinkage,
+            llvm::Constant::getNullValue(llTy), /* initializer, */
+            cname);
 
       info->lvt->addGlobalValue(cname, gVar, GEN_PTR, ! is_signed(type) );
     }
@@ -693,8 +738,8 @@ void VarSymbol::codegenDef() {
     llvm::Value *varAlloca = createTempVarLLVM(varType, cname);
     info->lvt->addValue(cname, varAlloca, GEN_PTR, ! is_signed(type));
     
-    if(ClassType *ctype = toClassType(type)) {
-      if(ctype->classTag == CLASS_CLASS ||
+    if(AggregateType *ctype = toAggregateType(type)) {
+      if(ctype->isClass() ||
          ctype->symbol->hasFlag(FLAG_WIDE) ||
          ctype->symbol->hasFlag(FLAG_WIDE_CLASS)) {
         if(isFnSymbol(defPoint->parentSymbol)) {
@@ -713,21 +758,6 @@ bool VarSymbol::isImmediate() {
 }
 
 
-// describes the intent (for use in an English sentence)
-const char* intentDescrName[INTENT_TYPE+1] = {
-  "blank intent",
-  "'in'",
-  "'inout'",
-  "'out'",
-  "'const'",
-  "'const in'",
-  "'const ref'",
-  "'ref'",
-  "'param'",
-  "'type'"
-};
-
-
 ArgSymbol::ArgSymbol(IntentTag iIntent, const char* iName, 
                      Type* iType, Expr* iTypeExpr,
                      Expr* iDefaultExpr, Expr* iVariableExpr) :
@@ -800,6 +830,7 @@ ArgSymbol*
 ArgSymbol::copyInner(SymbolMap* map) {
   ArgSymbol *ps = new ArgSymbol(intent, name, type, COPY_INT(typeExpr),
                                 COPY_INT(defaultExpr), COPY_INT(variableExpr));
+  ps->copyFlags(this);
   ps->cname = cname;
   ps->instantiatedFrom = instantiatedFrom;
   ps->instantiatedParam = instantiatedParam;
@@ -820,10 +851,8 @@ void ArgSymbol::replaceChild(BaseAST* old_ast, BaseAST* new_ast) {
 }
 
 bool argMustUseCPtr(Type* type) {
-  // Don't pass global ptrs by ref
-  if( fLLVMWideOpt && type->symbol->hasEitherFlag(FLAG_WIDE,FLAG_WIDE_CLASS) )
+  if (isWideString(type))
     return false;
-
   if (isRecord(type) || isUnion(type))
     return true;
   return false;
@@ -846,9 +875,19 @@ bool ArgSymbol::isConstant(void) {
   }
   return (intent == INTENT_BLANK || intent == INTENT_CONST) &&
     !isReferenceType(type) &&
+    !isSyncType(type) &&
     !isRecordWrappedType(type) /* array, domain, distribution */;
 }
 
+bool ArgSymbol::isConstValWillNotChange(void) {
+  //
+  // This is written to only be called post resolveIntents
+  //
+  assert (intent != INTENT_BLANK && intent != INTENT_CONST);
+  return (intent == INTENT_CONST_IN);
+}
+
+
 
 bool ArgSymbol::isParameter(void) {
   return (intent == INTENT_PARAM);
@@ -857,7 +896,21 @@ bool ArgSymbol::isParameter(void) {
 
 // describes this argument's intent (for use in an English sentence)
 const char* ArgSymbol::intentDescrString(void) {
-  return intentDescrName[intent];
+  switch (intent) {
+    case INTENT_BLANK: return "blank intent";
+    case INTENT_IN: return "'in'";
+    case INTENT_INOUT: return "'inout'";
+    case INTENT_OUT: return "'out'";
+    case INTENT_CONST: return "'const'";
+    case INTENT_CONST_IN: return "'const in'";
+    case INTENT_CONST_REF: return "'const ref'";
+    case INTENT_REF: return "'ref'";
+    case INTENT_PARAM: return "'param'";
+    case INTENT_TYPE: return "'type'";
+  }
+
+  INT_FATAL(this, "unknown intent");
+  return "unknown intent";
 }
 
 
@@ -939,13 +992,8 @@ TypeSymbol::copyInner(SymbolMap* map) {
   Type* new_type = COPY_INT(type);
   TypeSymbol* new_type_symbol = new TypeSymbol(name, new_type);
   new_type->addSymbol(new_type_symbol);
+  new_type_symbol->copyFlags(this);
   new_type_symbol->cname = cname;
-  if (this->hasFlag(FLAG_SYNC))
-    new_type_symbol->addFlag(FLAG_SYNC);
-  if (this->hasFlag(FLAG_SINGLE))
-    new_type_symbol->addFlag(FLAG_SINGLE);
-  if (this->hasFlag(FLAG_ATOMIC_TYPE))
-    new_type_symbol->addFlag(FLAG_ATOMIC_TYPE);
   return new_type_symbol;
 }
 
@@ -1010,13 +1058,13 @@ void TypeSymbol::codegenMetadata() {
   // avoid recursing.
   llvmTbaaNode = info->tbaaRootNode;
 
-  ClassType* ct = toClassType(type);
+  AggregateType* ct = toAggregateType(type);
 
   Type* superType = NULL;
   // Recursively generate the TBAA nodes for this type.
   if( ct ) {
     for_fields(field, ct) {
-      ClassType* fct = toClassType(field->type);
+      AggregateType* fct = toAggregateType(field->type);
       if(fct && field->hasFlag(FLAG_SUPER_CLASS)) {
         superType = field->type;
       }
@@ -1044,20 +1092,31 @@ void TypeSymbol::codegenMetadata() {
     return;
   }
 
-  // Now create tbaa metadata, one for const and one for not.
-  {
-    llvm::Value* Ops[2];
-    Ops[0] = llvm::MDString::get(ctx, cname);
-    Ops[1] = parent;
-    llvmTbaaNode = llvm::MDNode::get(ctx, Ops);
-  }
-  {
-    llvm::Value* Ops[3];
-    Ops[0] = llvm::MDString::get(ctx, cname);
-    Ops[1] = constParent;
-    Ops[2] = llvm::ConstantInt::get(llvm::Type::getInt64Ty(ctx), 1);
-    llvmConstTbaaNode = llvm::MDNode::get(ctx, Ops);
+  // Only things that aren't 'struct'-like should have simple TBAA
+  // metadata. If they can alias with their fields, we don't do simple TBAA.
+  // Integers, reals, bools, enums, references, wide pointers
+  // count as one thing. Records, strings, complexes should not
+  // get simple TBAA (they can get struct tbaa).
+  if( is_bool_type(type) || is_int_type(type) || is_uint_type(type) ||
+      is_real_type(type) || is_imag_type(type) || is_enum_type(type) ||
+      isClass(type) || hasEitherFlag(FLAG_REF,FLAG_WIDE) ||
+      hasEitherFlag(FLAG_DATA_CLASS,FLAG_WIDE_CLASS) ) {
+    // Now create tbaa metadata, one for const and one for not.
+    {
+      llvm::Value* Ops[2];
+      Ops[0] = llvm::MDString::get(ctx, cname);
+      Ops[1] = parent;
+      llvmTbaaNode = llvm::MDNode::get(ctx, Ops);
+    }
+    {
+      llvm::Value* Ops[3];
+      Ops[0] = llvm::MDString::get(ctx, cname);
+      Ops[1] = constParent;
+      Ops[2] = llvm::ConstantInt::get(llvm::Type::getInt64Ty(ctx), 1);
+      llvmConstTbaaNode = llvm::MDNode::get(ctx, Ops);
+    }
   }
+
   // Don't try to create tbaa.struct metadata for non-struct.
   if( isUnion(type) ||
       hasFlag(FLAG_STAR_TUPLE) ||
@@ -1082,7 +1141,7 @@ void TypeSymbol::codegenMetadata() {
 
     for_fields(field, ct) {
       llvm::Type* fieldType = field->type->symbol->codegen().type;
-      ClassType* fct = toClassType(field->type);
+      AggregateType* fct = toAggregateType(field->type);
       if(fct && field->hasFlag(FLAG_SUPER_CLASS)) {
         fieldType = info->lvt->getType(fct->classStructName(true));
       }
@@ -1172,7 +1231,9 @@ void FnSymbol::verify() {
   if (astTag != E_FnSymbol) {
     INT_FATAL(this, "Bad FnSymbol::astTag");
   }
-  if (normalized && !hasFlag(FLAG_AUTO_II)) {
+  if (_this && _this->defPoint->parentSymbol != this)
+    INT_FATAL(this, "Each method must contain a 'this' declaration.");
+  if (normalized) {
     CallExpr* last = toCallExpr(body->body.last());
     if (!last || !last->isPrimitive(PRIM_RETURN))
       INT_FATAL(this, "Last statement in normalized function is not a return");
@@ -1196,16 +1257,7 @@ FnSymbol* FnSymbol::getFnSymbol(void) {
 FnSymbol*
 FnSymbol::copyInner(SymbolMap* map) {
   FnSymbol* copy = new FnSymbol(name);
-  if (hasFlag(FLAG_CONSTRUCTOR))
-    copy->addFlag(FLAG_CONSTRUCTOR);
-  else if (hasFlag(FLAG_INIT_COPY_FN))
-    copy->addFlag(FLAG_INIT_COPY_FN);
-  else if (hasFlag(FLAG_AUTO_COPY_FN))
-    copy->addFlag(FLAG_AUTO_COPY_FN);
-  else if (hasFlag(FLAG_AUTO_DESTROY_FN))
-    copy->addFlag(FLAG_AUTO_DESTROY_FN);
-  if (hasFlag(FLAG_DONOR_FN))
-    copy->addFlag(FLAG_DONOR_FN);
+  copy->copyFlags(this);
   for_formals(formal, this) {
     copy->insertFormalAtTail(COPY_INT(formal->defPoint));
   }
@@ -1428,24 +1480,6 @@ void FnSymbol::codegenPrototype() {
 }
 
 
-static void
-codegenNullAssignments(FILE* outfile, const char* cname, ClassType* ct) {
-  if (isClass(ct))
-    fprintf(outfile, "%s = NULL;\n", cname);
-  else {
-    for_fields(field, ct) {
-      if (ClassType* fct = toClassType(field->type)) {
-        char buffer[1024];
-        strcpy(buffer, cname);
-        strcat(buffer, ".");
-        strcat(buffer, field->cname);
-        codegenNullAssignments(outfile, buffer, fct);
-      }
-    }
-  }
-}
-
-
 void FnSymbol::codegenDef() {
   GenInfo *info = gGenInfo;
   FILE* outfile = info->cfile;
@@ -1637,6 +1671,18 @@ FnSymbol::insertBeforeReturnAfterLabel(Expr* ast) {
 }
 
 
+void
+FnSymbol::insertBeforeDownEndCount(Expr* ast) {
+  CallExpr* ret = toCallExpr(body->body.last());
+  if (!ret || !ret->isPrimitive(PRIM_RETURN))
+    INT_FATAL(this, "function is not normal");
+  CallExpr* last = toCallExpr(ret->prev);
+  if (!last || strcmp(last->isResolved()->name, "_downEndCount"))
+    INT_FATAL(last, "Expected call to _downEndCount");
+  last->insertBefore(ast);
+}
+
+
 void
 FnSymbol::insertFormalAtHead(BaseAST* ast) {
   if (ArgSymbol* arg = toArgSymbol(ast))
@@ -1742,7 +1788,9 @@ void EnumSymbol::verify() {
 
 EnumSymbol*
 EnumSymbol::copyInner(SymbolMap* map) {
-  return new EnumSymbol(name);
+  EnumSymbol* copy = new EnumSymbol(this->name);
+  copy->copyFlags(this);
+  return copy;
 }
 
 void EnumSymbol::replaceChild(BaseAST* old_ast, BaseAST* new_ast) {
@@ -1801,8 +1849,8 @@ ModuleSymbol::copyInner(SymbolMap* map) {
 
 
 static int compareLineno(const void* v1, const void* v2) {
-  FnSymbol* fn1 = *(FnSymbol**)v1;
-  FnSymbol* fn2 = *(FnSymbol**)v2;
+  FnSymbol* fn1 = *(FnSymbol* const *)v1;
+  FnSymbol* fn2 = *(FnSymbol* const *)v2;
   if (fn1->linenum() > fn2->linenum())
     return 1;
   else if (fn1->linenum() < fn2->linenum())
@@ -1839,8 +1887,8 @@ void ModuleSymbol::codegenDef() {
   return;
 }
  
-Vec<ClassType*> ModuleSymbol::getClasses() {
-  Vec<ClassType*> classes;
+Vec<AggregateType*> ModuleSymbol::getClasses() {
+  Vec<AggregateType*> classes;
   for_alist(expr, block->body) {
     if (DefExpr* def = toDefExpr(expr))
       if (FnSymbol* fn = toFnSymbol(def->sym)) {
@@ -1849,7 +1897,7 @@ Vec<ClassType*> ModuleSymbol::getClasses() {
           for_alist(expr2, fn->body->body) {
             if (DefExpr* def2 = toDefExpr(expr2))
               if (TypeSymbol* type = toTypeSymbol(def2->sym)) 
-                if (ClassType* cl = toClassType(type->type)) {
+                if (AggregateType* cl = toAggregateType(type->type)) {
                   classes.add(cl);
                 }
           }
@@ -1963,6 +2011,7 @@ void LabelSymbol::verify() {
 LabelSymbol* 
 LabelSymbol::copyInner(SymbolMap* map) {
   LabelSymbol* copy = new LabelSymbol(name);
+  copy->copyFlags(this);
   copy->cname = cname;
   if (iterResumeGoto) {
     MapElem<GotoStmt*,GotoStmt*>* rec =
@@ -2089,7 +2138,9 @@ VarSymbol *new_UIntSymbol(uint64_t b, IF1_int_type size) {
   return s;
 }
 
-VarSymbol *new_RealSymbol(const char *n, long double b, IF1_float_type size) {
+static VarSymbol* new_FloatSymbol(const char* n, long double b, 
+                                  IF1_float_type size, IF1_num_kind kind, 
+                                  Type* type) {
   Immediate imm;
   switch (size) {
   case FLOAT_SIZE_32  : imm.v_float32  = b; break;
@@ -2097,44 +2148,31 @@ VarSymbol *new_RealSymbol(const char *n, long double b, IF1_float_type size) {
   default:
     INT_FATAL( "unknown FLOAT_SIZE");
   }
-  imm.const_kind = NUM_KIND_FLOAT;
+  imm.const_kind = kind;
   imm.num_index = size;
   VarSymbol *s = uniqueConstantsHash.get(&imm);
-  PrimitiveType* dtRetType = dtReal[size];
   if (s) {
     return s;
   }
-  s = new VarSymbol(astr("_literal_", istr(literal_id++)), dtRetType);
+  s = new VarSymbol(astr("_literal_", istr(literal_id++)), type);
   rootModule->block->insertAtTail(new DefExpr(s));
-  s->cname = astr(n);
+  if (!strchr(n, '.') && !strchr(n, 'e') && !(strchr(n, 'E'))) {
+    s->cname = astr(n, ".0");
+  } else {
+    s->cname = astr(n);
+  }
   s->immediate = new Immediate;
   *s->immediate = imm;
   uniqueConstantsHash.put(s->immediate, s);
   return s;
 }
 
+VarSymbol *new_RealSymbol(const char *n, long double b, IF1_float_type size) {
+  return new_FloatSymbol(n, b, size, NUM_KIND_REAL, dtReal[size]);
+}
+
 VarSymbol *new_ImagSymbol(const char *n, long double b, IF1_float_type size) {
-  Immediate imm;
-  switch (size) {
-  case FLOAT_SIZE_32  : imm.v_float32  = b; break;
-  case FLOAT_SIZE_64  : imm.v_float64  = b; break;
-  default:
-    INT_FATAL( "unknown FLOAT_SIZE");
-  }
-  imm.const_kind = NUM_KIND_IMAG;
-  imm.num_index = size;
-  VarSymbol *s = uniqueConstantsHash.get(&imm);
-  PrimitiveType* dtRetType = dtImag[size];
-  if (s) {
-    return s;
-  }
-  s = new VarSymbol(astr("_literal_", istr(literal_id++)), dtRetType);
-  rootModule->block->insertAtTail(new DefExpr(s));
-  s->cname = astr(n);
-  s->immediate = new Immediate;
-  *s->immediate = imm;
-  uniqueConstantsHash.put(s->immediate, s);
-  return s;
+  return new_FloatSymbol(n, b, size, NUM_KIND_IMAG, dtImag[size]);
 }
 
 VarSymbol *new_ComplexSymbol(const char *n, long double r, long double i,
@@ -2177,7 +2215,7 @@ immediate_type(Immediate *imm) {
       return dtUInt[imm->num_index];
     case NUM_KIND_INT:
       return dtInt[imm->num_index];
-    case NUM_KIND_FLOAT:
+    case NUM_KIND_REAL:
       return dtReal[imm->num_index];
     case NUM_KIND_IMAG:
       return dtImag[imm->num_index];
