@@ -1,5 +1,5 @@
 /**************************************************************************
-  Copyright (c) 2004-2013, Cray Inc.  (See LICENSE file for more details)
+  Copyright (c) 2004-2014, Cray Inc.  (See LICENSE file for more details)
 **************************************************************************/
 
 
@@ -384,10 +384,10 @@ void mark_view(BaseAST* ast, int id) {
   fflush(stdout);
 }
 
-static ClassType *
+static AggregateType *
 structuralTypeSymbol(Symbol *s) {
   if (TypeSymbol *ts = toTypeSymbol(s))
-    if (ClassType *st = toClassType(ts->type))
+    if (AggregateType *st = toAggregateType(ts->type))
       return st;
   return NULL;
 }
@@ -612,7 +612,7 @@ html_view_ast(BaseAST* ast, FILE* html_file, int pass) {
     if (DefExpr* e = toDefExpr(expr)) {
       if (toFnSymbol(e->sym) || 
           (toTypeSymbol(e->sym) &&
-           toClassType(e->sym->type))) {
+           toAggregateType(e->sym->type))) {
         fprintf(html_file, "</UL>\n");
         if (FnSymbol* fn = toFnSymbol(e->sym)) {
           fprintf(html_file, "<CHPLTAG=\"FN%d\">\n", fn->id);
@@ -662,8 +662,8 @@ void html_view(const char* passName) {
     fprintf(html_file, "<HTML>\n");
     fprintf(html_file, "<HEAD>\n");
     fprintf(html_file, "<TITLE> AST for Module %s after Pass %s </TITLE>\n", mod->name, passName);
-    fprintf(html_file, "<SCRIPT SRC=\"%s/compiler/etc/www/mktree.js\" LANGUAGE=\"JavaScript\"></SCRIPT>\n", chpl_home);
-    fprintf(html_file, "<LINK REL=\"stylesheet\" HREF=\"%s/compiler/etc/www/mktree.css\">\n", chpl_home);
+    fprintf(html_file, "<SCRIPT SRC=\"http://chapel.cray.com/developer/mktree.js\" LANGUAGE=\"JavaScript\"></SCRIPT>\n");
+    fprintf(html_file, "<LINK REL=\"stylesheet\" HREF=\"http://chapel.cray.com/developer/mktree.css\">\n");
     fprintf(html_file, "</HEAD><BODY%s>\n",
             fdump_html_wrap_lines ? "" : " style=\"white-space: nowrap;\"");
     if (currentPassNo > 1)
@@ -1169,10 +1169,10 @@ void whocalls(int id) {
       if (SymExpr* act2 = toSymExpr(call->get(2)))
         if (Symbol* ic = act2->var)
           if (Type* ty = ic->type)
-            if (FnSymbol* init = ty->initializer)
+            if (FnSymbol* init = ty->defaultInitializer)
               if (ArgSymbol* form1 = init->getFormal(1))
                 if (Type* fty = form1->type)
-                  if (FnSymbol* iterator = fty->initializer)
+                  if (FnSymbol* iterator = fty->defaultInitializer)
                     if (iterator->id == id)
                       printf("  for-loop blockInfo %d  %s  %s\n",
                              call->id, parentMsg(call, &forMatch,
