@@ -1,5 +1,5 @@
 /**************************************************************************
-  Copyright (c) 2004-2013, Cray Inc.  (See LICENSE file for more details)
+  Copyright (c) 2004-2014, Cray Inc.  (See LICENSE file for more details)
 **************************************************************************/
 
 
@@ -40,7 +40,7 @@ switch (to->const_kind) {
        default:
          INT_FATAL("Illegal case in coerce_immediate switch statement"); break;
        } break;
-     case NUM_KIND_FLOAT:
+     case NUM_KIND_REAL:
        switch (from->num_index) {
        case FLOAT_SIZE_16:
          INT_FATAL("Illegal case in coerce_immediate switch statement"); break;
@@ -95,7 +95,7 @@ switch (to->const_kind) {
        default:
          INT_FATAL("Illegal case in coerce_immediate switch statement"); break;
        } break;
-     case NUM_KIND_FLOAT:
+     case NUM_KIND_REAL:
        switch (from->num_index) {
        case FLOAT_SIZE_16:
          INT_FATAL("Illegal case in coerce_immediate switch statement"); break;
@@ -150,7 +150,7 @@ switch (to->const_kind) {
        default:
          INT_FATAL("Illegal case in coerce_immediate switch statement"); break;
        } break;
-     case NUM_KIND_FLOAT:
+     case NUM_KIND_REAL:
        switch (from->num_index) {
        case FLOAT_SIZE_16:
          INT_FATAL("Illegal case in coerce_immediate switch statement"); break;
@@ -205,7 +205,7 @@ switch (to->const_kind) {
        default:
          INT_FATAL("Illegal case in coerce_immediate switch statement"); break;
        } break;
-     case NUM_KIND_FLOAT:
+     case NUM_KIND_REAL:
        switch (from->num_index) {
        case FLOAT_SIZE_16:
          INT_FATAL("Illegal case in coerce_immediate switch statement"); break;
@@ -260,7 +260,7 @@ switch (to->const_kind) {
        default:
          INT_FATAL("Illegal case in coerce_immediate switch statement"); break;
        } break;
-     case NUM_KIND_FLOAT:
+     case NUM_KIND_REAL:
        switch (from->num_index) {
        case FLOAT_SIZE_16:
          INT_FATAL("Illegal case in coerce_immediate switch statement"); break;
@@ -320,7 +320,7 @@ switch (to->const_kind) {
        default:
          INT_FATAL("Illegal case in coerce_immediate switch statement"); break;
        } break;
-     case NUM_KIND_FLOAT:
+     case NUM_KIND_REAL:
        switch (from->num_index) {
        case FLOAT_SIZE_16:
          INT_FATAL("Illegal case in coerce_immediate switch statement"); break;
@@ -375,7 +375,7 @@ switch (to->const_kind) {
        default:
          INT_FATAL("Illegal case in coerce_immediate switch statement"); break;
        } break;
-     case NUM_KIND_FLOAT:
+     case NUM_KIND_REAL:
        switch (from->num_index) {
        case FLOAT_SIZE_16:
          INT_FATAL("Illegal case in coerce_immediate switch statement"); break;
@@ -430,7 +430,7 @@ switch (to->const_kind) {
        default:
          INT_FATAL("Illegal case in coerce_immediate switch statement"); break;
        } break;
-     case NUM_KIND_FLOAT:
+     case NUM_KIND_REAL:
        switch (from->num_index) {
        case FLOAT_SIZE_16:
          INT_FATAL("Illegal case in coerce_immediate switch statement"); break;
@@ -485,7 +485,7 @@ switch (to->const_kind) {
        default:
          INT_FATAL("Illegal case in coerce_immediate switch statement"); break;
        } break;
-     case NUM_KIND_FLOAT:
+     case NUM_KIND_REAL:
        switch (from->num_index) {
        case FLOAT_SIZE_16:
          INT_FATAL("Illegal case in coerce_immediate switch statement"); break;
@@ -540,7 +540,7 @@ switch (to->const_kind) {
        default:
          INT_FATAL("Illegal case in coerce_immediate switch statement"); break;
        } break;
-     case NUM_KIND_FLOAT:
+     case NUM_KIND_REAL:
        switch (from->num_index) {
        case FLOAT_SIZE_16:
          INT_FATAL("Illegal case in coerce_immediate switch statement"); break;
@@ -565,7 +565,7 @@ switch (to->const_kind) {
    default:
      INT_FATAL("Illegal case in coerce_immediate switch statement"); break;
    } break;
- case NUM_KIND_FLOAT:
+ case NUM_KIND_REAL:
    switch (to->num_index) {
    case FLOAT_SIZE_16:
      INT_FATAL("Illegal case in coerce_immediate switch statement"); break;
@@ -602,7 +602,7 @@ switch (to->const_kind) {
        default:
          INT_FATAL("Illegal case in coerce_immediate switch statement"); break;
        } break;
-     case NUM_KIND_FLOAT:
+     case NUM_KIND_REAL:
        switch (from->num_index) {
        case FLOAT_SIZE_16:
          INT_FATAL("Illegal case in coerce_immediate switch statement"); break;
@@ -659,7 +659,7 @@ switch (to->const_kind) {
        default:
          INT_FATAL("Illegal case in coerce_immediate switch statement"); break;
        } break;
-     case NUM_KIND_FLOAT:
+     case NUM_KIND_REAL:
        switch (from->num_index) {
        case FLOAT_SIZE_16:
          INT_FATAL("Illegal case in coerce_immediate switch statement"); break;
@@ -720,7 +720,7 @@ switch (to->const_kind) {
        default:
          INT_FATAL("Illegal case in coerce_immediate switch statement"); break;
        } break;
-     case NUM_KIND_FLOAT:
+     case NUM_KIND_REAL:
        switch (from->num_index) {
        case FLOAT_SIZE_16:
          INT_FATAL("Illegal case in coerce_immediate switch statement"); break;
@@ -776,7 +776,7 @@ switch (to->const_kind) {
      default:
        INT_FATAL("Illegal case in coerce_immediate switch statement"); break;
      } break;
-   case NUM_KIND_FLOAT:
+   case NUM_KIND_REAL:
      switch (from->num_index) {
      case FLOAT_SIZE_16:
        INT_FATAL("Illegal case in coerce_immediate switch statement"); break;
