@@ -1,5 +1,5 @@
 /**************************************************************************
-  Copyright (c) 2004-2013, Cray Inc.  (See LICENSE file for more details)
+  Copyright (c) 2004-2014, Cray Inc.  (See LICENSE file for more details)
 **************************************************************************/
 
 
@@ -13,8 +13,8 @@ public:
   IteratorTag tag;
   FnSymbol* iterator;
   FnSymbol* getIterator;
-  ClassType* iclass;
-  ClassType* irecord;
+  AggregateType* iclass;
+  AggregateType* irecord;
   FnSymbol* advance;
   FnSymbol* zip1;
   FnSymbol* zip2;
@@ -27,6 +27,4 @@ public:
 
 void lowerIterator(FnSymbol* fn);
 
-extern Vec<FnSymbol*> iteratorsWithRemovedRetInitSet;
-
 #endif
