@@ -1,5 +1,5 @@
 /**************************************************************************
-  Copyright (c) 2004-2013, Cray Inc.  (See LICENSE file for more details)
+  Copyright (c) 2004-2014, Cray Inc.  (See LICENSE file for more details)
 **************************************************************************/
 
 
@@ -7,8 +7,8 @@
 #define SCOPE_RESOLVE_H
 
 
-void build_constructors(ClassType* ct);
-void add_root_type(ClassType* ct);
+void build_constructors(AggregateType* ct);
+void add_root_type(AggregateType* ct);
 
 #ifdef HAVE_LLVM
 // this one is defined in externCResolve.cpp
