@@ -1,5 +1,5 @@
 /**************************************************************************
-  Copyright (c) 2004-2013, Cray Inc.  (See LICENSE file for more details)
+  Copyright (c) 2004-2014, Cray Inc.  (See LICENSE file for more details)
 **************************************************************************/
 
 
@@ -23,7 +23,7 @@
 #ifdef debugHoisting
   #define printDebug(string) printf string
 #else 
- #define printDebug(string) //do nothing
+  #define printDebug(string) //do nothing
 #endif
 
 
@@ -273,18 +273,58 @@ void collectNaturalLoopForEdge(Loop* loop, BasicBlock* header, BasicBlock* tail)
   } 
 }
 
+/*
+ * Takes a call that is a PRIM_SVEC_GET_MEMBER* and returns the symbol of the
+ * field. Normally the call is something of the form PRIM_SVEC_GET_MEMBER(p, 1) 
+ * and what this function gets out is the symbol that is the first field
+ * instead of just the number 1. 
+ */
+static Symbol* getSvecSymbol(CallExpr* call) {
+  INT_ASSERT(call->isPrimitive(PRIM_GET_SVEC_MEMBER)       ||
+             call->isPrimitive(PRIM_GET_SVEC_MEMBER_VALUE) ||
+             call->isPrimitive(PRIM_SET_SVEC_MEMBER));
+  
+  Type* type = call->get(1)->getValType();
+  AggregateType* tuple = toAggregateType(type);
+  SymExpr* fieldVal = toSymExpr(call->get(2));
+  VarSymbol* fieldSym = toVarSymbol(fieldVal->var);
+  int immediateVal = fieldSym->immediate->int_value();
+
+  INT_ASSERT(immediateVal >= 1 && immediateVal <= tuple->fields.length);
+  return tuple->getField(immediateVal);
+}
 
 // Returns rhs var if lhs aliases rhs
 //
 //
 static Symbol*
 rhsAlias(CallExpr* call) {
+
+  bool hasRef = false;
+  for_alist(expr, call->argList) {
+    if(SymExpr* symExpr = toSymExpr(expr)) {
+      Type* symType = symExpr->var->type->symbol->type;
+      bool isWideClass = symExpr->var->type->symbol->hasFlag(FLAG_WIDE_CLASS);
+      bool isWideRef = symExpr->var->type->symbol->hasFlag(FLAG_WIDE);
+      bool isWideStr = isWideString(symExpr->var->type);
+      if(isReferenceType(symType) || isRecordWrappedType(symType) ||
+        isWideClass || isWideRef || isWideStr) {
+        hasRef = true;
+      }
+    }
+  }
+  if (!hasRef) {
+    return NULL;
+  }
+
   if (call->isPrimitive(PRIM_MOVE) ||
+      call->isPrimitive(PRIM_ASSIGN) ||
       call->isPrimitive(PRIM_SET_MEMBER) ||
       call->isPrimitive(PRIM_SET_SVEC_MEMBER)) {
     SymExpr* lhs;
     Expr *rhsT;
-    if (call->isPrimitive(PRIM_MOVE)) {
+    if (call->isPrimitive(PRIM_MOVE) ||
+        call->isPrimitive(PRIM_ASSIGN)) {
       lhs = toSymExpr(call->get(1));
       rhsT = call->get(2);
     } else {
@@ -298,12 +338,16 @@ rhsAlias(CallExpr* call) {
     } else if (CallExpr* rhsCall = toCallExpr(rhsT)) {
       if (rhsCall->primitive) {
         if (rhsCall->isPrimitive(PRIM_GET_MEMBER_VALUE) ||
-            rhsCall->isPrimitive(PRIM_GET_MEMBER) ||
-            rhsCall->isPrimitive(PRIM_GET_SVEC_MEMBER_VALUE) ||
-            rhsCall->isPrimitive(PRIM_GET_SVEC_MEMBER)) {
+            rhsCall->isPrimitive(PRIM_GET_MEMBER)) {
           SymExpr* rhs = toSymExpr(rhsCall->get(2));
           INT_ASSERT(rhs);
           return rhs->var;
+        } else if (rhsCall->isPrimitive(PRIM_GET_SVEC_MEMBER) || 
+                   rhsCall->isPrimitive(PRIM_GET_SVEC_MEMBER_VALUE)) {
+          // get_svec's second argument is the integer offset to the field,
+          // instead of the field itself. We need to return the field so that
+          // we note that the lhs aliases the field and not just an integer 
+          return getSvecSymbol(rhsCall);
         } else if(rhsCall->isPrimitive(PRIM_ADDR_OF)) {
           SymExpr* rhs = toSymExpr(rhsCall->get(1));
           INT_ASSERT(rhs);
@@ -355,7 +399,8 @@ static bool isLoopInvariantPrimitive(PrimitiveOp* primitiveOp)
       case PRIM_OR:
       case PRIM_XOR:
       case PRIM_POW:
-    
+
+      case PRIM_ASSIGN:
       case PRIM_ADD_ASSIGN:
       case PRIM_SUBTRACT_ASSIGN:
       case PRIM_MULT_ASSIGN:
@@ -438,7 +483,7 @@ static void addDefOrUse(symToVecSymExprMap& localDefOrUseMap, Symbol* var, SymEx
 
 
 /*
- * Build the local def use maps for a loop and while we're at it built the local map which is the map from each 
+ * Build the local def use maps for a loop and while we're at it build the local map which is the map from each
  * symExpr to the block it it is defined in.
  */
 static void buildLocalDefUseMaps(Loop* loop, symToVecSymExprMap& localDefMap, symToVecSymExprMap& localUseMap, std::map<SymExpr*, int>& localMap) {
@@ -450,7 +495,11 @@ static void buildLocalDefUseMaps(Loop* loop, symToVecSymExprMap& localDefMap, sy
         if(callExpr->isPrimitive(PRIM_SET_MEMBER) ||
            callExpr->isPrimitive(PRIM_SET_SVEC_MEMBER)) {
           if(SymExpr* symExpr = toSymExpr(callExpr->get(2))) {
-            addDefOrUse(localDefMap, symExpr->var, symExpr);
+            if(callExpr->isPrimitive(PRIM_SET_SVEC_MEMBER)) {
+              addDefOrUse(localDefMap, getSvecSymbol(callExpr), symExpr);
+            } else {
+              addDefOrUse(localDefMap, symExpr->var, symExpr);
+            }
           }
           //and the "class" itself may also be defed. For instance if you change
           //the field of a record or union you have to note that the
@@ -459,8 +508,8 @@ static void buildLocalDefUseMaps(Loop* loop, symToVecSymExprMap& localDefMap, sy
           //pointer to a class is changed that will be detected by isDefAndOrUse
           if(SymExpr* symExpr = toSymExpr(callExpr->get(1))) {
             Type* type = symExpr->var->type->symbol->type;
-            if(ClassType* curClass = toClassType(type)) {
-              if(curClass->classTag != CLASS_CLASS) {
+            if(AggregateType* curClass = toAggregateType(type)) {
+              if(! curClass->isClass()) {
                 addDefOrUse(localDefMap, symExpr->var, symExpr);
               }
             }
@@ -489,7 +538,7 @@ static void buildLocalDefUseMaps(Loop* loop, symToVecSymExprMap& localDefMap, sy
               if(callExpr->isResolved()) {
                 addDefOrUse(localDefMap, symExpr->var, symExpr);
                 Type* type = symExpr->var->type->symbol->type;
-                if(ClassType* curClass = toClassType(type)) {
+                if(AggregateType* curClass = toAggregateType(type)) {
                   for_alist(classField, curClass->fields) {
                     if(DefExpr* classFieldDef = toDefExpr(classField)) {
                       addDefOrUse(localDefMap, classFieldDef->sym, symExpr);
@@ -533,7 +582,7 @@ static bool allOperandsAreLoopInvariant(Expr* expr, std::set<SymExpr*>& loopInva
   //are invariant
   if(CallExpr* callExpr = toCallExpr(expr)) {
     if(callExpr->primitive && isLoopInvariantPrimitive(callExpr->primitive)) {
-      if(callExpr->isPrimitive(PRIM_MOVE)) {  
+      if(callExpr->isPrimitive(PRIM_MOVE) || callExpr->isPrimitive(PRIM_ASSIGN)) {
         return allOperandsAreLoopInvariant(callExpr->get(2), loopInvariants, loopInvariantInstructions, loop, actualDefs);
       }  
       else {
@@ -664,14 +713,30 @@ static void computeLoopInvariants(std::vector<SymExpr*>& loopInvariants, Loop*
   //compute the map of aliases for each symbol 
   startTimer(computeAliasTimer);
   std::map<Symbol*, std::set<Symbol*> > aliases;
+
+  //Compute the aliases for the function's parameters. Any args passed by ref
+  //can potentially alias each other. 
+  for_alist(formal1, fn->formals) {
+    for_alist(formal2, fn->formals) {
+      if(formal1 == formal2) 
+        continue;
+      if(ArgSymbol* arg1 = toArgSymbol(toDefExpr(formal1)->sym)) {
+        if(ArgSymbol* arg2 = toArgSymbol(toDefExpr(formal2)->sym)) {
+          if(arg1->intent == INTENT_REF && arg2->intent == INTENT_REF) {
+            aliases[arg1].insert(arg2);
+            aliases[arg2].insert(arg1);
+          }
+        }
+      }
+    }
+  }
+
   for_vector(BasicBlock, block2, *fn->basicBlocks) {
     //if there are too many aliases, just return. Since nothing has been added
     //to the list of invariants, nothing will be hoisted from the current fn
     if(numAliases > MAX_NUM_ALIASES) {
 #ifdef detailedTiming 
-       FILE* tooManyAliasesFile = fopen("/data/cf/gtmp/chapel/eronagha"  
-           "/tooManyAliases.txt", "a");
-       
+       FILE* tooManyAliasesFile = fopen(astr(CHPL_HOME,"/LICMaliases.txt"), "a");
        fprintf(tooManyAliasesFile, "Skipping fn %s %d of module %s %d "
            "because there were too many aliases\n", fn->name, fn->id, 
            fn->getModule()->name, fn->getModule()->id);
@@ -682,30 +747,12 @@ static void computeLoopInvariants(std::vector<SymExpr*>& loopInvariants, Loop*
       return;
     }
  
-    //Compute the aliases for the function's parameters. Any args passed by ref
-    //can potentially alias each other. 
-    for_alist(formal1, fn->formals) {
-      for_alist(formal2, fn->formals) {
-        if(formal1 == formal2) 
-          continue;
-        if(ArgSymbol* arg1 = toArgSymbol(toDefExpr(formal1)->sym)) {
-          if(ArgSymbol* arg2 = toArgSymbol(toDefExpr(formal2)->sym)) {
-            if(arg1->intent == INTENT_REF && arg2->intent == INTENT_REF) {
-              aliases[arg1].insert(arg2);
-              aliases[arg2].insert(arg1);
-            }
-
-          }
-        }
-      }
-    }
-
     for_vector(Expr, expr, block2->exprs) {
       if(CallExpr* call = toCallExpr(expr)) {
         Symbol* rhs = rhsAlias(call);
         if(rhs != NULL) {
           Symbol* lhs = NULL;
-          if(call->isPrimitive(PRIM_MOVE)) 
+          if(call->isPrimitive(PRIM_MOVE) || call->isPrimitive(PRIM_ASSIGN)) 
             lhs = toSymExpr(call->get(1))->var;
           else
             lhs = toSymExpr(call->get(2))->var;
@@ -767,16 +814,61 @@ static void computeLoopInvariants(std::vector<SymExpr*>& loopInvariants, Loop*
         }
       }
     }    
-    
+   
+    // We have to note that for records if a field is deffed then the record
+    // itself is also deffed. This is handled in buildDefUseMaps for normal
+    // variables but we have to handle when a variable aliases a record's field
+    // and the alias is deffed.
+    if(isVarSymbol(symExpr->var) || isArgSymbol(symExpr->var)) {
+      // if the current variable is a record
+      if(AggregateType* ct = toAggregateType(symExpr->var->type->symbol->type)) {
+        if(isRecord(symExpr->var->type->symbol->type)) {
+          // go through each field and check for aliases
+          for_fields(field, ct) {
+            if(aliases.count(field) == 1) {
+              // for each alias
+              for_set(Symbol, alias, aliases[field]) {
+                if(localDefMap.count(alias) == 1) {
+                  // mark each def of the alias as a def of the record (so long
+                  // as that def isn't the alias creation.)
+                  for_vector(SymExpr, aliasDef, *localDefMap[alias]) {
+                    if(CallExpr* call = toCallExpr(aliasDef->parentExpr)) {
+                      if(field != rhsAlias(call)) {
+                        actualDefs[symExpr].insert(aliasDef);
+                      }
+                    }
+                  }
+                }
+              }
+            }
+          }
+        }
+      }
+    }
+
     //add the defs of the symbol itself
     if(localDefMap.count(symExpr->var) == 1) {
       for_vector(SymExpr, aliasSymExpr, *localDefMap[symExpr->var]) {
         actualDefs[symExpr].insert(aliasSymExpr);
       }
     }
-    
+
+    bool mightHaveBeenDeffedElseWhere = false;
+    //assume that anything passed in by ref has been changed elsewhere 
+    if(ArgSymbol* argSymbol = toArgSymbol(symExpr->var)) {
+      if(argSymbol->intent == INTENT_REF) {
+        mightHaveBeenDeffedElseWhere = true;
+      }
+    }
+    for_set(Symbol, aliasSym, aliases[symExpr->var]) {
+      if(ArgSymbol* argSymbol = toArgSymbol(aliasSym)) {
+        if(argSymbol->intent == INTENT_REF) {
+          mightHaveBeenDeffedElseWhere = true;
+        }
+      }
+    }
     //if there were no defs of the symbol, it is invariant 
-    if(actualDefs.count(symExpr) == 0) {
+    if(actualDefs.count(symExpr) == 0 && !mightHaveBeenDeffedElseWhere) {
       loopInvariantOperands.insert(symExpr);
     }
   }
@@ -800,15 +892,18 @@ static void computeLoopInvariants(std::vector<SymExpr*>& loopInvariants, Loop*
         if(actualDefs[symExpr2].size() == 1) {
           if(defsInLoop.count(symExpr2->var) == 1) {
             if(CallExpr* callExpr = toCallExpr(symExpr2->parentExpr)) {
-              if(callExpr->isPrimitive(PRIM_MOVE)) {
-                startTimer(allOperandsAreLoopInvariantTimer);   
-                bool loopInvarOps = allOperandsAreLoopInvariant(callExpr, loopInvariantOperands, loopInvariantInstructions, loop, actualDefs);
-                stopTimer(allOperandsAreLoopInvariantTimer);
-                if(loopInvarOps){
-                  loopInvariantInstructions.insert(symExpr2);
-                  loopInvariants.push_back(symExpr2);
+              if(callExpr->isPrimitive(PRIM_MOVE) || 
+                 callExpr->isPrimitive(PRIM_ASSIGN)) {
+                if(callExpr->get(1) == symExpr2) {
+                  startTimer(allOperandsAreLoopInvariantTimer);   
+                  bool loopInvarOps = allOperandsAreLoopInvariant(callExpr, loopInvariantOperands, loopInvariantInstructions, loop, actualDefs);
+                  stopTimer(allOperandsAreLoopInvariantTimer);
+                  if(loopInvarOps){
+                    loopInvariantInstructions.insert(symExpr2);
+                    loopInvariants.push_back(symExpr2);
+                  }
                 }
-              } 
+              }
             }
           }
         }
@@ -819,8 +914,8 @@ static void computeLoopInvariants(std::vector<SymExpr*>& loopInvariants, Loop*
  
 #ifdef debugHoisting
   printf("\n");
-  printf("HOISTABLE InvariantS\n");
-  for_vector(SymExpr, loopInvariant, loopInvariants) {
+  printf("HOISTABLE Invariants\n");
+  for_set(SymExpr, loopInvariant, loopInvariantInstructions) {
     printf("Symbol %s with id %d is a hoistable loop invariant\n", loopInvariant->var->name, loopInvariant->var->id);
   }
   
@@ -1012,7 +1107,7 @@ void loopInvariantCodeMotion(void) {
     
   //TODO use stl routine here
   forv_Vec(FnSymbol, fn, gFnSymbols) {
-  
+     
     //build the basic blocks, where the first bb is the entry block 
     startTimer(buildBBTimer);
     buildBasicBlocks(fn);
@@ -1035,10 +1130,10 @@ void loopInvariantCodeMotion(void) {
     std::vector<Loop*> loops;
     collectNaturalLoops(loops, basicBlocks, entryBlock, dominators);
     stopTimer(collectNaturalLoopsTimer);
-
+    
     //For each loop found 
     for_vector(Loop, curLoop, loops) {
-      
+
       //check that this loop doesn't have anything that 
       //would prevent code motion from occurring
       startTimer(canPerformCodeMotionTimer);
@@ -1074,9 +1169,9 @@ void loopInvariantCodeMotion(void) {
       }
                 
       freeLocalDefUseMaps(localDefMap, localUseMap);
-      numLoops+= loops.size();
     }
-   
+    numLoops += loops.size();
+    
     for_vector(Loop, loop, loops) {
       delete loop;
       loop = 0;
@@ -1093,10 +1188,8 @@ void loopInvariantCodeMotion(void) {
 #ifdef detailedTiming  
   FILE *timingFile;
   FILE *maxTimeFile;
-  timingFile = stdout;
-  maxTimeFile = stdout;
-  //timingFile = fopen("/data/cf/gtmp/chapel/eronagha/timing.txt", "a");    //TODO Where's an appropriate location?
-  //maxTimeFile = fopen("/data/cf/gtmp/chapel/eronagha/maxTime.txt", "a");
+  timingFile = fopen(astr(CHPL_HOME,"/LICMtiming.txt"), "a");
+  maxTimeFile = fopen(astr(CHPL_HOME,"/LICMmaxTime.txt"), "a");
 
   fprintf(timingFile, "For compilation of %s:                         \n", compileCommand );
   fprintf(timingFile, "Spent %2.3f seconds building basic blocks      \n", buildBBTimer.elapsed()); 
