@@ -1,5 +1,5 @@
 /**************************************************************************
-  Copyright (c) 2004-2013, Cray Inc.  (See LICENSE file for more details)
+  Copyright (c) 2004-2014, Cray Inc.  (See LICENSE file for more details)
 **************************************************************************/
 
 
@@ -35,15 +35,15 @@ static int srRecordReplaced = 0;
 // defMap - defMap for varSet
 // useMap - useMap for varSet
 //
-static Vec<ClassType*> typeVec;
+static Vec<AggregateType*> typeVec;
 static Vec<Symbol*> varSet;
-static Map<ClassType*,int> typeOrder;
-static Map<ClassType*,Vec<Symbol*>*> typeVarMap;
+static Map<AggregateType*,int> typeOrder;
+static Map<AggregateType*,Vec<Symbol*>*> typeVarMap;
 static Map<Symbol*,Vec<SymExpr*>*> defMap;
 static Map<Symbol*,Vec<SymExpr*>*> useMap;
 
-typedef Map<ClassType*,Vec<Symbol*>*> ClassTypeToVecSymbolMap;
-typedef MapElem<ClassType*,Vec<Symbol*>*> ClassTypeToVecSymbolMapElem;
+typedef Map<AggregateType*,Vec<Symbol*>*> AggregateTypeToVecSymbolMap;
+typedef MapElem<AggregateType*,Vec<Symbol*>*> AggregateTypeToVecSymbolMapElem;
 
 //
 // compute topological order for types; this functions assumes that
@@ -51,14 +51,14 @@ typedef MapElem<ClassType*,Vec<Symbol*>*> ClassTypeToVecSymbolMapElem;
 // for all class types
 //
 static int
-computeOrder(ClassType* ct) {
+computeOrder(AggregateType* ct) {
   if (typeOrder.get(ct) != -1)
     return typeOrder.get(ct);
 
   typeOrder.put(ct, -2);
   int order = 0;
   for_fields(field, ct) {
-    if (ClassType* fct = toClassType(field->type)) {
+    if (AggregateType* fct = toAggregateType(field->type)) {
       int fieldOrder = computeOrder(fct);
       if (fieldOrder >= order)
         order = fieldOrder+1;
@@ -73,8 +73,8 @@ computeOrder(ClassType* ct) {
 //
 static int
 compareTypesByOrder(const void* v1, const void* v2) {
-  ClassType* ct1 = *(ClassType**)v1;
-  ClassType* ct2 = *(ClassType**)v2;
+  AggregateType* ct1 = *(AggregateType* const *)v1;
+  AggregateType* ct2 = *(AggregateType* const *)v2;
   int order1 = typeOrder.get(ct1);
   int order2 = typeOrder.get(ct2);
   if (order1 < order2)
@@ -168,7 +168,7 @@ unifyClassInstances(Symbol* sym) {
 }
 
 static bool
-scalarReplaceClass(ClassType* ct, Symbol* sym) {
+scalarReplaceClass(AggregateType* ct, Symbol* sym) {
 
   if (fReportScalarReplace) srClass++;
   //
@@ -243,7 +243,7 @@ scalarReplaceClass(ClassType* ct, Symbol* sym) {
     sym->defPoint->insertBefore(new DefExpr(var));
     if (sym->hasFlag(FLAG_TEMP))
       var->addFlag(FLAG_TEMP);
-    if (ClassType* fct = toClassType(field->type))
+    if (AggregateType* fct = toAggregateType(field->type))
       if (Vec<Symbol*>* varVec = typeVarMap.get(fct))
         varVec->add(var);
   }
@@ -260,7 +260,6 @@ scalarReplaceClass(ClassType* ct, Symbol* sym) {
   //
   for_uses(se, useMap, sym) {
     if (CallExpr* call = toCallExpr(se->parentExpr)) {
-     if (call) {
       SET_LINENO(call);
       if (call->isPrimitive(PRIM_GET_MEMBER)) {
         SymExpr* member = toSymExpr(call->get(2));
@@ -303,8 +302,18 @@ scalarReplaceClass(ClassType* ct, Symbol* sym) {
         addDef(defMap, def);
         if (call->get(1)->typeInfo() == call->get(2)->typeInfo()->refType)
           call->insertAtTail(new CallExpr(PRIM_ADDR_OF, call->get(2)->remove()));
+      } else {
+        /*
+         * If we fall into this case, it suggests that we did not
+         * rewrite the call as expected.  In particular, the
+         * for_uses() loop above this one (line 199 in this commit)
+         * flagged this call as being legal to scalar replace, but
+         * then we didn't.  Presumably this would happen if someone
+         * added a new case to the conditional within that loop (line
+         * 211) but then forgot to add a case to this conditional.
+         */
+        INT_FATAL("It seems someone forgot to rewrite an Expression");
       }
-     }
     }
   }
 
@@ -312,7 +321,7 @@ scalarReplaceClass(ClassType* ct, Symbol* sym) {
 }
 
 static bool
-scalarReplaceRecord(ClassType* ct, Symbol* sym) {
+scalarReplaceRecord(AggregateType* ct, Symbol* sym) {
 
   if (fReportScalarReplace) srRecord++;
   //
@@ -323,6 +332,7 @@ scalarReplaceRecord(ClassType* ct, Symbol* sym) {
       CallExpr* call = toCallExpr(se->parentExpr);
       if (!call ||
           !call->isPrimitive(PRIM_MOVE) ||
+          // Do we need to add PRIM_ASSIGN here?
           !(isSymExpr(call->get(2)) ||
             toCallExpr(call->get(2))->isPrimitive(PRIM_GET_MEMBER_VALUE))) {
         return false;
@@ -341,6 +351,7 @@ scalarReplaceRecord(ClassType* ct, Symbol* sym) {
             call->isPrimitive(PRIM_GET_MEMBER) ||
             call->isPrimitive(PRIM_GET_MEMBER_VALUE) ||
             call->isPrimitive(PRIM_MOVE)))
+        // Do we need to add PRIM_ASSIGN here?
         return false;
     }
   }
@@ -359,7 +370,7 @@ scalarReplaceRecord(ClassType* ct, Symbol* sym) {
     sym->defPoint->insertBefore(new DefExpr(var));
     if (sym->hasFlag(FLAG_TEMP))
       var->addFlag(FLAG_TEMP);
-    if (ClassType* fct = toClassType(field->type))
+    if (AggregateType* fct = toAggregateType(field->type))
       if (Vec<Symbol*>* varVec = typeVarMap.get(fct))
         varVec->add(var);
   }
@@ -411,7 +422,14 @@ scalarReplaceRecord(ClassType* ct, Symbol* sym) {
       }
     }
   }
-  sym->defPoint->remove();
+
+  // It is an AST invariant that every method contains a declaration for its
+  // _this symbol.  So we can't remove the declaration if it is the containing
+  // method's _this.  If the containing function is not a method, then its
+  // _this field will be NULL (so will not match any valid sym).
+  if (FnSymbol* parent = toFnSymbol(sym->defPoint->parentSymbol))
+    if (parent->_this != sym)
+      sym->defPoint->remove();
 
   //
   // replace uses of sym with new vars
@@ -419,7 +437,8 @@ scalarReplaceRecord(ClassType* ct, Symbol* sym) {
   for_uses(se, useMap, sym) {
     if (CallExpr* call = toCallExpr(se->parentExpr)) {
       SET_LINENO(sym);
-      if (call && call->isPrimitive(PRIM_MOVE)) {
+      // Do we need to add a case for PRIM_ASSIGN?
+      if (call->isPrimitive(PRIM_MOVE)) {
         SymExpr* lhs = toSymExpr(call->get(1));
         for_fields(field, ct) {
           SymExpr* lhsCopy = lhs->copy();
@@ -430,17 +449,17 @@ scalarReplaceRecord(ClassType* ct, Symbol* sym) {
           addUse(useMap, lhsCopy);
         }
         call->remove();
-      } else if (call && call->isPrimitive(PRIM_GET_MEMBER)) {
+      } else if (call->isPrimitive(PRIM_GET_MEMBER)) {
         SymExpr* member = toSymExpr(call->get(2));
         SymExpr* use = new SymExpr(fieldMap.get(member->var));
         call->replace(new CallExpr(PRIM_ADDR_OF, use));
         addUse(useMap, use);
-      } else if (call && call->isPrimitive(PRIM_GET_MEMBER_VALUE)) {
+      } else if (call->isPrimitive(PRIM_GET_MEMBER_VALUE)) {
         SymExpr* member = toSymExpr(call->get(2));
         SymExpr* use = new SymExpr(fieldMap.get(member->var));
         call->replace(use);
         addUse(useMap, use);
-      } else if (call && call->isPrimitive(PRIM_SET_MEMBER)) {
+      } else if (call->isPrimitive(PRIM_SET_MEMBER)) {
         SymExpr* member = toSymExpr(call->get(2));
         call->primitive = primitives[PRIM_MOVE];
         call->get(2)->remove();
@@ -450,6 +469,17 @@ scalarReplaceRecord(ClassType* ct, Symbol* sym) {
         addDef(defMap, def);
         if (call->get(1)->typeInfo() == call->get(2)->typeInfo()->refType)
           call->insertAtTail(new CallExpr(PRIM_ADDR_OF, call->get(2)->remove()));
+      } else {
+        /*
+         * If we fall into this case, it suggests that we did not
+         * rewrite the call as expected.  In particular, the
+         * for_uses() loop above this one (line 341 in this commit)
+         * flagged this call as being legal to scalar replace, but
+         * then we didn't.  Presumably this would happen if someone
+         * added a new case to the conditional within that loop (line
+         * 344) but then forgot to add a case to this conditional.
+         */
+        INT_FATAL("Someone forgot to scalar replace an expression");
       }
     }
   }
@@ -483,7 +513,7 @@ scalarReplace() {
     // for scalar replacement
     //
     forv_Vec(TypeSymbol, ts, gTypeSymbols) {
-      ClassType* ct = toClassType(ts->type);
+      AggregateType* ct = toAggregateType(ts->type);
       if (ct) {
         typeOrder.put(ct, -1);
         if (ts->hasFlag(FLAG_ITERATOR_CLASS) ||
@@ -492,7 +522,7 @@ scalarReplace() {
              (ct->fields.length<=scalar_replace_limit))) {
           typeVec.add(ct);
           typeVarMap.put(ct, new Vec<Symbol*>());
-          if (ClassType* rct = toClassType(ct->refType))
+          if (AggregateType* rct = toAggregateType(ct->refType))
             typeVarMap.put(rct, new Vec<Symbol*>());
         }
       }
@@ -501,7 +531,7 @@ scalarReplace() {
     //
     // compute topological order of types
     //
-    forv_Vec(ClassType, ct, typeVec) {
+    forv_Vec(AggregateType, ct, typeVec) {
       computeOrder(ct);
     }
 
@@ -514,7 +544,7 @@ scalarReplace() {
     // compute typeVarMap and varSet
     //
     forv_Vec(VarSymbol, var, gVarSymbols) {
-      if (ClassType* ct = toClassType(var->type)) {
+      if (AggregateType* ct = toAggregateType(var->type)) {
         if (Vec<Symbol*>* varVec = typeVarMap.get(ct)) {
           if (isFnSymbol(var->defPoint->parentSymbol)) {
             varSet.set_add(var);
@@ -538,10 +568,10 @@ scalarReplace() {
       printf("SCALAR REPLACEMENT (limit=%d): %d types\n",
              scalar_replace_limit, typeVec.n);
     }
-    forv_Vec(ClassType, ct, typeVec) {
+    forv_Vec(AggregateType, ct, typeVec) {
       if (debugScalarReplacement)
         printf("%d: %s:\n", typeOrder.get(ct), ct->symbol->cname);
-      if (ClassType* rct = toClassType(ct->refType)) {
+      if (AggregateType* rct = toAggregateType(ct->refType)) {
         Vec<Symbol*>* refVec = typeVarMap.get(rct);
         forv_Vec(Symbol, var, *refVec) {
           eliminateSingleAssignmentReference(defMap, useMap, var);
@@ -582,7 +612,7 @@ scalarReplace() {
     typeVec.clear();
     varSet.clear();
     typeOrder.clear();
-    form_Map(ClassTypeToVecSymbolMapElem, e, typeVarMap) {
+    form_Map(AggregateTypeToVecSymbolMapElem, e, typeVarMap) {
       delete e->value;
     }
     typeVarMap.clear();
