@@ -1,5 +1,5 @@
 /**************************************************************************
-  Copyright (c) 2004-2013, Cray Inc.  (See LICENSE file for more details)
+  Copyright (c) 2004-2014, Cray Inc.  (See LICENSE file for more details)
 **************************************************************************/
 
 
@@ -15,27 +15,26 @@
 static bool mainReturnsInt;
 
 static void build_chpl_entry_points(void);
-static void build_getter(ClassType* ct, Symbol* field);
-static void build_union_assignment_function(ClassType* ct);
+static void build_getter(AggregateType* ct, Symbol* field);
+static void build_union_assignment_function(AggregateType* ct);
 static void build_enum_assignment_function(EnumType* et);
-static void build_record_assignment_function(ClassType* ct);
-static void build_record_cast_function(ClassType* ct);
-static void build_record_copy_function(ClassType* ct);
-static void build_record_hash_function(ClassType* ct);
-static void build_record_equality_function(ClassType* ct);
-static void build_record_inequality_function(ClassType* ct);
-static void build_union_assignment_function(ClassType* ct);
+static void build_record_assignment_function(AggregateType* ct);
+static void build_record_cast_function(AggregateType* ct);
+static void build_record_copy_function(AggregateType* ct);
+static void build_record_hash_function(AggregateType* ct);
+static void build_record_equality_function(AggregateType* ct);
+static void build_record_inequality_function(AggregateType* ct);
 static void build_enum_cast_function(EnumType* et);
 static void build_enum_enumerate_function(EnumType* et);
 
-//static void buildDefaultReadFunction(ClassType* type);
+//static void buildDefaultReadFunction(AggregateType* type);
 //static void buildDefaultReadFunction(EnumType* type);
 
-static void buildDefaultReadWriteFunctions(ClassType* type);
+static void buildDefaultReadWriteFunctions(AggregateType* type);
 
 static void buildStringCastFunction(EnumType* type);
 
-static void buildDefaultDestructor(ClassType* ct);
+static void buildDefaultDestructor(AggregateType* ct);
 
 
 void buildDefaultFunctions(void) {
@@ -46,7 +45,8 @@ void buildDefaultFunctions(void) {
   collect_asts(rootModule, asts);
   forv_Vec(BaseAST, ast, asts) {
     if (TypeSymbol* type = toTypeSymbol(ast)) {
-      if (ClassType* ct = toClassType(type->type)) {
+      AggregateType* ct = toAggregateType(type->type);
+      if (ct) {
         for_fields(field, ct) {
           if (!field->hasFlag(FLAG_IMPLICIT_ALIAS_FIELD)) {
             if (isVarSymbol(field)) {
@@ -58,19 +58,22 @@ void buildDefaultFunctions(void) {
             }
           }
         }
-      }
-      if (ClassType* ct = toClassType(type->type))
+
         if (!ct->symbol->hasFlag(FLAG_REF))
           buildDefaultDestructor(ct);
+      }
       if (type->hasFlag(FLAG_NO_DEFAULT_FUNCTIONS))
         continue;
       if (EnumType* et = toEnumType(type->type)) {
         //buildDefaultReadFunction(et);
         buildStringCastFunction(et);
-      } else if (ClassType* ct = toClassType(type->type)) {
+
+        build_enum_cast_function(et);
+        build_enum_assignment_function(et);
+        build_enum_enumerate_function(et);
+      } else if (ct) {
         buildDefaultReadWriteFunctions(ct);
-      }
-      if (ClassType* ct = toClassType(type->type)) {
+      
         if (isRecord(ct)) {
           if (!isRecordWrappedType(ct)) {
             build_record_equality_function(ct);
@@ -84,11 +87,6 @@ void buildDefaultFunctions(void) {
         if (isUnion(ct))
           build_union_assignment_function(ct);
       }
-      if (EnumType* et = toEnumType(type->type)) {
-        build_enum_cast_function(et);
-        build_enum_assignment_function(et);
-        build_enum_enumerate_function(et);
-      }
     }
   }
 }
@@ -141,7 +139,8 @@ static FnSymbol* function_exists(const char* name,
 }
 
 
-static void build_getter(ClassType* ct, Symbol *field) {
+static void build_getter(AggregateType* ct, Symbol *field) {
+  const bool fieldIsConst = field->hasFlag(FLAG_CONST);
   if (FnSymbol* fn = function_exists(field->name, 2, dtMethodToken, ct)) {
     Vec<BaseAST*> asts;
     collect_asts(fn, asts);
@@ -156,19 +155,25 @@ static void build_getter(ClassType* ct, Symbol *field) {
         }
       }
     }
+    fn->addFlag(FLAG_FIELD_ACCESSOR);
+    if (fieldIsConst)
+      fn->addFlag(FLAG_REF_TO_CONST);
     return;
   }
 
   FnSymbol* fn = new FnSymbol(field->name);
   fn->addFlag(FLAG_NO_IMPLICIT_COPY);
   fn->addFlag(FLAG_INLINE);
-//  fn->addFlag(FLAG_TEMP); // Getter functions act as if user-defined.
+//  fn->addFlag(FLAG_COMPILER_GENERATED); // Getter functions act as if user-defined.
   if (ct->symbol->hasFlag(FLAG_SYNC)) 
     fn->addFlag(FLAG_SYNC);
   if (ct->symbol->hasFlag(FLAG_SINGLE)) 
     fn->addFlag(FLAG_SINGLE);
   if (ct->symbol->hasFlag(FLAG_ATOMIC_TYPE))
     fn->addFlag(FLAG_ATOMIC_TYPE);
+  fn->addFlag(FLAG_FIELD_ACCESSOR);
+  if (fieldIsConst)
+    fn->addFlag(FLAG_REF_TO_CONST);
   fn->insertFormalAtTail(new ArgSymbol(INTENT_BLANK, "_mt", dtMethodToken));
   ArgSymbol* _this = new ArgSymbol(INTENT_BLANK, "this", ct);
   _this->addFlag(FLAG_ARG_THIS);
@@ -315,7 +320,7 @@ static void build_chpl_entry_points(void) {
     SET_LINENO(mainModule);
     chpl_user_main = new FnSymbol("main");
     chpl_user_main->retType = dtVoid;
-//    chpl_user_main->addFlag(FLAG_TEMP);
+//    chpl_user_main->addFlag(FLAG_COMPILER_GENERATED);
     mainModule->block->insertAtTail(new DefExpr(chpl_user_main));
     normalize(chpl_user_main);
   } else {
@@ -329,11 +334,11 @@ static void build_chpl_entry_points(void) {
   chpl_user_main->cname = "chpl_user_main";
 
   //
-  // chpl_gen_main is the entry point for the compiler-generated cdoe.
-  // It accounts for the initialization and memory tracking of the
-  // code
+  // chpl_gen_main is the entry point for the compiler-generated code.
+  // It invokes the user's code.
   //
   chpl_gen_main = new FnSymbol("chpl_gen_main");
+  chpl_gen_main->addFlag(FLAG_COMPILER_GENERATED);
 
   ArgSymbol* arg = new ArgSymbol(INTENT_BLANK, "_arg", dtMainArgument);
   chpl_gen_main->insertFormalAtTail(arg);
@@ -341,16 +346,25 @@ static void build_chpl_entry_points(void) {
 
   chpl_gen_main->cname = "chpl_gen_main";
   chpl_gen_main->addFlag(FLAG_EXPORT);  // chpl_gen_main is always exported.
-  chpl_gen_main->addFlag(FLAG_TEMP);
+  chpl_gen_main->addFlag(FLAG_LOCAL_ARGS);
+  chpl_gen_main->addFlag(FLAG_COMPILER_GENERATED);
   mainModule->block->insertAtTail(new DefExpr(chpl_gen_main));
   VarSymbol* main_ret = newTemp("_main_ret", dtInt[INT_SIZE_64]);
   VarSymbol* endCount = newTemp("_endCount");
   chpl_gen_main->insertAtTail(new DefExpr(main_ret));
   chpl_gen_main->insertAtTail(new DefExpr(endCount));
 
-  chpl_gen_main->insertAtTail(new CallExpr(PRIM_MOVE, endCount, new CallExpr("_endCountAlloc")));
-  chpl_gen_main->insertAtTail(new CallExpr(PRIM_SET_END_COUNT, endCount));
-  chpl_gen_main->insertAtTail(new CallExpr("chpl_startTrackingMemory"));
+  //
+  // In --minimal-modules compilation mode, we won't have any
+  // parallelism, so no need for end counts (or atomic/sync types to
+  // support them).
+  //
+  if (!fMinimalModules) {
+    chpl_gen_main->insertAtTail(new CallExpr(PRIM_MOVE, endCount, new CallExpr("_endCountAlloc")));
+    chpl_gen_main->insertAtTail(new CallExpr(PRIM_SET_END_COUNT, endCount));
+  }
+
+  chpl_gen_main->insertAtTail(new CallExpr("chpl_rt_preUserCodeHook"));
 
   if( ! fNoInternalModules ) {
     // We have to initialize the main module explicitly.
@@ -390,19 +404,29 @@ static void build_chpl_entry_points(void) {
           new_IntSymbol(0, INT_SIZE_64)));
   }
 
-  chpl_gen_main->insertAtTail(new CallExpr("_waitEndCount"));
-  //chpl_gen_main->insertAtTail(new CallExpr("_endCountFree", endCount));
+  chpl_gen_main->insertAtTail(new CallExpr("chpl_rt_postUserCodeHook"));
 
+  //
+  // In --minimal-modules compilation mode, we won't be waiting on an
+  // encount (see comment above)
+  //
+  if (!fMinimalModules) {
+
+    chpl_gen_main->insertAtTail(new CallExpr("_waitEndCount"));
+    //chpl_gen_main->insertAtTail(new CallExpr("_endCountFree", endCount));
+
+  }
   chpl_gen_main->insertAtTail(new CallExpr(PRIM_RETURN, main_ret));
 
   normalize(chpl_gen_main);
 }
 
-static void build_record_equality_function(ClassType* ct) {
+static void build_record_equality_function(AggregateType* ct) {
   if (function_exists("==", 2, ct))
     return;
 
   FnSymbol* fn = new FnSymbol("==");
+  fn->addFlag(FLAG_COMPILER_GENERATED);
   ArgSymbol* arg1 = new ArgSymbol(INTENT_BLANK, "_arg1", ct);
   arg1->markedGeneric = true;
   ArgSymbol* arg2 = new ArgSymbol(INTENT_BLANK, "_arg2", ct);
@@ -426,11 +450,12 @@ static void build_record_equality_function(ClassType* ct) {
 }
 
 
-static void build_record_inequality_function(ClassType* ct) {
+static void build_record_inequality_function(AggregateType* ct) {
   if (function_exists("!=", 2, ct))
     return;
 
   FnSymbol* fn = new FnSymbol("!=");
+  fn->addFlag(FLAG_COMPILER_GENERATED);
   ArgSymbol* arg1 = new ArgSymbol(INTENT_BLANK, "_arg1", ct);
   arg1->markedGeneric = true;
   ArgSymbol* arg2 = new ArgSymbol(INTENT_BLANK, "_arg2", ct);
@@ -457,6 +482,9 @@ static void build_enum_enumerate_function(EnumType* et) {
   // Build a function that returns a tuple of the enum's values
   // Each enum type has its own _enum_enumerate function.
   FnSymbol* fn = new FnSymbol("_enum_enumerate");
+// TODO: This flag should be enabled, so a user-defined version of the record
+//  assignment function can override the compiler-generated version.
+//  fn->addFlag(FLAG_COMPILER_GENERATED);
   ArgSymbol* arg = new ArgSymbol(INTENT_BLANK, "t", dtAny);
   arg->addFlag(FLAG_TYPE_VARIABLE);
   fn->insertFormalAtTail(arg);
@@ -479,7 +507,7 @@ static void build_enum_enumerate_function(EnumType* et) {
 static void build_enum_cast_function(EnumType* et) {
   // integral value to enumerated type cast function
   FnSymbol* fn = new FnSymbol("_cast");
-  fn->addFlag(FLAG_TEMP);
+  fn->addFlag(FLAG_COMPILER_GENERATED);
   ArgSymbol* arg1 = new ArgSymbol(INTENT_BLANK, "t", dtAny);
   arg1->addFlag(FLAG_TYPE_VARIABLE);
   ArgSymbol* arg2 = new ArgSymbol(INTENT_BLANK, "_arg2", dtIntegral);
@@ -524,7 +552,7 @@ static void build_enum_cast_function(EnumType* et) {
 
   // string to enumerated type cast function
   fn = new FnSymbol("_cast");
-  fn->addFlag(FLAG_TEMP);
+  fn->addFlag(FLAG_COMPILER_GENERATED);
   arg1 = new ArgSymbol(INTENT_BLANK, "t", dtAny);
   arg1->addFlag(FLAG_TYPE_VARIABLE);
   arg2 = new ArgSymbol(INTENT_BLANK, "_arg2", dtString);
@@ -560,11 +588,13 @@ static void build_enum_assignment_function(EnumType* et) {
     return;
 
   FnSymbol* fn = new FnSymbol("=");
-  ArgSymbol* arg1 = new ArgSymbol(INTENT_BLANK, "_arg1", et);
+  fn->addFlag(FLAG_ASSIGNOP);
+  fn->addFlag(FLAG_COMPILER_GENERATED);
+  ArgSymbol* arg1 = new ArgSymbol(INTENT_REF, "_arg1", et);
   ArgSymbol* arg2 = new ArgSymbol(INTENT_BLANK, "_arg2", et);
   fn->insertFormalAtTail(arg1);
   fn->insertFormalAtTail(arg2);
-  fn->insertAtTail(new CallExpr(PRIM_RETURN, arg2));
+  fn->insertAtTail(new CallExpr(PRIM_ASSIGN, arg1, arg2));
   DefExpr* def = new DefExpr(fn);
   et->symbol->defPoint->insertBefore(def);
   reset_ast_loc(def, et->symbol);
@@ -572,13 +602,16 @@ static void build_enum_assignment_function(EnumType* et) {
 }
 
 
-static void build_record_assignment_function(ClassType* ct) {
+static void build_record_assignment_function(AggregateType* ct) {
   if (function_exists("=", 2, ct))
     return;
 
   FnSymbol* fn = new FnSymbol("=");
-  ArgSymbol* arg1 = new ArgSymbol(INTENT_BLANK, "_arg1", ct);
-  arg1->markedGeneric = true;
+  fn->addFlag(FLAG_ASSIGNOP);
+  fn->addFlag(FLAG_COMPILER_GENERATED);
+
+  ArgSymbol* arg1 = new ArgSymbol(INTENT_REF, "_arg1", ct);
+  arg1->markedGeneric = true; // TODO: Check if we really want this.
 
   bool externRecord = ct->symbol->hasFlag(FLAG_EXTERN);
   // If the LHS is extern, the RHS must be of matching type; otherwise
@@ -587,7 +620,7 @@ static void build_record_assignment_function(ClassType* ct) {
                                   (externRecord ? ct : dtAny));
   fn->insertFormalAtTail(arg1);
   fn->insertFormalAtTail(arg2);
-  fn->retType = dtUnknown;
+  fn->retType = dtUnknown; // TODO: Maybe we can set this to dtVoid up front.
 
   if (externRecord) {
     fn->insertAtTail(new CallExpr(PRIM_MOVE, arg1, arg2));
@@ -599,7 +632,6 @@ static void build_record_assignment_function(ClassType* ct) {
       }
     }
   }
-  fn->insertAtTail(new CallExpr(PRIM_RETURN, arg1));
   DefExpr* def = new DefExpr(fn);
   ct->symbol->defPoint->insertBefore(def);
   reset_ast_loc(def, ct->symbol);
@@ -607,8 +639,11 @@ static void build_record_assignment_function(ClassType* ct) {
 }
 
 
-static void build_record_cast_function(ClassType* ct) {
+static void build_record_cast_function(AggregateType* ct) {
   FnSymbol* fn = new FnSymbol("_cast");
+// TODO: This flag should be enabled, so a user-defined version of the record
+//  assignment function can override the compiler-generated version.
+//  fn->addFlag(FLAG_COMPILER_GENERATED);
   ArgSymbol* t = new ArgSymbol(INTENT_BLANK, "t", dtAny);
   t->addFlag(FLAG_TYPE_VARIABLE);
   ArgSymbol* arg = new ArgSymbol(INTENT_BLANK, "arg", dtAny);
@@ -629,12 +664,14 @@ static void build_record_cast_function(ClassType* ct) {
   normalize(fn);
 }
 
-static void build_union_assignment_function(ClassType* ct) {
+static void build_union_assignment_function(AggregateType* ct) {
   if (function_exists("=", 2, ct))
     return;
 
   FnSymbol* fn = new FnSymbol("=");
-  ArgSymbol* arg1 = new ArgSymbol(INTENT_BLANK, "_arg1", ct);
+  fn->addFlag(FLAG_ASSIGNOP);
+  fn->addFlag(FLAG_COMPILER_GENERATED);
+  ArgSymbol* arg1 = new ArgSymbol(INTENT_REF, "_arg1", ct);
   ArgSymbol* arg2 = new ArgSymbol(INTENT_BLANK, "_arg2", ct);
   fn->insertFormalAtTail(arg1);
   fn->insertFormalAtTail(arg2);
@@ -651,7 +688,6 @@ static void build_union_assignment_function(ClassType* ct) {
             new CallExpr("=",
               new CallExpr(".", arg1, new_StringSymbol(tmp->name)),
               new CallExpr(".", arg2, new_StringSymbol(tmp->name)))));
-  fn->insertAtTail(new CallExpr(PRIM_RETURN, arg1));
   DefExpr* def = new DefExpr(fn);
   ct->symbol->defPoint->insertBefore(def);
   reset_ast_loc(def, ct->symbol);
@@ -659,7 +695,7 @@ static void build_union_assignment_function(ClassType* ct) {
 }
 
 
-static void build_record_copy_function(ClassType* ct) {
+static void build_record_copy_function(AggregateType* ct) {
   if (function_exists("chpl__initCopy", 1, ct))
     return;
 
@@ -667,10 +703,11 @@ static void build_record_copy_function(ClassType* ct) {
 
   FnSymbol* fn = new FnSymbol("chpl__initCopy");
   fn->addFlag(FLAG_INIT_COPY_FN);
+  fn->addFlag(FLAG_COMPILER_GENERATED);
   ArgSymbol* arg = new ArgSymbol(INTENT_BLANK, "x", ct);
   arg->markedGeneric = true;
   fn->insertFormalAtTail(arg);
-  CallExpr* call = new CallExpr(ct->initializer);
+  CallExpr* call = new CallExpr(ct->defaultInitializer);
   for_fields(tmp, ct) {
     if (!tmp->hasFlag(FLAG_IMPLICIT_ALIAS_FIELD))
       if (strcmp("_promotionType", tmp->name))
@@ -684,11 +721,14 @@ static void build_record_copy_function(ClassType* ct) {
 }
 
 
-static void build_record_hash_function(ClassType *ct) {
+static void build_record_hash_function(AggregateType *ct) {
   if (function_exists("chpl__defaultHash", 1, ct))
     return;
 
   FnSymbol *fn = new FnSymbol("chpl__defaultHash");
+// TODO: This flag should be enabled, so a user-defined version of the record
+//  assignment function can override the compiler-generated version.
+//  fn->addFlag(FLAG_COMPILER_GENERATED);
   fn->addFlag(FLAG_INLINE);
   ArgSymbol *arg = new ArgSymbol(INTENT_BLANK, "r", ct);
   arg->markedGeneric = true;
@@ -723,12 +763,20 @@ static void build_record_hash_function(ClassType *ct) {
   normalize(fn);
 }
 
-static void buildDefaultReadWriteFunctions(ClassType* ct) {
+static void buildDefaultReadWriteFunctions(AggregateType* ct) {
   bool hasReadWriteThis = false;
   bool hasReadThis = false;
   bool hasWriteThis = false;
   bool makeReadThisAndWriteThis = true;
 
+  //
+  // We have no QIO when compiling with --minimal-modules, so no need
+  // to build default R/W functions.
+  //
+  if (fMinimalModules) {
+    return;
+  }
+
   // If we have a readWriteThis, we'll call it from readThis/writeThis.
   if (function_exists("readWriteThis", 3, dtMethodToken, ct, NULL)) {
     hasReadWriteThis = true;
@@ -748,6 +796,7 @@ static void buildDefaultReadWriteFunctions(ClassType* ct) {
   // Make writeThis if we have neither writeThis nor readThis.
   if ( makeReadThisAndWriteThis && ! hasWriteThis ) {
     FnSymbol* fn = new FnSymbol("writeThis");
+    fn->addFlag(FLAG_COMPILER_GENERATED);
     fn->cname = astr("_auto_", ct->symbol->name, "_write");
     fn->_this = new ArgSymbol(INTENT_BLANK, "this", ct);
     fn->_this->addFlag(FLAG_ARG_THIS);
@@ -772,6 +821,7 @@ static void buildDefaultReadWriteFunctions(ClassType* ct) {
   }
   if ( makeReadThisAndWriteThis && ! hasReadThis ) {
     FnSymbol* fn = new FnSymbol("readThis");
+    fn->addFlag(FLAG_COMPILER_GENERATED);
     fn->cname = astr("_auto_", ct->symbol->name, "_read");
     fn->_this = new ArgSymbol(INTENT_BLANK, "this", ct);
     fn->_this->addFlag(FLAG_ARG_THIS);
@@ -803,6 +853,9 @@ static void buildStringCastFunction(EnumType* et) {
     return;
 
   FnSymbol* fn = new FnSymbol("_cast");
+// TODO: This flag should be enabled, so a user-defined version of the record
+//  copy function can override the compiler-generated version.
+//  fn->addFlag(FLAG_COMPILER_GENERATED);
   ArgSymbol* t = new ArgSymbol(INTENT_BLANK, "t", dtAny);
   t->addFlag(FLAG_TYPE_VARIABLE);
   fn->insertFormalAtTail(t);
@@ -830,13 +883,16 @@ static void buildStringCastFunction(EnumType* et) {
 }
 
 
-static void buildDefaultDestructor(ClassType* ct) {
+static void buildDefaultDestructor(AggregateType* ct) {
   if (function_exists("~chpl_destroy", 2, dtMethodToken, ct))
     return;
 
   SET_LINENO(ct->symbol);
 
   FnSymbol* fn = new FnSymbol("~chpl_destroy");
+// TODO: This flag should be enabled, so a user-defined version of the record
+//  copy function can override the compiler-generated version.
+//  fn->addFlag(FLAG_COMPILER_GENERATED);
   fn->addFlag(FLAG_DESTRUCTOR);
   fn->addFlag(FLAG_INLINE);
   fn->cname = astr("chpl__auto_destroy_", ct->symbol->name);
