@@ -1,5 +1,5 @@
 /**************************************************************************
-  Copyright (c) 2004-2013, Cray Inc.  (See LICENSE file for more details)
+  Copyright (c) 2004-2014, Cray Inc.  (See LICENSE file for more details)
 **************************************************************************/
 
 
@@ -30,6 +30,8 @@
 GenInfo* gGenInfo;
 int gMaxVMT = -1;
 fileinfo gAllExternCode;
+int stmtCount = 0;
+
 
 static const char*
 subChar(Symbol* sym, const char* ch, const char* x) {
@@ -92,7 +94,7 @@ static void legalizeName(Symbol* sym) {
     case '#': ch = subChar(sym, ch, "_POUND_"); break;
     case '?': ch = subChar(sym, ch, "_QUESTION_"); break;
     case '$': ch = subChar(sym, ch, "_DOLLAR_"); break;
-    case '~': ch = subChar(sym, ch, "_TILDA_"); break;
+    case '~': ch = subChar(sym, ch, "_TILDE_"); break;
     case '.': ch = subChar(sym, ch, "_DOT_"); break;
     case ' ': ch = subChar(sym, ch, "_SPACE_"); break;
     default: break;
@@ -168,7 +170,7 @@ genClassIDs(Vec<TypeSymbol*> & typeSymbols) {
 
   int count=0;
   forv_Vec(TypeSymbol, ts, typeSymbols) {
-    if (ClassType* ct = toClassType(ts->type)) {
+    if (AggregateType* ct = toAggregateType(ts->type)) {
       if (!isReferenceType(ct) && isClass(ct)) {
         genGlobalDefClassId(ts->cname, count);
         count++;
@@ -245,7 +247,7 @@ genVirtualMethodTable(Vec<TypeSymbol*>& types) {
     fprintf(hdrfile, "chpl_fn_p %s[] = {\n", vmt);
     bool comma = false;
     forv_Vec(TypeSymbol, ts, types) {
-      if (ClassType* ct = toClassType(ts->type)) {
+      if (AggregateType* ct = toAggregateType(ts->type)) {
         if (!isReferenceType(ct) && isClass(ct)) {
           if (comma)
             fprintf(hdrfile, ",\n");
@@ -278,7 +280,7 @@ genVirtualMethodTable(Vec<TypeSymbol*>& types) {
     llvm::Type *vmTableEntryType = funcPtrType;
     
     forv_Vec(TypeSymbol, ts, types) {
-      if (ClassType* ct = toClassType(ts->type)) {
+      if (AggregateType* ct = toAggregateType(ts->type)) {
         if (!isReferenceType(ct) && isClass(ct)) {
           int n = 0;
           if (Vec<FnSymbol*>* vfns = virtualMethodTable.get(ct)) {
@@ -319,8 +321,8 @@ genVirtualMethodTable(Vec<TypeSymbol*>& types) {
 
 static int
 compareSymbol(const void* v1, const void* v2) {
-  Symbol* s1 = *(Symbol**)v1;
-  Symbol* s2 = *(Symbol**)v2;
+  Symbol* s1 = *(Symbol* const *)v1;
+  Symbol* s2 = *(Symbol* const *)v2;
   ModuleSymbol* m1 = s1->getModule();
   ModuleSymbol* m2 = s2->getModule();
   if (m1 != m2) {
@@ -379,7 +381,7 @@ static const char* uniquifyName(const char* name,
   return newName;
 }
 
-static inline bool shouldCodegenAggregate(ClassType* ct)
+static inline bool shouldCodegenAggregate(AggregateType* ct)
 {
   // never codegen definitions of primitive or arithmetic types.
   if( toPrimitiveType(ct) ) return false;
@@ -406,7 +408,7 @@ static inline bool shouldCodegenAggregate(ClassType* ct)
 }
 
 
-static void codegen_aggregate_def(ClassType* ct) {
+static void codegen_aggregate_def(AggregateType* ct) {
   if (!shouldCodegenAggregate(ct)) return;
   if (ct->symbol->codegenned) return;
   ct->symbol->codegenned = true;
@@ -419,13 +421,13 @@ static void codegen_aggregate_def(ClassType* ct) {
   else if(ct->symbol->hasFlag(FLAG_DATA_CLASS))
     vt = getDataClassType(ct->symbol)->typeInfo();
   if (vt) {
-    if (ClassType* fct = toClassType(vt)) {
+    if (AggregateType* fct = toAggregateType(vt)) {
       codegen_aggregate_def(fct);
     }
   }
   // For other types, generate the field types
   for_fields(field, ct) {
-    if (ClassType* fct = toClassType(field->type)) {
+    if (AggregateType* fct = toAggregateType(field->type)) {
       codegen_aggregate_def(fct);
     }
   }
@@ -510,7 +512,7 @@ static void codegen_header() {
   //
   forv_Vec(TypeSymbol, ts, types) {
     if (ts->defPoint->parentExpr != rootModule->block) {
-      if (ClassType* ct = toClassType(ts->type)) {
+      if (AggregateType* ct = toAggregateType(ts->type)) {
         Vec<const char*> fieldNameSet;
         for_fields(field, ct) {
           legalizeName(field);
@@ -611,6 +613,7 @@ static void codegen_header() {
   genGlobalString("CHPL_ATOMICS", CHPL_ATOMICS);
   genGlobalString("CHPL_NETWORK_ATOMICS", CHPL_NETWORK_ATOMICS);
   genGlobalString("CHPL_GMP", CHPL_GMP);
+  genGlobalString("CHPL_HWLOC", CHPL_HWLOC);
   genGlobalString("CHPL_REGEXP", CHPL_REGEXP);
   genGlobalString("CHPL_WIDE_POINTERS", CHPL_WIDE_POINTERS);
   genGlobalString("CHPL_LLVM", CHPL_LLVM);
@@ -656,7 +659,7 @@ static void codegen_header() {
     // codegen records/unions/references/data class in topological order
     genComment("Records, Unions, Data Class, References (Hierarchically)");
     forv_Vec(TypeSymbol, ts, types) {
-      if (ClassType* ct = toClassType(ts->type))
+      if (AggregateType* ct = toAggregateType(ts->type))
         codegen_aggregate_def(ct);
     }
   }
@@ -730,41 +733,28 @@ static void codegen_header() {
   genGlobalInt("chpl_numGlobalsOnHeap", numGlobalsOnHeap);
   int globals_registry_static_size = (numGlobalsOnHeap ? numGlobalsOnHeap : 1);
   if( hdrfile ) {
-    fprintf(hdrfile, "\nptr_wide_ptr_t* chpl_globals_registry;\n");
-    fprintf(hdrfile, "\nptr_wide_ptr_t chpl_globals_registry_static[%d];\n",
+    fprintf(hdrfile, "\nptr_wide_ptr_t chpl_globals_registry[%d];\n",
                      globals_registry_static_size);
   } else {
 #ifdef HAVE_LLVM
     llvm::Type* ptr_wide_ptr_t = info->lvt->getType("ptr_wide_ptr_t");
     INT_ASSERT(ptr_wide_ptr_t);
 
-    llvm::Type* ptr_ptr_wide_ptr_t = llvm::PointerType::get(ptr_wide_ptr_t, 0);
-
+    if(llvm::GlobalVariable *GVar = llvm::cast_or_null<llvm::GlobalVariable>(
+          info->module->getNamedGlobal("chpl_globals_registry"))) {
+      GVar->eraseFromParent();
+    }
     llvm::GlobalVariable *chpl_globals_registryGVar =
       llvm::cast<llvm::GlobalVariable>(
           info->module->getOrInsertGlobal("chpl_globals_registry",
-            ptr_ptr_wide_ptr_t));
+            llvm::ArrayType::get(
+              ptr_wide_ptr_t,
+              globals_registry_static_size)));
     chpl_globals_registryGVar->setInitializer(
         llvm::Constant::getNullValue(
           chpl_globals_registryGVar->getType()->getContainedType(0)));
     info->lvt->addGlobalValue("chpl_globals_registry",
-        chpl_globals_registryGVar, GEN_PTR, true);
-
-    if(llvm::GlobalVariable *GVar = llvm::cast_or_null<llvm::GlobalVariable>(
-          info->module->getNamedGlobal("chpl_globals_registry_static"))) {
-      GVar->eraseFromParent();
-    }
-    llvm::GlobalVariable *chpl_globals_registry_staticGVar =
-      llvm::cast<llvm::GlobalVariable>(
-          info->module->getOrInsertGlobal("chpl_globals_registry_static",
-            llvm::ArrayType::get(
-              ptr_wide_ptr_t,
-              globals_registry_static_size)));
-    chpl_globals_registry_staticGVar->setInitializer(
-        llvm::Constant::getNullValue(
-          chpl_globals_registry_staticGVar->getType()->getContainedType(0)));
-    info->lvt->addGlobalValue("chpl_globals_registry_static",
-                              chpl_globals_registry_staticGVar, GEN_PTR, true);
+                              chpl_globals_registryGVar, GEN_PTR, true);
 #endif
   }
   genGlobalInt("chpl_heterogeneous", fHeterogeneous?1:0);
@@ -822,6 +812,9 @@ static void codegen_header() {
         INT_ASSERT(se);
         SET_LINENO(call);
         fprintf(hdrfile, ",\n&%s", se->var->cname);
+        // To preserve operand order, this should be insertAtTail.
+        // The change must also be made below (for LLVM) and in the signature
+        // of chpl_comm_broadcast_private().
         call->insertAtHead(new_IntSymbol(i));
         i++;
       }
@@ -856,7 +849,7 @@ static void codegen_header() {
               info->builder->CreatePointerCast(
                 info->lvt->getValue(se->var->cname).val,
                 private_broadcastTableEntryType)));
-
+        // To preserve operand order, this should be insertAtTail.
         call->insertAtHead(new_IntSymbol(broadcastID++));
       }
     }
@@ -894,7 +887,7 @@ static void codegen_header() {
 static void codegen_header_addons() {
   forv_Vec(TypeSymbol, ts, gTypeSymbols) {
     if (ts->defPoint->parentExpr != rootModule->block) {
-      if (ClassType* ct = toClassType(ts->type))
+      if (AggregateType* ct = toAggregateType(ts->type))
         codegen_aggregate_def(ct);
     }
   }
@@ -1165,6 +1158,11 @@ void codegen(void) {
 
   closeCFile(&hdrfile);
   closeCFile(&mainfile);
+
+  if (fPrintEmittedCodeSize)
+  {
+    fprintf(stderr, "Statements emitted: %d\n", stmtCount);
+  }
 }
 
 void makeBinary(void) {
