@@ -1,5 +1,5 @@
 /**************************************************************************
-  Copyright (c) 2004-2013, Cray Inc.  (See LICENSE file for more details)
+  Copyright (c) 2004-2014, Cray Inc.  (See LICENSE file for more details)
 **************************************************************************/
 
 
@@ -15,17 +15,17 @@ void flattenClasses(void) {
   //
   // collect nested classes
   //
-  Vec<ClassType*> nestedClasses;
+  Vec<AggregateType*> nestedClasses;
   forv_Vec(TypeSymbol, ts, gTypeSymbols) {
-    if (ClassType* ct = toClassType(ts->type))
-      if (toClassType(ct->symbol->defPoint->parentSymbol->type))
+    if (AggregateType* ct = toAggregateType(ts->type))
+      if (toAggregateType(ct->symbol->defPoint->parentSymbol->type))
         nestedClasses.add(ct);
   }
 
   //
   // move nested classes to module level
   //
-  forv_Vec(ClassType, ct, nestedClasses) {
+  forv_Vec(AggregateType, ct, nestedClasses) {
     ModuleSymbol* mod = ct->getModule();
     DefExpr *def = ct->symbol->defPoint;
     def->remove();
