@@ -1,5 +1,5 @@
 /**************************************************************************
-  Copyright (c) 2004-2013, Cray Inc.  (See LICENSE file for more details)
+  Copyright (c) 2004-2014, Cray Inc.  (See LICENSE file for more details)
 **************************************************************************/
 
 
@@ -46,6 +46,7 @@ static void replace_query_uses(ArgSymbol* formal, DefExpr* def, CallExpr* query,
 static void add_to_where_clause(ArgSymbol* formal, Expr* expr, CallExpr* query);
 static void fixup_query_formals(FnSymbol* fn);
 static void change_method_into_constructor(FnSymbol* fn);
+static void find_printModuleInit_stuff();
 
 void normalize(void) {
   // tag iterators and replace delete statements with calls to ~chpl_destroy
@@ -140,8 +141,9 @@ void normalize(void) {
       } else {
         DefExpr* thisDef = toDefExpr(fn->formals.get(2));
         INT_ASSERT(fn->name[0] == '~' && thisDef);
+        AggregateType* ct = toAggregateType(thisDef->sym->type);
         // make sure the name of the destructor matches the name of the class
-        if (strcmp(fn->name + 1, thisDef->sym->type->symbol->name)) {
+        if (ct && strcmp(fn->name + 1, ct->symbol->name)) {
           USR_FATAL(fn, "destructor name must match class name");
         } else {
           fn->name = astr("~chpl_destroy");
@@ -155,6 +157,8 @@ void normalize(void) {
       USR_FATAL(fn, "invalid method name");
     }
   }
+
+  find_printModuleInit_stuff();
 }
 
 // the following function is called from multiple places,
@@ -214,59 +218,62 @@ checkUseBeforeDefs() {
       // Walk the asts in this function.
       collect_asts_postorder(fn, asts);
       forv_Vec(BaseAST, ast, asts) {
-
+        // Adds definitions (this portion could probably be made into a
+        // separate function - see loopInvariantCodeMotion and copyPropagation)
         if (CallExpr* call = toCallExpr(ast)) {
-          // A symbol gets defined when it appears on the LHS of a move.
-          if (call->isPrimitive(PRIM_MOVE))
+          // A symbol gets defined when it appears on the LHS of a move or
+          // assignment.
+          if (call->isPrimitive(PRIM_MOVE) || call->isPrimitive(PRIM_ASSIGN))
             if (SymExpr* se = toSymExpr(call->get(1)))
               defined.set_add(se->var);
-        }
-
-        if (DefExpr* def = toDefExpr(ast)) {
+        } else if (DefExpr* def = toDefExpr(ast)) {
           // All arg sumbols are defined.
           if (isArgSymbol(def->sym))
             defined.set_add(def->sym);
-        }
-
-        if (SymExpr* sym = toSymExpr(ast)) {
-          CallExpr* call = toCallExpr(sym->parentExpr);
-          if (call && call->isPrimitive(PRIM_MOVE) && call->get(1) == sym)
-            continue;  // We already handled this case above.
-
-          if (toModuleSymbol(sym->var)) {
-            if (!toFnSymbol(fn->defPoint->parentSymbol)) {
-              if (!call || !call->isPrimitive(PRIM_USED_MODULES_LIST)) {
-                SymExpr* prev = toSymExpr(sym->prev);
-                if (!prev || prev->var != gModuleToken)
-                  USR_FATAL_CONT(sym, "illegal use of module '%s'", sym->var->name);
+        } else {
+          // The AST in question is not one of our methods of declaration so now
+          // we check if it is a (resolved/unresolved) symbol and make sure
+          // that symbol is not defined/declared before use
+          if (SymExpr* sym = toSymExpr(ast)) {
+            CallExpr* call = toCallExpr(sym->parentExpr);
+            if (call && 
+                (call->isPrimitive(PRIM_MOVE) || call->isPrimitive(PRIM_ASSIGN)) &&
+                call->get(1) == sym)
+              continue; // We already handled this case above.
+          
+            if (toModuleSymbol(sym->var)) {
+              if (!toFnSymbol(fn->defPoint->parentSymbol)) {
+                if (!call || !call->isPrimitive(PRIM_USED_MODULES_LIST)) {
+                  SymExpr* prev = toSymExpr(sym->prev);
+                  if (!prev || prev->var != gModuleToken)
+                    USR_FATAL_CONT(sym, "illegal use of module '%s'", sym->var->name);
+                }
               }
-            }
-          }
-
-          if (isVarSymbol(sym->var) || isArgSymbol(sym->var)) {
-            if (sym->var->defPoint->parentExpr != rootModule->block &&
-                (sym->var->defPoint->parentSymbol == fn ||
-                 (sym->var->defPoint->parentSymbol == mod && mod->initFn == fn))) {
-              if (!defined.set_in(sym->var) && !undefined.set_in(sym->var)) {
-                if (!sym->var->hasEitherFlag(FLAG_ARG_THIS,FLAG_EXTERN)) {
-                  USR_FATAL_CONT(sym, "'%s' used before defined (first used here)", sym->var->name);
-                  undefined.set_add(sym->var);
+            } else if (isVarSymbol(sym->var) || isArgSymbol(sym->var)) {
+              if (sym->var->defPoint->parentExpr != rootModule->block &&
+                  (sym->var->defPoint->parentSymbol == fn ||
+                   (sym->var->defPoint->parentSymbol == mod && mod->initFn == fn))) {
+                if (!defined.set_in(sym->var) && !undefined.set_in(sym->var)) {
+                  if (!sym->var->hasEitherFlag(FLAG_ARG_THIS,FLAG_EXTERN)) {
+                    USR_FATAL_CONT(sym, "'%s' used before defined (first used here)", sym->var->name);
+                    undefined.set_add(sym->var);
+                  }
                 }
               }
             }
-          }
-        }
-
-        if (UnresolvedSymExpr* sym = toUnresolvedSymExpr(ast)) {
-          CallExpr* call = toCallExpr(sym->parentExpr);
-          if (call && call->isPrimitive(PRIM_MOVE) && call->get(1) == sym)
-            continue;
-          if ((!call || (call->baseExpr != sym && !call->isPrimitive(PRIM_CAPTURE_FN))) && sym->unresolved) {
-            if (!undeclared.set_in(sym->unresolved)) {
-              if (!toFnSymbol(fn->defPoint->parentSymbol)) {
-                USR_FATAL_CONT(sym, "'%s' undeclared (first use this function)",
-                               sym->unresolved);
-                undeclared.set_add(sym->unresolved);
+          } else if (UnresolvedSymExpr* sym = toUnresolvedSymExpr(ast)) {
+            CallExpr* call = toCallExpr(sym->parentExpr);
+            if (call &&
+                (call->isPrimitive(PRIM_MOVE) || call->isPrimitive(PRIM_ASSIGN)) &&
+                call->get(1) == sym)
+              continue; // We already handled this case above.
+            if ((!call || (call->baseExpr != sym && !call->isPrimitive(PRIM_CAPTURE_FN))) && sym->unresolved) {
+              if (!undeclared.set_in(sym->unresolved)) {
+                if (!toFnSymbol(fn->defPoint->parentSymbol)) {
+                  USR_FATAL_CONT(sym, "'%s' undeclared (first use this function)",
+                                 sym->unresolved);
+                  undeclared.set_add(sym->unresolved);
+                }
               }
             }
           }
@@ -281,6 +288,8 @@ flattenGlobalFunctions() {
   forv_Vec(ModuleSymbol, mod, allModules) {
     for_alist(expr, mod->initFn->body->body) {
       if (DefExpr* def = toDefExpr(expr)) {
+        // FLAG_TEMP selects non-compiler-generated variables in the module init
+        // function to be moved out to module scope.
         if ((toVarSymbol(def->sym) && !def->sym->hasFlag(FLAG_TEMP)) ||
             toTypeSymbol(def->sym) ||
             toFnSymbol(def->sym)) {
@@ -354,7 +363,14 @@ static void insertRetMove(FnSymbol* fn, VarSymbol* retval, CallExpr* ret) {
   if (fn->retTag == RET_VAR)
     ret->insertBefore(new CallExpr(PRIM_MOVE, retval, new CallExpr(PRIM_ADDR_OF, ret_expr)));
   else if (fn->retExprType)
+  {
     ret->insertBefore(new CallExpr(PRIM_MOVE, retval, new CallExpr("=", retval, ret_expr)));
+    // Using assignment creates a new copy, which transfers ownership of *a*
+    // copy to the return value variable.
+    // Contrast this with a move, which merely shares ownership between the
+    // bitwise copies of the object.
+    retval->addFlag(FLAG_INSERT_AUTO_DESTROY);
+  }
   else if (!fn->hasFlag(FLAG_WRAPPER) && strcmp(fn->name, "iteratorIndex") &&
            strcmp(fn->name, "iteratorIndexHelp"))
     ret->insertBefore(new CallExpr(PRIM_MOVE, retval, new CallExpr(PRIM_DEREF, ret_expr)));
@@ -362,6 +378,26 @@ static void insertRetMove(FnSymbol* fn, VarSymbol* retval, CallExpr* ret) {
     ret->insertBefore(new CallExpr(PRIM_MOVE, retval, ret_expr));
 }
 
+
+// Look in the block statement determining a return value type, and see if it
+// looks like an array type expression.
+static bool
+returnTypeIsArray(BlockStmt* retExprType)
+{
+  CallExpr* call = toCallExpr(retExprType->body.tail);
+  if (! call)
+    return false;
+  UnresolvedSymExpr* urse = toUnresolvedSymExpr(call->baseExpr);
+  if (! urse)
+    return false;
+  if (strcmp(urse->unresolved, "chpl__buildArrayRuntimeType"))
+    // Does not match.
+    return false;
+  // Very likely an array type.
+  return true;
+}
+
+
 // Following normalization, each function contains only one return statement
 // preceded by a label.  The first half of the function counts the 
 // total number of returns and the number of void returns.
@@ -450,7 +486,17 @@ static void normalize_returns(FnSymbol* fn) {
 // Because we strip type information off of variable declarations and use 
 // the type of the initializer instead, initialization is obligatory.
 // I think we should definitely heed the declared type.  
-// Then at least these two lines can go away, and other simplifications may follow.
+// Then at least these two lines can go away, and other simplifications may
+// follow.
+      // We do not need to do this for iterators returning arrays
+      // because it adds initialization code that is later removed.  
+      // Also, we want arrays returned from iterators to behave like
+      // references, so we add the 'var' return intent here.
+      if (fn->hasFlag(FLAG_ITERATOR_FN) &&
+          returnTypeIsArray(retExprType))
+        // Treat iterators returning arrays as if they are always returned by ref.
+        fn->retTag = RET_VAR;
+      else
       {
       fn->insertAtHead(new CallExpr(PRIM_MOVE, retval, new CallExpr(PRIM_INIT, retExprType->body.tail->remove())));
       fn->insertAtHead(retExprType);
@@ -512,12 +558,12 @@ static void normalize_returns(FnSymbol* fn) {
 static void call_constructor_for_class(CallExpr* call) {
   if (SymExpr* se = toSymExpr(call->baseExpr)) {
     if (TypeSymbol* ts = toTypeSymbol(se->var)) {
-      if (ClassType* ct = toClassType(ts->type)) {
+      if (AggregateType* ct = toAggregateType(ts->type)) {
         // Select symExprs of class (or record) type.
         SET_LINENO(call);
 
-        // These tests can be moved up to a general ClassType object verifier.
-        if (!ct->initializer)
+        // These tests can be moved up to a general AggregateType object verifier.
+        if (!ct->defaultInitializer)
           INT_FATAL(call, "class type has no initializer");
         if (!ct->defaultTypeConstructor)
           INT_FATAL(call, "class type has no default type constructor");
@@ -529,12 +575,12 @@ static void call_constructor_for_class(CallExpr* call) {
 
         if (parent && parent->isPrimitive(PRIM_NEW)) {
           // Transform "new C ( ... )" into _construct_C ( ... ).
-          se->replace(new UnresolvedSymExpr(ct->initializer->name));
+          se->replace(new UnresolvedSymExpr(ct->defaultInitializer->name));
           parent->replace(call->remove());
         } else if (parentParent && parentParent->isPrimitive(PRIM_NEW) &&
                    call->partialTag) {
           // Transform "new C ( (_partial C) ... )" into _construct_C ( ... ).
-          se->replace(new UnresolvedSymExpr(ct->initializer->name));
+          se->replace(new UnresolvedSymExpr(ct->defaultInitializer->name));
           parentParent->replace(parent->remove());
         } else {
           if (ct->symbol->hasFlag(FLAG_SYNTACTIC_DISTRIBUTION))
@@ -605,6 +651,7 @@ static void insert_call_temps(CallExpr* call)
       call->isPrimitive(PRIM_GET_MEMBER_VALUE))
     return;
 
+  // TODO: Check if we need a call temp for PRIM_ASSIGN.
   CallExpr* parentCall = toCallExpr(call->parentExpr);
   if (parentCall && (parentCall->isPrimitive(PRIM_MOVE) ||
                      parentCall->isPrimitive(PRIM_NEW)))
@@ -695,8 +742,7 @@ fix_def_expr(VarSymbol* var) {
     CallExpr* partial;
     if (!type) {
       partial = new CallExpr("newAlias", gMethodToken, init->remove());
-      //      partial->partialTag = true;
-      //      partial->methodTag = true;
+      // newAlias is not a method, so we don't set the methodTag
       stmt->insertAfter(new CallExpr(PRIM_MOVE, var, new CallExpr("chpl__autoCopy", partial)));
     } else {
       partial = new CallExpr("reindex", gMethodToken, init->remove());
@@ -762,25 +808,34 @@ fix_def_expr(VarSymbol* var) {
     // the initialization expression if it exists
     //
     VarSymbol* typeTemp = newTemp("type_tmp");
-    stmt->insertBefore(new DefExpr(typeTemp));
-    stmt->insertBefore(
-      new CallExpr(PRIM_MOVE, typeTemp,
-        new CallExpr(PRIM_INIT, type->remove())));
-    if (init) {
-      stmt->insertAfter(new CallExpr(PRIM_MOVE, constTemp, typeTemp));
-      stmt->insertAfter(
-        new CallExpr(PRIM_MOVE, typeTemp,
-          new CallExpr("=", typeTemp, init->remove())));
+    if (!var->hasFlag(FLAG_NO_INIT))
+      stmt->insertBefore(new DefExpr(typeTemp));
+
+    CallExpr* initCall;
+    if (var->hasFlag(FLAG_NO_INIT)) {
+      initCall = new CallExpr(PRIM_MOVE, var,
+                   new CallExpr(PRIM_NO_INIT, type->remove()));
     } else {
-      if (constTemp->hasFlag(FLAG_TYPE_VARIABLE))
-        stmt->insertAfter(new CallExpr(PRIM_MOVE, constTemp, new CallExpr(PRIM_TYPEOF, typeTemp)));
-      else {
-        CallExpr* moveToConst = new CallExpr(PRIM_MOVE, constTemp, typeTemp);
-        Expr* newExpr = moveToConst;
-        if (var->hasFlag(FLAG_EXTERN)) {
-          newExpr = new BlockStmt(moveToConst, BLOCK_TYPE);
+      initCall = new CallExpr(PRIM_MOVE, typeTemp,
+                   new CallExpr(PRIM_INIT, type->remove()));
+    }
+    stmt->insertBefore(initCall);
+    if (!var->hasFlag(FLAG_NO_INIT)) {
+      if (init) {
+        stmt->insertAfter(new CallExpr(PRIM_MOVE, constTemp, typeTemp));
+        stmt->insertAfter(new CallExpr(PRIM_MOVE, typeTemp,
+                            new CallExpr("=", typeTemp, init->remove())));
+      } else {
+        if (constTemp->hasFlag(FLAG_TYPE_VARIABLE))
+          stmt->insertAfter(new CallExpr(PRIM_MOVE, constTemp, new CallExpr(PRIM_TYPEOF, typeTemp)));
+        else {
+          CallExpr* moveToConst = new CallExpr(PRIM_MOVE, constTemp, typeTemp);
+          Expr* newExpr = moveToConst;
+          if (var->hasFlag(FLAG_EXTERN)) {
+            newExpr = new BlockStmt(moveToConst, BLOCK_TYPE);
+          }
+          stmt->insertAfter(newExpr);
         }
-        stmt->insertAfter(newExpr);
       }
     }
 
@@ -859,12 +914,6 @@ static void fixup_array_formals(FnSymbol* fn) {
       CallExpr* call = toCallExpr(arg->typeExpr->body.tail);
       // Not sure why we select the tail here....
 
-      //if (call && call->isNamed("chpl__buildDomainExpr")) {
-        //CallExpr* arrayTypeCall = new CallExpr("chpl__buildArrayRuntimeType");
-        //call->insertBefore(arrayTypeCall);
-        //arrayTypeCall->insertAtTail(call->remove());
-        //call = arrayTypeCall;
-      //}
       if (call && call->isNamed("chpl__buildArrayRuntimeType")) {
         // We are building an array type.
         bool noDomain = (isSymExpr(call->get(1))) ? toSymExpr(call->get(1))->var == gNil : false;
@@ -1132,6 +1181,22 @@ fixup_query_formals(FnSymbol* fn) {
   }
 }
 
+static void
+find_printModuleInit_stuff() {
+  std::vector<Symbol*> symbols;
+  collectSymbolsSTL(printModuleInitModule, symbols);
+  for_vector(Symbol, symbol, symbols) {
+    if (symbol->hasFlag(FLAG_PRINT_MODULE_INIT_INDENT_LEVEL)) {
+      gModuleInitIndentLevel = toVarSymbol(symbol);
+      INT_ASSERT(gModuleInitIndentLevel);
+    } else if (symbol->hasFlag(FLAG_PRINT_MODULE_INIT_FN)) {
+      gPrintModuleInitFn = toFnSymbol(symbol);
+      INT_ASSERT(gPrintModuleInitFn);
+    }
+  }
+}
+
+
 static void change_method_into_constructor(FnSymbol* fn) {
   if (fn->numFormals() <= 1)
     return;
@@ -1152,14 +1217,14 @@ static void change_method_into_constructor(FnSymbol* fn) {
 
   // The type must be a class type.
   // No constructors for records? <hilde>
-  ClassType* ct = toClassType(fn->getFormal(2)->type);
+  AggregateType* ct = toAggregateType(fn->getFormal(2)->type);
   if (!ct)
     INT_FATAL(fn, "constructor on non-class type");
 
   // Call the initializer, passing in just the generic arguments.
   // This call ensures that the object is default-initialized before the user's
   // constructor body is called.
-  CallExpr* call = new CallExpr(ct->initializer);
+  CallExpr* call = new CallExpr(ct->defaultInitializer);
   for_formals(defaultTypeConstructorArg, ct->defaultTypeConstructor) {
     ArgSymbol* arg = NULL;
     for_formals(methodArg, fn) {
@@ -1189,13 +1254,13 @@ static void change_method_into_constructor(FnSymbol* fn) {
 
   fn->name = astr("_construct_", fn->name);
   // Save a string?
-  INT_ASSERT(!strcmp(fn->name, ct->initializer->name));
+  INT_ASSERT(!strcmp(fn->name, ct->defaultInitializer->name));
   fn->addFlag(FLAG_CONSTRUCTOR);
   // Hide the compiler-generated initializer 
   // which also serves as the default constructor.
   // hilde sez: Try leaving this visible, but make it inferior in case of multiple matches
   // (in disambiguateByMatch()).
-//  ct->initializer->addFlag(FLAG_INVISIBLE_FN);
+//  ct->defaultInitializer->addFlag(FLAG_INVISIBLE_FN);
 }
 
 // Note 1: Since param variables can only be of primitive or enumerated type,
