@@ -1,5 +1,5 @@
 /**************************************************************************
-  Copyright (c) 2004-2013, Cray Inc.  (See LICENSE file for more details)
+  Copyright (c) 2004-2014, Cray Inc.  (See LICENSE file for more details)
 **************************************************************************/
 
 
@@ -23,8 +23,11 @@
 //
 // static functions (forward declaration)
 //
-static void build_type_constructor(ClassType* ct);
-static void build_constructor(ClassType* ct);
+static void build_type_constructor(AggregateType* ct);
+static void build_constructor(AggregateType* ct);
+static ArgSymbol* create_generic_arg(VarSymbol* field);
+static void insert_implicit_this(FnSymbol* fn, Vec<const char*>& fieldNamesSet);
+static void move_constructor_to_outer(FnSymbol* fn, AggregateType* outerType);
 static void resolveUnresolvedSymExprs();
 static void resolveUnresolvedSymExpr(UnresolvedSymExpr* unresolvedSymExpr,
                                      Vec<UnresolvedSymExpr*>& skipSet);
@@ -40,7 +43,7 @@ static void resolveModuleCall(CallExpr* call,
 //   identifiers that are defined via query-expressions, e.g., 't' in
 //   'def f(x: ?t)'
 //
-//   TypeSymbol: defines a scope for EnumType and ClassType types for
+//   TypeSymbol: defines a scope for EnumType and AggregateType types for
 //   the enumerated type constants or the class/record fields
 //
 //   BlockStmt: defines a scope if the block is a normal block
@@ -92,7 +95,7 @@ getScope(BaseAST* ast) {
     } else if (FnSymbol* fn = toFnSymbol(expr->parentSymbol)) {
       return fn;
     } else if (TypeSymbol* ts = toTypeSymbol(expr->parentSymbol)) {
-      if (isEnumType(ts->type) || isClassType(ts->type)) {
+      if (isEnumType(ts->type) || isAggregateType(ts->type)) {
         return ts;
       }
     }
@@ -117,30 +120,35 @@ getScope(BaseAST* ast) {
 //
 static void
 addToSymbolTable(Vec<DefExpr*>& defs) {
-  forv_Vec(DefExpr, def, defs) {
-    if (!def->sym->hasFlag(FLAG_TEMP)) {
-      BaseAST* scope = getScope(def);
-      if (symbolTable.count(scope) == 0) {
-        symbolTable[scope] = new SymbolTableEntry();
-      }
-      SymbolTableEntry* entry = symbolTable[scope];
+  forv_Vec(DefExpr, def, defs)
+  {
+    // If the symbol is a compiler-generated variable, function or label, do not
+    // add it to the symbol table.
+    if (def->sym->hasFlag(FLAG_TEMP) ||
+        def->sym->hasFlag(FLAG_COMPILER_GENERATED))
+      continue;
 
-      if (entry->count(def->sym->name) != 0) {
-        Symbol* sym = (*entry)[def->sym->name];
-        FnSymbol* oldFn = toFnSymbol(sym);
-        FnSymbol* newFn = toFnSymbol(def->sym);
-        TypeSymbol* typeScope = toTypeSymbol(scope);
-        if (!typeScope || !isClassType(typeScope->type)) { // inheritance
-          if ((!oldFn || (oldFn && !oldFn->_this && oldFn->hasFlag(FLAG_NO_PARENS))) &&
-              (!newFn || (newFn && !newFn->_this && newFn->hasFlag(FLAG_NO_PARENS)))) {
-            USR_FATAL(sym, "'%s' has multiple definitions, redefined at:\n  %s", sym->name, def->sym->stringLoc());
-          }
+    BaseAST* scope = getScope(def);
+    if (symbolTable.count(scope) == 0) {
+      symbolTable[scope] = new SymbolTableEntry();
+    }
+    SymbolTableEntry* entry = symbolTable[scope];
+
+    if (entry->count(def->sym->name) != 0) {
+      Symbol* sym = (*entry)[def->sym->name];
+      FnSymbol* oldFn = toFnSymbol(sym);
+      FnSymbol* newFn = toFnSymbol(def->sym);
+      TypeSymbol* typeScope = toTypeSymbol(scope);
+      if (!typeScope || !isAggregateType(typeScope->type)) { // inheritance
+        if ((!oldFn || (!oldFn->_this && oldFn->hasFlag(FLAG_NO_PARENS))) &&
+            (!newFn || (!newFn->_this && newFn->hasFlag(FLAG_NO_PARENS)))) {
+          USR_FATAL(sym, "'%s' has multiple definitions, redefined at:\n  %s", sym->name, def->sym->stringLoc());
         }
-        if (!newFn || (newFn && !newFn->_this && newFn->hasFlag(FLAG_NO_PARENS)))
-          (*entry)[def->sym->name] = def->sym;
-      } else {
-        (*entry)[def->sym->name] = def->sym;
       }
+      if (!newFn || (newFn && !newFn->_this && newFn->hasFlag(FLAG_NO_PARENS)))
+        (*entry)[def->sym->name] = def->sym;
+    } else {
+      (*entry)[def->sym->name] = def->sym;
     }
   }
 }
@@ -235,7 +243,7 @@ lookup(BaseAST* scope,
   }
 
   if (TypeSymbol* ts = toTypeSymbol(scope))
-    if (ClassType* ct = toClassType(ts->type))
+    if (AggregateType* ct = toAggregateType(ts->type))
       if (Symbol* sym = ct->getField(name, false))
         symbols.set_add(sym);
 
@@ -299,8 +307,8 @@ lookup(BaseAST* scope,
       }
     } else {
       FnSymbol* fn = toFnSymbol(scope);
-      if (fn && fn->_this && toClassType(fn->_this->type)) {
-        ClassType* ct = toClassType(fn->_this->type);
+      if (fn && fn->_this && toAggregateType(fn->_this->type)) {
+        AggregateType* ct = toAggregateType(fn->_this->type);
         Symbol* sym = lookup(ct->symbol, name, alreadyVisited, scanModuleUses);
         if (sym)
           symbols.set_add(sym);
@@ -347,9 +355,9 @@ isMethodName(const char* name, Type* type) {
     if (isMethodName(name, pt))
       return true;
   }
-  if (ClassType* ct = toClassType(type)) {
+  if (AggregateType* ct = toAggregateType(type)) {
     Type *outerType = ct->symbol->defPoint->parentSymbol->type;
-    if (ClassType* outer = toClassType(outerType))
+    if (AggregateType* outer = toAggregateType(outerType))
       if (isMethodName(name, outer))
         return true;
   }
@@ -460,24 +468,25 @@ static void resolveEnumeratedTypes() {
 
 
 /********* build constructors ***************/
-void build_constructors(ClassType* ct)
+void build_constructors(AggregateType* ct)
 {
+  if (ct->defaultInitializer)
+    return;
+
+  SET_LINENO(ct);
+
   build_type_constructor(ct);
   build_constructor(ct);
 }
 
 
-static void build_type_constructor(ClassType* ct) {
-  if (ct->defaultTypeConstructor)
-    return;
-
-  SET_LINENO(ct);
-
+// Create the (default) type constructor for this class.
+static void build_type_constructor(AggregateType* ct) {
+  // Create the type constructor function,
   FnSymbol* fn = new FnSymbol(astr("_type_construct_", ct->symbol->name));
   fn->addFlag(FLAG_TYPE_CONSTRUCTOR);
-  ct->defaultTypeConstructor = fn;
   fn->cname = astr("_type_construct_", ct->symbol->cname);
-  fn->addFlag(FLAG_TEMP);
+  fn->addFlag(FLAG_COMPILER_GENERATED);
   fn->retTag = RET_TYPE;
 
   if (ct->symbol->hasFlag(FLAG_REF))
@@ -487,10 +496,15 @@ static void build_type_constructor(ClassType* ct) {
     fn->addFlag(FLAG_INLINE);
   }
 
+  // and insert it into the class type.
+  ct->defaultTypeConstructor = fn;
+
+  // Create "this".
   fn->_this = new VarSymbol("this", ct);
   fn->_this->addFlag(FLAG_ARG_THIS);
   fn->insertAtTail(new DefExpr(fn->_this));
 
+  // Walk all fields and select the generic ones.
   Vec<const char*> fieldNamesSet;
   for_fields(tmp, ct) {
     SET_LINENO(tmp);
@@ -501,6 +515,7 @@ static void build_type_constructor(ClassType* ct) {
 
       Expr* exprType = field->defPoint->exprType;
       Expr* init = field->defPoint->init;
+
       if (!strcmp(field->name, "_promotionType") ||
           field->hasFlag(FLAG_OMIT_FROM_CONSTRUCTOR)) {
         fn->insertAtTail(
@@ -516,23 +531,10 @@ static void build_type_constructor(ClassType* ct) {
         // if formal is generic
         //
         if (field->hasFlag(FLAG_TYPE_VARIABLE) || field->hasFlag(FLAG_PARAM) || (!exprType && !init)) {
-          ArgSymbol* arg = new ArgSymbol(INTENT_BLANK, field->name, field->type);
+
+          ArgSymbol* arg = create_generic_arg(field);
           fn->insertFormalAtTail(arg);
 
-          if (field->hasFlag(FLAG_PARAM))
-            arg->intent = INTENT_PARAM;
-          else
-            arg->addFlag(FLAG_TYPE_VARIABLE);
-
-          if (init)
-            arg->defaultExpr = new BlockStmt(init->copy(), BLOCK_SCOPELESS);
-
-          if (exprType)
-            arg->typeExpr = new BlockStmt(exprType->copy(), BLOCK_TYPE);
-
-          if (!exprType && arg->type == dtUnknown)
-            arg->type = dtAny;
-
           if (field->hasFlag(FLAG_PARAM) || field->hasFlag(FLAG_TYPE_VARIABLE))
             fn->insertAtTail(new CallExpr(PRIM_SET_MEMBER, fn->_this,
                                           new_StringSymbol(field->name), arg));
@@ -561,38 +563,21 @@ static void build_type_constructor(ClassType* ct) {
     }
   }
 
+// Add return
   fn->insertAtTail(new CallExpr(PRIM_RETURN, fn->_this));
-  ct->symbol->defPoint->insertBefore(new DefExpr(fn));
   fn->retType = ct;
 
-  //
-  // insert implicit uses of 'this' in constructor
-  //
-  Vec<BaseAST*> asts;
-  collect_asts(fn->body, asts);
-  forv_Vec(BaseAST, ast, asts) {
-    if (UnresolvedSymExpr* se = toUnresolvedSymExpr(ast))
-      if (fieldNamesSet.set_in(se->unresolved))
-        se->replace(buildDotExpr(fn->_this, se->unresolved));
-  }
+  ct->symbol->defPoint->insertBefore(new DefExpr(fn));
 
-  ClassType *outerType = toClassType(ct->symbol->defPoint->parentSymbol->type);
+  // Make implicit references to 'this' explicit.
+  insert_implicit_this(fn, fieldNamesSet);
+
+  AggregateType *outerType = toAggregateType(ct->symbol->defPoint->parentSymbol->type);
   if (outerType) {
     // Create an "outer" pointer to the outer class in the inner class
     VarSymbol* outer = new VarSymbol("outer");
 
-
-    // Remove the DefPoint for this constructor, add it to the outer
-    // class's method list.
-    outerType->methods.add(fn);
-    fn->_outer = new ArgSymbol(INTENT_BLANK, "outer", outerType);
-    fn->insertFormalAtHead(new DefExpr(fn->_outer));
-    fn->insertFormalAtHead(new DefExpr(new ArgSymbol(INTENT_BLANK, "_mt", dtMethodToken)));
-    fn->addFlag(FLAG_METHOD);
-    Expr* insertPoint = outerType->symbol->defPoint;
-    while (toTypeSymbol(insertPoint->parentSymbol))
-      insertPoint = insertPoint->parentSymbol->defPoint;
-    insertPoint->insertBefore(fn->defPoint->remove());
+    move_constructor_to_outer(fn, outerType);
 
     // Save the pointer to the outer class
     ct->fields.insertAtTail(new DefExpr(outer));
@@ -603,6 +588,7 @@ static void build_type_constructor(ClassType* ct) {
     ct->outer = outer;
   }
 
+  // Update the symbol table with added defs.
   Vec<DefExpr*> defs;
   collectDefExprs(fn, defs);
   addToSymbolTable(defs);
@@ -612,22 +598,16 @@ static void build_type_constructor(ClassType* ct) {
 // For the given class type, this builds the compiler-generated constructor
 // which is also called by user-defined constructors to pre-initialize all 
 // fields to their declared or type-specific initial values.
-static void build_constructor(ClassType* ct) {
-  if (ct->initializer)
-    return;
-
-  SET_LINENO(ct);
-
+static void build_constructor(AggregateType* ct) {
   if (isSyncType(ct))
     ct->defaultValue = NULL;
 
-  // Create the default constructor.
+  // Create the default constructor function symbol,
   FnSymbol* fn = new FnSymbol(astr("_construct_", ct->symbol->name));
   fn->addFlag(FLAG_DEFAULT_CONSTRUCTOR);
   fn->addFlag(FLAG_CONSTRUCTOR);
-  ct->initializer = fn;
   fn->cname = astr("_construct_", ct->symbol->cname);
-  fn->addFlag(FLAG_TEMP); // compiler inserted
+  fn->addFlag(FLAG_COMPILER_GENERATED);
 
   if (ct->symbol->hasFlag(FLAG_REF))
     fn->addFlag(FLAG_REF);
@@ -636,6 +616,9 @@ static void build_constructor(ClassType* ct) {
     fn->addFlag(FLAG_INLINE);
   }
 
+  // And insert it into the class type.
+  ct->defaultInitializer = fn;
+
   // Create "this".
   fn->_this = new VarSymbol("this", ct);
   fn->_this->addFlag(FLAG_ARG_THIS);
@@ -679,16 +662,16 @@ static void build_constructor(ClassType* ct) {
     if (isClass(ct)) {
       if (ct->dispatchParents.n > 0 && !ct->symbol->hasFlag(FLAG_EXTERN)) {
         // This class has a parent class.
-        if (!ct->dispatchParents.v[0]->initializer) {
+        if (!ct->dispatchParents.v[0]->defaultInitializer) {
           // If it doesn't yet have an initializer, make one.
-          build_constructors(toClassType(ct->dispatchParents.v[0]));
+          build_constructors(toAggregateType(ct->dispatchParents.v[0]));
         }
 
         // Get the parent constructor.
         // Note that since we only pay attention to the first entry in the
         // dispatchParents list, we are effectively implementing
         // single class inheritance, multiple interface inheritance.
-        FnSymbol* superCtor = ct->dispatchParents.v[0]->initializer;
+        FnSymbol* superCtor = ct->dispatchParents.v[0]->defaultInitializer;
 
         // Create a call to the superclass constructor.
         superCall = new CallExpr(superCtor->name);
@@ -770,6 +753,7 @@ static void build_constructor(ClassType* ct) {
     } else if (hadType && !field->hasFlag(FLAG_TYPE_VARIABLE) && !field->hasFlag(FLAG_PARAM)) {
       init = new CallExpr(PRIM_INIT, exprType->copy());
     }
+
     if (!field->hasFlag(FLAG_TYPE_VARIABLE) && !field->hasFlag(FLAG_PARAM)) {
       if (hadType)
         init = new CallExpr("_createFieldDefault", exprType->copy(), init);
@@ -777,12 +761,14 @@ static void build_constructor(ClassType* ct) {
         if (init)
           init = new CallExpr("chpl__initCopy", init);
     }
+
     if (init) {
       if (hadInit)
         arg->defaultExpr = new BlockStmt(init, BLOCK_SCOPELESS);
       else
         arg->defaultExpr = new BlockStmt(new SymExpr(gTypeDefaultToken));
     }
+
     if (exprType) {
       if (!isBlockStmt(exprType))
         arg->typeExpr = new BlockStmt(exprType, BLOCK_TYPE);
@@ -800,6 +786,11 @@ static void build_constructor(ClassType* ct) {
                                     new_StringSymbol(arg->name),
                                     new CallExpr("chpl__initCopy", arg)));
     else
+      // Since we don't copy the argument before stuffing it in a field, we will
+      // have to remove the autodestroy flag for specific cases.  Namely, if the
+      // function is a default constructor and the target of a PRIM_SET_MEMBER is
+      // a record, then the INSERT_AUTO_DESTROY flag must be removed.
+      // (See NOTE 1 in callDestructors.cpp.)
       fn->insertAtTail(new CallExpr(PRIM_SET_MEMBER, fn->_this, 
                                     new_StringSymbol(arg->name),
                                     arg));
@@ -808,30 +799,11 @@ static void build_constructor(ClassType* ct) {
   if (meme)
     fn->insertFormalAtTail(meme);
 
-  //
-  // insert implicit uses of 'this' in constructor
-  //
-  Vec<BaseAST*> asts;
-  collect_asts(fn->body, asts);
-  forv_Vec(BaseAST, ast, asts) {
-    if (UnresolvedSymExpr* se = toUnresolvedSymExpr(ast))
-      if (fieldNamesSet.set_in(se->unresolved))
-        se->replace(buildDotExpr(fn->_this, se->unresolved));
-  }
+  insert_implicit_this(fn, fieldNamesSet);
 
-  ClassType *outerType = toClassType(ct->symbol->defPoint->parentSymbol->type);
+  AggregateType *outerType = toAggregateType(ct->symbol->defPoint->parentSymbol->type);
   if (outerType) {
-    // Remove the DefPoint for this constructor, add it to the outer
-    // class's method list.
-    outerType->methods.add(fn);
-    fn->_outer = new ArgSymbol(INTENT_BLANK, "outer", outerType);
-    fn->insertFormalAtHead(new DefExpr(fn->_outer));
-    fn->insertFormalAtHead(new DefExpr(new ArgSymbol(INTENT_BLANK, "_mt", dtMethodToken)));
-    fn->addFlag(FLAG_METHOD);
-    Expr* insertPoint = outerType->symbol->defPoint;
-    while (toTypeSymbol(insertPoint->parentSymbol))
-      insertPoint = insertPoint->parentSymbol->defPoint;
-    insertPoint->insertBefore(fn->defPoint->remove());
+    move_constructor_to_outer(fn, outerType);
 
     // Save the pointer to the outer class
     fn->insertAtTail(new CallExpr(PRIM_SET_MEMBER,
@@ -862,6 +834,68 @@ static void build_constructor(ClassType* ct) {
   collectDefExprs(fn, defs);
   addToSymbolTable(defs);
 }
+
+
+static ArgSymbol* create_generic_arg(VarSymbol* field)
+{
+  ArgSymbol* arg = new ArgSymbol(INTENT_BLANK, field->name, field->type);
+
+  // We take it as a param argument if it is marked as a param field.
+  if (field->hasFlag(FLAG_PARAM))
+    arg->intent = INTENT_PARAM;
+  else
+    arg->addFlag(FLAG_TYPE_VARIABLE);
+
+  // Copy the field type if it exists.
+  Expr* exprType = field->defPoint->exprType;
+  if (exprType)
+    arg->typeExpr = new BlockStmt(exprType->copy(), BLOCK_TYPE);
+
+  // Copy the initialization expression if it exists.
+  Expr* init = field->defPoint->init;
+  if (init)
+    arg->defaultExpr = new BlockStmt(init->copy(), BLOCK_SCOPELESS);
+
+  // Translate an unknown field type into an unspecified arg type.
+  if (!exprType && arg->type == dtUnknown)
+    arg->type = dtAny;
+
+  return arg;
+}
+
+
+/// Replace implicit references to 'this' in the body of this 
+/// type constructor with explicit member reference (dot) expressions.
+static void insert_implicit_this(FnSymbol* fn, Vec<const char*>& fieldNamesSet)
+{
+  Vec<BaseAST*> asts;
+  collect_asts(fn->body, asts);
+  forv_Vec(BaseAST, ast, asts) {
+    if (UnresolvedSymExpr* se = toUnresolvedSymExpr(ast))
+      if (fieldNamesSet.set_in(se->unresolved))
+        // The name of this UnresolvedSymExpr matches a field name.
+        // So replace it with a dot expression.
+        se->replace(buildDotExpr(fn->_this, se->unresolved));
+  }
+}
+
+
+static void move_constructor_to_outer(FnSymbol* fn, AggregateType* outerType)
+{
+  // Remove the DefPoint for this constructor, add it to the outer
+  // class's method list.
+  outerType->methods.add(fn);
+  fn->_outer = new ArgSymbol(INTENT_BLANK, "outer", outerType);
+  fn->insertFormalAtHead(new DefExpr(fn->_outer));
+  fn->insertFormalAtHead(new DefExpr(new ArgSymbol(INTENT_BLANK, "_mt", dtMethodToken)));
+  fn->addFlag(FLAG_METHOD);
+  Expr* insertPoint = outerType->symbol->defPoint;
+  while (toTypeSymbol(insertPoint->parentSymbol))
+    insertPoint = insertPoint->parentSymbol->defPoint;
+  insertPoint->insertBefore(fn->defPoint->remove());
+}
+
+
 /********* end build constructor ***************/
 
 
@@ -945,7 +979,7 @@ process_import_expr(CallExpr* call) {
   call->getStmtExpr()->remove();
 }
 
-void add_root_type(ClassType* ct)
+void add_root_type(AggregateType* ct)
 {
   // make root records inherit from value
   // make root classes inherit from object
@@ -974,9 +1008,9 @@ void add_root_type(ClassType* ct)
 // record hierarchies
 //
 static void
-add_class_to_hierarchy(ClassType* ct, Vec<ClassType*>* localSeenPtr = NULL) {
-  static Vec<ClassType*> globalSeen; // classes already in hierarchy
-  Vec<ClassType*> localSeen;         // classes in potential cycle
+add_class_to_hierarchy(AggregateType* ct, Vec<AggregateType*>* localSeenPtr = NULL) {
+  static Vec<AggregateType*> globalSeen; // classes already in hierarchy
+  Vec<AggregateType*> localSeen;         // classes in potential cycle
 
   if (!localSeenPtr)
     localSeenPtr = &localSeen;
@@ -1000,7 +1034,7 @@ add_class_to_hierarchy(ClassType* ct, Vec<ClassType*>* localSeenPtr = NULL) {
     if (!ts)
       USR_FATAL(expr, "Illegal super class");
     //    printf("found it in %s\n", sym->getModule()->name);
-    ClassType* pt = toClassType(ts->type);
+    AggregateType* pt = toAggregateType(ts->type);
     if (!pt)
       USR_FATAL(expr, "Illegal super class %s", ts->name);
     if (isUnion(ct) && isUnion(pt))
@@ -1095,7 +1129,7 @@ static bool tryCResolve_set(ModuleSymbol* module, const char* name,
         addToSymbolTable(v);
         if (DefExpr* de = toDefExpr(c_expr)) {
           if (TypeSymbol* ts = toTypeSymbol(de->sym)) {
-            if (ClassType* ct = toClassType(ts->type)) {
+            if (AggregateType* ct = toAggregateType(ts->type)) {
               SET_LINENO(ct->symbol);
               //If this is a class DefExpr, make sure its initializer gets created.
               build_constructors(ct);
@@ -1144,7 +1178,7 @@ void scopeResolve(void) {
   //
   // compute class hierarchy
   //
-  forv_Vec(ClassType, ct, gClassTypes) {
+  forv_Vec(AggregateType, ct, gAggregateTypes) {
     add_class_to_hierarchy(ct);
   }
 
@@ -1164,7 +1198,7 @@ void scopeResolve(void) {
   //
   // add implicit fields for implementing alias-named-argument passing
   //
-  forv_Vec(ClassType, ct, gClassTypes) {
+  forv_Vec(AggregateType, ct, gAggregateTypes) {
     for_fields(field, ct) {
       if (aliasFieldSet.set_in(field->name)) {
         SET_LINENO(field);
@@ -1182,7 +1216,7 @@ void scopeResolve(void) {
   //
   // build constructors (type and value versions)
   //
-  forv_Vec(ClassType, ct, gClassTypes) {
+  forv_Vec(AggregateType, ct, gAggregateTypes) {
     SET_LINENO(ct->symbol);
     build_constructors(ct);
   }
@@ -1349,7 +1383,7 @@ void resolveUnresolvedSymExpr(UnresolvedSymExpr* unresolvedSymExpr,
             Type* type = method->_this->type;
             TypeSymbol* cts =
               (sym) ? toTypeSymbol(sym->defPoint->parentSymbol) : NULL;
-            if ((cts && isClassType(cts->type)) ||
+            if ((cts && isAggregateType(cts->type)) ||
                 isMethodName(name, type)) {
               CallExpr* call = toCallExpr(expr->parentExpr);
               if (call && call->baseExpr == expr &&
@@ -1360,14 +1394,14 @@ void resolveUnresolvedSymExpr(UnresolvedSymExpr* unresolvedSymExpr,
                 expr->replace(use);
                 skipSet.set_add(use);
               } else {
-                ClassType* ct = toClassType(type);
+                AggregateType* ct = toAggregateType(type);
                 int nestDepth = 0;
                 if (isMethodName(name, type)) {
                   while (ct && !isMethodNameLocal(name, ct)) {
                     // count how many classes out from current depth that
                     // this method is first defined in
                     nestDepth += 1;
-                    ct = toClassType
+                    ct = toAggregateType
                       (ct->symbol->defPoint->parentSymbol->type);
                   }
                 } else {
@@ -1375,7 +1409,7 @@ void resolveUnresolvedSymExpr(UnresolvedSymExpr* unresolvedSymExpr,
                     // count how many classes out from current depth that
                     // this symbol is first defined in
                     nestDepth += 1;
-                    ct = toClassType(ct->symbol->defPoint->parentSymbol->type);
+                    ct = toAggregateType(ct->symbol->defPoint->parentSymbol->type);
                   }
                 }
 
