@@ -1,5 +1,5 @@
 /**************************************************************************
-  Copyright (c) 2004-2013, Cray Inc.  (See LICENSE file for more details)
+  Copyright (c) 2004-2014, Cray Inc.  (See LICENSE file for more details)
 **************************************************************************/
 
 
@@ -54,7 +54,9 @@ static void nonLeaderParCheckInt(FnSymbol* fn, bool allowYields)
       USR_FATAL_CONT(call, "invalid use of parallel construct in serial iterator");
     }
     if ((call->isPrimitive(PRIM_BLOCK_ON)) ||
-        (call->isPrimitive(PRIM_BLOCK_ON_NB))) {
+        (call->isPrimitive(PRIM_BLOCK_BEGIN_ON)) ||
+        (call->isPrimitive(PRIM_BLOCK_COBEGIN_ON)) ||
+        (call->isPrimitive(PRIM_BLOCK_COFORALL_ON))) {
       // begin/cobegin/coforall *blocks* are eliminated earlier.
       // If they are not, check for PRIM_YIELD like below.
       INT_ASSERT(false);
@@ -359,7 +361,7 @@ replaceIteratorFormalsWithIteratorFields(FnSymbol* iterator, Symbol* ic,
 static Map<FnSymbol*,FnSymbol*> iteratorFnMap;
 static FnSymbol* argBundleCopyFn = NULL;
 static FnSymbol* argBundleFreeFn = NULL;
-static ClassType*  argBundleType = NULL;
+static AggregateType*  argBundleType = NULL;
 
 
 //
@@ -367,9 +369,9 @@ static ClassType*  argBundleType = NULL;
 // class, if one does not exist yet.
 //
 static void
-ensureArgBundleType(ClassType* ct) {
+ensureArgBundleType(AggregateType* ct) {
   if (argBundleType == NULL) {
-    argBundleType = new ClassType(CLASS_CLASS);
+    argBundleType = new AggregateType(AGGREGATE_CLASS);
     TypeSymbol* aabSym = new TypeSymbol("chpl_argBundle", argBundleType);
     aabSym->addFlag(FLAG_NO_OBJECT);
     ct->symbol->defPoint->insertBefore(new DefExpr(aabSym));
@@ -393,7 +395,7 @@ ensureArgBundleType(ClassType* ct) {
 // }
 //
 static void
-createArgBundleFreeFn(ClassType* ct, FnSymbol* loopBodyFnWrapper) {
+createArgBundleFreeFn(AggregateType* ct, FnSymbol* loopBodyFnWrapper) {
   if (argBundleFreeFn == NULL) {
     // Create the shared function that frees recursive argument bundles.
     argBundleFreeFn = new FnSymbol("chpl__freeRecursiveIteratorArgumentBundle");
@@ -479,7 +481,7 @@ createArgBundleFreeFn(ClassType* ct, FnSymbol* loopBodyFnWrapper) {
 //  chpl_argBundle chpl__copyRecursiveIteratorArgumentBundle(int loopBodyFnID, chpl_argBundle loopBodyFnArgs)
 //
 static void
-createArgBundleCopyFn(ClassType* ct, FnSymbol* loopBodyFnWrapper) {
+createArgBundleCopyFn(AggregateType* ct, FnSymbol* loopBodyFnWrapper) {
   if (argBundleCopyFn == NULL) {
     // Create the shared function that copies recursive argument bundles.
     // The initial implementation is just a shell.  After creation, it looks like:
@@ -571,7 +573,7 @@ createArgBundleCopyFn(ClassType* ct, FnSymbol* loopBodyFnWrapper) {
 // This creates the class type carrying the arguments to a (possibly recursive)
 // iterator function call and fills a temp with the passed-in values.
 //
-static ClassType* 
+static AggregateType* 
 bundleLoopBodyFnArgsForIteratorFnCall(CallExpr* iteratorFnCall,
                                       CallExpr* loopBodyFnCall,
                                       FnSymbol* loopBodyFnWrapper) {
@@ -579,14 +581,14 @@ bundleLoopBodyFnArgsForIteratorFnCall(CallExpr* iteratorFnCall,
   FnSymbol* loopBodyFn = loopBodyFnCall->isResolved();
 
   // Create the argument bundle type
-  ClassType* ct = new ClassType(CLASS_CLASS);
+  AggregateType* ct = new AggregateType(AGGREGATE_CLASS);
   TypeSymbol* ts = new TypeSymbol("_fn_arg_bundle", ct);
   ts->addFlag(FLAG_NO_OBJECT);
   ts->addFlag(FLAG_LOOP_BODY_ARGUMENT_CLASS);
   iteratorFnCall->parentSymbol->defPoint->insertBefore(new DefExpr(ts));
 
   // And its corresponding reference type.
-  ClassType* rct = new ClassType(CLASS_CLASS);
+  AggregateType* rct = new AggregateType(AGGREGATE_CLASS);
   TypeSymbol* rts = new TypeSymbol("_ref_fn_arg_bundle", rct);
   rts->addFlag(FLAG_NO_OBJECT);
   rts->addFlag(FLAG_REF);
@@ -933,7 +935,9 @@ static bool fnContainsOn(FnSymbol* fn)
   collectCallExprs(fn, calls);
   forv_Vec(CallExpr, call, calls) {
     if (call->isPrimitive(PRIM_BLOCK_ON) ||
-        call->isPrimitive(PRIM_BLOCK_ON_NB))
+        call->isPrimitive(PRIM_BLOCK_BEGIN_ON) ||
+        call->isPrimitive(PRIM_BLOCK_COBEGIN_ON) ||
+        call->isPrimitive(PRIM_BLOCK_COFORALL_ON))
       return true;
     if (FnSymbol* taskFn = resolvedToTaskFun(call))
       if (fnContainsOn(taskFn))
@@ -957,7 +961,7 @@ createIteratorFn(FnSymbol* iterator, CallExpr* iteratorFnCall, Symbol* index,
 
   // Now calls the newly-created iterator function.
   iteratorFnCall->baseExpr->replace(new SymExpr(iteratorFn));
-  ClassType* argsBundleType =
+  AggregateType* argsBundleType =
     bundleLoopBodyFnArgsForIteratorFnCall(iteratorFnCall, loopBodyFnCall, loopBodyFnWrapper);
 
   iteratorFn->body = iterator->body->copy();
@@ -1017,7 +1021,7 @@ expandRecursiveIteratorInline(CallExpr* call)
   // passed to it when this function is flattened)
   //
   Symbol* ic = toSymExpr(call->get(2))->var;
-  FnSymbol* iterator = ic->type->initializer->getFormal(1)->type->initializer;
+  FnSymbol* iterator = ic->type->defaultInitializer->getFormal(1)->type->defaultInitializer;
   CallExpr* iteratorFnCall = new CallExpr(iterator, ic, new_IntSymbol(ftableMap.get(loopBodyFnWrapper)));
   // replace function in iteratorFnCall with iterator function once that is created
 
@@ -1065,7 +1069,7 @@ expandBodyForIteratorInline(BlockStmt* body, BlockStmt* ibody, Vec<BaseAST*>& as
 static void
 expandIteratorInline(CallExpr* call) {
   Symbol* ic = toSymExpr(call->get(2))->var;
-  FnSymbol* iterator = ic->type->initializer->getFormal(1)->type->initializer;
+  FnSymbol* iterator = ic->type->defaultInitializer->getFormal(1)->type->defaultInitializer;
 
   if (iterator->hasFlag(FLAG_RECURSIVE_ITERATOR)) {
 
@@ -1236,7 +1240,7 @@ canInlineIterator(FnSymbol* iterator) {
 static void
 getRecursiveIterators(Vec<Symbol*>& iterators, Symbol* gIterator) {
   if (gIterator->type->symbol->hasFlag(FLAG_TUPLE)) {
-    ClassType* iteratorType = toClassType(gIterator->type);
+    AggregateType* iteratorType = toAggregateType(gIterator->type);
     for (int i=1; i <= iteratorType->fields.length; i++) {
       Symbol *iterator = toSymbol(iteratorType->getField(i));
       if (iterator)
@@ -1254,7 +1258,7 @@ canInlineSingleYieldIterator(Symbol* gIterator) {
   Vec<Symbol*> iterators;
   getRecursiveIterators(iterators, gIterator);
   for (int i = 0; i < iterators.n; i++) {
-    FnSymbol* iterator= iterators.v[i]->type->initializer->getFormal(1)->type->initializer;
+    FnSymbol* iterator= iterators.v[i]->type->defaultInitializer->getFormal(1)->type->defaultInitializer;
     BlockStmt *block = iterator->body;
 
     INT_ASSERT(block);
@@ -1289,8 +1293,8 @@ setupSimultaneousIterators(Vec<Symbol*>& iterators,
                            Symbol* gIndex,
                            BlockStmt* loop) {
   if (gIterator->type->symbol->hasFlag(FLAG_TUPLE)) {
-    ClassType* iteratorType = toClassType(gIterator->type);
-    ClassType* indexType = toClassType(gIndex->type);
+    AggregateType* iteratorType = toAggregateType(gIterator->type);
+    AggregateType* indexType = toAggregateType(gIndex->type);
     for (int i=1; i <= iteratorType->fields.length; i++) {
       Symbol* iterator = newTemp("_iterator", iteratorType->getField(i)->type);
       loop->insertBefore(new DefExpr(iterator));
@@ -1344,7 +1348,7 @@ getIteratorChildren(Vec<Type*>& children, Type* type) {
 
 static void
 buildIteratorCallInner(BlockStmt* block, Symbol* ret, int fnid, Symbol* iterator) {
-  IteratorInfo* ii = iterator->type->initializer->getFormal(1)->type->initializer->iteratorInfo;
+  IteratorInfo* ii = iterator->type->defaultInitializer->getFormal(1)->type->defaultInitializer->iteratorInfo;
   FnSymbol* fn = NULL;
   switch (fnid) {
   case ZIP1: fn = ii->zip1; break;
@@ -1410,7 +1414,7 @@ inlineSingleYieldIterator(CallExpr* call) {
   CallExpr *noop = new CallExpr(PRIM_NOOP);
   block->insertAtHead(noop);
   for (int i = 0; i < iterators.n; i++) {
-    FnSymbol *iterator = iterators.v[i]->type->initializer->getFormal(1)->type->initializer;
+    FnSymbol *iterator = iterators.v[i]->type->defaultInitializer->getFormal(1)->type->defaultInitializer;
     Vec<Expr*> exprs;
 
     BlockStmt *ibody = iterator->body->copy();
@@ -1480,8 +1484,8 @@ expand_for_loop(CallExpr* call) {
     INT_FATAL(call, "bad for loop primitive");
 
   if (!fNoInlineIterators &&
-      iterator->type->initializer->getFormal(1)->type->initializer->iteratorInfo &&
-      canInlineIterator(iterator->type->initializer->getFormal(1)->type->initializer) &&
+      iterator->type->defaultInitializer->getFormal(1)->type->defaultInitializer->iteratorInfo &&
+      canInlineIterator(iterator->type->defaultInitializer->getFormal(1)->type->defaultInitializer) &&
       (iterator->type->dispatchChildren.n == 0 ||
        (iterator->type->dispatchChildren.n == 1 &&
         iterator->type->dispatchChildren.v[0] == dtObject))) {
@@ -1511,7 +1515,7 @@ expand_for_loop(CallExpr* call) {
 
       // hilde sez: This seems like a kludge.
       // Why is the pragma applied to the initializer rather than the type itself?
-      if (isBoundedIterator(iterators.v[i]->type->initializer->getFormal(1)->type->initializer)) {
+      if (isBoundedIterator(iterators.v[i]->type->defaultInitializer->getFormal(1)->type->defaultInitializer)) {
         if (!firstCond) {
           firstCond = cond;
         } else if (!fNoBoundsChecks) {
@@ -1543,7 +1547,7 @@ inlineIterators() {
     if (block->parentSymbol) {
       if (block->blockInfo && block->blockInfo->isPrimitive(PRIM_BLOCK_FOR_LOOP)) {
         Symbol* iterator = toSymExpr(block->blockInfo->get(2))->var;
-        FnSymbol* ifn = iterator->type->initializer->getFormal(1)->type->initializer;
+        FnSymbol* ifn = iterator->type->defaultInitializer->getFormal(1)->type->defaultInitializer;
         if (ifn->hasFlag(FLAG_INLINE_ITERATOR)) {
           expandIteratorInline(block->blockInfo);
         }
@@ -1662,7 +1666,7 @@ static void fixNumericalGetMemberPrims()
   // via a number
   forv_Vec(CallExpr, call, gCallExprs) {
     if (call->parentSymbol && call->isPrimitive(PRIM_GET_MEMBER)) {
-      ClassType* ct = toClassType(call->get(1)->getValType());
+      AggregateType* ct = toAggregateType(call->get(1)->getValType());
       int64_t num;
       if (get_int(call->get(2), &num)) {
         Symbol* field = ct->getField(num+1); // add 1 for super
@@ -1703,7 +1707,7 @@ static void cleanupLeaderFollowerIteratorCalls()
             for_actuals(actual, call) {
               SymExpr* se = toSymExpr(actual);
               if (isArgSymbol(se->var) && call->parentSymbol != se->var->defPoint->parentSymbol) {
-                Symbol* field = toClassType(iteratorType)->getField(i);
+                Symbol* field = toAggregateType(iteratorType)->getField(i);
                 VarSymbol* tmp = NULL;
                 SET_LINENO(call);
                 if (field->type == se->var->type) {
@@ -1737,7 +1741,7 @@ static void handlePolymorphicIterators()
       INT_ASSERT(getIterator->defPoint->parentSymbol);
 
       // See if the iterator record is polymorphic.
-      ClassType* irecord = fn->iteratorInfo->irecord;
+      AggregateType* irecord = fn->iteratorInfo->irecord;
       if (irecord->dispatchChildren.n > 0) {
         // If so, then simulate dynamic dispatch by adding one conditional block
         // for each possible subtype.
@@ -1750,11 +1754,11 @@ static void handlePolymorphicIterators()
           VarSymbol* cid = newTemp(dtBool);
           BlockStmt* thenStmt = new BlockStmt();
           VarSymbol* recordTmp = newTemp("recordTmp", type);
-          VarSymbol* classTmp = newTemp("classTmp", type->initializer->iteratorInfo->getIterator->retType);
+          VarSymbol* classTmp = newTemp("classTmp", type->defaultInitializer->iteratorInfo->getIterator->retType);
           thenStmt->insertAtTail(new DefExpr(recordTmp));
           thenStmt->insertAtTail(new DefExpr(classTmp));
 
-          ClassType* ct = toClassType(type);
+          AggregateType* ct = toAggregateType(type);
           for_fields(field, ct) {
             VarSymbol* ftmp = newTemp("ftmp", getIterator->getFormal(1)->type->getField(field->name)->type);
             thenStmt->insertAtTail(new DefExpr(ftmp));
@@ -1769,11 +1773,11 @@ static void handlePolymorphicIterators()
               thenStmt->insertAtTail(new CallExpr(PRIM_SET_MEMBER, recordTmp, field, ftmp2));
             }
           }
-          thenStmt->insertAtTail(new CallExpr(PRIM_MOVE, classTmp, new CallExpr(type->initializer->iteratorInfo->getIterator, recordTmp)));
+          thenStmt->insertAtTail(new CallExpr(PRIM_MOVE, classTmp, new CallExpr(type->defaultInitializer->iteratorInfo->getIterator, recordTmp)));
           thenStmt->insertAtTail(new CallExpr(PRIM_MOVE, ret, new CallExpr(PRIM_CAST, ret->type->symbol, classTmp)));
           thenStmt->insertAtTail(new GotoStmt(GOTO_GETITER_END, label));
           ret->defPoint->insertAfter(new CondStmt(new SymExpr(cid), thenStmt));
-          ret->defPoint->insertAfter(new CallExpr(PRIM_MOVE, cid, new CallExpr(PRIM_TESTCID, tmp, type->initializer->iteratorInfo->irecord->getField(1)->type->symbol)));
+          ret->defPoint->insertAfter(new CallExpr(PRIM_MOVE, cid, new CallExpr(PRIM_TESTCID, tmp, type->defaultInitializer->iteratorInfo->irecord->getField(1)->type->symbol)));
           ret->defPoint->insertAfter(new DefExpr(cid));
           ret->defPoint->insertAfter(new CallExpr(PRIM_MOVE, tmp, new CallExpr(PRIM_GET_MEMBER_VALUE, getIterator->getFormal(1), irecord->getField(1))));
           ret->defPoint->insertAfter(new DefExpr(tmp));
@@ -1784,6 +1788,54 @@ static void handlePolymorphicIterators()
 }
 
 
+static void reconstructIRAutoCopy(FnSymbol* fn)
+{
+  Symbol* arg = fn->getFormal(1);
+  Symbol* ret = fn->getReturnSymbol();
+  BlockStmt* block = new BlockStmt();
+  block->insertAtTail(ret->defPoint->remove());
+  AggregateType* irt = toAggregateType(arg->type);
+  for_fields(field, irt) {
+    SET_LINENO(field);
+    if (FnSymbol* autoCopy = autoCopyMap.get(field->type)) {
+      Symbol* tmp1 = newTemp(field->name, field->type);
+      Symbol* tmp2 = newTemp(autoCopy->retType);
+      block->insertAtTail(new DefExpr(tmp1));
+      block->insertAtTail(new DefExpr(tmp2));
+      block->insertAtTail(new CallExpr(PRIM_MOVE, tmp1, new CallExpr(PRIM_GET_MEMBER_VALUE, arg, field)));
+      block->insertAtTail(new CallExpr(PRIM_MOVE, tmp2, new CallExpr(autoCopy, tmp1)));
+      block->insertAtTail(new CallExpr(PRIM_SET_MEMBER, ret, field, tmp2));
+    } else {
+      Symbol* tmp = newTemp(field->name, field->type);
+      block->insertAtTail(new DefExpr(tmp));
+      block->insertAtTail(new CallExpr(PRIM_MOVE, tmp, new CallExpr(PRIM_GET_MEMBER_VALUE, arg, field)));
+      block->insertAtTail(new CallExpr(PRIM_SET_MEMBER, ret, field, tmp));
+    }
+  }
+  block->insertAtTail(new CallExpr(PRIM_RETURN, ret));
+  fn->body->replace(block);
+}
+
+
+static void reconstructIRAutoDestroy(FnSymbol* fn)
+{
+  Symbol* arg = fn->getFormal(1);
+  BlockStmt* block = new BlockStmt();
+  AggregateType* irt = toAggregateType(arg->type);
+  for_fields(field, irt) {
+    SET_LINENO(field);
+    if (FnSymbol* autoDestroy = autoDestroyMap.get(field->type)) {
+      Symbol* tmp = newTemp(field->name, field->type);
+      block->insertAtTail(new DefExpr(tmp));
+      block->insertAtTail(new CallExpr(PRIM_MOVE, tmp, new CallExpr(PRIM_GET_MEMBER_VALUE, arg, field)));
+      block->insertAtTail(new CallExpr(autoDestroy, tmp));
+    }
+  }
+  block->insertAtTail(new CallExpr(PRIM_RETURN, gVoid));
+  fn->body->replace(block);
+}
+
+
 static void reconstructIRautoCopyAutoDestroy()
 {
   //
@@ -1792,49 +1844,10 @@ static void reconstructIRautoCopyAutoDestroy()
   forv_Vec(FnSymbol, fn, gFnSymbols) {
     if (fn->numFormals() == 1 && fn->getFormal(1)->type->symbol->hasFlag(FLAG_ITERATOR_RECORD)) {
       SET_LINENO(fn);
-      if (fn->hasFlag(FLAG_AUTO_COPY_FN)) {
-        Symbol* arg = fn->getFormal(1);
-        Symbol* ret = fn->getReturnSymbol();
-        BlockStmt* block = new BlockStmt();
-        block->insertAtTail(ret->defPoint->remove());
-        ClassType* irt = toClassType(arg->type);
-        for_fields(field, irt) {
-          SET_LINENO(field);
-          if (FnSymbol* autoCopy = autoCopyMap.get(field->type)) {
-            Symbol* tmp1 = newTemp(field->name, field->type);
-            Symbol* tmp2 = newTemp(autoCopy->retType);
-            block->insertAtTail(new DefExpr(tmp1));
-            block->insertAtTail(new DefExpr(tmp2));
-            block->insertAtTail(new CallExpr(PRIM_MOVE, tmp1, new CallExpr(PRIM_GET_MEMBER_VALUE, arg, field)));
-            block->insertAtTail(new CallExpr(PRIM_MOVE, tmp2, new CallExpr(autoCopy, tmp1)));
-            block->insertAtTail(new CallExpr(PRIM_SET_MEMBER, ret, field, tmp2));
-          } else {
-            Symbol* tmp = newTemp(field->name, field->type);
-            block->insertAtTail(new DefExpr(tmp));
-            block->insertAtTail(new CallExpr(PRIM_MOVE, tmp, new CallExpr(PRIM_GET_MEMBER_VALUE, arg, field)));
-            block->insertAtTail(new CallExpr(PRIM_SET_MEMBER, ret, field, tmp));
-          }
-        }
-        block->insertAtTail(new CallExpr(PRIM_RETURN, ret));
-        fn->body->replace(block);
-      }
-      if (fn->hasFlag(FLAG_AUTO_DESTROY_FN)) {
-        SET_LINENO(fn);
-        Symbol* arg = fn->getFormal(1);
-        BlockStmt* block = new BlockStmt();
-        ClassType* irt = toClassType(arg->type);
-        for_fields(field, irt) {
-          SET_LINENO(field);
-          if (FnSymbol* autoDestroy = autoDestroyMap.get(field->type)) {
-            Symbol* tmp = newTemp(field->name, field->type);
-            block->insertAtTail(new DefExpr(tmp));
-            block->insertAtTail(new CallExpr(PRIM_MOVE, tmp, new CallExpr(PRIM_GET_MEMBER_VALUE, arg, field)));
-            block->insertAtTail(new CallExpr(autoDestroy, tmp));
-          }
-        }
-        block->insertAtTail(new CallExpr(PRIM_RETURN, gVoid));
-        fn->body->replace(block);
-      }
+      if (fn->hasFlag(FLAG_AUTO_COPY_FN))
+        reconstructIRAutoCopy(fn);
+      if (fn->hasFlag(FLAG_AUTO_DESTROY_FN))
+        reconstructIRAutoDestroy(fn);
     }
   }
 }
@@ -1875,11 +1888,12 @@ void lowerIterators() {
     if (fn->hasFlag(FLAG_ITERATOR_FN)) {
       collapseBlocks(fn->body);
       removeUnnecessaryGotos(fn);
+#if DEBUG_CP < 2    // That is, disabled if DEBUG_CP >= 2
       if (!fNoCopyPropagation)
         localCopyPropagation(fn);
-      if (!fNoDeadCodeElimination) {
+      if (!fNoDeadCodeElimination)
         deadCodeElimination(fn);
-      }
+#endif
     }
   }
   forv_Vec(FnSymbol, fn, gFnSymbols) {
