@@ -1,5 +1,5 @@
 /**************************************************************************
-  Copyright (c) 2004-2013, Cray Inc.  (See LICENSE file for more details)
+  Copyright (c) 2004-2014, Cray Inc.  (See LICENSE file for more details)
 **************************************************************************/
 
 
@@ -1492,11 +1492,6 @@ void makeBinaryLLVM(void) {
 
   options += " -pthread";
 
-  for (int i=0; i<numLibFlags; i++) {
-    options += " ";
-    options += libFlag[i];
-  }
-
   // Now, if we're doing a multilocale build, we have to make a launcher.
   // For this reason, we create a makefile. codegen_makefile
   // also gives us the name of the temporary place to save
@@ -1527,6 +1522,14 @@ void makeBinaryLLVM(void) {
     command += args[i];
   }
 
+  // Put user-requested libraries at the end of the compile line,
+  // they should at least be after the .o files and should be in
+  // order where libraries depend on libraries to their right.
+  for (int i=0; i<numLibFlags; i++) {
+    command += " ";
+    command += libFlag[i];
+  }
+
   mysystem(command.c_str(), "Make Binary - Linking");
 
   // Now run the makefile to move from tmpbinname to the proper program
