@@ -1,5 +1,5 @@
 /**************************************************************************
-  Copyright (c) 2004-2013, Cray Inc.  (See LICENSE file for more details)
+  Copyright (c) 2004-2014, Cray Inc.  (See LICENSE file for more details)
 **************************************************************************/
 
 
@@ -70,6 +70,25 @@ void ensureDirExists(const char* dirname, const char* explanation) {
 }
 
 
+static const char* removeSpacesFromString(char* str) {
+  if (strchr(str, ' ')) {
+    char* str2 = (char*)malloc(strlen(str)+1);
+    char* dst = str2;
+    char* src = str;
+    while (*src != '\0') {
+      *dst = *src;
+      src++;
+      if (*dst != ' ')
+        dst++;
+    }
+    *dst = '\0';
+    return str2;
+  } else {
+    return str;
+  }
+}
+
+
 static void ensureTmpDirExists(void) {
   if (saveCDir[0] == '\0') {
     if (tmpdirname == NULL) {
@@ -89,7 +108,7 @@ static void ensureTmpDirExists(void) {
       if (passwdinfo == NULL) {
         userid = "anon";
       } else {
-        userid = passwdinfo->pw_name;
+        userid = removeSpacesFromString(passwdinfo->pw_name);
       }
       
       tmpdirname = astr(tmpdirprefix, userid, mypidstr, tmpdirsuffix);
@@ -230,7 +249,7 @@ void appendCFile(fileinfo* fi, const char* name, const char* ext) {
 }
 void closeCFile(fileinfo* fi, bool beautifyIt) {
   fclose(fi->fptr);
-  if (beautifyIt)
+  if (beautifyIt && saveCDir[0])
     beautify(fi);
 }
 
@@ -356,7 +375,9 @@ static const char* mysystem_getresult(const char* command,
   char* result = (char*)malloc(256*sizeof(char));
   mysystem(astr(command, " > ", fullSystemFilename), description, ignorestatus);
   fileinfo* systemFile = openTmpFile(systemFilename, "r");
-  fscanf(systemFile->fptr, "%s", result);
+  if (fscanf(systemFile->fptr, "%s", result) != 1){
+    strcpy(result,"");
+  }
   closefile(systemFile);
   return astr(result);  // canonicalize
 }
@@ -628,18 +649,24 @@ void addDashMsToUserPath(void) {
 
 
 void setupModulePaths(void) {
-  intModPath.add(astr(CHPL_HOME, "/modules/internal/localeModels/",
+  const char* modulesRoot = (fMinimalModules ? "modules-minimal" : "modules");
+
+  intModPath.add(astr(CHPL_HOME, "/", modulesRoot, "/internal/localeModels/",
                       CHPL_LOCALE_MODEL));
-  intModPath.add(astr(CHPL_HOME, "/modules/internal/threads/", CHPL_THREADS));
-  intModPath.add(astr(CHPL_HOME, "/modules/internal/tasks/", CHPL_TASKS));
-  intModPath.add(astr(CHPL_HOME, "/modules/internal/comm/", CHPL_COMM));
-  intModPath.add(astr(CHPL_HOME, "/modules/internal"));
-  stdModPath.add(astr(CHPL_HOME, "/modules/standard/gen/", CHPL_TARGET_PLATFORM,
+  intModPath.add(astr(CHPL_HOME, "/", modulesRoot, "/internal/threads/", 
+                      CHPL_THREADS));
+  intModPath.add(astr(CHPL_HOME, "/", modulesRoot, "/internal/tasks/", 
+                      CHPL_TASKS));
+  intModPath.add(astr(CHPL_HOME, "/", modulesRoot, "/internal/comm/", 
+                      CHPL_COMM));
+  intModPath.add(astr(CHPL_HOME, "/", modulesRoot, "/internal"));
+  stdModPath.add(astr(CHPL_HOME, "/", modulesRoot, "/standard/gen/", 
+                      CHPL_TARGET_PLATFORM,
                       "-", CHPL_TARGET_COMPILER));
-  stdModPath.add(astr(CHPL_HOME, "/modules/standard"));
-  stdModPath.add(astr(CHPL_HOME, "/modules/layouts"));
-  stdModPath.add(astr(CHPL_HOME, "/modules/dists"));
-  stdModPath.add(astr(CHPL_HOME, "/modules/dists/dims"));
+  stdModPath.add(astr(CHPL_HOME, "/", modulesRoot, "/standard"));
+  stdModPath.add(astr(CHPL_HOME, "/", modulesRoot, "/layouts"));
+  stdModPath.add(astr(CHPL_HOME, "/", modulesRoot, "/dists"));
+  stdModPath.add(astr(CHPL_HOME, "/", modulesRoot, "/dists/dims"));
   const char* envvarpath = getenv("CHPL_MODULE_PATH");
   if (envvarpath) {
     char path[FILENAME_MAX+1];
