@@ -1,5 +1,5 @@
 /**************************************************************************
-  Copyright (c) 2004-2013, Cray Inc.  (See LICENSE file for more details)
+  Copyright (c) 2004-2014, Cray Inc.  (See LICENSE file for more details)
 **************************************************************************/
 
 
@@ -52,6 +52,10 @@ namespace {
 
 
 static const bool DEBUG = false;
+static const bool extraChecks = false;
+// Set a function name here to get lots of debugging output.
+static const char* debugThisFn = "";
+
 
 // If there is a gap between memory that we are loading,
 // for example due to padding, or just because we didn't
@@ -95,10 +99,9 @@ Value* getLoadStorePointer(Instruction* I)
 static
 Value* rebasePointer(Value* ptr, Value* oldBase, Value* newBase, const Twine &name,
                      IRBuilder<>* builder, const DataLayout &TD,
-                     Value** oldBaseI, Value** newBaseI)
+                     Value* oldBaseI, Value* newBaseI)
 {
   Type* iPtrTy = TD.getIntPtrType(ptr->getType());
-  Type* iNewBaseTy = TD.getIntPtrType(newBase->getType());
   Type* localPtrTy = ptr->getType()->getPointerElementType()->getPointerTo(0);
 
   Value* ret;
@@ -106,22 +109,14 @@ Value* rebasePointer(Value* ptr, Value* oldBase, Value* newBase, const Twine &na
   if( ptr != oldBase ) {
     // compute newBase + (ptr - oldBase)
     Value* pI = builder->CreatePtrToInt(ptr, iPtrTy, name + ".ptr.i");
-    Value* obI = *oldBaseI;
-    if( ! obI ) {
-      obI = builder->CreatePtrToInt(oldBase, iPtrTy, name + ".oldb.i");
-      *oldBaseI = obI;
-    }
-    Value* newI = *newBaseI;
-    if( ! newI ) {
-      newI = builder->CreatePtrToInt(newBase, iNewBaseTy, name + ".newb.i");
-      *newBaseI = newI;
-    }
+    assert( oldBaseI );
+    assert( newBaseI );
     // then subtract
-    Value* diff = builder->CreateSub(pI, obI, name + ".diff");
+    Value* diff = builder->CreateSub(pI, oldBaseI, name + ".diff");
     // then make sure same type
-    Value* ext = builder->CreateSExtOrTrunc(diff, newI->getType(), ".ext.i");
+    Value* ext = builder->CreateSExtOrTrunc(diff, newBaseI->getType(), ".ext.i");
     // Now add
-    Value* sum = builder->CreateAdd(newI, ext, name + ".sum");
+    Value* sum = builder->CreateAdd(newBaseI, ext, name + ".sum");
     ret = builder->CreateIntToPtr(sum, localPtrTy, name + ".cast");
   } else {
     ret = builder->CreatePointerCast(newBase, localPtrTy, name + ".cast");
@@ -129,6 +124,78 @@ Value* rebasePointer(Value* ptr, Value* oldBase, Value* newBase, const Twine &na
   return ret;
 }
 
+// Given a start and end load/store instruction (in the same basic block),
+// reorder the instructions so that the addressing instructions are
+// first, the load/store instructions are next, and then the
+// uses of loaded values are last. This reordering is valid when
+// the other instructions do not read or write memory.
+// Returns the last instruction in the reordering.
+static
+Instruction* reorderAddressingMemopsUses(Instruction *FirstLoadOrStore,
+                                         Instruction *LastLoadOrStore,
+                                         bool DebugThis)
+{
+  SmallPtrSet<Instruction*, 8> memopsUses;
+  Instruction *LastMemopUse = NULL;
+
+  for (BasicBlock::iterator BI = FirstLoadOrStore; !isa<TerminatorInst>(BI); ++BI) {
+    Instruction* insn = BI;
+    bool isUseOfMemop = false;
+
+    if( isa<StoreInst>(insn) || isa<LoadInst>(insn) ) {
+      memopsUses.insert(insn);
+      continue;
+    }
+    // Check -- are any operands to this instruction memopsUses?
+    for (User::op_iterator i = insn->op_begin(), e = insn->op_end(); i != e; ++i) {
+      Value *v = *i;
+      if(Instruction *uses_insn = dyn_cast<Instruction>(v)) {
+        if( memopsUses.count(uses_insn) ){
+          isUseOfMemop = true;
+          break;
+        }
+      }
+    }
+
+    if( isUseOfMemop ) memopsUses.insert(insn);
+
+    if( insn == LastLoadOrStore ) break;
+  }
+
+  LastMemopUse = LastLoadOrStore;
+
+  // Reorder the instructions here.
+  // Move all addressing instructions before StartInst.
+  // Move all uses of loaded values before LastLoadOrStore (which will be removed).
+  for (BasicBlock::iterator BI = FirstLoadOrStore; !isa<TerminatorInst>(BI);) {
+    Instruction* insn = BI++; // don't invalidate iterator.
+    // Leave loads/stores where they are (they will be removed)
+    if( isa<StoreInst>(insn) || isa<LoadInst>(insn) ) {
+      if( DebugThis ) {
+        errs() << "found load/store: "; insn->dump();
+      }
+    } else if( memopsUses.count(insn) ) {
+      if( DebugThis ) {
+        errs() << "found memop use: "; insn->dump();
+      }
+      // Move uses of memops to after the final memop.
+      insn->removeFromParent();
+      insn->insertAfter(LastMemopUse);
+      LastMemopUse = insn;
+    } else {
+      if( DebugThis ) {
+        errs() << "found other: "; insn->dump();
+      }
+      // Move addressing instructions to before the first memop.
+      insn->removeFromParent();
+      insn->insertBefore(FirstLoadOrStore);
+    }
+    if( insn == LastLoadOrStore ) break;
+  }
+
+  return LastMemopUse;
+}
+
 // The next several fns are stolen almost totally unmodified from MemCpyOptimizer.
 // modified code areas say CUSTOM.
 
@@ -381,7 +448,7 @@ void MemOpRanges::addRange(int64_t Start, int64_t Size, int64_t Slack, Value *Pt
       AU.addPreserved<MemoryDependenceAnalysis>();
     }
 
-    Instruction *tryAggregating(Instruction *I, Value *StartPtr);
+    Instruction *tryAggregating(Instruction *I, Value *StartPtr, bool DebugThis);
   };
 
   char AggregateGlobalOpsOpt::ID = 0;
@@ -399,7 +466,8 @@ FunctionPass *createAggregateGlobalOpsOptPass(unsigned globalSpace)
 /// other loads or stores that could be aggregated with this one.
 /// Returns the last instruction added (if one was added) since we might have
 /// removed some loads or stores and that might invalidate an iterator.
-Instruction *AggregateGlobalOpsOpt::tryAggregating(Instruction *StartInst, Value *StartPtr) {
+Instruction *AggregateGlobalOpsOpt::tryAggregating(Instruction *StartInst, Value *StartPtr,
+    bool DebugThis) {
   if (TD == 0) return 0;
 
   Module* M = StartInst->getParent()->getParent()->getParent();
@@ -411,7 +479,10 @@ Instruction *AggregateGlobalOpsOpt::tryAggregating(Instruction *StartInst, Value
   bool isLoad = isa<LoadInst>(StartInst);
   bool isStore = isa<StoreInst>(StartInst);
   Instruction *lastAddedInsn = NULL;
+  Instruction *LastLoadOrStore = NULL;
  
+  SmallVector<Instruction*, 8> toRemove;
+
   // Okay, so we now have a single global load/store. Scan to find
   // all subsequent stores of the same value to offset from the same pointer.
   // Join these together into ranges, so we can decide whether contiguous blocks
@@ -448,6 +519,7 @@ Instruction *AggregateGlobalOpsOpt::tryAggregating(Instruction *StartInst, Value
         break;
 
       Ranges.addStore(Offset, NextStore);
+      LastLoadOrStore = NextStore;
     } else {
       LoadInst *NextLoad = cast<LoadInst>(BI);
       if (!NextLoad->isSimple()) break;
@@ -458,14 +530,21 @@ Instruction *AggregateGlobalOpsOpt::tryAggregating(Instruction *StartInst, Value
         break;
 
       Ranges.addLoad(Offset, NextLoad);
+      LastLoadOrStore = NextLoad;
     }
   }
+
   // If we have no ranges, then we just had a single store with nothing that
   // could be merged in.  This is a very common case of course.
   if (!Ranges.moreThanOneOp())
     return 0;
 
-  IRBuilder<> builder(StartInst);
+  // Divide the instructions between StartInst and LastLoadOrStore into
+  // addressing, memops, and uses of memops (uses of loads)
+  reorderAddressingMemopsUses(StartInst, LastLoadOrStore, DebugThis);
+
+  Instruction* insertBefore = StartInst;
+  IRBuilder<> builder(insertBefore);
 
   // Now that we have full information about ranges, loop over the ranges and
   // emit memcpy's for anything big enough to be worthwhile.
@@ -474,17 +553,16 @@ Instruction *AggregateGlobalOpsOpt::tryAggregating(Instruction *StartInst, Value
     const MemOpRange &Range = *I;
     Value* oldBaseI = NULL;
     Value* newBaseI = NULL;
-    Instruction* insertBefore = Range.TheStores[0];
 
     if (Range.TheStores.size() == 1) continue; // Don't bother if there's only one thing...
 
     builder.SetInsertPoint(insertBefore);
 
-    // Otherwise, we do want to transform this!  Create a new mecpy.
+    // Otherwise, we do want to transform this!  Create a new memcpy.
     // Get the starting pointer of the block.
     StartPtr = Range.StartPtr;
 
-    if( DEBUG ) {
+    if( DebugThis ) {
       errs() << "base is:";
       StartPtr->dump();
     }
@@ -504,19 +582,13 @@ Instruction *AggregateGlobalOpsOpt::tryAggregating(Instruction *StartInst, Value
     alloc = makeAlloca(int8Ty, "agg.tmp", insertBefore,
                        Range.End-Range.Start, Alignment);
 
-    // If StartPtr is not Range.TheStores[0], we will need to compute
-    // the old base address from Range.TheStores[0] plus its offset;
-    // otherwise we can run into situations where we havn't computed
-    // the old base address yet before the first load/store.
-    Value* firstPtr = getLoadStorePointer(Range.TheStores[0]);
-    if( firstPtr != StartPtr ) {
-      int64_t Offset = 0;
-      bool ok = IsPointerOffset(firstPtr, StartPtr, Offset, *TD);
-      assert(ok); // must be same base if we got here!
-      // Now just subtract offset from pointer.
-      Value* cast = builder.CreatePointerCast(firstPtr, globalInt8PtrTy, "agg.bcast");
-      Value* gep = builder.CreateConstInBoundsGEP1_64(cast, Offset, "agg.bgep");
-      StartPtr = gep;
+    // Generate the old and new base pointers before we output
+    // anything else.
+    {
+      Type* iPtrTy = TD->getIntPtrType(alloc->getType());
+      Type* iNewBaseTy = TD->getIntPtrType(alloc->getType());
+      oldBaseI = builder.CreatePtrToInt(StartPtr, iPtrTy, "agg.tmp.oldb.i");
+      newBaseI = builder.CreatePtrToInt(alloc, iNewBaseTy, "agg.tmp.newb.i");
     }
 
     // If storing, do the stores we had into our alloca'd region.
@@ -526,16 +598,14 @@ Instruction *AggregateGlobalOpsOpt::tryAggregating(Instruction *StartInst, Value
            SE = Range.TheStores.end(); SI != SE; ++SI) {
         StoreInst* oldStore = cast<StoreInst>(*SI);
 
-        if( DEBUG ) {
+        if( DebugThis ) {
           errs() << "have store in range:";
           oldStore->dump();
         }
 
-        builder.SetInsertPoint(oldStore);
-
         Value* ptrToAlloc = rebasePointer(oldStore->getPointerOperand(),
                                           StartPtr, alloc, "agg.tmp",
-                                          &builder, *TD, &oldBaseI, &newBaseI);
+                                          &builder, *TD, oldBaseI, newBaseI);
         // Old load must not be volatile or atomic... or we shouldn't have put
         // it in ranges
         assert(!(oldStore->isVolatile() || oldStore->isAtomic()));
@@ -605,16 +675,14 @@ Instruction *AggregateGlobalOpsOpt::tryAggregating(Instruction *StartInst, Value
            SI = Range.TheStores.begin(),
            SE = Range.TheStores.end(); SI != SE; ++SI) {
         LoadInst* oldLoad = cast<LoadInst>(*SI);
-        if( DEBUG ) {
+        if( DebugThis ) {
           errs() << "have load in range:";
           oldLoad->dump();
         }
 
-        builder.SetInsertPoint(oldLoad);
-
         Value* ptrToAlloc = rebasePointer(oldLoad->getPointerOperand(),
                                           StartPtr, alloc, "agg.tmp",
-                                          &builder, *TD, &oldBaseI, &newBaseI);
+                                          &builder, *TD, oldBaseI, newBaseI);
         // Old load must not be volatile or atomic... or we shouldn't have put
         // it in ranges
         assert(!(oldLoad->isVolatile() || oldLoad->isAtomic()));
@@ -626,15 +694,22 @@ Instruction *AggregateGlobalOpsOpt::tryAggregating(Instruction *StartInst, Value
       }
     }
 
-    // Zap all the old loads/stores
+    // Save old loads/stores for removal
     for (SmallVector<Instruction*, 16>::const_iterator
          SI = Range.TheStores.begin(),
          SE = Range.TheStores.end(); SI != SE; ++SI) {
-      //MD->removeInstruction(*SI);
-      (*SI)->eraseFromParent();
+      Instruction* insn = *SI;
+      toRemove.push_back(insn);
     }
   }
- 
+
+  // Zap all the old loads/stores
+  for (SmallVector<Instruction*, 16>::const_iterator
+       SI = toRemove.begin(),
+       SE = toRemove.end(); SI != SE; ++SI) {
+    (*SI)->eraseFromParent();
+  }
+
   return lastAddedInsn;
 }
 
@@ -643,13 +718,19 @@ Instruction *AggregateGlobalOpsOpt::tryAggregating(Instruction *StartInst, Value
 //
 bool AggregateGlobalOpsOpt::runOnFunction(Function &F) {
   bool MadeChange = false;
+  bool DebugThis = DEBUG;
+  
+  if( debugThisFn[0] && F.getName() == debugThisFn ) {
+    DebugThis = true;
+  }
+
   //MD = &getAnalysis<MemoryDependenceAnalysis>();
   TD = getAnalysisIfAvailable<DataLayout>();
   //TLI = &getAnalysis<TargetLibraryInfo>();
 
   // Walk all instruction in the function.
   for (Function::iterator BB = F.begin(), BBE = F.end(); BB != BBE; ++BB) {
-    if( DEBUG ) {
+    if( DebugThis ) {
       errs() << "Working on BB ";
       BB->dump();
     }
@@ -659,7 +740,7 @@ bool AggregateGlobalOpsOpt::runOnFunction(Function &F) {
       Instruction *I = BI++;
 
       if( isGlobalLoadOrStore(I, globalSpace, true, true) ) {
-        Instruction* lastAdded = tryAggregating(I, getLoadStorePointer(I));
+        Instruction* lastAdded = tryAggregating(I, getLoadStorePointer(I), DebugThis);
         if( lastAdded ) {
           MadeChange = true;
           BI = lastAdded;
@@ -667,14 +748,14 @@ bool AggregateGlobalOpsOpt::runOnFunction(Function &F) {
       }
     }
 
-    if( DEBUG && MadeChange ) {
+    if( DebugThis && MadeChange ) {
       errs() << "After transform BB is ";
       BB->dump();
     }
 
   }
 
-  if( DEBUG ) {
+  if( extraChecks ) {
     verifyFunction(F);
   }
 
