@@ -1,5 +1,5 @@
 /**************************************************************************
-  Copyright (c) 2004-2013, Cray Inc.  (See LICENSE file for more details)
+  Copyright (c) 2004-2014, Cray Inc.  (See LICENSE file for more details)
 **************************************************************************/
 
 
@@ -695,7 +695,7 @@ namespace {
               assert(0 && "Unknown special function");
             }
           } else if( calledFn && isa<MemIntrinsic>(call) ) {
-            if( isa<MemCpyInst>(call) ) {
+            if( isa<MemCpyInst>(call) || isa<MemMoveInst>(call) ) {
               // handle memcpy
               // is the source a global pointer?
               // memcpy(dst, src, n)
