@@ -1,5 +1,5 @@
 /**************************************************************************
-  Copyright (c) 2004-2013, Cray Inc.  (See LICENSE file for more details)
+  Copyright (c) 2004-2014, Cray Inc.  (See LICENSE file for more details)
 **************************************************************************/
 
 
@@ -10,14 +10,19 @@
 #include "chpltypes.h"
 
 //
-// The type of a global locale ID, and functions to assemble and
-// disassemble global locale IDs.
+// This is the type of a global locale ID.
 //
 typedef struct {
   int32_t node;
   int32_t subloc;
 } chpl_localeID_t;
 
+//
+// This is the initializer for a chpl_localeID_t.  This macro is
+// referenced explicitly in the compiler, in symbol.cpp.
+//
+#define CHPL_LOCALEID_T_INIT  {0, 0}
+
 //
 // This is the external copy constructor for a chpl_localeID_t, specified
 // by the module code for a numa locale model.
@@ -27,6 +32,10 @@ chpl_localeID_t chpl__initCopy_chpl_rt_localeID_t(chpl_localeID_t initial) {
   return initial;
 }
 
+//
+// These functions are used by the module code to assemble and
+// disassemble global locale IDs.
+//
 static ___always_inline
 chpl_localeID_t chpl_rt_buildLocaleID(c_nodeid_t node, c_sublocid_t subloc) {
   chpl_localeID_t loc = { node, subloc };
