@@ -1,5 +1,5 @@
 /**************************************************************************
-  Copyright (c) 2004-2013, Cray Inc.  (See LICENSE file for more details)
+  Copyright (c) 2004-2014, Cray Inc.  (See LICENSE file for more details)
 **************************************************************************/
 
 
@@ -19,10 +19,12 @@
 #include "error.h"
 
 #include <signal.h>
+#include <sched.h>
 #include <stdint.h>
 #include <stdlib.h>
 #include <string.h>
 #include <assert.h>
+#include <time.h>
 
 #ifdef GASNET_NEEDS_MAX_SEGSIZE
 #define CHPL_COMM_GASNET_SETENV chpl_comm_gasnet_set_max_segsize();
@@ -60,6 +62,24 @@ static gasnet_seginfo_t* seginfo_table = NULL;
     }                                                                   \
   } while(0)
 
+//
+// Adapted from the above, this one is for the case where the caller
+// needs to do the fncall separately (see chpl_comm_barrier()).  Here
+// the fn is not actually called; fncall is only used to produce the
+// message.
+//
+#define GASNET_Safe_Retval(fncall, _retval) do {                        \
+    if (_retval != GASNET_OK) {                                         \
+      fprintf(stderr, "ERROR calling: %s\n"                             \
+              " at: %s:%i\n"                                            \
+              " error: %s (%s)\n",                                      \
+              #fncall, __FILE__, __LINE__,                              \
+              gasnet_ErrorName(_retval), gasnet_ErrorDesc(_retval));    \
+      fflush(stderr);                                                   \
+      gasnet_exit(_retval);                                             \
+    }                                                                   \
+  } while(0)
+
 //
 // This is the type of object we use to manage GASNet acknowledgements.
 //
@@ -129,13 +149,6 @@ typedef struct {
 static void AM_fork_fast(gasnet_token_t token, void* buf, size_t nbytes) {
   fork_t *f = buf;
 
-  if (chpl_verbose_comm) {
-    char mybuf[128];
-    sprintf(mybuf, "%d: running (fast) remote task created by %d\n",
-            chpl_nodeID, f->caller);
-    write(2, mybuf, strlen(mybuf));
-  }
-
   if (f->arg_size)
     chpl_ftable_call(f->fid, &f->arg);
   else
@@ -337,12 +350,25 @@ int32_t chpl_comm_getMaxThreads(void) {
   return GASNETI_MAX_THREADS-1;
 }
 
-static done_t alldone;
-static volatile int pollingRunning = 0;
+//
+// On all locales, we'll do the primary polling in a thread of control
+// managed by the tasking layer, so that it can coordinate the use of
+// hardware resources for polling and user tasks.  This symmetry will
+// also allow the tasking layer to minimize its locale-based behavioral
+// differences and simplify our analysis of performance effects due to
+// polling.  We'll refer to this thread of control as the "polling task"
+// even though the tasking layer can implement it however it likes, as a
+// task or thread or whatever.
+//
+static volatile int pollingRunning;
+static volatile int pollingQuit;
 
 static void polling(void* x) {
   pollingRunning = 1;
-  GASNET_BLOCKUNTIL(alldone.flag);
+  while (!pollingQuit) {
+    (void) gasnet_AMPoll();
+    chpl_task_yield();
+  }
   pollingRunning = 0;
 }
 
@@ -463,7 +489,7 @@ void chpl_comm_init(int *argc_p, char ***argv_p) {
 void chpl_comm_post_mem_init(void) { }
 
 int chpl_comm_numPollingTasks(void) {
-  return (chpl_nodeID == 0);
+  return 1;
 }
 
 //
@@ -475,13 +501,14 @@ int chpl_comm_run_in_gdb(int argc, char* argv[], int gdbArgnum, int* status) {
 
 void chpl_comm_post_task_init(void) {
   //
-  // Start polling task on locale 0.  (On other locales, main enters
-  // into a barrier wait, so the polling task is unnecessary.)
+  // Start a polling task on each locale.
   //
-  if (chpl_nodeID == 0) {
-    INIT_DONE_OBJ(alldone, 1);
-    if (chpl_task_createCommTask(polling, NULL))
-      chpl_internal_error("unable to start polling task for gasnet");
+  pollingRunning = 0;
+  pollingQuit = 0;
+  if (chpl_task_createCommTask(polling, NULL))
+    chpl_internal_error("unable to start polling task for gasnet");
+  while (!pollingRunning) {
+    sched_yield();
   }
 
   // clear diags
@@ -506,10 +533,6 @@ void chpl_comm_desired_shared_heap(void** start_p, size_t* size_p) {
 #endif
 }
 
-void chpl_comm_alloc_registry(int numGlobals) {
-  chpl_globals_registry = chpl_globals_registry_static;
-}
-
 void chpl_comm_broadcast_global_vars(int numGlobals) {
   int i;
   if (chpl_nodeID != 0) {
@@ -579,25 +602,38 @@ void chpl_comm_broadcast_private(int id, int32_t size, int32_t tid) {
 }
 
 void chpl_comm_barrier(const char *msg) {
+  int id = (int) msg[0];
+  int retval;
+
   if (chpl_verbose_comm && !chpl_comm_no_debug_private)
     printf("%d: barrier for '%s'\n", chpl_nodeID, msg);
-  gasnet_barrier_notify(0, GASNET_BARRIERFLAG_ANONYMOUS);
-  GASNET_Safe(gasnet_barrier_wait(0, GASNET_BARRIERFLAG_ANONYMOUS));
+
+  //
+  // We don't want to just do a gasnet_barrier_wait() here, because
+  // GASNet will put us to work polling, and we already have a polling
+  // task that the tasking layer has presumably placed to best effect.
+  // We don't want to compete with that.  Also, the implementation is
+  // required to do chpl_task_yield() while waiting for the barrier to
+  // satisfy; see chpl_comm.h.  This prevents us from monopolizing the
+  // processor while waiting.
+  //
+  gasnet_barrier_notify(id, 0);
+  while ((retval = gasnet_barrier_try(id, 0)) == GASNET_ERR_NOT_READY) {
+    chpl_task_yield();
+  }
+  GASNET_Safe_Retval(gasnet_barrier_try(id, 0), retval);
 }
 
 void chpl_comm_pre_task_exit(int all) {
   if (all) {
-    chpl_comm_barrier("chpl_comm_pre_task_exit");
+    chpl_comm_barrier("stop polling");
 
-    if (chpl_nodeID == 0) {
-      //
-      // Only locale 0 actually runs a polling task.  Tell that task to
-      // halt, and then wait for it to do so.
-      //
-      GASNET_Safe(gasnet_AMRequestShort2(chpl_nodeID, SIGNAL,
-                                         AckArg0(&alldone),
-                                         AckArg1(&alldone)));
-      while (pollingRunning) {}
+    //
+    // Tell the polling task to halt, then wait for it to do so.
+    //
+    pollingQuit = 1;
+    while (pollingRunning) {
+      sched_yield();
     }
   }
 }
@@ -605,10 +641,9 @@ void chpl_comm_pre_task_exit(int all) {
 static void exit_common(int status) {
   static int loopback = 0;
 
+  pollingQuit = 1;
+
   if (chpl_nodeID == 0) {
-    GASNET_Safe(gasnet_AMRequestShort2(chpl_nodeID, SIGNAL,
-                                       AckArg0(&alldone),
-                                       AckArg1(&alldone)));
     if (loopback) {
       gasnet_exit(2);
     }
@@ -620,15 +655,14 @@ static void exit_common(int status) {
 }
 
 static void exit_any_dirty(int status) {
-  // kill the polling task on locale 0, but other than that...
+  // kill the polling task, but other than that...
   // clean up nothing; just ask GASNet to exit
   // GASNet will then kill all other locales.
   static int loopback = 0;
 
+  pollingQuit = 1;
+
   if (chpl_nodeID == 0) {
-    GASNET_Safe(gasnet_AMRequestShort2(chpl_nodeID, SIGNAL,
-                                       AckArg0(&alldone),
-                                       AckArg1(&alldone)));
     if (loopback) {
       gasnet_exit(2);
     }
