@@ -1,5 +1,5 @@
 /**************************************************************************
-  Copyright (c) 2004-2013, Cray Inc.  (See LICENSE file for more details)
+  Copyright (c) 2004-2014, Cray Inc.  (See LICENSE file for more details)
 **************************************************************************/
 
 
@@ -78,11 +78,13 @@ void qbytes_free_iobuf(qbytes_t* b) {
 
 void debug_print_bytes(qbytes_t* b)
 {
-  fprintf(stderr, "bytes %p: data=%p len=%lli ref_cnt=%li free_function=%p flags=%i\n",
-          b, b->data, (long long int) b->len, (long int) DO_GET_REFCNT(b),
+  fprintf(stderr, "bytes %p: data=%p len=%lli ref_cnt=%" PRIu64 " free_function=%p flags=%i\n",
+          b, b->data, (long long int) b->len, DO_GET_REFCNT(b),
           b->free_function, b->flags);
 }
 
+// On return the ref count is 1.
+// The callee owns the returned qbytes buffer until qbytes_release is called.
 void _qbytes_init_generic(qbytes_t* ret, void* give_data, int64_t len, qbytes_free_t free_function)
 {
   ret->data = give_data;
@@ -92,6 +94,7 @@ void _qbytes_init_generic(qbytes_t* ret, void* give_data, int64_t len, qbytes_fr
   ret->free_function = free_function;
 }
 
+// On return, the ref count is 1.
 err_t qbytes_create_generic(qbytes_t** out, void* give_data, int64_t len, qbytes_free_t free_function)
 {
   qbytes_t* ret = NULL;
@@ -99,6 +102,7 @@ err_t qbytes_create_generic(qbytes_t** out, void* give_data, int64_t len, qbytes
   ret = (qbytes_t*) qio_calloc(1, sizeof(qbytes_t));
   if( ! ret ) return ENOMEM;
 
+  // On return the ref count is 1.
   _qbytes_init_generic(ret, give_data, len, free_function);
 
   *out = ret;
@@ -106,6 +110,9 @@ err_t qbytes_create_generic(qbytes_t** out, void* give_data, int64_t len, qbytes
   return 0;
 }
 
+// On return, the ref count in ret is 1.
+// The caller owns the returned qbytes buffer until qbytes_release
+// is called on it.
 err_t _qbytes_init_iobuf(qbytes_t* ret)
 {
   void* data = NULL;
@@ -120,6 +127,7 @@ err_t _qbytes_init_iobuf(qbytes_t* ret)
   //if( err ) return err;
   memset(data, 0, qbytes_iobuf_size);
 
+  // The ref count in ret is initially 1.
   _qbytes_init_generic(ret, data, qbytes_iobuf_size, qbytes_free_iobuf);
 
   return 0;
@@ -132,7 +140,10 @@ err_t qbytes_create_iobuf(qbytes_t** out)
   err_t err;
 
   ret = (qbytes_t*) qio_calloc(1, sizeof(qbytes_t));
-  if( ! ret ) return ENOMEM;
+  if( ! ret ) {
+    *out = NULL;
+    return ENOMEM;
+  }
 
   err = _qbytes_init_iobuf(ret);
   if( err ) {
@@ -160,15 +171,21 @@ err_t _qbytes_init_calloc(qbytes_t* ret, int64_t len)
   return 0;
 }*/
 
+// The ref count returned in 'out' is initially 1.
+// The caller is responsible for calling qbytes_release on it.
 err_t qbytes_create_calloc(qbytes_t** out, int64_t len)
 {
   qbytes_t* ret = NULL;
   void* data;
 
   ret = (qbytes_t*) qio_calloc(1, sizeof(qbytes_t) + len);
-  if( ! ret ) return ENOMEM;
+  if( ! ret ) {
+    *out = NULL;
+    return ENOMEM;
+  }
 
   data = ret + 1; // ie ret + sizeof(qbytes_t)
+  // On return, the ref count in ret is 1.
   _qbytes_init_generic(ret, data, len, qbytes_free_null);
 
   *out = ret;
@@ -288,11 +305,15 @@ err_t qbuffer_create(qbuffer_t** out)
   err_t err;
 
   ret = (qbuffer_t*) qio_malloc(sizeof(qbuffer_t));
-  if( ! ret ) return ENOMEM;
+  if( ! ret ) {
+    *out = NULL;
+    return ENOMEM;
+  }
 
   err = qbuffer_init(ret);
   if( err ) {
     qio_free(ret);
+    *out = NULL;
     return err;
   }
 
@@ -850,21 +871,31 @@ err_t qbuffer_flatten(qbuffer_t* buf, qbuffer_iter_t start, qbuffer_iter_t end,
   MAYBE_STACK_SPACE(struct iovec, iov_onstack);
   err_t err;
  
-  if( num_bytes < 0 || num_parts < 0 || start.offset < buf->offset_start || end.offset > buf->offset_end ) return EINVAL;
+  if( num_bytes < 0 || num_parts < 0 || start.offset < buf->offset_start || end.offset > buf->offset_end ) {
+    *bytes_out = 0;
+    return EINVAL;
+  }
 
   err = qbytes_create_calloc(&ret, num_bytes);
-  if( err ) return err;
+  if( err ) {
+    *bytes_out = 0;
+    return err;
+  }
 
   MAYBE_STACK_ALLOC(struct iovec, num_parts, iov, iov_onstack);
   if( ! iov ) {
+    // The buffer was successfully allocated, so we have to release it here.
     qbytes_release(ret);
+    *bytes_out = 0;
     return ENOMEM;
   }
 
   err = qbuffer_to_iov(buf, start, end, num_parts, iov, NULL, &iovcnt);
   if( err ) {
     MAYBE_STACK_FREE(iov, iov_onstack);
+    // The buffer was successfully allocated, so we have to release it here.
     qbytes_release(ret);
+    *bytes_out = 0;
     return err;
   }
 
