@@ -1,5 +1,5 @@
 /**************************************************************************
-  Copyright (c) 2004-2013, Cray Inc.  (See LICENSE file for more details)
+  Copyright (c) 2004-2014, Cray Inc.  (See LICENSE file for more details)
 **************************************************************************/
 
 
@@ -20,6 +20,7 @@
 #include "chpl-locale-model.h"
 #include "chpl-mem.h"
 #include "chpl-tasks.h"
+#include "chplsys.h"
 #include "error.h"
 #include <stdio.h>
 #include <string.h>
@@ -37,7 +38,6 @@
 typedef struct task_pool_struct* task_pool_p;
 
 typedef struct {
-  c_sublocid_t requestedSubloc;  // requested sublocal for task
   chpl_bool    serial_state;     // true: serialize execution
 } task_private_data_t;
 
@@ -71,6 +71,17 @@ struct chpl_task_list {
 };
 
 
+//
+// This is a descriptor for movedTaskWrapper().
+//
+typedef struct {
+  chpl_fn_p fp;
+  void* arg;
+  chpl_bool countRunning;
+  task_private_data_t chpl_data;
+} movedTaskWrapperDesc_t;
+
+
 typedef struct lockReport {
   const char*        filename;
   int                lineno;
@@ -89,6 +100,8 @@ typedef struct {
 
 static chpl_bool        initialized = false;
 
+static volatile chpl_bool canCountRunningTasks = false;
+
 static chpl_thread_mutex_t threading_lock;     // critical section lock
 static chpl_thread_mutex_t extra_task_lock;    // critical section lock
 static chpl_thread_mutex_t task_id_lock;       // critical section lock
@@ -111,17 +124,19 @@ static int                 idle_thread_cnt;    // number of threads looking
 static uint64_t            progress_cnt;       // number of unblock operations,
                                                //   as a proxy for progress
 
-static chpl_thread_mutex_t block_report_lock;  // critical section lock
-static chpl_thread_mutex_t taskTable_lock;     // critical section lock
-
+static chpl_thread_mutex_t block_report_lock;   // critical section lock
 static lockReport_t* lockReportHead = NULL;
 static lockReport_t* lockReportTail = NULL;
 
+static chpl_bool do_taskReport = false;
+static chpl_thread_mutex_t taskTable_lock;     // critical section lock
+
 static chpl_string idleTaskName = "|idle|";
 
 static chpl_fn_p comm_task_fn;
 
 static void                    comm_task_wrapper(void*);
+static void                    movedTaskWrapper(void* a);
 static chpl_taskID_t           get_next_task_id(void);
 static thread_private_data_t*  get_thread_private_data(void);
 static task_pool_p             get_current_ptask(void);
@@ -144,44 +159,71 @@ static task_pool_p             add_to_task_pool(chpl_fn_p,
                                                 task_private_data_t,
                                                 chpl_task_list_p);
 
+//
+// Condition variable methods
+//
+static void chpl_thread_condvar_init(chpl_thread_condvar_t* cv);
+
+//
+// Sync variable methods
+//
+static chpl_bool chpl_thread_sync_suspend(chpl_sync_aux_t *s,
+                                   struct timeval *deadline);
+static void chpl_thread_sync_awaken(chpl_sync_aux_t *s);
 
 // Sync variables
 
 static void sync_wait_and_lock(chpl_sync_aux_t *s,
                                chpl_bool want_full,
                                int32_t lineno, chpl_string filename) {
+  chpl_bool suspend_using_cond;
+
   chpl_thread_mutexLock(&s->lock);
 
+  // If true, we want to use conditionals, otherwise we want to spin
+  suspend_using_cond = (chpl_thread_getNumThreads() >=
+                        chpl_numCoresOnThisLocale());
+
   while (s->is_full != want_full) {
-    chpl_thread_mutexUnlock(&s->lock);
-
-    while (s->is_full != want_full) {
-      if (set_block_loc(lineno, filename)) {
-        // all other tasks appear to be blocked
-        struct timeval deadline, now;
-        gettimeofday(&deadline, NULL);
-        deadline.tv_sec += 1;
-        do {
-          chpl_thread_yield();
-          if (s->is_full != want_full)
-            gettimeofday(&now, NULL);
-        } while (s->is_full != want_full
-                 && (now.tv_sec < deadline.tv_sec
-                     || (now.tv_sec == deadline.tv_sec
-                         && now.tv_usec < deadline.tv_usec)));
-        if (s->is_full != want_full)
-          check_for_deadlock();
-      }
-      else {
-        do {
-          chpl_thread_yield();
-        } while (s->is_full != want_full);
-      }
-
-      unset_block_loc();
+    if (!suspend_using_cond) {
+      chpl_thread_mutexUnlock(&s->lock);
     }
+    if (set_block_loc(lineno, filename)) {
+      // all other tasks appear to be blocked
+      struct timeval deadline, now;
+      chpl_bool timed_out = false;
+      // default value so that always allows condition to be true if not
+      // using conditionals
 
-    chpl_thread_mutexLock(&s->lock);
+      gettimeofday(&deadline, NULL);
+      deadline.tv_sec += 1;
+      do {
+        if (suspend_using_cond)
+          timed_out = chpl_thread_sync_suspend(s, &deadline);
+        else
+          chpl_thread_yield();
+        
+        if (s->is_full != want_full && !timed_out)
+          gettimeofday(&now, NULL);
+      } while (s->is_full != want_full
+               && !timed_out
+               && (now.tv_sec < deadline.tv_sec
+                   || (now.tv_sec == deadline.tv_sec
+                       && now.tv_usec < deadline.tv_usec)));
+      if (s->is_full != want_full)
+        check_for_deadlock();
+    }
+    else {
+      do {
+        if (suspend_using_cond)
+          (void) chpl_thread_sync_suspend(s, NULL);
+        else
+          chpl_thread_yield();
+      } while (s->is_full != want_full);
+    }
+    unset_block_loc();
+    if (!suspend_using_cond)
+      chpl_thread_mutexLock(&s->lock);
   }
 
   if (blockreport)
@@ -206,13 +248,39 @@ void chpl_sync_waitEmptyAndLock(chpl_sync_aux_t *s,
   sync_wait_and_lock(s, false, lineno, filename);
 }
 
+static chpl_bool chpl_thread_sync_suspend(chpl_sync_aux_t *s,
+                                   struct timeval *deadline) {
+  chpl_thread_condvar_t* cond;
+  cond = s->is_full ? &s->signal_empty : &s->signal_full;
+
+  if (deadline == NULL) {
+    (void) pthread_cond_wait(cond, (pthread_mutex_t*) &s->lock);
+    return false;
+  }
+  else {
+    struct timespec ts;
+    ts.tv_sec  = deadline->tv_sec;
+    ts.tv_nsec = deadline->tv_usec * 1000UL;
+    return (pthread_cond_timedwait(cond, (pthread_mutex_t*) &s->lock, &ts)
+            == ETIMEDOUT);
+  }
+}
+
+static void chpl_thread_sync_awaken(chpl_sync_aux_t *s) {
+  if (pthread_cond_signal(s->is_full ?
+                          &s->signal_full : &s->signal_empty))
+    chpl_internal_error("pthread_cond_signal() failed");
+}
+
 void chpl_sync_markAndSignalFull(chpl_sync_aux_t *s) {
   s->is_full = true;
+  chpl_thread_sync_awaken(s);
   chpl_sync_unlock(s);
 }
 
 void chpl_sync_markAndSignalEmpty(chpl_sync_aux_t *s) {
   s->is_full = false;
+  chpl_thread_sync_awaken(s);
   chpl_sync_unlock(s);
 }
 
@@ -221,14 +289,20 @@ chpl_bool chpl_sync_isFull(void *val_ptr,
   return s->is_full;
 }
 
+static void chpl_thread_condvar_init(chpl_thread_condvar_t* cv) {
+  if (pthread_cond_init((pthread_cond_t*) cv, NULL))
+    chpl_internal_error("pthread_cond_init() failed");
+}
+
 void chpl_sync_initAux(chpl_sync_aux_t *s) {
   s->is_full = false;
   chpl_thread_mutexInit(&s->lock);
+  chpl_thread_condvar_init(&s->signal_full);
+  chpl_thread_condvar_init(&s->signal_empty);
 }
 
 void chpl_sync_destroyAux(chpl_sync_aux_t *s) { }
 
-
 // Tasks
 
 void chpl_task_init(void) {
@@ -246,15 +320,6 @@ void chpl_task_init(void) {
 
   chpl_thread_init(thread_begin, thread_end);
 
-  if (taskreport) {
-    chpl_thread_mutexInit(&taskTable_lock);
-  }
-
-  if (blockreport) {
-    progress_cnt = 0;
-    chpl_thread_mutexInit(&block_report_lock);
-  }
-
   //
   // Set main thread private data, so that things that require access
   // to it, like chpl_task_getID() and chpl_task_setSerial(), can be
@@ -286,12 +351,17 @@ void chpl_task_init(void) {
     tp->lockRprt            = NULL;
 
     // Set up task-private data for locale (architectural) support.
-    tp->ptask->chpl_data.requestedSubloc = c_sublocid_any;
     tp->ptask->chpl_data.serial_state = true;     // Set to false in chpl_task_callMain().
 
     chpl_thread_setPrivateData(tp);
   }
 
+  if (blockreport) {
+    progress_cnt = 0;
+    chpl_thread_mutexInit(&block_report_lock);
+    initializeLockReportForThread();
+  }
+
   if (blockreport || taskreport) {
     signal(SIGINT, SIGINT_handler);
   }
@@ -315,9 +385,23 @@ void chpl_task_callMain(void (*chpl_main)(void)) {
 
 void chpl_task_stdModulesInitialized(void) {
   //
-  // Register this main task in the task table.  We couldn't do this
-  // at creation time because the standard modules had not yet been
-  // initialized and therefore the task table didn't exist yet.
+  // It's not safe to call the module code to count the main task as
+  // running until after the modules have been initialized.
+  //
+  canCountRunningTasks = true;
+  chpl_taskRunningCntInc(0, NULL);
+
+  //
+  // The task table is implemented in Chapel code in the modules, so
+  // we can't use it, and thus can't support task reporting on ^C or
+  // deadlock, until the other modules on which it depends have been
+  // initialized and the supporting code here is set up.  In this
+  // function we're guaranteed that is true, because it is called only
+  // after all the standard module initialization is complete.
+  //
+
+  //
+  // Register this main task in the task table.
   //
   if (taskreport) {
     thread_private_data_t* tp = chpl_thread_getPrivateData();
@@ -326,11 +410,14 @@ void chpl_task_stdModulesInitialized(void) {
                           tp->ptask->lineno, tp->ptask->filename,
                           (uint64_t) (intptr_t) tp->ptask);
     chpldev_taskTable_set_active(tp->ptask->id);
+
+    chpl_thread_mutexInit(&taskTable_lock);
   }
 
-  if (blockreport) {
-    initializeLockReportForThread();
-  }
+  //
+  // Now we can do task reporting if the user requested it.
+  //
+  do_taskReport = taskreport;
 }
 
 
@@ -361,7 +448,7 @@ static void comm_task_wrapper(void* arg) {
 
   //
   // The comm (polling) task shouldn't really need this information.
-  tp->ptask->chpl_data.requestedSubloc = c_sublocid_any;
+  //
   tp->ptask->chpl_data.serial_state = true;
 
   tp->lockRprt = NULL;
@@ -379,11 +466,9 @@ void chpl_task_addToTaskList(chpl_fn_int_t fid, void* arg,
                              chpl_bool is_begin_stmt,
                              int lineno,
                              chpl_string filename) {
-  task_private_data_t chpl_data = { subloc, chpl_task_getSerial() };
+  task_private_data_t chpl_data = { chpl_task_getSerial() };
 
-  assert(subloc == 0
-         || subloc == c_sublocid_any
-         || subloc == c_sublocid_curr);
+  assert(subloc == 0 || subloc == c_sublocid_any);
 
   if (task_list_locale == chpl_nodeID) {
     chpl_task_list_p ltask;
@@ -490,7 +575,7 @@ void chpl_task_processTaskList(chpl_task_list_p task_list) {
 
     set_current_ptask(&nested_task);
 
-    if (taskreport) {
+    if (do_taskReport) {
       chpl_thread_mutexLock(&taskTable_lock);
       chpldev_taskTable_add(nested_task.id,
                             nested_task.lineno, nested_task.filename,
@@ -521,7 +606,7 @@ void chpl_task_processTaskList(chpl_task_list_p task_list) {
     // end critical section
     chpl_thread_mutexUnlock(&extra_task_lock);
 
-    if (taskreport) {
+    if (do_taskReport) {
       chpl_thread_mutexLock(&taskTable_lock);
       chpldev_taskTable_set_active(curr_ptask->id);
       chpldev_taskTable_remove(nested_task.id);
@@ -607,7 +692,7 @@ void chpl_task_executeTasksInList(chpl_task_list_p task_list) {
         // end critical section
         chpl_thread_mutexUnlock(&extra_task_lock);
 
-        if (taskreport) {
+        if (do_taskReport) {
           chpl_thread_mutexLock(&taskTable_lock);
           chpldev_taskTable_set_suspended(curr_ptask->id);
           chpldev_taskTable_set_active(nested_ptask->id);
@@ -619,7 +704,7 @@ void chpl_task_executeTasksInList(chpl_task_list_p task_list) {
 
         (*task_to_run_fun)(task_to_run_arg);
 
-        if (taskreport) {
+        if (do_taskReport) {
           chpl_thread_mutexLock(&taskTable_lock);
           chpldev_taskTable_set_active(curr_ptask->id);
           chpldev_taskTable_remove(nested_ptask->id);
@@ -668,15 +753,23 @@ void chpl_task_startMovedTask(chpl_fn_p fp,
                               c_sublocid_t subloc,
                               chpl_taskID_t id,
                               chpl_bool serial_state) {
-  task_private_data_t chpl_data = { subloc, serial_state };
+  movedTaskWrapperDesc_t* pmtwd;
 
   assert(subloc == 0 || subloc == c_sublocid_any);
   assert(id == chpl_nullTaskID);
 
+  pmtwd = (movedTaskWrapperDesc_t*)
+          chpl_mem_alloc(sizeof(*pmtwd),
+                         CHPL_RT_MD_THREAD_PRIVATE_DATA,
+                         0, 0);
+  *pmtwd = (movedTaskWrapperDesc_t)
+           { fp, a, canCountRunningTasks,
+             (task_private_data_t) { serial_state } };
+
   // begin critical section
   chpl_thread_mutexLock(&threading_lock);
 
-  (void) add_to_task_pool(fp, a, chpl_data, NULL);
+  (void) add_to_task_pool(movedTaskWrapper, pmtwd, pmtwd->chpl_data, NULL);
   schedule_next_task(1);
 
   // end critical section
@@ -684,24 +777,30 @@ void chpl_task_startMovedTask(chpl_fn_p fp,
 }
 
 
-c_sublocid_t chpl_task_getSubloc(void) {
-  return 0;
+static void movedTaskWrapper(void* a) {
+  movedTaskWrapperDesc_t* pmtwd = (movedTaskWrapperDesc_t*) a;
+  if (pmtwd->countRunning)
+    chpl_taskRunningCntInc(0, NULL);
+  (pmtwd->fp)(pmtwd->arg);
+  if (pmtwd->countRunning)
+    chpl_taskRunningCntDec(0, NULL);
+  chpl_mem_free(pmtwd, 0, 0);
 }
 
 
-void chpl_task_setSubloc(c_sublocid_t subloc) {
-  assert(subloc == 0
-         || subloc == c_sublocid_any
-         || subloc == c_sublocid_curr);
-  get_current_ptask()->chpl_data.requestedSubloc = subloc;
-}
+//
+// chpl_task_getSubloc() is in tasks-fifo.h.
+//
 
 
-c_sublocid_t chpl_task_getRequestedSubloc(void) {
-  if (!initialized)
-    return c_sublocid_any;
-  return get_current_ptask()->chpl_data.requestedSubloc;
-}
+//
+// chpl_task_setSubloc() is in tasks-fifo.h.
+//
+
+
+//
+// chpl_task_getRequestedSubloc() is in tasks-fifo.h.
+//
 
 
 chpl_taskID_t chpl_task_getId(void) {
@@ -734,26 +833,15 @@ c_sublocid_t chpl_task_getNumSublocales(void) {
 #endif
 }
 
-uint64_t chpl_task_getCallStackSize(void) {
+size_t chpl_task_getCallStackSize(void) {
   return chpl_thread_getCallStackSize();
 }
 
 uint32_t chpl_task_getNumQueuedTasks(void) { return queued_task_cnt; }
 
 uint32_t chpl_task_getNumRunningTasks(void) {
-  int numRunningTasks;
-
-  // begin critical section
-  chpl_thread_mutexLock(&threading_lock);
-  chpl_thread_mutexLock(&extra_task_lock);
-
-  numRunningTasks = running_task_cnt + extra_task_cnt;
-
-  // end critical section
-  chpl_thread_mutexUnlock(&extra_task_lock);
-  chpl_thread_mutexUnlock(&threading_lock);
-
-  return numRunningTasks;
+  chpl_internal_error("chpl_task_getNumRunningTasks() called");
+  return 1;
 }
 
 int32_t  chpl_task_getNumBlockedTasks(void) {
@@ -774,7 +862,7 @@ int32_t  chpl_task_getNumBlockedTasks(void) {
     return numBlockedTasks;
   }
   else
-    return -1;
+    return 0;
 }
 
 
@@ -888,7 +976,7 @@ static void SIGINT_handler(int sig) {
   if (blockreport)
     report_locked_threads();
 
-  if (taskreport)
+  if (do_taskReport)
     report_all_tasks();
 
   chpl_exit_any(1);
@@ -1009,7 +1097,7 @@ static void check_for_deadlock(void) {
 
   report_locked_threads();
 
-  if (taskreport)
+  if (do_taskReport)
     report_all_tasks();
 
   chpl_exit_any(1);
@@ -1036,7 +1124,7 @@ thread_begin(void* ptask_void) {
     initializeLockReportForThread();
 
   while (true) {
-    if (taskreport) {
+    if (do_taskReport) {
       chpl_thread_mutexLock(&taskTable_lock);
       chpldev_taskTable_set_active(ptask->id);
       chpl_thread_mutexUnlock(&taskTable_lock);
@@ -1044,7 +1132,7 @@ thread_begin(void* ptask_void) {
 
     (*ptask->fun)(ptask->arg);
 
-    if (taskreport) {
+    if (do_taskReport) {
       chpl_thread_mutexLock(&taskTable_lock);
       chpldev_taskTable_remove(ptask->id);
       chpl_thread_mutexUnlock(&taskTable_lock);
@@ -1076,9 +1164,19 @@ thread_begin(void* ptask_void) {
     //
     // wait for a not-yet-begun task to be present in the task pool
     //
+
+    // In revision 22137, we investigated whether it was beneficial to
+    // implement this while loop in a hybrid style, where depending on
+    // the number of tasks available, idle threads would either yield or
+    // wait on a condition variable to waken them.  Through analysis, we
+    // realized this could potential create a case where a thread would
+    // become stranded, waiting for a condition signal that would never
+    // come.  A potential solution to this was to keep a count of threads
+    // that were waiting on the signal, but since there was a performance
+    // impact from keeping it as a hybrid as opposed to merely yielding,
+    // it was decided that we would return to the simple yield case.
     while (!task_pool_head) {
       chpl_thread_mutexUnlock(&threading_lock);
-
       while (!task_pool_head) {
         if (set_block_loc(0, idleTaskName)) {
           // all other tasks appear to be blocked
@@ -1093,21 +1191,22 @@ thread_begin(void* ptask_void) {
                    && (now.tv_sec < deadline.tv_sec
                        || (now.tv_sec == deadline.tv_sec
                            && now.tv_usec < deadline.tv_usec)));
-          if (!task_pool_head)
+          if (!task_pool_head) {
             check_for_deadlock();
+          }
         }
         else {
           do {
-            chpl_thread_yield();
+            chpl_thread_yield();              
           } while (!task_pool_head);
         }
 
         unset_block_loc();
       }
-
       chpl_thread_mutexLock(&threading_lock);
     }
 
+
     if (blockreport)
       progress_cnt++;
 
@@ -1320,7 +1419,7 @@ static task_pool_p add_to_task_pool(chpl_fn_p fp,
 
   queued_task_cnt++;
 
-  if (taskreport) {
+  if (do_taskReport) {
     chpl_thread_mutexLock(&taskTable_lock);
     chpldev_taskTable_add(ptask->id,
                           ptask->lineno, ptask->filename,
