@@ -1,6 +1,6 @@
 /*   $Source: /var/local/cvs/gasnet/dcmf-conduit/gasnet_coll_barrier_dcmf.h,v $
- *     $Date: 2009/10/28 03:01:34 $
- * $Revision: 1.5 $
+ *     $Date: 2013/06/07 19:05:05 $
+ * $Revision: 1.6 $
  * Description:  GASNet team barrier implementation on DCMF
  * Copyright 2009, E. O. Lawrence Berekely National Laboratory
  * Terms of use are as specified in license.txt
@@ -20,6 +20,13 @@ typedef enum {
   G_DCMF_BARRIER_PROTO_NUM
 } gasnete_dcmf_barrier_proto_t;
 
+/* Barrier state */
+typedef enum {
+  GASNETE_COLL_BARRIER_IDLE = 0,
+  GASNETE_COLL_BARRIER_BUSY,
+  GASNETE_COLL_BARRIER_DONE
+} gasnete_dcmf_barrier_state_t;
+
 extern DCMF_CollectiveProtocol_t *gasnete_dcmf_barriers[G_DCMF_BARRIER_PROTO_NUM];
 extern int gasnete_dcmf_num_barriers; /**< num. of available barrier protocols */
 
