@@ -1,6 +1,6 @@
 /*   $Source: /var/local/cvs/gasnet/extended-ref/gasnet_coll_internal.h,v $
- *     $Date: 2012/10/12 23:46:45 $
- * $Revision: 1.68 $
+ *     $Date: 2013/06/19 21:30:02 $
+ * $Revision: 1.75 $
  * Description: GASNet Collectives conduit header
  * Copyright 2002, Dan Bonachea <bonachea@cs.berkeley.edu>
  * Terms of use are as specified in license.txt
@@ -220,28 +220,8 @@ extern int gasnete_coll_consensus_wait(gasnete_coll_team_t team GASNETE_THREAD_F
 typedef void (*gasnete_all_barrier_notify)(gasnete_coll_team_t team, int id, int flags);
 typedef int (*gasnete_all_barrier_wait)(gasnete_coll_team_t team, int id, int flags);
 typedef int (*gasnete_all_barrier_try)(gasnete_coll_team_t team, int id, int flags);
-
-/*XXX: TEMPORARY ONLY*/
-/* If the conduit hasn't defined team barrier define it here*/
-#ifndef gasnete_coll_teambarrier
-#define gasnete_coll_teambarrier(TEAM) do {\
-    gasnete_coll_barrier_notify(TEAM, 0, GASNET_BARRIERFLAG_ANONYMOUS GASNETE_THREAD_GET); \
-    gasnete_coll_barrier_wait(TEAM, 0, GASNET_BARRIERFLAG_ANONYMOUS GASNETE_THREAD_GET); \
-  } while(0)
-#endif
-
-#ifndef gasnete_coll_teambarrier_notify
-#define gasnete_coll_teambarrier_notify(TEAM) do {\
-    gasnete_coll_barrier_notify(TEAM, 0, GASNET_BARRIERFLAG_ANONYMOUS GASNETE_THREAD_GET); \
-  } while(0)
-#endif
-
-#ifndef gasnete_coll_teambarrier_wait
-#define gasnete_coll_teambarrier_wait(TEAM) do {\
-    gasnete_coll_barrier_wait(TEAM, 0, GASNET_BARRIERFLAG_ANONYMOUS GASNETE_THREAD_GET); \
-  } while(0)
-#endif
-
+typedef int (*gasnete_all_barrier)(gasnete_coll_team_t team, int id, int flags);
+typedef int (*gasnete_all_barrier_result)(gasnete_coll_team_t team, int *id);
 
 typedef enum {
   GASNETE_COLL_BARRIER_ENVDEFAULT=0,
@@ -327,6 +307,8 @@ struct gasnete_coll_team_t_ {
   gasnete_all_barrier_notify barrier_notify;
   gasnete_all_barrier_try barrier_try;
   gasnete_all_barrier_wait barrier_wait;
+  gasnete_all_barrier barrier;
+  gasnete_all_barrier_result barrier_result;
   gasneti_progressfn_t barrier_pf;
   
 #ifndef GASNETE_COLL_P2P_OVERRIDE
@@ -345,7 +327,7 @@ struct gasnete_coll_team_t_ {
   GASNETE_COLL_TEAM_EXTRA
 #endif
 
-#ifdef GASNET_FCA_ENABLED
+#ifdef GASNETI_USE_FCA
   fca_comm_data_t fca_comm_data;
   int use_fca;
 #endif
