@@ -1,6 +1,6 @@
 /*   $Source: /var/local/cvs/gasnet/gasnet_diagnostic.c,v $
- *     $Date: 2012/08/14 22:11:37 $
- * $Revision: 1.38 $
+ *     $Date: 2013/08/06 07:28:37 $
+ * $Revision: 1.44 $
  * Description: GASNet internal diagnostics
  * Copyright 2002, Dan Bonachea <bonachea@cs.berkeley.edu>
  * Terms of use are as specified in license.txt
@@ -115,7 +115,7 @@ extern int gasneti_run_diagnostics(int iter_cnt, int threadcnt, const char *test
   if (partner == gasnet_nodes()) partner = gasnet_mynode();
   test_errs = 0;
   iters = iter_cnt;
-  iters2 = iters*100;
+  iters2 = (iters <= INT_MAX/100) ? iters*100 : iters;
   iters0 = MAX(1,iters/100);
   peer = gasnet_mynode()^1;
   if (peer == gasnet_nodes()) peer = gasnet_mynode();
@@ -233,10 +233,10 @@ static void malloc_test(int id) {
 
   PTHREAD_BARRIER(num_threads);
   
-  maxobjs = MIN(iters,10000/num_threads);
+  maxobjs = MIN(iters0,10000/num_threads);
   ptrs = gasneti_calloc(maxobjs,sizeof(void*));
   for (i = 0; i < maxobjs; i++) assert_always(ptrs[i] == NULL);
-  for (i = 0; i < iters2/num_threads; i++) {
+  for (i = 0; i < iters/num_threads; i++) {
     gasneti_memcheck_one();
     if (cnt == maxobjs || (cnt > 0 && TEST_RAND_ONEIN(2))) {
       size_t idx = TEST_RAND(0,cnt-1);
@@ -371,6 +371,7 @@ static void mutex_test(int id) {
   static gasneti_mutex_t lock1 = GASNETI_MUTEX_INITIALIZER;
   static gasneti_mutex_t lock2;
   static unsigned int counter;
+  unsigned int count = iters2 / num_threads;
   int i;
 
   PTHREAD_BARRIER(num_threads);
@@ -401,7 +402,7 @@ static void mutex_test(int id) {
 
   PTHREAD_BARRIER(num_threads);
 
-    for (i=0;i<iters2;i++) {
+    for (i=0;i<count;i++) {
       if (i&1) {
         gasneti_mutex_lock(&lock1);
       } else {
@@ -416,8 +417,8 @@ static void mutex_test(int id) {
 
   PTHREAD_BARRIER(num_threads);
 
-    if (counter != (num_threads * iters2)) 
-      ERR("failed mutex test: counter=%i expecting=%i", counter, (num_threads * iters2));
+    if (counter != (num_threads * count)) 
+      ERR("failed mutex test: counter=%i expecting=%i", counter, (num_threads * count));
 
   PTHREAD_BARRIER(num_threads);
 }
@@ -427,6 +428,7 @@ static void spinlock_test(int id) {
   static gasneti_atomic_t lock1 = GASNETI_SPINLOCK_INITIALIZER;
   static gasneti_atomic_t lock2;
   static unsigned int counter;
+  unsigned int count = iters2 / num_threads;
   int i;
 
   PTHREAD_BARRIER(num_threads);
@@ -450,7 +452,7 @@ static void spinlock_test(int id) {
 
   PTHREAD_BARRIER(num_threads);
 
-    for (i=0;i<iters2;i++) {
+    for (i=0;i<count;i++) {
       if (i&1) {
         gasneti_spinlock_lock(&lock1);
       } else {
@@ -465,8 +467,8 @@ static void spinlock_test(int id) {
 
   PTHREAD_BARRIER(num_threads);
 
-    if (counter != (num_threads * iters2)) 
-      ERR("failed spinlock test: counter=%i expecting=%i", counter, (num_threads * iters2));
+    if (counter != (num_threads * count)) 
+      ERR("failed spinlock test: counter=%i expecting=%i", counter, (num_threads * count));
 
   PTHREAD_BARRIER(num_threads);
 }
@@ -480,7 +482,8 @@ static void semaphore_test(int id) {
   static gasneti_semaphore_t sema1 = GASNETI_SEMAPHORE_INITIALIZER(GASNETI_SEMAPHORE_MAX,0);
   static gasneti_semaphore_t sema2;
   static gasneti_atomic_t counter;
-  gasneti_atomic_val_t limit = MIN(1000000, num_threads * iters2);
+  int count = iters2 / num_threads;
+  gasneti_atomic_val_t limit = MIN(1000000, num_threads * count);
   int i;
 
   PTHREAD_BARRIER(num_threads);
@@ -512,7 +515,7 @@ static void semaphore_test(int id) {
 
   PTHREAD_BARRIER(num_threads);
 
-    for (i=0;i<iters2;i++) {
+    for (i=0;i<count;i++) {
       if (gasneti_semaphore_trydown(&sema1))
         gasneti_semaphore_up(&sema1);
     }
@@ -590,7 +593,7 @@ static void atomic128_test(int id) {
   }
 
   gasneti_atomic128_set(var128, iters, 0, 0);
-  for (i=0;i<=iters;i++) {
+  for (i=0;i<iters;i++) {
     if (gasneti_atomic128_compare_and_swap(var128, iters+i-1, i-1, iters+i-2, i-2, 0))
       ERR("gasneti_atomic128_compare_and_swap succeeded at i=%i when it should have failed", i);
     if (gasneti_atomic128_compare_and_swap(var128, iters+i+1, i+1, iters+i-2, i-2, 0))
@@ -620,7 +623,7 @@ static void atomicswap_test(int id) {
 
   TEST_HEADER("atomic SWAP test"); else return;
 
-  limit = num_threads * MIN(GASNETI_ATOMIC_MAX/num_threads, 65536);
+  limit = MIN(GASNETI_ATOMIC_MAX, 8192);
 
   if (0 == id) {
     gasneti_atomic_set(&var, GASNETI_ATOMIC_MAX, 0);
@@ -678,7 +681,8 @@ static void lifo_test(int id) {
   static gasneti_lifo_head_t lifo1 = GASNETI_LIFO_INITIALIZER;
   static gasneti_lifo_head_t lifo2;
   static gasneti_atomic_t counter;
-  int limit = MIN(1000000, num_threads * iters2);
+  int count = iters2 / num_threads;
+  int limit = MIN(1000000, num_threads * count);
   int i;
 
   PTHREAD_BARRIER(num_threads);
@@ -686,7 +690,7 @@ static void lifo_test(int id) {
 
   { 
     void * tmp = test_malloc(sizeof(void *));
-    for (i = 0; i < iters2; ++i) {
+    for (i = 0; i < count; ++i) {
       gasneti_lifo_push(&lifo1, tmp);
       tmp = gasneti_lifo_pop(&lifo1);
       if (tmp == NULL)
@@ -731,7 +735,7 @@ static void lifo_test(int id) {
 
   PTHREAD_BARRIER(num_threads);
 
-    for (i=0;i<iters2;i++) {
+    for (i=0;i<count;i++) {
       void * tmp = gasneti_lifo_pop(&lifo1);
       if (tmp != NULL) {
         gasneti_lifo_push(&lifo2, tmp);
@@ -749,7 +753,7 @@ static void lifo_test(int id) {
   {
     void * head = NULL;
 
-    for (i=0;i<iters2;i++) {
+    for (i=0;i<count;i++) {
       void * tmp = gasneti_lifo_pop(&lifo2);
       if (tmp != NULL) {
 	*(void **)tmp = head;
@@ -797,6 +801,10 @@ static void progressfn_tester(int *counter) {
   if (!iamactive) return;
 
   /* do some work that should be legal inside a progress fn */
+#if 0
+  if ((gasneti_weakatomic_read(&progressfn_req_sent,0) -
+       gasneti_weakatomic_read(&progressfn_rep_rcvd,0)) <= 128U)
+#endif
   { static int tmp = 47;
     int sz;
     gasnet_put_nbi(peer, peersegmid, &tmp, sizeof(tmp));
@@ -855,6 +863,7 @@ static void progressfns_test(int id) {
       gasnet_AMPoll();
     }
 
+    PTHREAD_BARRIER(num_threads);
     /* ensure they did run */
     cnt_c = pf_cnt_counted; cnt_b = pf_cnt_boolean;
     assert_always(cnt_c > 0); assert_always(cnt_b > 0);
