@@ -1,6 +1,6 @@
 /*   $Source: /var/local/cvs/gasnet/gasnet_help.h,v $
- *     $Date: 2012/08/21 14:35:30 $
- * $Revision: 1.113 $
+ *     $Date: 2013/08/27 01:23:44 $
+ * $Revision: 1.116 $
  * Description: GASNet Header Helpers (Internal code, not for client use)
  * Copyright 2002, Dan Bonachea <bonachea@cs.berkeley.edu>
  * Terms of use are as specified in license.txt
@@ -613,9 +613,12 @@ typedef void (*gasneti_progressfn_t)(void);
           lock contention and cache thrashing between spin-pollers
         - manage AMPoll calls internal to GASNet (including those from polluntil) 
           and explicit client AMPoll calls
+       Lowest priority design goals, which have not been achieved:
         - allow concurrent handler execution - if the spin poller recvs an AM, 
           it should release another spin poller before invoking the handler
+          [lock reacquisition becomes a bottleneck]
         - the single spin-poller should not need to pay locking overheads in the loop
+          [a limitation of the infrastructure: no way to differentiate "single"]
     */
     extern gasneti_atomic_t gasneti_throttle_haveusefulwork;
     extern gasneti_mutex_t gasneti_throttle_spinpoller;
@@ -631,17 +634,18 @@ typedef void (*gasneti_progressfn_t)(void);
 
     /* and finally, the throttled poll implementation */
     GASNETI_INLINE(gasneti_AMPoll)
-    int gasneti_AMPoll() {
-       int retval;
+    int gasneti_AMPoll(void) {
+       int retval = GASNET_OK;
        gasneti_AMPoll_spinpollers_check();
        gasneti_memcheck_one();
-       if (gasneti_atomic_read(&gasneti_throttle_haveusefulwork,0) > 0) 
-         return GASNET_OK; /* another thread sending - skip the poll */
-       if (gasneti_mutex_trylock(&gasneti_throttle_spinpoller) != 0)
-         return GASNET_OK; /* another thread spin-polling - skip the poll */
-       retval = gasnetc_AMPoll();
-       gasneti_mutex_unlock(&gasneti_throttle_spinpoller);
-       GASNETI_PROGRESSFNS_RUN();
+       /* if another thread is spin-polling then skip both the poll and progress fns: */
+       if_pt (!gasneti_mutex_trylock(&gasneti_throttle_spinpoller)) {
+          /* if another thread is sending then skip the poll: */
+          if_pt (!gasneti_atomic_read(&gasneti_throttle_haveusefulwork,0))
+             retval = gasnetc_AMPoll();
+          gasneti_mutex_unlock(&gasneti_throttle_spinpoller);
+          GASNETI_PROGRESSFNS_RUN();
+       }
        return retval;
     }
   #endif
