@@ -1,6 +1,6 @@
 /*   $Source: /var/local/cvs/gasnet/gasnet_internal.c,v $
- *     $Date: 2013/03/25 05:31:03 $
- * $Revision: 1.235 $
+ *     $Date: 2013/08/28 02:36:57 $
+ * $Revision: 1.239 $
  * Description: GASNet implementation of internal helpers
  * Copyright 2002, Dan Bonachea <bonachea@cs.berkeley.edu>
  * Terms of use are as specified in license.txt
@@ -296,25 +296,6 @@ extern void gasneti_defaultAMHandler(gasnet_token_t token) {
                      (int)gasnet_mynode(), (int)gasnet_nodes(), (int)srcnode);
 }
 /* ------------------------------------------------------------------------------------ */
-#define DEF_SIGNAL(name) { name, #name, NULL }
-static struct {
-  int signum;
-  const char *signame;
-  gasneti_sighandlerfn_t oldhandler;
-} gasneti_signals[] = {
-  /* abort signals */
-  DEF_SIGNAL(SIGABRT),
-  DEF_SIGNAL(SIGILL),
-  DEF_SIGNAL(SIGSEGV),
-  DEF_SIGNAL(SIGBUS),
-  DEF_SIGNAL(SIGFPE),
-  /* termination signals */
-  DEF_SIGNAL(SIGQUIT),
-  DEF_SIGNAL(SIGINT),
-  DEF_SIGNAL(SIGTERM),
-  DEF_SIGNAL(SIGHUP),
-  DEF_SIGNAL(SIGPIPE)
-};
 
 #ifndef GASNETC_FATALSIGNAL_CALLBACK
 #define GASNETC_FATALSIGNAL_CALLBACK(sig)
@@ -330,16 +311,9 @@ static void do_raise(int sig) {
 }
 
 void gasneti_defaultSignalHandler(int sig) {
-  gasneti_sighandlerfn_t oldhandler = NULL;
   gasneti_sighandlerfn_t oldsigpipe = NULL;
-  const char *signame = NULL;
-  int i;
-  for (i = 0; i < sizeof(gasneti_signals)/sizeof(gasneti_signals[0]); i++) {
-    if (gasneti_signals[i].signum == sig) {
-      oldhandler = gasneti_signals[i].oldhandler;
-      signame = gasneti_signals[i].signame;
-    }
-  }
+  const char *signame =  gasnett_signame_fromval(sig);
+
   gasneti_assert(signame);
 
   switch (sig) {
@@ -387,15 +361,6 @@ void gasneti_defaultSignalHandler(int sig) {
   }
 }
 
-void gasneti_registerSignalHandlers(gasneti_sighandlerfn_t handler) {
-  int i;
-  for (i = 0; i < sizeof(gasneti_signals)/sizeof(gasneti_signals[0]); i++) {
-    gasneti_signals[i].oldhandler = 
-      gasneti_reghandler(gasneti_signals[i].signum, handler);
-  }
-  gasneti_ondemand_init(); /* allow user override of signal handlers */
-}
-
 extern int gasneti_set_waitmode(int wait_mode) {
   const char *desc = NULL;
   GASNETI_CHECKINIT();
@@ -771,19 +736,8 @@ static void gasneti_check_portable_conduit(void) { /* check for portable conduit
           { "/dev/hw/cpunum",      S_IFDIR, "SGI Altix", 0 },
           { "/dev/xpmem",          S_IFCHR, "SGI Altix", 0 },
         #endif
-        #if PLATFORM_OS_AIX
-          { "/dev/nampd0",         S_IFCHR, "IBM LAPI", 1 }, /* could also run lslpp -l | grep lapi */
-        #endif
         { "/dev/infiniband/uverbs0", S_IFCHR, "InfiniBand IBV", 2 },  /* OFED 1.0 */
-        { "/dev/vipkl",          S_IFCHR, "InfiniBand VAPI", 2 },  /* Mellanox drivers */
-        { "/dev/ib_dsc",         S_IFCHR, "InfiniBand VAPI", 2 },  /* wotan - could also run system_profiler */
-        { "/dev/gm3",            S_IFCHR, "Myrinet", 3 }, /* could also look in /proc/devices and /proc/pci */
-        { "/dev/elan3/control0", S_IFCHR, "Quadrics QsNetI", 4 },
-        { "/dev/elan4/control0", S_IFCHR, "Quadrics QsNetII", 4 },
-        { "/proc/qsnet/version", S_IFREG, "Quadrics QsNet", 4 },
         #if !GASNET_SEGMENT_EVERYTHING
-          { "/dev/ukbridge",         S_IFCHR, "Cray Portals", 5 },
-          { "/proc/portals/meminfo", S_IFREG, "Cray Portals", 5 },
           { "/dev/kgni0",            S_IFCHR, "Cray Gemini", 6 },
           { "/proc/kgnilnd",         S_IFDIR, "Cray Gemini", 6 },
         #endif
@@ -804,9 +758,9 @@ static void gasneti_check_portable_conduit(void) { /* check for portable conduit
       #if PLATFORM_ARCH_CRAYX1
         if (strlen(natives)) strcat(natives,", ");
         strcat(natives,"Cray SHMEM (X1)");
-      #elif PLATFORM_OS_CATAMOUNT || PLATFORM_OS_CNL
+      #elif PLATFORM_OS_CNL
         if (strlen(natives)) strcat(natives,", ");
-        strcat(natives,"Cray Portals (XT) or Gemini (XE and XK)");
+        strcat(natives,"Cray Gemini (XE and XK) or Aries (XC)");
       #elif PLATFORM_OS_BGP
         if (strlen(natives)) strcat(natives,", ");
         strcat(natives,"IBM DCMF (BG/P)");
@@ -1082,7 +1036,7 @@ static void gasneti_nodemap_dflt(gasneti_bootstrapExchangefn_t exchangefn) {
  *
  * NOTE: may modify gasneti_nodemap[] if env var GASNET_SUPERNODE_MAXSIZE is set.
  * TODO: splitting by socket or other criteria for/with GASNET_SUPERNODE_MAXSIZE.
- * TODO: keep widths around for conduits to use? (vapi and gemini both use)
+ * TODO: keep widths around for conduits to use? (ibv and gemini both use)
  */
 extern void gasneti_nodemapParse(void) {
   gasnet_node_t host_count = 0;
