@@ -1,6 +1,6 @@
 /*   $Source: /var/local/cvs/gasnet/gasnet_membar.h,v $
- *     $Date: 2013/03/18 02:28:51 $
- * $Revision: 1.127 $
+ *     $Date: 2013/09/06 21:07:07 $
+ * $Revision: 1.129 $
  * Description: GASNet header for portable memory barrier operations
  * Copyright 2002, Dan Bonachea <bonachea@cs.berkeley.edu>
  * Terms of use are as specified in license.txt
@@ -140,15 +140,12 @@
    #define GASNETI_LOCAL_RMB_BODY	GASNETI_ASM_SPECIAL("lfence")
    #define GASNETI_LOCAL_MB_BODY	GASNETI_ASM_SPECIAL("mfence")
  #elif PLATFORM_COMPILER_CRAY
-   GASNETI_INLINE(_gasneti_compiler_fence)
-   void _gasneti_compiler_fence(void) {
-     static int volatile x;
-     x = 1;
-   }
-   #define gasneti_compiler_fence() _gasneti_compiler_fence()
-   #define gasneti_local_wmb()      __builtin_ia32_sfence()
+   #define gasneti_compiler_fence() _Pragma("_CRI suppress")
+   #define gasneti_local_wmb()      do { gasneti_compiler_fence(); \
+                                         __builtin_ia32_sfence(); } while (0)
    #define gasneti_local_rmb()      __builtin_ia32_lfence()
-   #define gasneti_local_mb()       __builtin_ia32_mfence()
+   #define gasneti_local_mb()       do { gasneti_compiler_fence(); \
+                                         __builtin_ia32_mfence(); } while (0)
  #else
    GASNETI_INLINE(gasneti_local_wmb)
    void gasneti_local_wmb(void) {
@@ -166,6 +163,12 @@
    #define gasneti_local_mb() _gasneti_local_mb()
  #endif
 /* ------------------------------------------------------------------------------------ */
+#elif PLATFORM_ARCH_MIC
+   GASNETI_INLINE(gasneti_local_wmb)
+   void gasneti_local_wmb(void) {
+     __asm__ __volatile__("":::"memory");
+   }
+/* ------------------------------------------------------------------------------------ */
 #elif PLATFORM_ARCH_IA64 /* Itanium */
     /* Empirically observed that IA64 requires a full "mf" for both wmb and rmb (see bug 1000).
      * The reason is that the Itanium memory model only ensures ordering in one direction when
