@@ -1,6 +1,6 @@
 /*   $Source: /var/local/cvs/gasnet/gasnet_mmap.c,v $
- *     $Date: 2013/03/25 04:32:04 $
- * $Revision: 1.124 $
+ *     $Date: 2013/09/25 02:16:26 $
+ * $Revision: 1.130.2.1 $
  * Description: GASNet memory-mapping utilities
  * Copyright 2002, Dan Bonachea <bonachea@cs.berkeley.edu>
  * Terms of use are as specified in license.txt
@@ -68,6 +68,8 @@
 #elif PLATFORM_OS_HPUX
   #define GASNETI_MMAP_FLAGS (MAP_ANONYMOUS | MAP_NORESERVE | MAP_PRIVATE)
   #define GASNETI_MMAP_NOTFIXED_FLAG MAP_VARIABLE
+#elif PLATFORM_ARCH_MIC
+  #define GASNETI_MMAP_FLAGS (MAP_ANONYMOUS | MAP_PRIVATE | MAP_POPULATE | MAP_HUGETLB)
 #endif
 
 #ifndef GASNETI_MMAP_FLAGS
@@ -100,8 +102,8 @@
   #define GASNETI_MMAP_NOTFIXED_FLAG 0
 #endif
 
-#if GASNET_PSHM && (PLATFORM_OS_BGP || PLATFORM_OS_CYGWIN)
-  /* BG/P: MAP_FIXED is ignored for fd obtained from pshm_open() */
+#if GASNET_PSHM && (PLATFORM_OS_BGP || PLATFORM_OS_BGQ || PLATFORM_OS_CYGWIN)
+  /* BG/P and /Q: MAP_FIXED is ignored for fd obtained from pshm_open() */
   /* CYGWIN: may not honor the address passed to shmat() */
   #define GASNETI_PSHM_MAP_FIXED_IGNORED 1
 #endif
@@ -123,9 +125,13 @@ static void *gasneti_mmap_internal(void *segbase, uintptr_t segsize) {
   #endif
 
   t1 = gasneti_ticks_now();
+#if defined(GASNETI_USE_HUGETLBFS)
+  ptr = gasneti_huge_mmap(segbase, segsize);
+#else
   ptr = mmap(segbase, segsize, (PROT_READ|PROT_WRITE), 
       (GASNETI_MMAP_FLAGS | (segbase==NULL?GASNETI_MMAP_NOTFIXED_FLAG:GASNETI_MMAP_FIXED_FLAG)), 
       gasneti_mmapfd, 0);
+#endif
   mmap_errno = errno;
   t2 = gasneti_ticks_now();
 
@@ -442,11 +448,44 @@ extern void gasneti_huge_munmap(void *addr, uintptr_t size) {
 #endif
 
 #if GASNET_PSHM
+
+#if defined(PLATFORM_OS_BGQ) /* must delay close until unmap() */
+  static struct {
+    void *addr;
+    int fd;
+  } *gasneti_addr2fd = NULL;
+  static void gasneti_addr2fd_add(void *addr, int fd) {
+    int i;
+    if (NULL == gasneti_addr2fd) {
+       gasneti_addr2fd = gasneti_calloc(1+gasneti_pshm_nodes, sizeof(*gasneti_addr2fd));
+    }
+    for (i=0; i<=gasneti_pshm_nodes; ++i) {
+      if (gasneti_addr2fd[i].addr == NULL) {
+        gasneti_addr2fd[i].addr = addr;
+        gasneti_addr2fd[i].fd = fd;
+        return;
+      }
+    }
+    gasneti_fatalerror("addr2fd table is full");
+  }
+  static int gasneti_addr2fd_del(void *addr) {
+    int i;
+    for (i=0; i<=gasneti_pshm_nodes; ++i) {
+      if (gasneti_addr2fd[i].addr == addr) {
+        gasneti_addr2fd[i].addr = NULL;
+        return gasneti_addr2fd[i].fd;
+      }
+    }
+    gasneti_fatalerror("address not found in addr2fd table");
+  }
+#endif /* BGQ */
+
 static void gasneti_pshm_unlink(int pshm_rank);
 
 /* create the object/region/segment and return its address */
 static void * gasneti_pshm_mmap(int pshm_rank, void *segbase, size_t segsize) {
-#if defined(PLATFORM_OS_BGP) /* pshm_unlink() is apparently a no-op on BG/P */
+#if defined(PLATFORM_OS_BGP) || defined(PLATFORM_OS_BGQ)
+  /* shm_unlink() is apparently a no-op on BG/P and earlier driver versions on /Q */
   const int create = ((pshm_rank == gasneti_pshm_nodes) && !gasneti_pshm_mynode);
 #else
   const int create = (pshm_rank == gasneti_pshm_mynode) ||
@@ -491,6 +530,13 @@ static void * gasneti_pshm_mmap(int pshm_rank, void *segbase, size_t segsize) {
           fd = shm_open(filename, flags, S_IRUSR | S_IWUSR);
         } while ((fd == -1) && (errno == EEXIST) && retries_remain--);
       }
+    #elif defined(PLATFORM_OS_BGQ)
+      /* XXX: configure or compile time way to know? */
+      if ((fd == -1) && (errno == ENOENT) && (pshm_rank == gasneti_pshm_mynode)) {
+        /* We didn't use O_CREAT+O_EXCL on the first try, because shm_unlink()
+         * does nothing prior to V1R2M1.  So, try again with those flags. */
+        fd = shm_open(filename, flags | O_CREAT | O_EXCL, S_IRUSR | S_IWUSR);
+      }
     #endif
   #else
     #error
@@ -506,6 +552,12 @@ static void * gasneti_pshm_mmap(int pshm_rank, void *segbase, size_t segsize) {
     ptr = mmap(segbase, segsize, (PROT_READ|PROT_WRITE), mmap_flags, fd, 0);
   }
 
+ #if defined(PLATFORM_OS_BGQ)
+  /* must delay close() untile just before munmap() */
+  if (ptr != MAP_FAILED) {
+    gasneti_addr2fd_add(ptr, fd);
+  } else
+ #endif
   {
     const int save_errno = errno;
     (void) close(fd);
@@ -658,6 +710,10 @@ static void gasneti_cleanup_shm(void) {
     gasneti_free(gasneti_pshmname);
     gasneti_pshmname = NULL;
   }
+  #if defined(PLATFORM_OS_BGQ)
+    gasneti_free(gasneti_addr2fd);
+    gasneti_addr2fd = NULL;
+  #endif
 #elif defined(GASNETI_PSHM_XPMEM)
   gasneti_free(gasneti_pshm_segids);
   gasneti_pshm_segids = NULL;
@@ -847,6 +903,10 @@ extern void gasneti_munmap(void *segbase, uintptr_t segsize) {
         gasneti_fatalerror("msync("GASNETI_LADDRFMT",%lu) failed: %s\n",
 	        GASNETI_LADDRSTR(segbase), (unsigned long)segsize, strerror(errno));
     #endif
+    #if GASNET_PSHM && defined(PLATFORM_OS_BGQ)
+      /* we delayed the close() until now */
+      close(gasneti_addr2fd_del(segbase));
+    #endif
     if (munmap(segbase, segsize) != 0) 
       gasneti_fatalerror("munmap("GASNETI_LADDRFMT",%lu) failed: %s\n",
 	      GASNETI_LADDRSTR(segbase), (unsigned long)segsize, strerror(errno));
@@ -1595,15 +1655,14 @@ void gasneti_segmentAttachLocal(uintptr_t segsize, uintptr_t minheapoffset,
 
 #if GASNET_PSHM
 static uintptr_t
-gasneti_AttachRemote(uintptr_t segsize, const gasnet_node_t pshm_node,
+gasneti_AttachRemote(const gasnet_node_t pshm_node,
                      uintptr_t minheapoffset, gasnet_seginfo_t *seginfo) {
   void *segbase = NULL;
   uintptr_t topofheap;
   gasnet_node_t node = gasneti_nodemap_local[pshm_node];
+  uintptr_t segsize = seginfo[node].size;
 
   gasneti_assert(seginfo);
-  gasneti_assert(gasneti_segexch);
-  gasneti_memcheck(gasneti_segexch);
   gasneti_assert(node != gasneti_mynode);
 
     topofheap = gasneti_myheapend;
@@ -1700,7 +1759,7 @@ void gasneti_segmentAttach(uintptr_t segsize, uintptr_t minheapoffset,
     gasneti_nodeinfo[gasneti_mynode].offset = 0;
     for(i=0; i<gasneti_pshm_nodes; i++){
       if (i == gasneti_pshm_mynode) continue;
-      ar = gasneti_AttachRemote(seginfo[i].size, i, minheapoffset, seginfo);
+      ar = gasneti_AttachRemote(i, minheapoffset, seginfo);
       if (ar) break;
     }
 
