@@ -21,6 +21,9 @@ GASNETI_IDENT(gasnetc_IdentString_HaveSSHSpawner, "$GASNetSSHSpawner: 1 $");
 #if HAVE_MPI_SPAWNER
 GASNETI_IDENT(gasnetc_IdentString_HaveMPISpawner, "$GASNetMPISpawner: 1 $");
 #endif
+#if HAVE_PMI_SPAWNER
+GASNETI_IDENT(gasnetc_IdentString_HavePMISpawner, "$GASNetPMISpawner: 1 $");
+#endif
 
 gasnet_handlerentry_t const *gasnetc_get_handlertable(void);
 #if HAVE_ON_EXIT
@@ -33,7 +36,7 @@ gasneti_handler_fn_t gasnetc_handler[GASNETC_MAX_NUMHANDLERS]; /* handler table
 
 gasnet_mxm_module_t   gasnet_mxm_module;
 
-//#define MXM_MUTEX_AMPOLL_LOCK
+/*#define MXM_MUTEX_AMPOLL_LOCK*/
 
 #ifndef GASNET_SEQ
 #ifndef MXM_MUTEX_AMPOLL_LOCK
@@ -45,7 +48,7 @@ gasnet_mxm_module_t   gasnet_mxm_module;
 #endif
 
 #ifdef MXM_MUTEX_AMPOLL_LOCK
-static gasneti_mutex_t gasnetc_ampoll_lock;
+static gasneti_mutex_t gasnetc_ampoll_lock = GASNETI_MUTEX_INITIALIZER;
 #endif
 #ifdef MXM_ATOMIC_AMPOLL_LOCK
 static gasneti_atomic_t gasnetc_ampoll_atomic_lock = gasneti_atomic_init(1);
@@ -119,6 +122,7 @@ void (*gasneti_bootstrapBarrier_p)(void) = NULL;
 void (*gasneti_bootstrapExchange_p)(void *src, size_t len, void *dest) = NULL;
 void (*gasneti_bootstrapAlltoall_p)(void *src, size_t len, void *dest) = NULL;
 void (*gasneti_bootstrapBroadcast_p)(void *src, size_t len, void *dest, int rootnode) = NULL;
+void (*gasneti_bootstrapCleanup_p)(void) = NULL;
 
 #if GASNET_TRACE
 static unsigned int	gasnetc_pinned_blocks = 0;
@@ -152,15 +156,17 @@ static void gasnetc_bootstrapBarrier(void) {
 }
 /* -------------------------------------------------------------------------- */
 
-static void gasneti_bootstrapInit(
+static int gasneti_bootstrapInit(
     int *argc_p, char ***argv_p,
     gasnet_node_t *nodes_p,
     gasnet_node_t *mynode_p)
 {
-    char *spawner = gasneti_getenv_withdefault("GASNET_SPAWNER", "(not set)");
-    if (!strcmp(spawner, "ssh") ||
-            ((*argc_p >= 2) && !strncmp((*argv_p)[1], "-GASNET-SPAWN-", 14))) {
+    char *spawner = gasneti_getenv_withdefault("GASNET_IB_SPAWNER", "(not set)");
+    int res = GASNET_ERR_NOT_INIT;
+
 #if HAVE_SSH_SPAWNER
+    /* Sigh.  We can't assume GASNET_IB_SPAWNER has been set except in the master */
+    if (GASNET_OK == (res = gasneti_bootstrapInit_ssh(argc_p, argv_p, nodes_p, mynode_p))) {
         gasneti_bootstrapInit_ssh(argc_p, argv_p, nodes_p, mynode_p);
         gasneti_bootstrapFini_p     = &gasneti_bootstrapFini_ssh;
         gasneti_bootstrapAbort_p    = &gasneti_bootstrapAbort_ssh;
@@ -168,11 +174,12 @@ static void gasneti_bootstrapInit(
         gasneti_bootstrapExchange_p = &gasneti_bootstrapExchange_ssh;
         gasneti_bootstrapAlltoall_p = &gasneti_bootstrapAlltoall_ssh;
         gasneti_bootstrapBroadcast_p= &gasneti_bootstrapBroadcast_ssh;
-#else
-        gasneti_fatalerror("Requested ssh-spawner is not supported in this build");
+        gasneti_bootstrapCleanup_p  = &gasneti_bootstrapCleanup_ssh;
+    } else
 #endif
-    } else {
 #if HAVE_MPI_SPAWNER
+    if (!strcmp(spawner, "mpi")) {
+        res = gasneti_bootstrapInit_mpi(argc_p, argv_p, nodes_p, mynode_p);
         gasneti_bootstrapInit_mpi(argc_p, argv_p, nodes_p, mynode_p);
         gasneti_bootstrapFini_p	= &gasneti_bootstrapFini_mpi;
         gasneti_bootstrapAbort_p	= &gasneti_bootstrapAbort_mpi;
@@ -180,20 +187,31 @@ static void gasneti_bootstrapInit(
         gasneti_bootstrapExchange_p	= &gasneti_bootstrapExchange_mpi;
         gasneti_bootstrapAlltoall_p	= &gasneti_bootstrapAlltoall_mpi;
         gasneti_bootstrapBroadcast_p= &gasneti_bootstrapBroadcast_mpi;
-#else
-        if (!strcmp(spawner, "mpi")) {
-            gasneti_fatalerror("Requested mpi-spawner is not supported in this build");
-        } else {
-            gasneti_fatalerror("Requested spawner \"%s\" is unknown", spawner);
-        }
+        gasneti_bootstrapCleanup_p  = &gasneti_bootstrapCleanup_mpi;
+    } else
 #endif
+#if HAVE_PMI_SPAWNER
+    if (GASNET_OK == (res = gasneti_bootstrapInit_pmi(argc_p, argv_p, nodes_p, mynode_p))) {
+        gasneti_bootstrapFini_p = &gasneti_bootstrapFini_pmi;
+        gasneti_bootstrapAbort_p    = &gasneti_bootstrapAbort_pmi;
+        gasneti_bootstrapBarrier_p  = &gasneti_bootstrapBarrier_pmi;
+        gasneti_bootstrapExchange_p = &gasneti_bootstrapExchange_pmi;
+        gasneti_bootstrapAlltoall_p = &gasneti_bootstrapAlltoall_pmi;
+        gasneti_bootstrapBroadcast_p= &gasneti_bootstrapBroadcast_pmi;
+        gasneti_bootstrapCleanup_p  = &gasneti_bootstrapCleanup_pmi;
+    } else
+#endif
+    {
+        gasneti_fatalerror("Requested spawner \"%s\" is unknown or not supported in this build", spawner);
     }
 
+    return res;
 }
 
 
 /* -------------------------------------------------------------------------- */
 
+#if MXM_API < MXM_VERSION(2,0)
 static int
 gasneti_mxm_get_ep_address(gasnet_mxm_ep_conn_info_t *ep_info, mxm_ptl_id_t id)
 {
@@ -209,6 +227,7 @@ gasneti_mxm_get_ep_address(gasnet_mxm_ep_conn_info_t *ep_info, mxm_ptl_id_t id)
                   mxm_error_string(mxm_status));
     return mxm_status;
 }
+#endif
 
 /* -------------------------------------------------------------------------- */
 
@@ -307,8 +326,8 @@ static int gasneti_load_settings(void)
 
 size_t inline gasneti_AMMaxMedium(void)
 {
-    //return gasnet_mxm_module.max_am_med -
-    //          (sizeof(gasnet_handlerarg_t) * GASNETC_MAX_ARGS));
+    /*return gasnet_mxm_module.max_am_med -*/
+    /*          (sizeof(gasnet_handlerarg_t) * GASNETC_MAX_ARGS));*/
     return gasnet_mxm_module.max_am_med;
 }
 
@@ -317,7 +336,7 @@ size_t inline gasneti_AMMaxMedium(void)
 /* Maximum pinning capabilities */
 typedef struct gasnetc_pin_info_t_ {
     uintptr_t    memory;    /* How much pinnable (per proc) */
-    //uint32_t   regions;
+    /*uint32_t   regions;*/
     int          ppn;       /* How many procs on the same host */
 } gasnetc_pin_info_t;
 static gasnetc_pin_info_t gasnetc_pin_info;
@@ -348,14 +367,14 @@ extern int gasnetc_pin(void *addr, size_t size, gasnetc_memreg_t *reg)
     gasneti_assert(((uintptr_t)addr % GASNET_PAGESIZE) == 0);
     gasneti_assert(((uintptr_t)size % GASNET_PAGESIZE) == 0);
 
-#if MXM_API < MXM_VERSION(1,5)
+#if MXM_API >= MXM_VERSION(2,0)
+    res = mxm_mem_map(gasnet_mxm_module.mxm_context, &addr, &size, 0, 0, 0); 
+#elif MXM_API < MXM_VERSION(1,5)
     res = mxm_reg_mr(gasnet_mxm_module.mxm_ep, MXM_PTL_RDMA,
                      addr, size, &reg->lkey, &reg->rkey);
 #elif MXM_API == MXM_VERSION(1,5)
     res = mxm_mem_register(gasnet_mxm_module.mxm_context,
                            addr, size, &reg->memh);
-#else
-#error MXM version is not supported
 #endif
 
     if_pf (MXM_OK != res)
@@ -364,6 +383,9 @@ extern int gasnetc_pin(void *addr, size_t size, gasnetc_memreg_t *reg)
     reg->addr     = (uintptr_t)addr;
     reg->len      = size;
     reg->end      = (uintptr_t)addr + (size - 1);
+#if MXM_API >= MXM_VERSION(2,0)
+    mxm_mem_get_key(gasnet_mxm_module.mxm_context, addr, &reg->m_key);
+#endif
 
 #if GASNET_TRACE
     gasnetc_pinned_blocks += 1;
@@ -377,15 +399,18 @@ extern int gasnetc_pin(void *addr, size_t size, gasnetc_memreg_t *reg)
 
 static void gasnetc_unpin(gasnetc_memreg_t *reg)
 {
-#if MXM_API < MXM_VERSION(1,5)
+#if MXM_API >= MXM_VERSION(2,0)
+    if_pf (MXM_OK != mxm_mem_unmap(gasnet_mxm_module.mxm_context, (void *)reg->addr, reg->len, 0)) {
+        MXM_ERROR("Failed deregistering memory region\n");
+    }
+
+#elif MXM_API < MXM_VERSION(1,5)
     if_pf (MXM_OK != mxm_dereg_mr(gasnet_mxm_module.mxm_ep, MXM_PTL_RDMA,
                                   (void *)reg->addr, reg->len)) {
         MXM_ERROR("Failed deregistering memory region\n");
     }
 #elif MXM_API == MXM_VERSION(1,5)
     mxm_mem_destroy(reg->memh);
-#else
-#error MXM version is not supported
 #endif
 
 #if GASNET_TRACE
@@ -402,14 +427,14 @@ static void *gasnetc_try_pin_inner(size_t size, gasnetc_memreg_t *reg)
 
     addr = gasneti_mmap(size);
     if (addr != MAP_FAILED) {
-        //MXM_DEBUG("MMap for size %lu succeeded, trying to register\n", size);
+        /*MXM_DEBUG("MMap for size %lu succeeded, trying to register\n", size);*/
         if (gasnetc_pin(addr, size, reg)) {
             MXM_DEBUG("Registering for size %lu failed\n", size);
             gasnetc_unpin(reg);
             gasneti_munmap(addr, size);
             return NULL;
         } else {
-            //MXM_DEBUG("Registering for size %lu succeeded\n", size);
+            /*MXM_DEBUG("Registering for size %lu succeeded\n", size);*/
         }
     } else {
         MXM_DEBUG("MMap for size %lu failed\n", size);
@@ -548,18 +573,17 @@ static inline int gasnetc_post_recv(void)
     p_req->tag_mask = 0; /* match any tag */
     p_base->completed_cb = NULL;
     p_base->conn = NULL; /* receive from any source */
-    p_base->flags = 0;
     p_base->mq = gasnet_mxm_module.mxm_mq;
     p_base->state = MXM_REQ_NEW;
     p_base->data_type = MXM_REQ_DATA_BUFFER;
     p_base->data.buffer.ptr = (void *)gasnet_mxm_module.recv_reg.addr;
     p_base->data.buffer.length = gasnet_mxm_module.recv_reg.len;
 #if MXM_API < MXM_VERSION(1,5)
+    p_base->flags = 0;
     p_base->data.buffer.mkey = gasnet_mxm_module.recv_reg.lkey;
 #elif MXM_API == MXM_VERSION(1,5)
+    p_base->flags = 0;
     p_base->data.buffer.memh = gasnet_mxm_module.recv_reg.memh;
-#else
-#error MXM version is not supported
 #endif
     return mxm_req_recv(p_req);
 }
@@ -573,20 +597,20 @@ static int gasnetc_init(int *argc, char ***argv)
     struct sockaddr_mxm_ib_local    sockaddr_bind_rdma;
     mxm_ep_opts_t                   mxm_ep_opts;
     mxm_context_opts_t              mxm_opts;
-#elif MXM_API == MXM_VERSION(1,5)
+#else
     mxm_ep_opts_t                 * mxm_ep_opts;
     mxm_context_opts_t            * mxm_opts;
-#else
-#error MXM version is not supported
 #endif
     gasnet_mxm_ep_conn_info_t       local_ep;
     mxm_error_t                     mxm_status;
     int                             i;
     uint32_t                        res;
+#if MXM_API >= MXM_VERSION(2,0)
+    size_t                          mxm_addr_len;
+#endif
 
     uint32_t jobid = 0;
     unsigned long cur_ver;
-    memset(&gasnet_mxm_module, 0, sizeof(gasnet_mxm_module));
 
     /*  check system sanity */
     gasnetc_check_config();
@@ -596,6 +620,8 @@ static int gasnetc_init(int *argc, char ***argv)
 
     gasneti_freezeForDebugger();
 
+    memset(&gasnet_mxm_module, 0, sizeof(gasnet_mxm_module));
+
 #if GASNET_DEBUG_VERBOSE
     /* note - can't call trace macros during gasnet_init because trace system not yet initialized */
     fprintf(stderr,"gasnetc_init(): about to spawn...\n");
@@ -608,7 +634,10 @@ static int gasnetc_init(int *argc, char ***argv)
      * Using existing bootstrap init via SSH.
      * This also initializes gasneti_nodes and gasneti_mynode globals.
      */
-    gasneti_bootstrapInit(argc, argv, &gasneti_nodes, &gasneti_mynode);
+    res = gasneti_bootstrapInit(argc, argv, &gasneti_nodes, &gasneti_mynode);
+    if (res != GASNET_OK) {
+        return res;
+    }
 
     gasneti_init_done = 1; /* enable early to allow tracing */
 
@@ -644,17 +673,17 @@ static int gasnetc_init(int *argc, char ***argv)
     mxm_fill_context_opts(&mxm_opts);
     mxm_opts.async_mode = MXM_ASYNC_MODE_THREAD;
     mxm_status = mxm_init(&mxm_opts, &gasnet_mxm_module.mxm_context);
-#elif MXM_API == MXM_VERSION(1,5)
+#else 
     res = mxm_config_read_context_opts(&mxm_opts);
     if_pf (res != MXM_OK) {
         MXM_ERROR("Failed to parse MXM configuration");
         return GASNET_ERR_NOT_INIT;
     }
     mxm_opts->async_mode = MXM_ASYNC_MODE_THREAD;
+#if MXM_API == MXM_VERSION(1,5)
     mxm_opts->ptl_bitmap = MXM_BIT(MXM_PTL_SELF) | MXM_BIT(MXM_PTL_RDMA);
+#endif
     mxm_status = mxm_init(mxm_opts, &gasnet_mxm_module.mxm_context);
-#else
-#error MXM version is not supported
 #endif
 
     if (mxm_status != MXM_OK) {
@@ -709,21 +738,20 @@ static int gasnetc_init(int *argc, char ***argv)
 
     mxm_status = mxm_ep_create(gasnet_mxm_module.mxm_context,
                                &mxm_ep_opts, &gasnet_mxm_module.mxm_ep);
-#elif MXM_API == MXM_VERSION(1,5)
+#else
     mxm_status = mxm_config_read_ep_opts(&mxm_ep_opts);
     if (mxm_status != MXM_OK) {
         MXM_ERROR("Failed to parse MXM configuration (%s)\n",
                   mxm_error_string(mxm_status));
         return GASNET_ERR_NOT_INIT;
     }
+#if MXM_API == MXM_VERSION(1,5)
     mxm_ep_opts->job_id = jobid;
     mxm_ep_opts->local_rank = gasneti_nodemap_local_rank;
     mxm_ep_opts->num_local_procs = gasneti_nodemap_local_count;
+#endif
     mxm_status = mxm_ep_create(gasnet_mxm_module.mxm_context,
                         mxm_ep_opts, &gasnet_mxm_module.mxm_ep);
-    mxm_config_free(mxm_ep_opts);
-#else
-#error MXM version is not supported
 #endif
 
     if (mxm_status != MXM_OK) {
@@ -733,27 +761,36 @@ static int gasnetc_init(int *argc, char ***argv)
     }
 
 #if MXM_API < MXM_VERSION(1,5)
-    gasnet_mxm_module.rndv_thresh = mxm_ep_opts.rdma.rndv_thresh;
     gasnet_mxm_module.zcopy_thresh = mxm_ep_opts.rdma.zcopy_thresh;
-#elif MXM_API == MXM_VERSION(1,5)
-    gasnet_mxm_module.rndv_thresh = mxm_ep_opts->rdma.rndv_thresh;
+#elif MXM_API == MXM_VERSION(1,5) 
     gasnet_mxm_module.zcopy_thresh = mxm_ep_opts->rdma.zcopy_thresh;
+    mxm_config_free(mxm_ep_opts);
 #else
-#error MXM version is not supported
+    gasnet_mxm_module.zcopy_thresh = mxm_ep_opts->zcopy_thresh;
+    mxm_config_free_ep_opts(mxm_ep_opts);
 #endif
 
     /*
      * Get address for each PTL on this endpoint and exchange it
      */
+#if MXM_API < MXM_VERSION(2,0)
     if (MXM_OK != gasneti_mxm_get_ep_address(&local_ep, MXM_PTL_SELF))
         return GASNET_ERR_NOT_INIT;
     if (MXM_OK != gasneti_mxm_get_ep_address(&local_ep, MXM_PTL_RDMA))
         return GASNET_ERR_NOT_INIT;
+#else
+    mxm_addr_len = sizeof(local_ep.ep_addr);
+    if (MXM_OK != (mxm_status = mxm_ep_get_address(gasnet_mxm_module.mxm_ep, local_ep.ep_addr, &mxm_addr_len))) {
+        MXM_ERROR("Unable to extract endpoint address (%s)\n",
+                                  mxm_error_string(mxm_status));
+        return GASNET_ERR_NOT_INIT;
+    }
+#endif
     /*
      * MXM fills in addresses for MXM_PTL_SELF and MXM_PTL_RDMA.
      * We don't care about the rest - tell valgrind to ignore it.
      */
-    //VALGRIND_MAKE_MEM_DEFINED(&local_ep, sizeof (local_ep));
+    /*VALGRIND_MAKE_MEM_DEFINED(&local_ep, sizeof (local_ep));*/
 
     gasnet_mxm_module.remote_eps =
         gasneti_calloc(gasneti_nodes, sizeof(local_ep));
@@ -879,7 +916,7 @@ static int gasnetc_init(int *argc, char ***argv)
 
         gasneti_assert(gasnetc_pin_info.memory != 0);
         gasneti_assert(gasnetc_pin_info.memory != (uintptr_t)(-1));
-        //gasneti_assert(gasnetc_pin_info.regions != 0);
+        /*gasneti_assert(gasnetc_pin_info.regions != 0);*/
 
 
 #if GASNET_SEGMENT_FAST
@@ -900,7 +937,7 @@ static int gasnetc_init(int *argc, char ***argv)
 #error Bad segment config
 #endif
 
-#if (1)
+#if (0) /* The ssh or mpi spwaner code takes care of this */
     /* Enable this if you wish to use the default GASNet services for broadcasting
         the environment from one compute node to all the others (for use in gasnet_getenv(),
         which needs to return environment variable values from the "spawning console").
@@ -955,9 +992,6 @@ static int gasnetc_init(int *argc, char ***argv)
     gasnetc_bootstrapBarrier();
 
     gasneti_auxseg_init(); /* adjust max seg values based on auxseg */
-#ifdef MXM_MUTEX_AMPOLL_LOCK
-    gasneti_mutex_init(&gasnetc_ampoll_lock);
-#endif
     return GASNET_OK;
 }
 
@@ -1223,6 +1257,8 @@ extern int gasnetc_attach(gasnet_handlerentry_t *table, int numentries,
     /* ensure extended API is initialized across nodes */
     gasnetc_bootstrapBarrier();
 
+    gasneti_bootstrapCleanup();
+
     return GASNET_OK;
 }
 
@@ -1284,12 +1320,16 @@ static void gasneti_mxm_finalize(void)
     if (gasnet_mxm_module.mxm_ep)
         mxm_ep_destroy(gasnet_mxm_module.mxm_ep);
 
-    /*
-     * TODO: deregister memory
-     *       mxm_error_t mxm_dereg_mr(mxm_ep_h ep,
-     *                                mxm_ptl_id_t ptlid,
-     *                                void *addr, size_t len);
-     */
+    gasnetc_unpin(&gasnet_mxm_module.recv_reg);
+
+    if (gasneti_attach_done) {
+        size_t remain = gasneti_seginfo[gasneti_mynode].size;
+        int j = gasnetc_max_regs*gasneti_mynode;
+        while (remain) {
+            gasnetc_unpin(&gasnet_mxm_module.reg[j++]);
+            remain -= MIN(remain, gasnetc_pin_maxsz);
+        }
+    }
 
     if (gasnet_mxm_module.mxm_context)
         mxm_cleanup(gasnet_mxm_module.mxm_context);
@@ -1617,8 +1657,8 @@ static int gasnetc_get_exit_role(void)
          * Don't know our role yet.
          * Send a system-category AM Request to determine our role.
          */
-        rc = gasnetc_SystemRequest(GASNETC_EXIT_FLOW_ROOT_NODE, // destination
-                                   1,                           // nargs
+        rc = gasnetc_SystemRequest(GASNETC_EXIT_FLOW_ROOT_NODE, /* destination*/
+                                   1,                           /* nargs*/
                                    (gasnet_handlerarg_t)SYSTEM_EXIT_ROLE_REQ);
 
         gasneti_assert(rc == GASNET_OK);
@@ -1739,8 +1779,8 @@ static int gasnetc_exit_master(int exitcode, int64_t timeout_us)
             return -1;
 
         MXM_DEBUG_EXIT_FLOW("Master sending exit request to slave node %d\n", i);
-        rc = gasnetc_SystemRequest(i, // destination
-                                   2, // nargs
+        rc = gasnetc_SystemRequest(i, /* destination*/
+                                   2, /* nargs*/
                                    (gasnet_handlerarg_t) SYSTEM_EXIT_REQ,
                                    (gasnet_handlerarg_t) exitcode);
         if (rc != GASNET_OK)
@@ -2061,7 +2101,7 @@ void gasnetc_exit(int exitcode)
 
 extern int gasnetc_AMGetMsgSource(gasnet_token_t token, gasnet_node_t *srcindex) {
     gasnet_node_t sourceid;
-//  GASNETI_CHECKATTACH();
+/*  GASNETI_CHECKATTACH();*/
     GASNETI_CHECK_ERRR((!token),BAD_ARG,"bad token");
     GASNETI_CHECK_ERRR((!srcindex),BAD_ARG,"bad src ptr");
 
@@ -2087,7 +2127,7 @@ static int gasnetc_AMPoll_nocheckattach(void) {
     if (gasneti_mutex_trylock(&gasnetc_ampoll_lock) == 0) {
 #endif
 #ifdef MXM_ATOMIC_AMPOLL_LOCK
-        if (gasneti_atomic_decrement_and_test(&gasnetc_ampoll_atomic_lock,0)) {
+        if (gasneti_atomic_decrement_and_test(&gasnetc_ampoll_atomic_lock,GASNETI_ATOMIC_ACQ)) {
 #endif
 
 
@@ -2115,7 +2155,7 @@ static int gasnetc_AMPoll_nocheckattach(void) {
         }
 #endif
 #ifdef MXM_ATOMIC_AMPOLL_LOCK
-        gasneti_atomic_set(&gasnetc_ampoll_atomic_lock,1,GASNETI_ATOMIC_WMB_POST);
+        gasneti_atomic_set(&gasnetc_ampoll_atomic_lock,1,GASNETI_ATOMIC_REL);
     }
 #endif
     return GASNET_OK;
@@ -2154,10 +2194,10 @@ extern int gasnetc_AMRequestShortM(
          */
 
         retval = gasnetc_RequestGeneric(gasnetc_Short, dest, handler,
-                                        NULL,        // void *src_addr
-                                        0,           // int nbytes
-                                        NULL,        // void *dst_addr
-                                        1,           // uint8_t is_sync
+                                        NULL,        /* void *src_addr*/
+                                        0,           /* int nbytes*/
+                                        NULL,        /* void *dst_addr*/
+                                        1,           /* uint8_t is_sync*/
                                         numargs, argptr);
     }
     va_end(argptr);
@@ -2187,10 +2227,10 @@ extern int gasnetc_AMRequestMediumM(
                  and send the active message
          */
         retval = gasnetc_RequestGeneric(gasnetc_Medium, dest, handler,
-                                        source_addr, // void *src_addr
-                                        nbytes,      // int nbytes
-                                        NULL,        // void *dst_addr
-                                        1,           // uint8_t is_sync
+                                        source_addr, /* void *src_addr*/
+                                        nbytes,      /* int nbytes*/
+                                        NULL,        /* void *dst_addr*/
+                                        1,           /* uint8_t is_sync*/
                                         numargs, argptr);
     }
     va_end(argptr);
@@ -2221,10 +2261,10 @@ extern int gasnetc_AMRequestLongM( gasnet_node_t dest,        /* destination nod
          */
 
         retval = gasnetc_RequestGeneric(gasnetc_Long, dest, handler,
-                                        source_addr, // void *src_addr
-                                        nbytes,      // int nbytes
-                                        dest_addr,   // void *dst_addr
-                                        1,           // uint8_t is_sync
+                                        source_addr, /* void *src_addr*/
+                                        nbytes,      /* int nbytes*/
+                                        dest_addr,   /* void *dst_addr*/
+                                        1,           /* uint8_t is_sync*/
                                         numargs, argptr);
     }
     va_end(argptr);
@@ -2255,10 +2295,10 @@ extern int gasnetc_AMRequestLongAsyncM( gasnet_node_t dest,        /* destinatio
          */
 
         retval = gasnetc_RequestGeneric(gasnetc_Long, dest, handler,
-                                        source_addr, // void *src_addr
-                                        nbytes,      // int nbytes
-                                        dest_addr,   // void *dst_addr
-                                        0,           // uint8_t is_sync
+                                        source_addr, /* void *src_addr*/
+                                        nbytes,      /* int nbytes*/
+                                        dest_addr,   /* void *dst_addr*/
+                                        0,           /* uint8_t is_sync*/
                                         numargs, argptr);
     }
     va_end(argptr);
@@ -2287,11 +2327,11 @@ extern int gasnetc_AMReplyShortM(
                  and send the active message
          */
 
-        retval = gasnetc_ReplyGeneric(gasnetc_Short, // gasnetc_category_t category
-                                      token,         // gasnet_token_t token
-                                      handler,       // gasnet_handler_t handler
-                                      NULL, 0,       // void *src_addr, int nbytes
-                                      NULL,          // void *dst_addr
+        retval = gasnetc_ReplyGeneric(gasnetc_Short, /* gasnetc_category_t category*/
+                                      token,         /* gasnet_token_t token*/
+                                      handler,       /* gasnet_handler_t handler*/
+                                      NULL, 0,       /* void *src_addr, int nbytes*/
+                                      NULL,          /* void *dst_addr*/
                                       numargs, argptr);
     }
     va_end(argptr);
@@ -2321,12 +2361,12 @@ extern int gasnetc_AMReplyMediumM(
                  and send the active message
          */
 
-        retval = gasnetc_ReplyGeneric(gasnetc_Medium,// gasnetc_category_t category
-                                      token,         // gasnet_token_t token
-                                      handler,       // gasnet_handler_t handler
-                                      source_addr,   // void *src_addr
-                                      nbytes,        // int nbytes
-                                      NULL,          // void *dst_addr
+        retval = gasnetc_ReplyGeneric(gasnetc_Medium,/* gasnetc_category_t category*/
+                                      token,         /* gasnet_token_t token*/
+                                      handler,       /* gasnet_handler_t handler*/
+                                      source_addr,   /* void *src_addr*/
+                                      nbytes,        /* int nbytes*/
+                                      NULL,          /* void *dst_addr*/
                                       numargs, argptr);
     }
     va_end(argptr);
@@ -2357,12 +2397,12 @@ extern int gasnetc_AMReplyLongM(
                  and send the active message
          */
 
-        retval = gasnetc_ReplyGeneric(gasnetc_Long,  // gasnetc_category_t category
-                                      token,         // gasnet_token_t token
-                                      handler,       // gasnet_handler_t handler
-                                      source_addr,   // void *src_addr
-                                      nbytes,        // int nbytes
-                                      dest_addr,     // void *dst_addr
+        retval = gasnetc_ReplyGeneric(gasnetc_Long,  /* gasnetc_category_t category*/
+                                      token,         /* gasnet_token_t token*/
+                                      handler,       /* gasnet_handler_t handler*/
+                                      source_addr,   /* void *src_addr*/
+                                      nbytes,        /* int nbytes*/
+                                      dest_addr,     /* void *dst_addr*/
                                       numargs, argptr);
     }
     va_end(argptr);
@@ -2377,13 +2417,13 @@ static int gasnetc_SystemRequest(gasnet_node_t dest,
     int retval;
     va_list argptr;
     va_start(argptr, numargs);
-    retval = gasnetc_RequestGeneric(gasnetc_System,// gasnetc_category_t category
-                                    dest,          // destination
-                                    0,             // gasnet_handler_t handler
-                                    NULL,          // void *src_addr
-                                    0,             // int nbytes
-                                    NULL,          // void *dst_addr
-                                    1,             // uint8_t is_sync
+    retval = gasnetc_RequestGeneric(gasnetc_System,/* gasnetc_category_t category*/
+                                    dest,          /* destination*/
+                                    0,             /* gasnet_handler_t handler*/
+                                    NULL,          /* void *src_addr*/
+                                    0,             /* int nbytes*/
+                                    NULL,          /* void *dst_addr*/
+                                    1,             /* uint8_t is_sync*/
                                     numargs, argptr);
     va_end(argptr);
     GASNETI_RETURN(retval);
@@ -2397,11 +2437,11 @@ int gasnetc_SystemReply(gasnet_token_t token,
     int retval;
     va_list argptr;
     va_start(argptr, numargs);
-    retval = gasnetc_ReplyGeneric(gasnetc_System, // gasnetc_category_t category
-                                  token,          // gasnet_token_t token
-                                  0,              // gasnet_handler_t handler
-                                  NULL, 0,        // void *src_addr, int nbytes
-                                  NULL,           // void *dst_addr
+    retval = gasnetc_ReplyGeneric(gasnetc_System, /* gasnetc_category_t category*/
+                                  token,          /* gasnet_token_t token*/
+                                  0,              /* gasnet_handler_t handler*/
+                                  NULL, 0,        /* void *src_addr, int nbytes*/
+                                  NULL,           /* void *dst_addr*/
                                   numargs, argptr);
     va_end(argptr);
     GASNETI_RETURN(retval);
