@@ -18,7 +18,6 @@
 #define GASNET_USE_MXM_SELF_PTL 0
 
 #define GASNET_USE_ZCOPY 1
-#define GASNET_USE_ZCOPY_RCV 0
 
 
 /* -------------------------------------------------------------------------- */
@@ -162,22 +161,27 @@ extern void gasnetc_barrier_fence(void)
         continue;
 #endif
         mxm_sreq = &sreqs[dest].mxm_sreq;
+#if MXM_API < MXM_VERSION(2,0)
         mxm_sreq->opcode = MXM_REQ_OP_FENCE;
+        mxm_sreq->base.flags = MXM_REQ_FLAG_SEND_SYNC;
+#else
+        mxm_sreq->opcode = MXM_REQ_OP_PUT_SYNC;
+        mxm_sreq->flags  = MXM_REQ_SEND_FLAG_FENCE;
+        mxm_sreq->op.mem.remote_vaddr = 0;
+        mxm_sreq->op.mem.remote_mkey  = &mxm_empty_mem_key;
+#endif
 
         mxm_sreq->base.state = MXM_REQ_NEW;
         mxm_sreq->base.conn = gasnet_mxm_module.connections[dest];
         mxm_sreq->base.mq = gasnet_mxm_module.mxm_mq;
-        mxm_sreq->base.flags = MXM_REQ_FLAG_SEND_SYNC;
 
         mxm_sreq->base.data.buffer.ptr = NULL;
         mxm_sreq->base.data.buffer.length = 0;
 
 #if MXM_API < MXM_VERSION(1,5)
         mxm_sreq->base.data.buffer.mkey = MXM_MKEY_NONE;
-#elif MXM_API == MXM_VERSION(1,5)
-        mxm_sreq->base.data.buffer.memh = NULL;
 #else
-#error MXM version is not supported
+        mxm_sreq->base.data.buffer.memh = NULL;
 #endif
         mxm_sreq->base.data_type = MXM_REQ_DATA_BUFFER;
 
@@ -218,8 +222,8 @@ static gasnetc_memreg_t * gasnetc_find_reg(void *src_addr, int nbytes, int rank)
 #if GASNET_USE_ZCOPY
     int i;
     int rank_reg_offset = gasnetc_max_regs * rank;
-    if ( (nbytes < gasnet_mxm_module.zcopy_thresh) ||
-            (!GASNET_ADDR_IN_RANGE(gasneti_seginfo[rank].addr,
+
+    if ((!GASNET_ADDR_IN_RANGE(gasneti_seginfo[rank].addr,
                                    gasneti_seginfo[rank].size,
                                    src_addr, nbytes)) )
         return NULL;
@@ -251,7 +255,12 @@ static gasnetc_memreg_t * gasnetc_find_reg(void *src_addr, int nbytes, int rank)
 inline uint32_t gasnetc_find_lkey(void *src_addr, int nbytes)
 {
 #if GASNET_SEGMENT_FAST
-    gasnetc_memreg_t * reg = gasnetc_find_reg(src_addr, nbytes, gasnet_mynode());
+    gasnetc_memreg_t * reg;
+
+    if (nbytes < gasnet_mxm_module.zcopy_thresh)
+        return NULL;
+
+    reg = gasnetc_find_reg(src_addr, nbytes, gasnet_mynode());
     return (reg) ? reg->lkey : MXM_MKEY_NONE;
 #else
     return MXM_MKEY_NONE;
@@ -260,24 +269,20 @@ inline uint32_t gasnetc_find_lkey(void *src_addr, int nbytes)
 
 inline uint32_t gasnetc_find_rkey(void *src_addr, int nbytes, int rank)
 {
-#if GASNET_USE_ZCOPY_RCV
-#if GASNET_SEGMENT_FAST
-    gasnetc_memreg_t * reg = gasnetc_find_reg(src_addr, nbytes, rank);
-    return (reg) ? reg->rkey : MXM_MKEY_NONE;
-#else
     return MXM_MKEY_NONE;
-#endif
-#else
-    return MXM_MKEY_NONE;
-#endif
 }
 
-#elif MXM_API == MXM_VERSION(1,5)
+#elif MXM_API < MXM_VERSION(2,0)
 
 inline mxm_mem_h gasnetc_find_memh(void *addr, int nbytes)
 {
 #if GASNET_SEGMENT_FAST
-    gasnetc_memreg_t * reg = gasnetc_find_reg(addr, nbytes, gasnet_mynode());
+    gasnetc_memreg_t * reg;
+
+    if (nbytes < gasnet_mxm_module.zcopy_thresh)
+        return NULL;
+
+    reg = gasnetc_find_reg(addr, nbytes, gasnet_mynode());
     return (reg) ? reg->memh : NULL;
 #else
     return NULL;
@@ -290,7 +295,18 @@ inline mxm_mem_h gasnetc_find_remote_memh(void *addr, int nbytes, int rank)
 }
 
 #else
-#error MXM version is not supported
+
+inline mxm_mem_key_t *gasnetc_find_remote_mkey(void *addr, int nbytes, int rank)
+{
+#if (GASNET_SEGMENT_FAST)
+    gasnetc_memreg_t * reg = gasnetc_find_reg(addr, nbytes, rank);
+    return  reg ? &reg->m_key : &mxm_empty_mem_key ;
+#else
+    return  &mxm_empty_mem_key;
+#endif
+}
+
+
 #endif
 
 /* -------------------------------------------------------------------------- */
@@ -377,6 +393,8 @@ int gasnetc_AM_Generic(gasnetc_category_t category,
     mxm_sreq->base.state = MXM_REQ_NEW;
     mxm_sreq->base.conn = gasnet_mxm_module.connections[dest];
     mxm_sreq->base.mq = gasnet_mxm_module.mxm_mq;
+
+#if MXM_API < MXM_VERSION(2,0)
     mxm_sreq->base.flags = 0;
 
     /*
@@ -386,6 +404,9 @@ int gasnetc_AM_Generic(gasnetc_category_t category,
      * the message. E.g. MXM will not use ZCOPY.
      */
     mxm_sreq->base.flags |= MXM_REQ_FLAG_BLOCKING;
+#else
+    mxm_sreq->flags = MXM_REQ_SEND_FLAG_BLOCKING;
+#endif
 
     mxm_sreq->base.completed_cb = NULL;
     mxm_sreq->base.context = NULL;
@@ -420,10 +441,8 @@ int gasnetc_AM_Generic(gasnetc_category_t category,
 
 #if MXM_API < MXM_VERSION(1,5)
         mxm_sreq->base.data.buffer.mkey = MXM_MKEY_NONE;
-#elif MXM_API == MXM_VERSION(1,5)
-        mxm_sreq->base.data.buffer.memh = NULL;
 #else
-#error MXM version is not supported
+        mxm_sreq->base.data.buffer.memh = NULL;
 #endif
 
         mxm_sreq->base.data_type = MXM_REQ_DATA_BUFFER;
@@ -441,10 +460,8 @@ int gasnetc_AM_Generic(gasnetc_category_t category,
                                                   GASNETI_MEDBUF_ALIGNMENT);
 #if MXM_API < MXM_VERSION(1,5)
             p_sreq->sendiov[sge_idx].mkey = MXM_MKEY_NONE;
-#elif MXM_API == MXM_VERSION(1,5)
-            p_sreq->sendiov[sge_idx].memh = NULL;
 #else
-#error MXM version is not supported
+            p_sreq->sendiov[sge_idx].memh = NULL;
 #endif
             sge_idx++;
         }
@@ -456,8 +473,6 @@ int gasnetc_AM_Generic(gasnetc_category_t category,
             p_sreq->sendiov[sge_idx].mkey = gasnetc_find_lkey(src_addr, nbytes);
 #elif MXM_API == MXM_VERSION(1,5)
             p_sreq->sendiov[sge_idx].memh = gasnetc_find_memh(src_addr, nbytes);
-#else
-#error MXM version is not supported
 #endif
             sge_idx++;
         }
@@ -486,7 +501,11 @@ int gasnetc_AM_Generic(gasnetc_category_t category,
             put_mxm_sreq->base.state = MXM_REQ_NEW;
             put_mxm_sreq->base.conn = gasnet_mxm_module.connections[dest];
             put_mxm_sreq->base.mq = gasnet_mxm_module.mxm_mq;
+#if MXM_API < MXM_VERSION(2,0)
             put_mxm_sreq->base.flags = MXM_REQ_FLAG_BLOCKING;
+#else
+            put_mxm_sreq->flags = MXM_REQ_SEND_FLAG_BLOCKING;
+#endif
 
             put_mxm_sreq->base.data_type = MXM_REQ_DATA_BUFFER;
             put_mxm_sreq->opcode = MXM_REQ_OP_PUT;
@@ -501,7 +520,7 @@ int gasnetc_AM_Generic(gasnetc_category_t category,
             put_mxm_sreq->base.data.buffer.memh = gasnetc_find_memh(src_addr, nbytes);
             put_mxm_sreq->op.mem.remote_memh = gasnetc_find_remote_memh(dst_addr, nbytes, dest);
 #else
-#error MXM version is not supported
+            put_mxm_sreq->op.mem.remote_mkey = gasnetc_find_remote_mkey(dst_addr, nbytes, dest);
 #endif
 
             /*
@@ -537,10 +556,8 @@ int gasnetc_AM_Generic(gasnetc_category_t category,
             p_sreq->sendiov[sge_idx].length = numargs * sizeof(gasnet_handlerarg_t);
 #if MXM_API < MXM_VERSION(1,5)
             p_sreq->sendiov[sge_idx].mkey = MXM_MKEY_NONE;
-#elif MXM_API == MXM_VERSION(1,5)
-            p_sreq->sendiov[sge_idx].memh = NULL;
 #else
-#error MXM version is not supported
+            p_sreq->sendiov[sge_idx].memh = NULL;
 #endif
             sge_idx++;
         }
@@ -551,10 +568,8 @@ int gasnetc_AM_Generic(gasnetc_category_t category,
         p_sreq->sendiov[sge_idx].length = sizeof(p_sreq->long_info);
 #if MXM_API < MXM_VERSION(1,5)
         p_sreq->sendiov[sge_idx].mkey = MXM_MKEY_NONE;
-#elif MXM_API == MXM_VERSION(1,5)
-        p_sreq->sendiov[sge_idx].memh = NULL;
 #else
-#error MXM version is not supported
+        p_sreq->sendiov[sge_idx].memh = NULL;
 #endif
         sge_idx++;
 
