@@ -1,6 +1,6 @@
 /*   $Source: /var/local/cvs/gasnet/other/portable_platform.h,v $
- *     $Date: 2013/04/27 04:11:50 $
- * $Revision: 1.34.2.1 $
+ *     $Date: 2013/09/18 23:01:54 $
+ * $Revision: 1.37 $
  *     $Note: Tilera support added. (Chapel--hildeth) $
  * Description: Portable platform detection header
  * Copyright 2006, Dan Bonachea <bonachea@cs.berkeley.edu>
@@ -634,6 +634,12 @@
   #define PLATFORM_ARCH_FAMILYNAME POWERPC
   #define PLATFORM_ARCH_BIG_ENDIAN 1
 
+#elif defined(__KNC__) || defined(__MIC__)
+  #define PLATFORM_ARCH_MIC 1
+  #define PLATFORM_ARCH_FAMILYNAME MIC
+  #define _PLATFORM_ARCH_64 1
+  #define PLATFORM_ARCH_LITTLE_ENDIAN 1
+
 #elif defined(__x86_64) || defined(__x86_64__) || \
     defined(__athlon) || defined(__athlon__) || \
     defined(__amd64)  || defined(__amd64__)
@@ -683,6 +689,13 @@
     /* IRIX */
     #define PLATFORM_ARCH_BIG_ENDIAN 1
   #endif
+  #ifdef _MIPS_SZPTR
+    #if _MIPS_SZPTR == 32
+      #define _PLATFORM_ARCH_32 1
+    #elif _MIPS_SZPTR == 64
+      #define _PLATFORM_ARCH_64 1
+    #endif
+  #endif
 
 #elif defined(__sparc) || defined(__sparc__) || \
     defined(__sparclet__) || defined(__sparclite__) || \
