@@ -1,6 +1,6 @@
 /*   $Source: /var/local/cvs/gasnet/portals4-conduit/gasnet_core.c,v $
- *     $Date: 2013/04/23 01:08:24 $
- * $Revision: 1.1.1.1.2.1 $
+ *     $Date: 2013/07/28 23:54:59 $
+ * $Revision: 1.4 $
  * Description: GASNet portals4 conduit Implementation
  * Copyright 2002, Dan Bonachea <bonachea@cs.berkeley.edu>
  * Terms of use are as specified in license.txt
@@ -43,7 +43,6 @@ static void gasnetc_check_config(void) {
 
 static int gasnetc_init(int *argc, char ***argv) {
     int ret;
-    int rank, size;
     
     /*  check system sanity */
     gasnetc_check_config();
@@ -66,12 +65,9 @@ static int gasnetc_init(int *argc, char ***argv) {
     /* gasnetc_p4_init has barrier semantics such that
        gasnetc_bootstrapBarrier and gasnetc_bootstrapExchange will
        work upon return of init. */
-    ret = gasnetc_p4_init(&rank, &size);
+    ret = gasnetc_p4_init(&gasneti_mynode, &gasneti_nodes);
     if (GASNET_OK != ret) gasneti_fatalerror("gasnetc_p4_init failed: %d\n", ret);
 
-    gasneti_mynode = rank;
-    gasneti_nodes = size;
-
     /* node map set in gasnetc_p4_init */
 
 #if GASNET_PSHM
@@ -299,12 +295,17 @@ extern int gasnetc_attach(gasnet_handlerentry_t *table, int numentries,
   return GASNET_OK;
 }
 /* ------------------------------------------------------------------------------------ */
+
+static int gasnetc_exit_in_progress = 0;
+
 #if HAVE_ON_EXIT
 static void gasnetc_on_exit(int exitcode, void *arg) {
+  if (!gasnetc_exit_in_progress)
     gasnetc_exit(exitcode);
 }
 #else
 static void gasnetc_atexit(void) {
+  if (!gasnetc_exit_in_progress)
     gasnetc_exit(0);
 }
 #endif
@@ -312,6 +313,7 @@ static void gasnetc_atexit(void) {
 extern void gasnetc_exit(int exitcode) {
   /* once we start a shutdown, ignore all future SIGQUIT signals or we risk reentrancy */
   gasneti_reghandler(SIGQUIT, SIG_IGN);
+  gasnetc_exit_in_progress = 1;
 
   {  /* ensure only one thread ever continues past this point */
     static gasneti_mutex_t exit_lock = GASNETI_MUTEX_INITIALIZER;
