@@ -1,6 +1,6 @@
 /*   $Source: /var/local/cvs/gasnet/tests/testtools.c,v $
- *     $Date: 2010/04/26 01:45:00 $
- * $Revision: 1.100 $
+ *     $Date: 2013/05/31 20:52:11 $
+ * $Revision: 1.103 $
  * Description: helpers for GASNet tests
  * Copyright 2002, Dan Bonachea <bonachea@cs.berkeley.edu>
  * Terms of use are as specified in license.txt
@@ -195,10 +195,31 @@ int main(int argc, char **argv) {
     gasnett_tick_t ticktimemin = GASNETT_TICK_MIN;
     gasnett_tick_t ticktimemax = GASNETT_TICK_MAX;
 
+    double overhead = gasnett_tick_overheadus();
+    double granularity = gasnett_tick_granularityus();
+
+    /* Aiming for 'total' microseconds of busy waits, with 25% of it
+       in the last iteration, but need a non-trivial number of ticks
+       to pass int each iteration. */
+    uint64_t total = 20 * 1000000;
+    uint64_t us_delay = (0.75*total) / timeiters;
+    uint64_t min_delay = MAX(1, 50*granularity);
+    if (us_delay < min_delay) {
+       us_delay = min_delay;
+       timeiters = (0.75*total) / min_delay;
+    }
+
     if (!(ticktimemin < ticktimemax)) ERR("!(min < max)");
     if (!(gasnett_ticks_now() > ticktimemin)) ERR("!(now > min)");
     if (!(gasnett_ticks_now() < ticktimemax)) ERR("!(now < max)");
 
+    if (granularity <= 0.0 || overhead <= 0.0 ||
+        (granularity+0.1) < 0.5*overhead) 
+        /* allow some leeway for noise at granularities approaching cycle speed */
+        /*granularity < 0.5*overhead)*/
+        ERR("nonsensical timer overhead/granularity measurements:\n"
+             "  overhead: %.3fus  granularity: %.3fus\n",overhead, granularity);
+
     begin = gasnett_ticks_now();  /* outer time point */
     beginref = gasnett_gettimeofday_us();
     for (i=0; i < timeiters; i++) {
@@ -206,6 +227,7 @@ int main(int argc, char **argv) {
       if (i == timeiters - 1) {
         start = begin; /* use outer time point for base of last iteration */
         startref = beginref;
+        us_delay = total; /* consume the remainder (about 25%) of the total interval */
       } else {
         start = gasnett_ticks_now(); /* inner time point */
         startref = gasnett_gettimeofday_us();
@@ -226,7 +248,7 @@ int main(int argc, char **argv) {
                  (unsigned long long)next, (unsigned long long)GASNETT_TICK_MAX);
           d_junk *= 1.0001;
           last = next;
-        } while (gasnett_ticks_to_us(last-start) < 100000);
+        } while (gasnett_ticks_to_us(last-start) < us_delay);
       }
       end = gasnett_ticks_now();
       endref = gasnett_gettimeofday_us();
@@ -248,15 +270,6 @@ int main(int argc, char **argv) {
 
     }
   }
-  { double overhead = gasnett_tick_overheadus();
-    double granularity = gasnett_tick_granularityus();
-    if (granularity <= 0.0 || overhead <= 0.0 ||
-        (granularity+0.1) < 0.5*overhead) 
-        /* allow some leeway for noise at granularities approaching cycle speed */
-        /*granularity < 0.5*overhead)*/
-        ERR("nonsensical timer overhead/granularity measurements:\n"
-             "  overhead: %.3fus  granularity: %.3fus\n",overhead, granularity);
-  }
 
   TEST_HEADER("Testing zero-byte counting...")
   { /* gasnett_count0s*() */
@@ -373,9 +386,9 @@ int main(int argc, char **argv) {
     if (gasnett_atomic_read(&var,0) != (gasnett_atomic_val_t)(2*iters))
       ERR("gasnett_atomic_set/gasnett_atomic_read got wrong value");
 
-    for (i=1;i<=iters;i++) {
+    for (i=0;i<iters;i++) {
       gasnett_atomic_increment(&var,0);
-      if (gasnett_atomic_read(&var,0) != (gasnett_atomic_val_t)(2*iters + i))
+      if (gasnett_atomic_read(&var,0) != (gasnett_atomic_val_t)(2*iters + (i+1)))
         ERR("gasnett_atomic_increment got wrong value");
     }
 
@@ -385,14 +398,14 @@ int main(int argc, char **argv) {
         ERR("gasnett_atomic_decrement got wrong value");
     }
 
-    for (i=1;i<=iters;i++) {
+    for (i=0;i<iters;i++) {
       gasnett_atomic_set(&var, i, 0);
       gasnett_atomic_increment(&var,0);
       if (gasnett_atomic_read(&var,0) != (gasnett_atomic_val_t)(i+1))
         ERR("gasnett_atomic_set/gasnett_atomic_increment got wrong value");
     }
 
-    for (i=1;i<=iters;i++) {
+    for (i=0;i<iters;i++) {
       gasnett_atomic_set(&var, i, 0);
       gasnett_atomic_decrement(&var,0);
       if (gasnett_atomic_read(&var,0) != (gasnett_atomic_val_t)(i-1))
@@ -413,7 +426,7 @@ int main(int argc, char **argv) {
 
     #if defined(GASNETT_HAVE_ATOMIC_CAS)
       gasnett_atomic_set(&var, 0, 0);
-      for (i=0;i<=iters;i++) {
+      for (i=0;i<iters;i++) {
 	if (gasnett_atomic_compare_and_swap(&var, i-1, i-2, 0))
           ERR("gasnett_atomic_compare_and_swap succeeded at i=%i when it should have failed", i);
 	if (gasnett_atomic_compare_and_swap(&var, i+1, i-2, 0))
@@ -429,7 +442,7 @@ int main(int argc, char **argv) {
 
     #if defined(GASNETT_HAVE_ATOMIC_ADD_SUB)
       gasnett_atomic_set(&var, 1, 0);
-      for (i=1;i<=iters;i++) {
+      for (i=1;i<iters;i++) {
         if ((gasnett_atomic_add(&var, i, 0) != (gasnett_atomic_val_t)(2*i)) ||
             (gasnett_atomic_read(&var,0) != (gasnett_atomic_val_t)(2*i)))
           ERR("gasnett_atomic_add got wrong value");
@@ -564,7 +577,7 @@ int main(int argc, char **argv) {
       }
 
       gasnett_atomic32_set(&var32, 0, 0);
-      for (i=0;i<=iters;i++) {
+      for (i=0;i<iters;i++) {
 	if (gasnett_atomic32_compare_and_swap(&var32, i-1, i-2, 0))
           ERR("gasnett_atomic32_compare_and_swap succeeded at i=%i when it should have failed", i);
 	if (gasnett_atomic32_compare_and_swap(&var32, i+1, i-2, 0))
@@ -615,7 +628,7 @@ int main(int argc, char **argv) {
       }
 
       gasnett_atomic64_set(&var64, 0, 0);
-      for (i=0;i<=iters;i++) {
+      for (i=0;i<iters;i++) {
 	if (gasnett_atomic64_compare_and_swap(&var64, i-1, i-2, 0))
           ERR("gasnett_atomic64_compare_and_swap succeeded at i=%i when it should have failed", i);
 	if (gasnett_atomic64_compare_and_swap(&var64, i+1, i-2, 0))
@@ -995,11 +1008,11 @@ void * thread_fn(void *arg) {
   TEST_HEADER("parallel compare-and-swap test...") {
     #if defined(GASNETT_HAVE_ATOMIC_CAS)
       static gasnett_atomic_t counter2 = gasnett_atomic_init(0);
-      static uint32_t shared_counter = 0;
-      uint32_t woncnt = 0;
-      uint32_t share = ((unsigned)iters >= (0xffffffffU / NUM_THREADS)) ? (0xffffffffU / NUM_THREADS) : iters;
-      uint32_t goal = NUM_THREADS * share;
-      uint32_t i, oldval;
+      static gasnett_atomic_val_t shared_counter = 0;
+      gasnett_atomic_val_t woncnt = 0;
+      gasnett_atomic_val_t share = ((unsigned)iters >= (GASNETT_ATOMIC_MAX / NUM_THREADS)) ? (GASNETT_ATOMIC_MAX / NUM_THREADS) : iters;
+      gasnett_atomic_val_t goal = NUM_THREADS * share;
+      gasnett_atomic_val_t i, oldval;
 
       /* Look for missing or doubled updates by taking an equal share of increments */
       while (woncnt < share &&
