@@ -33,6 +33,7 @@
 #include <errno.h>
 #include <sys/time.h>
 #include <unistd.h>
+#include <sched.h>
 
 #include "qthread/qthread.h"
 #include "qthread/qtimer.h"
@@ -112,57 +113,33 @@ struct chpl_task_list {
     chpl_task_list_p next;
 };
 
-typedef struct {
-  c_sublocid_t requestedSubloc;  // requested sublocal for task
-  chpl_bool    serial_state;     // true: serialize execution
-} task_private_data_t;
+pthread_t chpl_qthread_comm_pthread;
 
-typedef struct {
-    void                     *fn;
-    void                     *args;
-    chpl_string              task_filename;
-    int                      lineno;
-    task_private_data_t      chpl_data;
-} chapel_wrapper_args_t;
+chpl_qthread_tls_t chpl_qthread_comm_task_tls = { { c_sublocid_any_val,
+                                                    false },
+                                                  NULL, 0, NULL, 0 };
 
-// Structure of task-local storage
-typedef struct chapel_tls_s {
-    /* Task private data: serial state, etc. */
-    task_private_data_t chpl_data;
-    /* Reports */
-    chpl_string lock_filename;
-    size_t      lock_lineno;
-    const char *task_filename;
-    size_t      task_lineno;
-} chapel_tls_t;
+//
+// structs chpl_qthread_private_data_t, chpl_qthread_wrapper_args_t and
+// chpl_qthread_tls_t have been moved to tasks-qthreads.h
+//
 
-// Wrap qthread_get_tasklocal() and assert that it is always available.
-static inline chapel_tls_t * chapel_get_tasklocal(void)
-{
-    chapel_tls_t * tls = 
-        (chapel_tls_t *)qthread_get_tasklocal(sizeof(chapel_tls_t));
-
-    assert(tls);
-
-    return tls;
-}
-
-// FIXME: this is the same as chapel_get_tasklocal() except that it does
-//        not have the assertion.
-static inline chapel_tls_t * chapel_get_tasklocal_possibly_from_non_task(void)
-{
-    chapel_tls_t * tls = 
-        (chapel_tls_t *)qthread_get_tasklocal(sizeof(chapel_tls_t));
-
-    return tls;
-}
+//
+// chpl_qthread_get_tasklocal() is in tasks-qthreads.h
+//
 
 static syncvar_t exit_ret = SYNCVAR_STATIC_EMPTY_INITIALIZER;
 
+static volatile chpl_bool canCountRunningTasks = false;
+
 void chpl_task_yield(void)
 {
     PROFILE_INCR(profile_task_yield,1);
-    qthread_yield();
+    if (qthread_shep() == NO_SHEPHERD) {
+        sched_yield();
+    } else {
+        qthread_yield();
+    }
 }
 
 // Sync variables
@@ -186,7 +163,7 @@ void chpl_sync_unlock(chpl_sync_aux_t *s)
 static inline void about_to_block(int32_t     lineno,
                                   chpl_string filename)
 {
-    chapel_tls_t * data = chapel_get_tasklocal();
+    chpl_qthread_tls_t * data = chpl_qthread_get_tasklocal();
     assert(data);
 
     data->lock_lineno   = lineno;
@@ -274,7 +251,7 @@ static void chapel_display_thread(qt_key_t     addr,
                                   void        *tls,
                                   void        *callarg)
 {
-    chapel_tls_t *rep = (chapel_tls_t *)tls;
+    chpl_qthread_tls_t *rep = (chpl_qthread_tls_t *)tls;
 
     if (rep) {
         if ((rep->lock_lineno > 0) && rep->lock_filename) {
@@ -313,15 +290,14 @@ static void SIGINT_handler(int sig)
 // Tasks
 
 #ifndef QTHREAD_MULTINODE
-static int       done_initializing = 0;
-static syncvar_t canexit           = SYNCVAR_STATIC_EMPTY_INITIALIZER;
-static int       done_finalizing   = 0;
+static syncvar_t canexit            = SYNCVAR_STATIC_EMPTY_INITIALIZER;
+static volatile int done_finalizing = 0;
 
 static void *initializer(void *junk)
 {
     qthread_initialize();
     MACHINE_FENCE;
-    done_initializing = 1;
+    chpl_qthread_done_initializing = 1;
 
     qthread_syncvar_readFF(NULL, &canexit);
 
@@ -334,14 +310,17 @@ static void *initializer(void *junk)
 
 void chpl_task_init(void)
 {
+    int32_t   numCoresPerLocale;
     int32_t   numThreadsPerLocale;
-    uint64_t  callStackSize;
+    size_t    callStackSize;
     pthread_t initer;
     char      newenv_sheps[100] = { 0 };
     char      newenv_stack[100] = { 0 };
 
     // Set up available hardware parallelism
-    if ((numThreadsPerLocale = chpl_task_getenvNumThreadsPerLocale()) == 0)
+    numThreadsPerLocale = chpl_task_getenvNumThreadsPerLocale();
+    numCoresPerLocale = chpl_numCoresOnThisLocale();
+    if (numThreadsPerLocale == 0)
         numThreadsPerLocale = chpl_comm_getMaxThreads();
     if (0 < numThreadsPerLocale) {
         // We are assuming the user wants to constrain the hardware
@@ -352,13 +331,13 @@ void chpl_task_init(void)
         qt_internal_unset_envstr("NUM_SHEPHERDS");
         qt_internal_unset_envstr("NUM_WORKERS_PER_SHEPHERD");
 
-        if (chpl_numCoresOnThisLocale() < numThreadsPerLocale) {
+        if (numCoresPerLocale < numThreadsPerLocale) {
             if (2 == verbosity) {
                 printf("QTHREADS: Ignored --numThreadsPerLocale=%d to prevent oversubsription of the system.\n", numThreadsPerLocale);
             }
 
             // Do not oversubscribe the system, use all available resources.
-            numThreadsPerLocale = chpl_numCoresOnThisLocale();
+            numThreadsPerLocale = numCoresPerLocale;
         }
 
         // Set environment variable for Qthreads
@@ -371,24 +350,21 @@ void chpl_task_init(void)
         // environment variables.
     } else {
         // Default to using all hardware resources.
-        numThreadsPerLocale = chpl_numCoresOnThisLocale();
+        numThreadsPerLocale = numCoresPerLocale;
         snprintf(newenv_sheps, 99, "%i", (int)numThreadsPerLocale);
         setenv("QT_HWPAR", newenv_sheps, 1);
     }
 
     // Precendence (high-to-low):
-    // 1) CHPL_RT_CALL_STACK_SIZE
+    // 1) Chapel minimum
     // 2) QTHREAD_STACK_SIZE
-    // 3) Chapel default
-    callStackSize = chpl_task_getenvCallStackSize();
-    if (callStackSize != 0) {
-        snprintf(newenv_stack, 99, "%lu", (unsigned long)callStackSize);
-        setenv("QT_STACK_SIZE", getenv("CHPL_RT_CALL_STACK_SIZE"), 1);
-    } else if (qt_internal_get_env_str("STACK_SIZE", NULL) == NULL) {
-        uint64_t callStackSize = 32 * 1024 * sizeof(size_t);
-        snprintf(newenv_stack, 99, "%lu", (unsigned long)callStackSize);
-        setenv("QT_STACK_SIZE", newenv_stack, 1);
-    }
+    // In practice we never get to #2, because the Chapel minimum is
+    // always > 0, but we cover that case as a backstop.
+    callStackSize = chpl_task_getMinCallStackSize();
+    if (callStackSize <= 0)
+        callStackSize = 1024 * 1024 * sizeof(size_t);
+    snprintf(newenv_stack, 99, "%zu", callStackSize);
+    setenv("QT_STACK_SIZE", newenv_stack, 1);
 
     // Turn on informative Qthreads setting messages with Chapel's verbose flag
     if (verbosity == 2) {
@@ -396,7 +372,7 @@ void chpl_task_init(void)
     }
 
     pthread_create(&initer, NULL, initializer, NULL);
-    while (done_initializing == 0) SPINLOCK_BODY();
+    while (chpl_qthread_done_initializing == 0) SPINLOCK_BODY();
 
     if (blockreport || taskreport) {
         if (signal(SIGINT, SIGINT_handler) == SIG_ERR) {
@@ -416,7 +392,7 @@ void chpl_task_exit(void)
     if (qthread_shep() == NO_SHEPHERD) {
         /* sometimes, tasking is told to shutdown even though it hasn't been
          * told to start yet */
-        if (done_initializing == 1) {
+        if (chpl_qthread_done_initializing == 1) {
             qthread_syncvar_fill(&canexit);
             while (done_finalizing == 0) SPINLOCK_BODY();
         }
@@ -428,8 +404,8 @@ void chpl_task_exit(void)
 
 static aligned_t chapel_wrapper(void *arg)
 {
-    chapel_wrapper_args_t *rarg = arg;
-    chapel_tls_t * data = chapel_get_tasklocal();
+    chpl_qthread_wrapper_args_t *rarg = arg;
+    chpl_qthread_tls_t * data = chpl_qthread_get_tasklocal();
 
     data->task_filename = rarg->task_filename;
     data->task_lineno = rarg->lineno;
@@ -437,8 +413,16 @@ static aligned_t chapel_wrapper(void *arg)
     data->lock_filename = NULL;
     data->lock_lineno = 0;
 
+    if (rarg->countRunning) {
+        chpl_taskRunningCntInc(0, NULL);
+    }
+
     (*(chpl_fn_p)(rarg->fn))(rarg->args);
 
+    if (rarg->countRunning) {
+        chpl_taskRunningCntDec(0, NULL);
+    }
+
     return 0;
 }
 
@@ -448,8 +432,8 @@ static aligned_t chapel_wrapper(void *arg)
 // not use methods that require task context (e.g., task-local storage).
 void chpl_task_callMain(void (*chpl_main)(void))
 {
-    const chapel_wrapper_args_t wrapper_args = 
-        {chpl_main, NULL, NULL, 0, {c_sublocid_any, false}};
+    const chpl_qthread_wrapper_args_t wrapper_args = 
+        {chpl_main, NULL, NULL, 0, false, {c_sublocid_any_val, false}};
 
     qthread_debug(CHAPEL_CALLS, "[%d] begin chpl_task_callMain()\n", chpl_localeID);
 
@@ -466,13 +450,23 @@ void chpl_task_callMain(void (*chpl_main)(void))
     qthread_debug(CHAPEL_CALLS, "[%d] end chpl_task_callMain()\n", chpl_localeID);
 }
 
+void chpl_task_stdModulesInitialized(void)
+{
+    //
+    // It's not safe to call the module code to count the main task as
+    // running until after the modules have been initialized.  That's
+    // when this function is called, so now count the main task.
+    //
+    canCountRunningTasks = true;
+    chpl_taskRunningCntInc(0, NULL);
+}
+
 int chpl_task_createCommTask(chpl_fn_p fn,
                              void     *arg)
 {
 #ifndef QTHREAD_MULTINODE
-    pthread_t polling_thread;
-
-    return pthread_create(&polling_thread, NULL, (void *(*)(void *))fn, arg);
+    return pthread_create(&chpl_qthread_comm_pthread,
+                          NULL, (void *(*)(void *))fn, arg);
 #else
     return 0;
 #endif
@@ -489,26 +483,27 @@ void chpl_task_addToTaskList(chpl_fn_int_t     fid,
 {
     qthread_shepherd_id_t const here_shep_id = qthread_shep();
     chpl_bool serial_state = chpl_task_getSerial();
-    chapel_wrapper_args_t wrapper_args = 
-        {chpl_ftable[fid], arg, filename, lineno, {subloc, serial_state}};
+    chpl_qthread_wrapper_args_t wrapper_args = 
+        {chpl_ftable[fid], arg, filename, lineno, false,
+         {subloc, serial_state}};
+
+    assert(subloc != c_sublocid_none);
 
     PROFILE_INCR(profile_task_addToTaskList,1);
 
     if (serial_state) {
         syncvar_t ret = SYNCVAR_STATIC_EMPTY_INITIALIZER;
         qthread_fork_syncvar_copyargs_to(chapel_wrapper, &wrapper_args,
-                                         sizeof(chapel_wrapper_args_t), &ret,
+                                         sizeof(chpl_qthread_wrapper_args_t), &ret,
                                          here_shep_id);
         qthread_syncvar_readFF(NULL, &ret);
     } else if (subloc == c_sublocid_any) {
-        qthread_fork_syncvar_copyargs(chapel_wrapper, &wrapper_args,
-                                      sizeof(chapel_wrapper_args_t), NULL);
+        qthread_fork_copyargs(chapel_wrapper, &wrapper_args,
+                              sizeof(chpl_qthread_wrapper_args_t), NULL);
     } else {
-        if (subloc == c_sublocid_curr)
-            subloc = (c_sublocid_t) here_shep_id;
-        qthread_fork_syncvar_copyargs_to(chapel_wrapper, &wrapper_args,
-                                         sizeof(chapel_wrapper_args_t), NULL,
-                                         (qthread_shepherd_id_t) subloc);
+        qthread_fork_copyargs_to(chapel_wrapper, &wrapper_args,
+                                 sizeof(chpl_qthread_wrapper_args_t), NULL,
+                                 (qthread_shepherd_id_t) subloc);
     }
 }
 
@@ -533,75 +528,36 @@ void chpl_task_startMovedTask(chpl_fn_p      fp,
                               chpl_taskID_t  id,
                               chpl_bool      serial_state)
 {
-    assert(subloc != c_sublocid_curr);
+    assert(subloc != c_sublocid_none);
     assert(id == chpl_nullTaskID);
 
-    chapel_wrapper_args_t wrapper_args = 
-        {fp, arg, NULL, 0, {subloc, serial_state}};
+    chpl_qthread_wrapper_args_t wrapper_args = 
+        {fp, arg, NULL, 0, canCountRunningTasks, {subloc, serial_state}};
 
     PROFILE_INCR(profile_task_startMovedTask,1);
 
-#if 1
-    // We are timing out when the subloc is passed as 0 (zero).  Can
-    // we not time share tasks on a single shepherd?  Perhaps we can
-    // only time share as many tasks on a shepherd as that shepherd
-    // has workers?  For now, force the subloc to be "any".
-    subloc = c_sublocid_any;
-#endif
     if (subloc == c_sublocid_any) {
-        qthread_fork_syncvar_copyargs(chapel_wrapper, &wrapper_args,
-                                      sizeof(chapel_wrapper_args_t), NULL);
+        qthread_fork_copyargs(chapel_wrapper, &wrapper_args,
+                              sizeof(chpl_qthread_wrapper_args_t), NULL);
     } else {
-        qthread_fork_syncvar_copyargs_to(chapel_wrapper, &wrapper_args,
-                                         sizeof(chapel_wrapper_args_t), NULL,
-                                         (qthread_shepherd_id_t) subloc);
+        qthread_fork_copyargs_to(chapel_wrapper, &wrapper_args,
+                                 sizeof(chpl_qthread_wrapper_args_t), NULL,
+                                 (qthread_shepherd_id_t) subloc);
     }
 }
 
-c_sublocid_t chpl_task_getSubloc(void)
-{
-    return (c_sublocid_t) qthread_shep();
-}
+//
+// chpl_task_getSubloc() is in tasks-qthreads.h
+//
 
-void chpl_task_setSubloc(c_sublocid_t subloc)
-{
-    qthread_shepherd_id_t curr_shep;
+//
+// chpl_task_setSubloc() is in tasks-qthreads.h
+//
 
-    // Only change sublocales if the caller asked for a particular one,
-    // which is not the current one, and we're a (movable) task.
-    //
-    // Note: It's likely that this won't work in all cases where we need
-    //       it.  In particular, we envision needing to move execution
-    //       from sublocale to sublocale while initializing the memory
-    //       layer, in order to get the NUMA domain affinity right for
-    //       the subparts of the heap.  But this will be happening well
-    //       before tasking init and in any case would be done from the
-    //       main thread of execution, which doesn't have a shepherd.
-    //       The code below wouldn't work in that situation.
-    if (subloc != c_sublocid_curr &&
-        (curr_shep = qthread_shep()) != NO_SHEPHERD) {
-        chapel_tls_t * data = chapel_get_tasklocal_possibly_from_non_task();
-        if (data) {
-            data->chpl_data.requestedSubloc = subloc;
-        }
+//
+// chpl_task_getRequestedSubloc() is in tasks-qthreads.h
+//
 
-        if (subloc != c_sublocid_any &&
-            (qthread_shepherd_id_t) subloc != curr_shep) {
-            qthread_migrate_to((qthread_shepherd_id_t) subloc);
-        }
-    }
-}
-
-c_sublocid_t chpl_task_getRequestedSubloc(void)
-{
-    chapel_tls_t * data = chapel_get_tasklocal_possibly_from_non_task();
-    if (data) {
-        return data->chpl_data.requestedSubloc;
-    }
-    else {
-        return c_sublocid_any;
-    }
-}
 
 // Returns '(unsigned int)-1' if called outside of the tasking layer.
 chpl_taskID_t chpl_task_getId(void)
@@ -620,7 +576,7 @@ void chpl_task_sleep(int secs)
  * data segment holds a chpl_bool denoting the serial state. */
 chpl_bool chpl_task_getSerial(void)
 {
-    chapel_tls_t * data = chapel_get_tasklocal();
+    chpl_qthread_tls_t * data = chpl_qthread_get_tasklocal();
 
     PROFILE_INCR(profile_task_getSerial,1);
 
@@ -629,7 +585,7 @@ chpl_bool chpl_task_getSerial(void)
 
 void chpl_task_setSerial(chpl_bool state)
 {
-    chapel_tls_t * data = chapel_get_tasklocal();
+    chpl_qthread_tls_t * data = chpl_qthread_get_tasklocal();
     data->chpl_data.serial_state = state;
 
     PROFILE_INCR(profile_task_setSerial,1);
@@ -637,21 +593,28 @@ void chpl_task_setSerial(chpl_bool state)
 
 c_sublocid_t chpl_task_getNumSublocales(void)
 {
-    c_sublocid_t num_sublocs = (c_sublocid_t) qthread_num_shepherds();
-
     // FIXME: What we really want here is the number of NUMA
     // sublocales we are supporting.  For now we use the number of
     // shepherds as a proxy for that.
+
 #ifdef CHPL_LOCALE_MODEL_NUM_SUBLOCALES
-    return ((num_sublocs < CHPL_LOCALE_MODEL_NUM_SUBLOCALES)
-            ? num_sublocs
-            : CHPL_LOCALE_MODEL_NUM_SUBLOCALES);
+#if CHPL_LOCALE_MODEL_NUM_SUBLOCALES < 0
+#error Forced number of sublocales cannot be negative.
+#endif
+    if (CHPL_LOCALE_MODEL_NUM_SUBLOCALES == 0) {
+        return 0;
+    } else {
+        c_sublocid_t num_sublocs = (c_sublocid_t) qthread_num_shepherds();
+        return ((num_sublocs < CHPL_LOCALE_MODEL_NUM_SUBLOCALES)
+                ? num_sublocs
+                : CHPL_LOCALE_MODEL_NUM_SUBLOCALES);
+    }
 #else
-    return (c_sublocid_t) ((num_sublocs < 2) ? 0 : num_sublocs);
+    return (c_sublocid_t) qthread_num_shepherds();
 #endif
 }
 
-uint64_t chpl_task_getCallStackSize(void)
+size_t chpl_task_getCallStackSize(void)
 {
     PROFILE_INCR(profile_task_getCallStackSize,1);
 
@@ -666,13 +629,15 @@ uint32_t chpl_task_getNumQueuedTasks(void)
 
 uint32_t chpl_task_getNumRunningTasks(void)
 {
-    return qthread_readstate(WORKER_OCCUPATION);
-}                                                         /* 1, i.e. this one */
+    chpl_internal_error("chpl_task_getNumRunningTasks() called");
+    return 1;
+}
 
-// XXX: not sure what the correct value should be here!
 int32_t chpl_task_getNumBlockedTasks(void)
 {
-    return -1;
+    // This isn't accurate, but in the absence of better information
+    // it's the best we can do.
+    return 0;
 }
 
 // Threads
